function F = customfit(x,xdata)
% ---
% --- User supplied custom fit function called from 'showresult.m'
% --- 
% ---
% --- Standard parameter (used in all g2 fit functions):
% ---
% --- x(1) : Baseline
% --- x(2) : Contrast
% --- x(3) : Gamma = 1/tau
% ---
% ---
% --- Custom parameter (used only for custom g2 fit functions):
% ---
% --- x(4) : ...
% --- x(5) : ...
% --- ...
% ---
% ---
% --- Example : streched exponential fit
% ---
% --- F = x(1) + x(2) * exp( - 2 * (x(3) * xdata).^x(4) )
% ---
% --- x(4) is the streching exponent in this case
% ---
% ---
% --- General comment:
% --- A great problem in fitting is a good choice of starting parameters
% --- & to define sensible limits for these parameters. The success
% --- probability to have a converging fit is often determined by this
% --- choice. So be carefull!!!
% ---


% --- Single exponential in a homodyne detection scheme
% F = x(1) + x(2) * exp(- 2 * x(3) * xdata)                                ; % please uncomment if you enter a custom function

% --- Streched exponential in a homodyne detection scheme
% F = x(1) + x(2) * exp( - 2 * (x(3) * xdata).^x(4) )                      ;

% --- Please enter the custom fitting function here

% --- Double exponential fitting in a homodyne detection scheme
F = x(1) + x(2) * (      x(5)  * exp( - 2 * x(3) * xdata)             ...
                  + (1 - x(5)) * exp( - 2 * x(4) * xdata) ).^2           ; % double Exponential fitting

% ---
% EOF
