function [cstart,clow,chigh] = customfitparameter(xdata,ydata)
% ---
% --- General comment:
% --- A great problem in fitting is a good choice of starting parameters
% --- & to define sensible limits for these parameters. The success
% --- probability to have a converging fit is often determined by this
% --- choice. So be carefull!!!
% ---
% --- User supplied custom fit parameter called from 'showresult.m'
% --- 
% --- The fit function needs a set of starting fitting parameters.
% --- It works best if also a lower and upper limit for the fitting
% --- parameters is defined. Please make sure that the number of parameters
% --- match to the custom fit function!!!
% ---
% --- Standard parameter (used in all g2 fit functions):
% ---
% --- x(1) : Baseline
% --- x(2) : Contrast
% --- x(3) : Gamma1 = 1/tau1
% ---
% ---
% --- Custom parameter (used only for custom g2 fit functions):
% ---
% --- x(4) : Gamma2 = 1/tau2...
% --- x(5) : ratio
% --- ...
% ---
% --- The data can often supply some information about how the fitting
% --- parameters should be chosen. 

% ---
% --- This function is written for a double exponential custom
% --- fitting function of the form:
% --- F = x(1) + x(2) * (      x(5)  * exp( - 2 * x(3) * xdata)        ...
% ---                   + (1 - x(5)) * exp( - 2 * x(4) * xdata) ).^2      ;
% ---
 

% ---
% --- please comment this code if you enter custom fitting parameter 
% ---
     if (numel(xdata) > 3)
          baseline = sum(ydata(end-2:end)) / numel(ydata(end-2:end))      ; % take the average of the last 3 data points as baseline start parameter
          contrast = sum(ydata(1:3)) / numel(ydata(1:3)) - baseline       ; % take the average of the first 3 data points minus baseline as contrast start parameter
          contrast = max(0.01, contrast)                                  ; % make sure the contrast is positive
          gamma1   = 1 /2 * 1 / xdata(floor(1/3*numel(xdata)))            ; % put the starting gamma1 value at 1/3 of the time axis
          gamma2   = 1 /2 * 1 / xdata(ceil (2/3*numel(xdata)))            ; % put the starting gamma2 value at 2/3 of the time axis
          ratio    = 0.5                                                  ; % default ratio start parameter (don't know better)
     else
          baseline = 1.0                                                  ; % emergency default baseline start parameter
          contrast = 0.3                                                  ; % emergency default contrast start parameter
          gamma1   = 1.0                                                  ; % emergency default gamma1 start parameter
          gamma2   = 10.                                                  ; % emergency default gamma2 start parameter
          ratio    = 0.5                                                  ; % emergency default ratio start parameter
     end
     cstart = [baseline contrast gamma1 gamma2 ratio]                     ; % starting fitting parameters
     clow   = [baseline/2 contrast/5 gamma1/1e03 gamma2/1e03 1e-06]       ; % define lower limits
     chigh  = [baseline*2 contrast*5 gamma1*1e03 gamma2*1e03 1-1e-06]     ; % define higher limits
     clear baseline contrast gamma1 gamma2 ratio                          ;


% ---
% EOF
