function loadbatchinfo(varargin)
% --- LOADBATCHINFO Load batchinfo file.
%
% --- Zhang Jiang
% --- $Revision: 1.0 $  $Date: 2004/12/03 $
% --- $Revision: 1.1 $  $Date: 2005/09/26 $ by MS
% ---            --> allow to call loadbatchinfo in a batchmode
% --- $Revision: 1.2 $  $Date: 2005/12/05 $ by MS
% ---            --> allow to use already background subtracted data
% ---            --> improve the handling of beam_i & beam_i_vacuum
% ---


%==========================================================================
% --- go the previous stored path
%==========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
currentPath  = pwd                                                         ;
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                     ;
end


%==========================================================================
% --- check if batchmode is used or else open GUI dialog box
%==========================================================================
if ( nargin == 1 )                                                           % check if a batchinfo file name is supplied (a better check is needed)
    file = varargin{1}                                                     ; % assign the input variable to file
    restorePath(currentPath)                                               ; % restore path
else
    % --- use try-catch to go the image directory in case user delete the 
    % --- image path while xpcsgui is in process
    try
        go_path_str = ['cd ','''',ccdimginfo.imgPath,'''']                 ;
        eval(go_path_str)                                                  ;
    catch
        go_path_str = ['cd ','''',currentPath,'''']                        ;
        eval(go_path_str)                                                  ;
    end
    % --- dlg to select file
    [filename, filepath] = uigetfile(                                   ...
        {'*.batchinfo', 'Batchinfo Files (*.batchinfo)'},               ...
         'Select Info File',                                            ...
         'MultiSelect', 'off')                                             ;
    if isequal([filename,filepath],[0,0])
        restorePath(currentPath)                                           ;
        return                                                             ;
    end
    file = fullfile(filepath,filename)                                     ; % the whole filename
    restorePath(currentPath)                                               ; % restore path
end


%==========================================================================
% --- open info file loading information
%==========================================================================
[fid,message] = fopen(file)                                                ; % open file
if fid == -1                                                                 % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(fid)                                                            ;
    return                                                                 ;
end


%==========================================================================
% --- reset everyting (kill all previous figures related to xpcsgui and
% --- clear ccdimginfo in main xpcsgui figure and in current workspace
%==========================================================================
resetxpcsgui                                                               ;                
clear ccdimginfo                                                           ; 


%==========================================================================
% --- get info file extension
%==========================================================================
[pathstr,name,ext,versn]      = fileparts(file)                            ;
ccdimginfo.batchinfoExtension = ext                                        ;
ccdimginfo.imgPath            = pathstr                                    ;
ccdimginfo.batchinfoFile      = file                                       ;


%==========================================================================
% --- read batchinfo file
%==========================================================================
while feof(fid) == 0
    scanline = fgetl(fid)                                                  ;
    equal_pos = find(scanline == '=')                                      ;
    left_str = scanline(1:equal_pos-1)                                     ;
    left_str(findstr(left_str,' '))=''                                     ;
    right_str = scanline(equal_pos+1:end)                                  ;
    eval(['ccdimginfo.',left_str,'=','''',right_str,''';'])                ;
end
fclose(fid)                                                                ; % close file


%==========================================================================
% --- process the batchinfo information
%==========================================================================
ccdimginfo.info_name(findstr(ccdimginfo.info_name,'"')) = ''               ;
ccdimginfo.parent(findstr(ccdimginfo.parent,'"'))       = ''               ;
ccdimginfo.child(findstr(ccdimginfo.child,'"'))         = ''               ;
ccdimginfo.name(findstr(ccdimginfo.name,'"'))           = ''               ;
ccdimginfo.name(find(ccdimginfo.name == ' '))           = ''               ; % remove space
ccdimginfo.suffix(findstr(ccdimginfo.suffix,'"'))       = ''               ;


ccdimginfo.samx             = str2num(ccdimginfo.samx)                     ;
ccdimginfo.samz             = str2num(ccdimginfo.samz)                     ;
ccdimginfo.samth            = str2num(ccdimginfo.samth)                    ;
ccdimginfo.bstop            = str2num(ccdimginfo.bstop)                    ;
ccdimginfo.ccdz             = str2num(ccdimginfo.ccdz)                     ;
ccdimginfo.ccdx             = str2num(ccdimginfo.ccdx)                     ;


ccdimginfo.detector         = str2num(ccdimginfo.detector)                 ;
ccdimginfo.rr               = str2num(ccdimginfo.rr)                       ;
ccdimginfo.x0               = str2num(ccdimginfo.x0)                       ; 
ccdimginfo.y0               = str2num(ccdimginfo.y0)                       ;
ccdimginfo.ccdx0            = str2num(ccdimginfo.ccdx0)                    ;
ccdimginfo.ccdz0            = str2num(ccdimginfo.ccdz0)                    ;
ccdimginfo.hgap             = str2num(ccdimginfo.hgap)                     ;
ccdimginfo.vgap             = str2num(ccdimginfo.vgap)                     ;
ccdimginfo.airgap           = str2num(ccdimginfo.airgap)                   ;


ccdimginfo.geometry         = str2num(ccdimginfo.geometry)                 ; % needs to be done before ccdimginfo.nominal_angle
if ccdimginfo.geometry == 1
    ccdimginfo.nominal_angle    = str2num(ccdimginfo.nominal_angle)        ;
    ccdimginfo.xspec            = str2num(ccdimginfo.xspec)                ;
    ccdimginfo.ccdxspec         = str2num(ccdimginfo.ccdxspec)             ;
    ccdimginfo.yspec            = str2num(ccdimginfo.yspec)                ;
    ccdimginfo.ccdzspec         = str2num(ccdimginfo.ccdzspec)             ;
elseif ccdimginfo.geometry == 0                                            % define specular beam to be -1 for transmission geometry
    ccdimginfo.nominal_angle    = -1                                       ; % define nominal angle to be -1 for transmission geometry
    ccdimginfo.xspec            = -1                                       ;
    ccdimginfo.ccdxspec         = -1                                       ;
    ccdimginfo.yspec            = -1                                       ;
    ccdimginfo.ccdzspec         = -1                                       ;
end


ccdimginfo.topup(findstr(ccdimginfo.topup,'"')) = ''                       ;
ccdimginfo.energy           = str2num(ccdimginfo.energy)                   ; % x-ray energy in keV
ccdimginfo.ccd_speed        = str2num(ccdimginfo.ccd_speed)                ;
ccdimginfo.kinetics         = str2num(ccdimginfo.kinetics)                 ;
if ccdimginfo.kinetics == 1
    ccdimginfo.kinwinsize   = str2num(ccdimginfo.kinwinsize)               ;
    ccdimginfo.slicetop     = str2num(ccdimginfo.slicetop)                 ;
else
    ccdimginfo.kinwinsize   = -1                                           ;
    ccdimginfo.slicetop     = -1                                           ;
end


ccdimginfo.rows             = str2num(ccdimginfo.rows)                     ;
ccdimginfo.cols             = str2num(ccdimginfo.cols)                     ;
ccdimginfo.col_beg          = str2num(ccdimginfo.col_beg)                  ;
ccdimginfo.col_end          = str2num(ccdimginfo.col_end)                  ;
ccdimginfo.row_beg          = str2num(ccdimginfo.row_beg)                  ;
ccdimginfo.row_end          = str2num(ccdimginfo.row_end)                  ;
if ( ccdimginfo.detector == 6 )
    ccdimginfo.col_beg      = ccdimginfo.col_beg + 1                       ;
    ccdimginfo.col_end      = ccdimginfo.col_end + 1                       ;
    ccdimginfo.row_beg      = ccdimginfo.row_beg + 1                       ;
    ccdimginfo.row_end      = ccdimginfo.row_end + 1                       ;    
end


% --- information for the data images
ccdimginfo.ndata0           = str2num(ccdimginfo.ndata0)                   ;
ccdimginfo.ndataend         = str2num(ccdimginfo.ndataend)                 ;
ccdimginfo.preset           = str2num(ccdimginfo.preset)                   ;


% --- information for the dark images
% --- allow for special treatment of the SMD camera
if ( isfield(ccdimginfo,'ndark0') == 1 )
    ccdimginfo.ndark0           = str2num(ccdimginfo.ndark0)               ;
else
    ccdimginfo.ndark0           = 0.0 * ccdimginfo.ndata0 + 99999          ; % dummy value
end
if ( isfield(ccdimginfo,'ndarkend') == 1 )
    ccdimginfo.ndarkend         = str2num(ccdimginfo.ndarkend)             ;
else
    ccdimginfo.ndarkend         = 0.0 * ccdimginfo.ndataend + 99998        ; % dummy value ( ndarkend < ndark0 !!! )
end
if ( isfield(ccdimginfo,'dark_preset') == 1 )
    ccdimginfo.dark_preset      = str2num(ccdimginfo.dark_preset)          ;
else
    ccdimginfo.dark_preset      = ccdimginfo.preset                        ; % set dark preset to data preset
end


% --- ring current values at beginning and end of a ccdseries
ccdimginfo.ring_i_beg       = str2num(ccdimginfo.ring_i_beg)               ; % ring current at beginning of ccd series
ccdimginfo.ring_i_end       = str2num(ccdimginfo.ring_i_end)               ; % ring current at beginning of ccd series
ccdimginfo.ring_i           = str2num(ccdimginfo.ring_i)                   ; % this is the averaged ring current value for each ccd series


% --- here intensity & ring current values from takeflux are arranged
% --- these values are optional since takeflux is not mandatory !!!
% --- create two flags (ccdimginfo.iflux & ccdimginfo.tflux) to take
% --- care of this problem !!!
% ---
% --- incident flux measurements (if beam exists than ring exists too!)
% ---
if ( isfield(ccdimginfo,'beam_i_vacuum') == 1 )
    ccdimginfo.iflux             = 1                                       ; % set incident flux measurement flag to one
    ccdimginfo.beam_i_vacuum     = str2num(ccdimginfo.beam_i_vacuum)       ; % this is the incident flux at the beam stop
    if ( numel(ccdimginfo.beam_i_vacuum) ~= numel(ccdimginfo.preset) )       % if it was not measured for each ccd series
        ccdimginfo.beam_i_vacuum = 0.0 * ccdimginfo.preset              ...
                                 + mean(ccdimginfo.beam_i_vacuum)          ; % create vector with the averaged incident flux
    end
else
    ccdimginfo.iflux             = 0                                       ; % set incident flux measurement flag to zero
    ccdimginfo.beam_i_vacuum     = 0.0*ccdimginfo.preset + exp(1)*1.0e+009 ; % create a vector of dummy values (assume it is e * I_trans)
end
% ---
if ( isfield(ccdimginfo,'ring_i_vacuum') == 1 )
    ccdimginfo.ring_i_vacuum     = str2num(ccdimginfo.ring_i_vacuum)       ;
    if ( numel(ccdimginfo.ring_i_vacuum) ~= numel(ccdimginfo.preset) )       % if it was not measured for each ccd series
        ccdimginfo.ring_i_vacuum = 0.0 * ccdimginfo.preset              ...
                                 + mean(ccdimginfo.ring_i_vacuum)          ; % create vector with the averaged ring current during the measurement of the incident flux
    end
else
    ccdimginfo.ring_i_vacuum     =  0.0 * ccdimginfo.preset + 1.0e+002     ; % create a vector of dummy values
end
% ---
% --- transmitted flux measurements (if beam exists than ring exists too!)
% --
if ( isfield(ccdimginfo,'beam_i') == 1 )
    ccdimginfo.tflux      = 1                                              ; % set transmitted flux measurement flag to one
    ccdimginfo.beam_i     = str2num(ccdimginfo.beam_i)                     ; % this is the transmitted intensity through the sample
    if ( numel(ccdimginfo.beam_i) ~= numel(ccdimginfo.preset) )              % if it was not measured for each ccd series
        ccdimginfo.beam_i = 0.0 * ccdimginfo.preset                     ...
                          + mean(ccdimginfo.beam_i)                        ; % create vector with the averaged transmitted flux
    end
else
    ccdimginfo.tflux      = 0                                              ; % set transmitted flux measurement flag to zero
    ccdimginfo.beam_i     = 0.0 * ccdimginfo.preset + 1.0e+009             ; % create a vector of dummy values
end
% ---
if ( isfield(ccdimginfo,'ring_i_sample') == 1 )
    ccdimginfo.ring_i_sample     = str2num(ccdimginfo.ring_i_sample)       ;
    if ( numel(ccdimginfo.ring_i_sample) ~= numel(ccdimginfo.preset) )       % if it was not measured for each ccd series
        ccdimginfo.ring_i_sample = 0.0 * ccdimginfo.preset              ...
                                 + mean(ccdimginfo.ring_i_sample)          ; % this is the averaged ring current during the measurement of the transmitted flux
    end
else
    ccdimginfo.ring_i_sample    = 0.0 * ccdimginfo.preset + 1.0e+002       ; % create a vector of dummy values
end


% --- determine wavelength of x-ray in unit of Angstrom
PLANCK                = 6.626068E-34                                       ;
SPEEDOFLIGHT          = 299792458                                          ;
ELEMENTARYCHARGE      = 1.60217646E-19                                     ;
ccdimginfo.wavelength = 1e10 * PLANCK * SPEEDOFLIGHT                    ...
                      /(ELEMENTARYCHARGE*1000*ccdimginfo.energy)           ;
clear PLANK SPEEDOFLIGHT ELEMENTARYCHARGE


% --- determine the batch start and end time
start_time              = ccdimginfo.start_time                            ;
end_time                = ccdimginfo.end_time                              ;
posDoubleQuoteStartTime = findstr('"',start_time)                          ;
posDoubleQuoteEndTime   = findstr('"',end_time)                            ;
ccdimginfo.start_time   = cell(length(ccdimginfo.ndata0),1)                ;
ccdimginfo.end_time     = cell(length(ccdimginfo.ndata0),1)                ;
for iBatch = 1:length(ccdimginfo.ndata0)
    ccdimginfo.start_time{iBatch} =                                     ...
        start_time( posDoubleQuoteStartTime(2*iBatch-1) + 1             ... 
                  : posDoubleQuoteStartTime(2*iBatch)   - 1)               ;
    ccdimginfo.end_time{iBatch}   =                                     ...
        end_time  ( posDoubleQuoteEndTime  (2*iBatch-1) + 1             ...
                  : posDoubleQuoteEndTime  (2*iBatch)   - 1)               ;
end


% --- for kinetics mode, determine the first and last usable slices;
% --- & determine positions of each slice and save to ccdimginfo.sliceinfo
if ccdimginfo.kinetics == 1                                                 % kinetic mode
    ccdimginfo.firstslice = 2                                              ;
    ccdimginfo.lastslice  = floor(ccdimginfo.row_end/ccdimginfo.kinwinsize);      
    shiftOffset           = ccdimginfo.slicetop-ccdimginfo.row_end         ; % offset for the first used row (negative value!!!)
    for iSlice = 1 : ccdimginfo.lastslice
        sliceInfo(iSlice,1) = shiftOffset      ...
                            + (iSlice-1) * ccdimginfo.kinwinsize           ; % bottom row of slice
        sliceInfo(iSlice,2) = shiftOffset - 1  ...
                            +  iSlice    * ccdimginfo.kinwinsize           ; % top row of slice
    end
    clear shiftOffset j
else                                                                        % full frame or roi mode
    ccdimginfo.firstslice   = -1                                           ;
    ccdimginfo.lastslice    = -1                                           ;
    sliceInfo(1,1)= ccdimginfo.row_beg                                     ;
    sliceInfo(1,2)= ccdimginfo.row_end                                     ;
end
ccdimginfo.sliceinfo = sliceInfo                                           ;
clear sliceInfo


% --- get detector settings
[ccdimginfo] = detectorinfo(ccdimginfo)                                    ;


% --- lower-level discrimination; 0 == no lld; minus means absolute lld,
% --- plus means relative lld, etc., -12: 12 ADU, 4 : 4 x dark RMS;
ccdimginfo.lld                = -15                                        ;
ccdimginfo.dpl                = 4                                          ; % # of delays per multi tau level
ccdimginfo.slicedpl           = 4                                          ; % # of slice delays per multi tau level
ccdimginfo.ssn                = 0                                          ; % use smoothed symetric normalization in dynamic analysis
ccdimginfo.ssnmin             = 100                                        ; % mininum # of pixels to smooth
ccdimginfo.memory             = 750                                        ; % computer memory used to do analysis 
ccdimginfo.analysistype       = 1                                          ; % perform 0) static analysis 1) dynamic analysis
ccdimginfo.savepath           = fullfile(ccdimginfo.imgPath,'result')      ; % default path to save analysis result (subdirectory result below image file)               


% --- initialize the batches, ndata0, ndataend, ndark, ndarkend
ccdimginfo.batchestodo   = 1:length(ccdimginfo.ndata0)                     ;
ccdimginfo.ndata0todo    = ccdimginfo.ndata0                               ;
ccdimginfo.ndataendtodo  = ccdimginfo.ndataend                             ;
ccdimginfo.ndark0todo    = ccdimginfo.ndark0                               ;
ccdimginfo.ndarkendtodo  = ccdimginfo.ndarkend                             ;


% --- generate file names for all the batches and store in a cell structure
ccdimginfo.imagefile = cell(length(ccdimginfo.ndata0),1)                   ;
if ( ccdimginfo.detector ~= 6 )
    ccdimginfo.imagefile = cell(length(ccdimginfo.ndata0),1)               ;
    for iBatch = 1:length(ccdimginfo.ndata0)
        frameBeg = '00000'                                                 ;
        frameEnd = '00000'                                                 ;
        frameBeg(5-length(num2str(ccdimginfo.ndata0(iBatch)))+1:end) =  ...
                 num2str(num2str(ccdimginfo.ndata0(iBatch)))               ;
        frameEnd(5-length(num2str(ccdimginfo.ndarkend(iBatch)))+1:end) =...
                 num2str(num2str(ccdimginfo.ndarkend(iBatch)))             ;
        ccdimginfo.imagefile{iBatch} = fullfile(ccdimginfo.imgPath,     ...
                        [ccdimginfo.name,frameBeg,'-',frameEnd,'.imm'])    ;
    end
else
    ccdimginfo.imagefile = cell(length(ccdimginfo.ndata0),1)               ;
    for iBatch = 1:length(ccdimginfo.ndata0)
        frameBeg = sprintf('%04i',ccdimginfo.ndata0(iBatch))               ;
        frameEnd = sprintf('%04i',ccdimginfo.ndataend(iBatch))             ;
        ccdimginfo.imagefile{iBatch} = fullfile(ccdimginfo.imgPath,     ...
                        [ccdimginfo.name,frameBeg,'-',frameEnd,'.imm'])    ;
    end
    ccdimginfo.darkfile = cell(length(ccdimginfo.ndark0),1)                ;
    for iBatch = 1:length(ccdimginfo.ndark0)
        frameBeg = sprintf('%04i',ccdimginfo.ndark0(iBatch))               ;
        frameEnd = sprintf('%04i',ccdimginfo.ndarkend(iBatch))             ;
        ccdimginfo.darkfile{iBatch} = fullfile(ccdimginfo.imgPath,      ...
                        [ccdimginfo.name,frameBeg,'-',frameEnd,'.imm'])    ;
    end    
end


% --- define static and dynamic q phi partition information. Default
% values are listed below. Partition methods:
% 1. from existing result file (not working yet)
% 2. evenly spaced
% 3. equal dq/q (q & q_z partition); evenly spaced for phi partiton
% 4. manually set (not working yet)
% 5. no paritions (use whole image)


% Define methods for sq,sphi,dq,dphi
ccdimginfo.sqMethod         = 2                                            ;
ccdimginfo.sphiMethod       = 2                                            ;
ccdimginfo.dqMethod         = 2                                            ;
ccdimginfo.dphiMethod       = 2                                            ;


% Define existing file name and path for method 1
ccdimginfo.sqfile           = ''                                           ;
ccdimginfo.sphifile         = ''                                           ;
ccdimginfo.dqfile           = ''                                           ;
ccdimginfo.dphifile         = ''                                           ;


% Define default number of partitions for method 2,3 & 5
ccdimginfo.snoq             = 100                                          ;
ccdimginfo.snophi           = 1                                            ;
ccdimginfo.dnoq             = 36                                           ;
ccdimginfo.dnophi           = 1                                            ;
% ---
if ( ccdimginfo.sqMethod   == 5 )
    ccdimginfo.snoq   = 1                                                  ;
end
if ( ccdimginfo.sphiMethod == 5 )
    ccdimginfo.snophi = 1                                                  ;
end
if ( ccdimginfo.dqMethod   == 5 )
    ccdimginfo.dnoq   = 1                                                  ;
end
if ( ccdimginfo.dqMethod   == 5 )
    ccdimginfo.dnophi = 1                                                  ;
end


% Define q,phi span for method 4
ccdimginfo.sqspanstr        = ''                                           ;
ccdimginfo.sphispanstr      = ''                                           ;
ccdimginfo.dqspanstr        = ''                                           ;
ccdimginfo.dphispanstr      = ''                                           ;


% for any partition method, generate partition q and phi points
ccdimginfo.sqspan           = []                                           ;
ccdimginfo.sphispan         = []                                           ;
ccdimginfo.dqspan           = []                                           ;
ccdimginfo.dphispan         = []                                           ;


% --- define mask information method:
% 1. no mask (use all pixles) 
% 2. new mask
% 3. from existing result file
ccdimginfo.maskMethod       = 1                                            ;            


% --- define mask points for the whole slice/image 
% --- (nx2 matrix to define the mask polygon)
xpixels = ccdimginfo.col_end-ccdimginfo.col_beg+1                          ;
if ccdimginfo.kinetics == 1
    ypixels = ccdimginfo.kinwinsize                                        ;
else
    ypixels = ccdimginfo.row_end-ccdimginfo.row_beg+1                      ;
end
ccdimginfo.maskpoints = [247      128      ;                              ...
                         518      128      ;                              ...
                         518      1        ;                              ...
                         247      1         ]                                 ;
                     
% --- define mask roi as corner points of the whole slice/image roi
if ccdimginfo.kinetics == 1
    ccdimginfo.maskroi = [1                  1                    ;     ...
                          xpixels            ccdimginfo.kinwinsize ]       ;
else
    ccdimginfo.maskroi = [1                  1                 ;        ...
                          xpixels            ypixels            ]          ;
end

                    
% --- define the DEFAULT logical (1/0) mask, and save it to a 2-D matrix
% ccdimginfo.usermask which has the same size of slice/image, and which is
% not a sparse matrix, since most of the time also the whole slice/image is
% choosen by user so that the sparse matrix is usally much larger than
% non-sparse matrix.
ccdimginfo.usermask = single(ones(ypixels,xpixels))                        ;
clear xpixels ypixels                                                      ;

% --- if from existing result file
ccdimginfo.maskfile         = ''                                           ;


% --- define some switches for function 'showresult.m'
ccdimginfo.ASCIIon     = 1                                                 ; % switch for ASCII output ( 0 : 'off' ; 1 : 'on' ) (headers to be improved)
ccdimginfo.FIT1on      = 1                                                 ; % switch for single   exponential data fitting ( 0 : 'off' ; 1 : 'on' )
ccdimginfo.FIT2on      = 0                                                 ; % switch for streched exponential data fitting ( 0 : 'off' ; 1 : 'on' ) 
ccdimginfo.CUSTOMFITon = 0                                                 ; % switch for custom data fitting ( 0 : 'off' ; 1 : 'on' ) 
ccdimginfo.PNGon       = 0                                                 ; % switch for PNG   output ( 0 : 'off' ; 1 : 'on' ) (!!! very slow !!! ~180s)
ccdimginfo.WATERFALLon = 0                                                 ; % switch to show waterfall plot( 0 : 'off' ; 1 : 'on' )


% --- define some values & switches for the thickness normalization
% normalization options
% 0 : only I_norm_0 = I_meas / adupphot / efficiency / exposure time         % [general case]
% 1 : I_norm_1 = I_norm_0 / ( (dpix_x*dpix_y)/rr^2 )                         % [transmission mode]
% 2 : I_norm_2 = I_norm_1 / I_trans                                          % [transmission mode]
% 3a: I_norm_3 = I_norm_2 / thickness or                                     % [transmission mode]
% 3b: I_norm_3 = I_norm_2 / ( atten_length*log(I_0/I_trans) )                % [transmission mode]
% 4 : I_norm_4 = I_norm_0 / I_0                                              % [reflection mode]
ccdimginfo.normalization = 0                                               ; % switch to keep track of normalization type [0,1,2,3,4]
% ---
ccdimginfo.thicknessflag = 0                                               ; % switch to keep track if thickness value or attenuation length is used [0,1,2]
ccdimginfo.atten_length  = -1                                              ; % sample attenuation length in [mm]
ccdimginfo.thickness     = -1                                              ; % sample thickness in [mm]


%==========================================================================
% --- save ccdimginfo to figure
%==========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;


%==========================================================================
% --- update message window in xpcsmain figure
%==========================================================================
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',{' ';'   Click pushbuttons on Control Panel to view and change batch information, analysis and system settings before clicking Run to start analysis.'},...
    'HorizontalAlignment','left')                                          ;

hPushbuttonLoad      = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonLoad')      ;
hPushbuttonBatchinfo = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonBatchinfo') ;
hPushbuttonAnalysis  = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonAnalysis')  ;
hPushbuttonSystem    = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonSystem')    ;
hPushbuttonRun       = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonRun')       ;
hPushbuttonStop      = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonStop')      ;
hPushbuttonResult    = findall(hFigXPCSMain,'Tag','xpcsmain_PushbuttonResult')    ;

set(hPushbuttonLoad     ,'Enable','on')                                    ;
set(hPushbuttonBatchinfo,'Enable','on')                                    ;
set(hPushbuttonAnalysis ,'Enable','on')                                    ;
set(hPushbuttonSystem   ,'Enable','on')                                    ;
set(hPushbuttonRun      ,'Enable','on')                                    ;
set(hPushbuttonStop     ,'Enable','off')                                   ;
set(hPushbuttonResult   ,'Enable','on')                                    ;


%==========================================================================
% --- restore to current path
%==========================================================================
function restorePath(currentPath)
path_str = ['cd ','''',currentPath,'''']                                   ;
eval(path_str)                                                             ;


% ---
% EOF
