function showaverageresult
% --- SHOWAVERAGERESULT averages & shows previously obtained results
% --- from data with has been taken with equal exposure times
% ---
% --- Usage:
% --- [xpcsgui should be running to obtain the structure 'ccdimginfo']
% --- Usage (in the future a button in 'Viewresult' is planned):
% --- showaverageresult
% ---
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/02/16 $
% ---


% =========================================================================
% --- define switch SUPERTITLE
% =========================================================================
SUPERTITLEon = 1                                                           ;


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;


% =========================================================================
% --- check % if necessary define some switches
% =========================================================================
if ( isfield(ccdimginfo,'ASCIIon') == 1 )
    ASCIIon     = ccdimginfo.ASCIIon                                       ; % switch for ASCII output ( 0 : 'off' ; 1 : 'on' ) (headers to be improved)
else
    ASCIIon     = 1                                                        ; % default value
end
if ( isfield(ccdimginfo,'FIT1on') == 1 )
    FIT1on      = ccdimginfo.FIT1on                                        ; % switch for single   exponential data fitting ( 0 : 'off' ; 1 : 'on' )
else
    FIT1on      = 1                                                        ; % default value
end
if ( isfield(ccdimginfo,'FIT2on') == 1 )
    FIT2on      = ccdimginfo.FIT2on                                        ; % switch for stretched exponential data fitting ( 0 : 'off' ; 1 : 'on' ) 
else
    FIT2on      = 0                                                        ; % default value 
end
if ( isfield(ccdimginfo,'CUSTOMFITon') == 1 )
    CUSTOMFITon = ccdimginfo.CUSTOMFITon                                   ; % switch for custom data fitting( 0 : 'off' ; 1 : 'on' )
else
    CUSTOMFITon = 0                                                        ; % default value
end
if ( isfield(ccdimginfo,'PNGon') == 1 )
    PNGon       = ccdimginfo.PNGon                                         ; % switch for PNG   output ( 0 : 'off' ; 1 : 'on' ) (!!! very slow !!! ~180s)
else
    PNGon       = 0                                                        ; % default value
end
if ( isfield(ccdimginfo,'WATERFALLon') == 1 )
    WATERFALLon = ccdimginfo.WATERFALLon                                   ; % switch to show waterfall plot( 0 : 'off' ; 1 : 'on' )
else
    WATERFALLon = 0                                                        ; % default value
end
if ( isfield(ccdimginfo,'PLOTG1on') == 1 )
    PLOTG1on = ccdimginfo.PLOTG1on                                         ; % switch to show waterfall plot( 0 : 'off' ; 1 : 'on' )
else
    PLOTG1on = 0                                                           ; % default value
end


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1 = ccdimginfo.maskroi(1,2)                                               ; % row    start of ROI
R2 = ccdimginfo.maskroi(2,2)                                               ; % row    end   of ROI
C1 = ccdimginfo.maskroi(1,1)                                               ; % column start of ROI
C2 = ccdimginfo.maskroi(2,1)                                               ; % column end   of ROI


% =========================================================================
% --- define comment in figure title line
% =========================================================================
str1 = ccdimginfo.name                                                     ;
str2 = 'AVERAGES'                                                          ;
figureTitleLineName = [str1,str2]                                          ;
clear str1 str2                                                            ;


% =========================================================================
% --- define starting figure number & some figure properties
% =========================================================================
fignumber        = 1                                                       ;
ScreenDimensions = get(0,'ScreenSize')                                     ;
xScreenSize      = ScreenDimensions(3)                                     ;
yScreenSize      = ScreenDimensions(4)                                     ;
clear ScreenDimensions                                                     ;
if ( xScreenSize > 1200 )
    xFigSize         = 960                                                 ;
    yFigSize         = 3/4 * xFigSize                                      ;
else
    xFigSize         = 720                                                 ;
    yFigSize         = 3/4 * xFigSize                                      ;
end
xFigShift    = 25                                                          ;
yFigShift    = 25                                                          ;
yFigMenuSize = 75                                                          ;


% =========================================================================
% --- before starting, close previously displayed results
% =========================================================================
delete(findall(0,'Tag','SAR_staticaverages'))                              ;
delete(findall(0,'Tag','SAR_intensityVSq'))                                ;
delete(findall(0,'Tag','SAR_intensityVSphi'))                              ;
for i = 1 : 10
    figuretag = sprintf('SAR_correlationfigure_%02i',i)                    ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
delete(findall(0,'Tag','SAR_fit1results'))                                 ;
delete(findall(0,'Tag','SAR_fit2results'))                                 ;
delete(findall(0,'Tag','SAR_customfitresults'))                            ;
delete(findall(0,'Tag','SAR_waterfall'))                                   ;
for i = 1 : 10
    figuretag = sprintf('SAR_intermediatescatteringfigure_%02i',i)         ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag
% ---
delete(findall(0,'Tag','staticaverages'))                                  ;
delete(findall(0,'Tag','intensityVSq'))                                    ;
delete(findall(0,'Tag','intensityVSphi'))                                  ;
for i = 1 : 10
    figuretag = sprintf('correlationfigure_%02i',i)                        ;
    delete(findall(0,'Tag',figuretag))                                     ;    
end
delete(findall(0,'Tag','fit1results'))                                     ;
delete(findall(0,'Tag','fit2results'))                                     ;
delete(findall(0,'Tag','customfitresults'))                                ;
delete(findall(0,'Tag','waterfall'))                                       ;
for i = 1 : 10
    figuretag = sprintf('intermediatescatteringfigure_%02i',i)             ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag
% ---
delete(findall(0,'Tag','SMR_staticaverages'))                              ;
delete(findall(0,'Tag','SMR_intensityVSq'))                                ;
delete(findall(0,'Tag','SMR_intensityVSphi'))                              ;
for i = 1 : 10
    figuretag = sprintf('SMR_correlationfigure_%02i',i)                    ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
delete(findall(0,'Tag','SMR_fit1results'))                                 ;
delete(findall(0,'Tag','SMR_fit2results'))                                 ;
delete(findall(0,'Tag','SMR_customfitresults'))                            ;
for i = 1 : 10
    figuretag = sprintf('SMR_intermediatescatteringfigure_%02i',i)         ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag


% =========================================================================
% --- choose only batches that were analyzed with the same settings
% --- some of Simon's batches are not compliant, so the exposure time
% --- of these batches has to be compared too !!!
% =========================================================================
b2average = ccdimginfo.batchestodo                                         ; % a vector with possible batch numbers
if ( numel(b2average) >= 2 )                                                 % check if batches with equal exposure times exist
    for i = 1 : numel(b2average)
        preset(i) = ccdimginfo.preset(b2average(i))                        ;
        n = 0                                                              ;
        for j = i : numel(b2average)
            if ( preset(i) == ccdimginfo.preset(b2average(j)) )
                n = n + 1                                                  ;
            end
        end
        Equals(i) = n                                                      ;
    end
    [noEqualBatches,indexOfMostEqualBatches] = max(Equals)                 ;
    preset0 = ccdimginfo.preset(b2average(indexOfMostEqualBatches))        ;
    clear indexOfMostEqualBatches noEqualBatches Equals n preset           ;
    % ---
    n = 0                                                                  ;
    for i = 1 : numel(b2average)
        if ( ccdimginfo.preset(b2average(i)) == preset0 )
            n = n + 1                                                      ;
            Compliant(n) = i                                               ;
        end
    end
    if ( exist('Compliant','var') == 1 )
        b2average = b2average(Compliant)                                   ; % keep only compliant batches in the list
    end
end
clear Compliant n preset1                                                  ;


% =========================================================================
% --- Static averages
% --- Plot 1: averaged image (top left: linear ; bottom left: logarithmic
% --- top right: integrated intensity vs. time)
% =========================================================================
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber)     ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')             ;
pause(0.01)                                                                ;
t1 = clock                                                                 ;
% ---
hFigSAR_SA = figure('Name',[figureTitleLineName,' - static averages']    ...
      ,'Tag','SAR_staticaverages')                                         ;
clf; cla                                                                   ;
% ---
figLeft   = 125                                                            ; % absolute start position of the left border of the figure in pixel
figHeight =  50                                                            ; % absolute start position of the lower border of the figure in pixel
set(gcf,'Position',[figLeft figHeight xFigSize yFigSize])                  ;
% ---
% --- top subplot
% ---
figure(hFigSAR_SA)                                                         ;
subplot(2,2,1)                                                             ;
% ---
aItSize = size(ccdimginfo.result.aIt{b2average(1)} .* ccdimginfo.usermask) ;
aIt = zeros(aItSize)                                                       ;
for i = 1 : numel(b2average)
    aIt = aIt + ccdimginfo.result.aIt{b2average(i)} .* ccdimginfo.usermask ;
end
% ---
imagesc(aIt)                                                               ;
% title('Image of the average intensity distribution','FontSize',14)         ; 
xlabel('column no.','FontSize',14)                                         ;
ylabel('row no.','FontSize',14)                                            ;
set(gca,'FontSize',12,'Linewidth',1.5)                                     ;
set(gca,'YDir','normal')                                                   ;
pause(0.010)                                                               ;
% ---
% --- bottom subplot
% ---
figure(hFigSAR_SA)                                                         ;
subplot(2,2,3)                                                             ;
% ---
log_aIt = log10(aIt + 1.0e-03)                                             ;
% ---
imagesc(log_aIt)                                                           ;
title('Log_{10} scale of the average image','FontSize',14)                 ;
xlabel('column no.','FontSize',14)                                         ;
ylabel('row no.','FontSize',14)                                            ;
set(gca,'FontSize',12,'Linewidth',1.5)                                     ;
set(gca,'YDir','normal')                                                   ;
pause(0.010)                                                               ;
% ---
% --- top right subplot
% ---
Starttime = ccdimginfo.start_time{b2average(1)}                            ; % get the start time of the first batch to average
Starttime = Starttime(4:end)                                               ; % remove day string
Starttime = datevec(Starttime ,'mmm dd HH:MM:SS yyyy')                     ;
for n = 1 : numel(b2average)
    nBatch = b2average(n)                                                  ;
    if (ccdimginfo.kinetics == 0)
        slicespacing  = 0                                                  ;
    else
        slicespacing  = ccdimginfo.preset(nBatch)                          ;        
    end
    framespacing = framespacingFromDelayVector(nBatch)                     ;
    % ---
    totalIntensity = ccdimginfo.result.totalIntensity{nBatch}              ;
    % ---
    Batchstart = ccdimginfo.start_time{nBatch}                             ; % get start time of batch
    Batchstart = Batchstart(4:end)                                         ; % remove day string
    Batchstart = datevec(Batchstart,'mmm dd HH:MM:SS yyyy')                ;
    timeBetweenBatches = etime(Batchstart,Starttime)                       ;   
    % ---
    for i = 1 : size(totalIntensity,1)                                       % loop over frames
        for j = 1 : size(totalIntensity,2)                                   % loop over slices
            k    = (n-1) * numel(totalIntensity)                        ...
                 + (i-1) * size(totalIntensity,2) + j                      ; % get index
            x(k) = timeBetweenBatches                                   ...  
                 + (i-1) * framespacing + j * slicespacing                 ; % create time vector
            y(k) = totalIntensity(i,j)                                     ; % create intensity vector
        end
    end
    clear totalIntensity slicespacing                                      ;
    clear framespacing framespacings                                       ;
end
clear i j k n                                                              ;
% ---
figure(hFigSAR_SA)                                                         ;
subplot(2,2,2)                                                             ;
% ---
plot(x(:),y(:),'ko','MarkerSize',4)                                        ;
% ---
xlabel('elapsed time [s]','FontSize',14)                                   ;
ylabel('intensity [photons/sec]','FontSize',14)                            ;
% ---
x_min = min(x)                                                             ;
x_max = 1.02 * max(x)                                                      ;
xlim([x_min x_max])                                                        ;
% ---
y_min = 0.80 * min(y)                                                      ;
y_max = 1.20 * max(y)                                                      ;
ylim([y_min y_max])                                                        ;
% ---
set(gca,'FontSize',12,'Linewidth',1.5)                                     ;
pause(0.010)                                                               ;
% ---
% --- option to add a supertitle to the multiplot
% ---
if ( SUPERTITLEon == 1 )
    dataCollectionTime = [' - collected ',ccdimginfo.start_time{b2average(1)}] ;
    superTitleString   = [figureTitleLineName,dataCollectionTime]              ;
    supertitle(superTitleString)                                               ;
end
% ---
% --- print option
% ---
if ( PNGon == 1 )
    set(gcf,'PaperPositionMode','auto')                                    ;
    figtitle = [ccdimginfo.name,'average_image','.png']                    ; % define filename figure
    figname  = fullfile(ccdimginfo.savepath,figtitle)                      ; % build full file name
    print(fignumber,'-dpng','-r600','-opengl',figname)                     ; % create png file of plot
    pause(0.010)                                                           ;
end
% ---
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
             'xpcsmain_EditMessage'),'string')                             ;
strB   = sprintf('   Finish figure #%03i of Averages in %05.2f seconds.' ...
                 ,fignumber,etime(t2,t1))                                  ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{strB}],'HorizontalAlignment','left')                 ;
pause(0.010)                                                               ;
clear t t1 t2 strA strB                                                    ;
% ---
clear aIt log_aIt x y x_min x_max y_min y_max                              ;


% =========================================================================
% --- Static averages
% --- Plot 2: I(q) vs. q (if more than one q was averaged)
% =========================================================================
if ( ccdimginfo.snoq > 1 )
    fignumber = fignumber + 1                                              ;
    % ---
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')         ;
    pause(0.01)                                                            ;
    t1 = clock                                                             ;
    % ---
    hFigSAR_IQ = figure('Name',[figureTitleLineName,' - intensity vs. q']...
                      ,'Tag','SAR_intensityVSq')                           ;
    clf; cla                                                               ;
    % ---
    figLeft   = figLeft   + xFigShift                                      ; % position of the left border of the figure in pixel
    figHeight = figHeight + yFigShift                                      ; % position of the lower border of the figure in pixel
    if ( (figHeight + yFigSize + yFigMenuSize) > yScreenSize )
       figHeight = figHeight - yFigShift                                   ;
    end
    set(gcf,'Position',[figLeft figHeight xFigSize yFigSize])              ;
    % ---
    warning off MATLAB:DivideByZero                                        ;
    usermask = ccdimginfo.usermask(R1:R2,C1:C2)                            ;
    qmap     = ccdimginfo.qmap  (R1:R2,C1:C2) ./ usermask                  ; % set   q values of masked pixels to infinite
    phimap   = ccdimginfo.phimap(R1:R2,C1:C2) ./ usermask                  ; % set phi values of masked pixels to infinite
    % ---
    for i = 1 : numel(b2average)
        nBatch = b2average(i)                                              ;
        info   = zeros(ccdimginfo.snoq,ccdimginfo.snophi,3)                ; % initialize info
        x      = zeros(1,ccdimginfo.snoq)                                  ; % initialize q values
        y      = zeros(1,ccdimginfo.snoq)                                  ; % initialize intensity values
        Iqphi  = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                  ;
        % ---
        Iqphibatch = ccdimginfo.result.Iqphi{nBatch}                       ;
        for n = 1 : ccdimginfo.snoq
            xbatch(n) = 0                                                  ;
            ybatch(n) = 0                                                  ;
            ibatch(n) = 0                                                  ;
            for m = 1 : ccdimginfo.snophi
                if ( numel(usermask(usermask(ccdimginfo.smask{n,m})>0)) ~= 0)    % look for good pixels in static q phi partition
                    Unm   = usermask(ccdimginfo.smask{n,m})                    ;
                    Qnm   = qmap    (ccdimginfo.smask{n,m})                    ;
                    Phinm = phimap  (ccdimginfo.smask{n,m})                    ;
                    info(n,m,1) = mean (Qnm  (isfinite(Qnm  )))                ;
                    info(n,m,2) = mean (Phinm(isfinite(Phinm)))                ;
                    info(n,m,3) = numel(Unm  (Unm   >  0))                     ;
                    % ---
                    xbatch(n) = xbatch(n) + info(n,m,1) * info(n,m,3)      ; % calculate the sum of all q for all valid pixels
                    ybatch(n) = ybatch(n) + Iqphibatch (n,m) * info(n,m,3) ; % Iqphibatch has been calculated as a mean of info(n,m,3) pixels
                    ibatch(n) = ibatch(n) + info(n,m,3)                    ; % store the total number of valid pixels
                else
                    info(n,m,1)     = 0                                    ;
                    info(n,m,2)     = 0                                    ;
                    info(n,m,3)     = 0                                    ;
                    Iqphibatch(n,m) = 0                                    ; % set the value of Iqphi to zero if no pixels are in static q phi partition
                end
            end
        end
        % ---
        xbatch = xbatch./ibatch                                            ;
        ybatch = ybatch./ibatch                                            ;
        % ---
        x     = x + xbatch                                                 ;
        y     = y + ybatch                                                 ;
        Iqphi = Iqphi + Iqphibatch                                         ;
       clear xbatch ybatch ibatch Iqphibatch                               ;
    end
    x = x(:)                                                               ;
    y = y(:)                                                               ;
    % ---
    warning on MATLAB:DivideByZero                                         ;
    clear Unm Qnm Phinm                                                    ;
    clear usermask qmap phimap                                             ;
    % ---
    % --- plot I(Q)
    % ---
    figure(hFigSAR_IQ)                                                      ;
    semilogy(x(isfinite(x)),y(isfinite(y)),'ko'                         ...
            ,'MarkerSize',8,'Linewidth',2)                                 ;
    % ---
    xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',16)         ;
    % ---
    % --- different ylabels are possible due to different normalizations
    % ---
    if ~isfield(ccdimginfo,'normalization')
        ccdimginfo.normalization  = 0                                      ; % just create a dummy value for older versions
    end
    if ( ccdimginfo.normalization  == 0 | ccdimginfo.normalization  == 1 )
        ylabel('Intensity  [photons/sec]','FontSize',16)                   ; % standard normalizations        
    end
    if ( ccdimginfo.normalization  == 2 )
        ylabel('Intensity  [arb. units]','FontSize',16)                    ; % transmitted flux was known, but no thickness normalization could be done        
    end
    if ( ccdimginfo.normalization  == 3 )
        ylabel('Cross section  [1/cm]','FontSize',16)                      ; % incident & transmitted flux known + thickness normalization       
    end
    if ( ccdimginfo.normalization  == 4 )
        ylabel('I/I_{0}  [arb. units]','FontSize',16)                      ; % reflection mode & incident flux was known       
    end
    % ---
    if ( ccdimginfo.snophi > 1 )
        hold on                                                            ;
        dummy = zeros(size(Iqphi))                                         ;
        for m = 1 : ccdimginfo.snophi
            dummy(:,m) = Iqphi(:,m) / (4*(m+1))                            ; % create spacing between Iqphi's of different phi's
        end
        figure(hFigSAR_IQ)                                                 ;
        plot(x,dummy,'o-','MarkerSize',5,'Linewidth',1.5)                  ;
        hold off                                                           ;
        clear dummy                                                        ;
    end
    % ---
    % --- define plot layout
    % ---
    set(gca,'TickLength',[0.025 0.025])                                    ;
    set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')            ;
    set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')            ;
    set(gca,'FontSize',16,'Linewidth',1.5)                                 ;
    pause(0.010)                                                           ;
    % ---
    % --- option to add a supertitle to the multiplot
    % ---
    if ( SUPERTITLEon == 1 )
        dataCollectionTime=[' - collected ',ccdimginfo.start_time{b2average(1)}] ;
        superTitleString  =[figureTitleLineName,dataCollectionTime]              ;
        STh               = supertitle(superTitleString)                         ;
    end
    % ---
    % --- print option
    % ---
    if ( PNGon == 1 )
        set(gcf,'PaperPositionMode','auto')                                ;
        figtitle = [ccdimginfo.name,'average_IQ','.png']                   ; % define filename figure
        figname  = fullfile(ccdimginfo.savepath,figtitle)                  ; % build full file name
        print(fignumber,'-dpng','-r600','-opengl',figname)                 ; % create png file of plot
        pause(0.010)                                                       ;
    end
    % ---
    % --- ascii output option
    % ---
    if ( ASCIIon == 1 )
        filetitle = [ccdimginfo.name,'average_IQ.ASCII']                   ; % define filename figure
        filename  = fullfile(ccdimginfo.savepath,filetitle)                ; % build full file name
        out       = fopen(filename,'w')                                    ;
        % ---
        fprintf(out,'# File name      :    %s \n',ccdimginfo.name)         ;
        fprintf(out,'# \n')                                                ;
        fprintf(out,'# Some Parameter : \n')                               ;
        fprintf(out,'# No. of static q   partitions : %5i\n',           ...
                ccdimginfo.snoq)                                           ;
        fprintf(out,'# No. of static phi partitions : %5i\n',           ...
                ccdimginfo.snophi)                                         ;
        fprintf(out,'# \n')                                                ;
        fprintf(out,'# \n')                                                ;
        % ---
        str1   = '#q            ave. I(Q) '                                ;
        output = [x y]                                                     ;
        str2   = '%10.2e %12.2e'                                           ;
        if ( ccdimginfo.snophi > 1 )
            for m = 1 : ccdimginfo.snophi
                strdummy = sprintf('  Phi(%3i)  ',m)                       ;
                str1     = [str1,strdummy]                                 ;
                z        = squeeze(Iqphi(:,m))                             ;
                output   = [output z(:)]                                   ;
                str2     = [str2,'%12.2e']                                 ;
            end
        end
        str1 = [str1,'\n']                                                 ;
        str2 = [str2,'\n']                                                 ;
        % ---
        fprintf(out,str1)                                                  ;
        fprintf(out,str2,output')                                          ;
        % ---
        fclose(out)                                                        ;
        clear out filename filetitle str1 str2 strdummy output z           ;
        pause(0.010)                                                       ;
    end
    % ---
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                     ,fignumber,etime(t2,t1))                                   ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{strB}],'HorizontalAlignment','left')             ;
    pause(0.010)                                                           ;
    clear t t1 t2 strA strB                                                ;
    % ---
    clear info Iqphi x y i n m                                             ;
end


% =========================================================================
% --- Static averages
% --- Plot 3: I(phi) vs. phi (if more than one q was averaged)
% =========================================================================
if ( ccdimginfo.snophi > 1 )
    fignumber = fignumber + 1                                              ;
    % ---
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')         ;
    pause(0.010)                                                           ;
    t1 = clock                                                             ;
    % ---
    hFigSAR_IPhi = figure('Name',[figureTitleLineName,' - intensity vs. phi'] ...
                        ,'Tag','SAR_intensityVSphi')                             ;
    clf; cla                                                               ;
    % ---
    figLeft   = figLeft   + xFigShift                                      ; % position of the left border of the figure in pixel
    figHeight = figHeight + yFigShift                                      ; % position of the lower border of the figure in pixel
    if ( (figHeight + yFigSize + yFigMenuSize) > yScreenSize )
       figHeight = figHeight - yFigShift                                   ;
    end
    set(gcf,'Position',[figLeft figHeight xFigSize yFigSize])              ;
    % ---
    warning off MATLAB:DivideByZero                                        ;
    usermask = ccdimginfo.usermask(R1:R2,C1:C2)                            ;
    qmap     = ccdimginfo.qmap  (R1:R2,C1:C2) ./ usermask                  ; % set   q values of masked pixels to infinite
    phimap   = ccdimginfo.phimap(R1:R2,C1:C2) ./ usermask                  ; % set phi values of masked pixels to infinite
    % ---
    for i = 1 : numel(b2average)
        nBatch = b2average(i)                                              ;
        info   = zeros(ccdimginfo.snoq,ccdimginfo.snophi,3)                ; % initialize info
        x      = zeros(1,ccdimginfo.snophi)                                ; % initialize q values
        y      = zeros(1,ccdimginfo.snophi)                                ; % initialize intensity values
        Iqphi  = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                  ;
        % ---
        Iqphibatch = ccdimginfo.result.Iqphi{nBatch}                       ;
        for m = 1 : ccdimginfo.snophi
            xbatch(m) = 0                                                  ;
            ybatch(m) = 0                                                  ;
            ibatch(m) = 0                                                  ;
            for n = 1 : ccdimginfo.snoq
                if ( numel(usermask(usermask(ccdimginfo.smask{n,m})>0)) ~= 0) % look for good pixels in static q phi partition
                    Unm   = usermask(ccdimginfo.smask{n,m})                ;
                    Qnm   = qmap    (ccdimginfo.smask{n,m})                ;
                    Phinm = phimap  (ccdimginfo.smask{n,m})                ;
                    info(n,m,1) = mean (Qnm  (isfinite(Qnm  )))            ;
                    info(n,m,2) = mean (Phinm(isfinite(Phinm)))            ;
                    info(n,m,3) = numel(Unm  (Unm   >  0))                 ;
                    % ---
                    xbatch(m) = xbatch(m) + info(n,m,2) * info(n,m,3)      ; % calculate the sum of all q for all valid pixels
                    ybatch(m) = ybatch(m) + Iqphibatch (n,m) * info(n,m,3) ; % Iqphi has been calculated as a mean of info(n,m,3) pixels
                    ibatch(m) = ibatch(m) + info(n,m,3)                    ; % store the total number of valid pixels
                else
                    info(n,m,1)     = 0                                    ;
                    info(n,m,2)     = 0                                    ;
                    info(n,m,3)     = 0                                    ;
                    Iqphibatch(n,m) = 0                                    ; % set the value of Iqphi to zero if no pixels are in static q phi partition
                end
            end
        end
        % ---
        xbatch = xbatch./ibatch                                            ;
        ybatch = ybatch./ibatch                                            ;
        % ---
        x     = x + xbatch                                                 ;
        y     = y + ybatch                                                 ;
        Iqphi = Iqphi + Iqphibatch                                         ;
        clear xbatch ybatch ibatch Iqphibatch                              ;
    end
    x = x(:)                                                               ;
    y = y(:)                                                               ;
    % ---
    if ( ccdimginfo.geometry == 0 )
        x = round(1000*180/pi*x)/1000                                      ; % convert phi values from radiant to degree (3 digits after the dot)
    end
    % ---
    warning on MATLAB:DivideByZero                                         ;
    clear Unm Qnm Phinm                                                    ;
    clear usermask qmap phimap                                             ;
    % ---
    % --- plot I(phi)
    % ---
    figure(hFigSAR_IPhi)                                                    ;
    semilogy(x(isfinite(x)),y(isfinite(y)),'ko-'                        ...
            ,'MarkerSize',8,'Linewidth',2)                                 ;
    % ---
    if ( ccdimginfo.geometry == 0 )
        xlabel('Phi [deg]','FontSize',16)                                  ;   % transmission mode
    else
        xlabel('Q$_{z}$ [{\AA}$^{-1}$]','Interpreter','latex','FontSize',16) ; % in reflection mode 'phi' is really Q_z !!!
    end
    % --- different ylabels are possible due to different normalizations
    if ~isfield(ccdimginfo,'normalization')
        ccdimginfo.normalization  = 0                                      ; % just create a dummy value for older versions
    end
    if ( ccdimginfo.normalization  == 0 | ccdimginfo.normalization  == 1 )
        ylabel('Intensity  [photons/sec]','FontSize',16)                   ; % standard normalizations        
    end
    if ( ccdimginfo.normalization  == 2 )
        ylabel('Intensity  [arb. units]','FontSize',16)                    ; % transmitted flux was known, but no thickness normalization could be done        
    end
    if ( ccdimginfo.normalization  == 3 )
        ylabel('Cross section  [1/cm]','FontSize',16)                      ; % incident & transmitted flux known + thickness normalization       
    end
    if ( ccdimginfo.normalization  == 4 )
        ylabel('I/I_{0}  [arb. units]','FontSize',16)                      ; % reflection mode & incident flux was known       
    end
    % ---
    if ( ccdimginfo.snoq > 1 )
        hold on                                                            ;
        dummy = zeros(size(Iqphi))                                         ;
        for n = 1 : ccdimginfo.snoq
            dummy(n,:) = Iqphi(n,:) / (4*(n+1))                            ; % create spacing between Iqphi's of different q's
        end
        figure(hFigSAR_IPhi)                                               ;
        plot(x,dummy','o-','MarkerSize',5,'Linewidth',1.5)                 ;
        hold off                                                           ;
        clear dummy                                                        ;
    end
    % ---
    % --- define plot layout
    % ---
    set(gca,'TickLength',[0.025 0.025])                                    ;
    set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')            ;
    set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')            ;
    set(gca,'FontSize',16,'Linewidth',1.5)                                 ;
    pause(0.010)                                                           ;
    % ---
    % --- option to add a supertitle to the multiplot
    % ---
    if ( SUPERTITLEon == 1 )
        dataCollectionTime=[' - collected ',ccdimginfo.start_time{b2average(1)}] ;
        superTitleString  =[figureTitleLineName,dataCollectionTime]              ;
        STh               = supertitle(superTitleString)                         ;
    end
    % ---
    % --- print option
    % ---
    if ( PNGon == 1 )
        set(gcf,'PaperPositionMode','auto')                                ;
        figtitle = [ccdimginfo.name,'average_IPhi','.png']                 ; % define filename figure
        figname  = fullfile(ccdimginfo.savepath,figtitle)                  ; % build full file name
        print(fignumber,'-dpng','-r600','-opengl',figname)                 ; % create png file of plot
        pause(0.010)                                                       ;
    end
    % ---
    % --- ascii output option
    % ---
    if ( ASCIIon == 1 )
        filetitle = [ccdimginfo.name,'average_IPhi.ASCII']                 ; % define filename
        filename  = fullfile(ccdimginfo.savepath,filetitle)                ; % build full file name
        out       = fopen(filename,'w')                                    ;
        % ---
        fprintf(out,'# File name      :    %s\n',ccdimginfo.name)          ;
        fprintf(out,'# \n')                                                ;
        fprintf(out,'# Some Parameter : \n')                               ;
        fprintf(out,'# No. of static q   partitions : %5i\n',           ...
                ccdimginfo.snoq)                                           ;
        fprintf(out,'# No. of static phi partitions : %5i\n',           ...
                ccdimginfo.snophi)                                         ;
        fprintf(out,'# \n')                                                ;
        fprintf(out,'# \n')                                                ;
        % ---
        str1   = '#Phi         ave. I(Phi)'                                ;
        output = [x y]                                                     ;
        str2   = '%10.2e %12.2e'                                           ;
        if ( ccdimginfo.snoq > 1 )
            for n = 1 : ccdimginfo.snoq
                strdummy = sprintf('  Q(%4i)   ',n)                        ;
                str1     = [str1,strdummy]                                 ;
                z        = squeeze(Iqphi(n,:))                             ;
                output   = [output z(:)]                                   ;
                str2     = [str2,'%12.2e']                                 ;
            end
        end
        str1 = [str1,'\n']                                                 ;
        str2 = [str2,'\n']                                                 ;
        % ---
        fprintf(out,str1)                                                  ;
        fprintf(out,str2,output')                                          ;
        % ---
        fclose(out)                                                        ;
        clear out filename filetitle str1 str2 strdummy output z           ;
        pause(0.010)                                                       ;
    end
    % ---
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                     ,fignumber,etime(t2,t1))                                   ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{strB}],'HorizontalAlignment','left')             ;
    pause(0.010)                                                           ;
    clear t t1 t2 strA strB                                                ;
    % ---
    clear info Iqphi x y i n m                                             ;
end


% =========================================================================
% --- Start of plots results from the dynamic analysis
% =========================================================================
if ( ccdimginfo.analysistype == 1 )

    good       = 0.020                                                     ; % limit value of resnorm for a good fit
    BADon      = 1                                                         ; % switch to show unreliable results or not
    LEGENDon   = 1                                                         ; % switch to display a legend if BADon == 1
    
% =========================================================================
% --- Plots containing up to 9 single correlation functions 
% =========================================================================
fignum2 = fignumber                                                        ; % store number of figures before loop
% ---
x     = 0.0 * ccdimginfo.result.delay{b2average(1)}                        ; % initialize the averaged delay vector
g2avg = 0.0 * ccdimginfo.result.g2avg{b2average(1)}                        ; % initialize the averaged g2 functions
if ( isfield(ccdimginfo.result,'g2avgErr') == 1 )
    g2avgErr = ccdimginfo.result.g2avgErr{b2average(1)}                    ; % initialize the averaged error of the g2 functions
else
    g2avgErr = 0.0 * g2avg                                                 ; % create dummy value for older versions
end
% ---
for i = 1 : numel(b2average)
    nBatch   = b2average(i)                                                ;
    xbatch   = ccdimginfo.result.delay{nBatch}                             ; % get the delay vector
    x        = x + xbatch                                                  ; % make x a column vector
    g2avg    = g2avg + ccdimginfo.result.g2avg{nBatch}                     ; % get the averaged g2 functions
    if ( isfield(ccdimginfo.result,'g2avgErr') == 1 )
        g2avgErr = g2avgErr + ccdimginfo.result.g2avgErr{nBatch}           ; % get the averaged error of the g2 functions
    else
        g2avgErr = g2avgErr + 0.0                                          ; % create dummy value for older versions
    end
end
% ---
x        = x / numel(b2average)                                            ;
x        = x(:)                                                            ; % create a column vector
g2avg    = g2avg / numel(b2average)                                        ;
g2avgErr = sqrt(1/numel(b2average)) * g2avgErr / numel(b2average)          ;
% ---
warning off MATLAB:DivideByZero                                            ;
corr_info = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,3)                     ; % initialize corr_info 
for i = 1 : numel(b2average)
    nBatch   = b2average(i)                                                ;
    usermask = ccdimginfo.usermask(R1:R2,C1:C2)                            ;
    usermask(ccdimginfo.result.NEPhoton{nBatch}) = 0                       ; % mask pixel which did not receive enough photons
    qmap     = ccdimginfo.qmap  (R1:R2,C1:C2) ./ usermask                  ; % set   q values of masked pixels to infinite
    phimap   = ccdimginfo.phimap(R1:R2,C1:C2) ./ usermask                  ; % set phi values of masked pixels to infinite
    % ---
    corr_infobatch = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,3)            ; % initialize corr_infobatch 
    for n = 1 : ccdimginfo.dnoq
        for m = 1 : ccdimginfo.dnophi
            if ( numel(usermask(usermask(ccdimginfo.dmask{n,m})>0)) ~= 0)        % look for good pixels in dynamic q phi partition
                Unm   = usermask(ccdimginfo.dmask{n,m})                    ;
                Qnm   = qmap    (ccdimginfo.dmask{n,m})                    ;
                Phinm = phimap  (ccdimginfo.dmask{n,m})                    ;
                corr_infobatch(n,m,1) = mean (Qnm  (isfinite(Qnm  )))      ;
                corr_infobatch(n,m,2) = mean (Phinm(isfinite(Phinm)))      ;
                corr_infobatch(n,m,3) = numel(Unm  (Unm   >  0))           ;
            else
                corr_infobatch(n,m,1) = 0                                  ;
                corr_infobatch(n,m,2) = 0                                  ;
                corr_infobatch(n,m,3) = 0                                  ;
            end
        end
    end
    corr_info(:,:,1) = corr_info(:,:,1) + corr_infobatch(:,:,1)            ; % sum of mean q's
    corr_info(:,:,2) = corr_info(:,:,2) + corr_infobatch(:,:,2)            ; % sum of mean phi's
    corr_info(:,:,3) = corr_info(:,:,3) + corr_infobatch(:,:,3)            ; % sum of good pixels
end
corr_info(:,:,1) = corr_info(:,:,1) / numel(b2average)                     ; % calculate mean q
corr_info(:,:,2) = corr_info(:,:,2) / numel(b2average)                     ; % calculate mean phi
if (ccdimginfo.geometry == 0 )
    corr_info(:,:,2) = round(100*180/pi*corr_info(:,:,2))/100              ; % convert to degree (two digits after the dot)
end
clear Unm Qnm Phinm qmap phimap corr_infobatch                             ;
warning on MATLAB:DivideByZero                                             ;
% ---
% --- initialize some variables to store fitting parameter
% ---
if ( FIT1on == 1 )
    fit1_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,3)              ; % storage for fitting parameter of single Exp.
    sig1_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,3)              ; % storage for error of fitting parameter of single Exp.
    gof1_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,2)              ;
    gof1_results(:,:,1) = 50 * good                                        ; % set the resnorm much higher than 'good'
end
if ( FIT2on == 1 )
    fit2_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,4)              ; % storage for fitting parameter of stretched Exp.
    sig2_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,4)              ; % storage for error of fitting parameter of stretched Exp.
    gof2_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,2)              ;
    gof2_results(:,:,1) = 50 * good                                        ; % set the resnorm much higher than 'good'
end
if ( CUSTOMFITon == 1 )
    xdummy = [1:1:100]; ydummy = 100 - xdummy                              ;
    [cdummy1,cdummy2,cdummy3] = customfitparameter(xdummy,ydummy)          ;
    noParameter  = numel(cdummy1)                                          ;
    cfit_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,noParameter)    ; % storage for fitting parameter of stretched Exp.
    csig_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,noParameter)    ; % storage for error of fitting parameter of custom fit
    cgof_results = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,2)              ;
    cgof_results(:,:,1) = 50 * good                                        ; % set the resnorm much higher than 'good'
    clear xdummy ydummy cdummy1 cdummy2 cdummy3 noParameter                ;
end
% ---
multiplotnumber = ceil((ccdimginfo.dnoq*ccdimginfo.dnophi)/9)              ; % calculate how many multiplots are created
% ---
for l = 1 : multiplotnumber
    fignumber = l + fignum2                                                ;
    % ---
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')         ;
    pause(0.01)                                                            ;
    t1 = clock                                                             ;
    % ---
    figuretag = sprintf('SAR_correlationfigure_%02i',l)                    ;
    hFigSAR_CF(l) = figure('Name',[figureTitleLineName,' - ',figuretag]  ...
          ,'Tag',figuretag)                                                ;
    clf; cla                                                               ;
    % ---
    startx = figLeft   + l * xFigShift                                     ;
    starty = figHeight + l * yFigShift                                     ;
    while ( (starty + yFigSize + yFigMenuSize) > yScreenSize )
        starty = starty - yFigShift                                        ;
    end
    set(gcf,'Position',[startx starty xFigSize yFigSize])                  ;
    % ---
    for i = 1 : 9
        k = i + (l-1) * 9                                                  ; % subplot counter
        % ---
        m = ceil(k/ccdimginfo.dnoq)                                        ; % calculate the corresponding m (phi partition) value 
        n = k - (m-1) * ccdimginfo.dnoq                                    ; % calculate the corresponding n (q partition)   value
        % ---
        if ( k<=ccdimginfo.dnoq*ccdimginfo.dnophi && corr_info(n,m,3) > 0 )    % look for good pixels in dynamic q phi partition
            % =============================================================
            % --- get g2 data and errors
            % =============================================================
            for j = 1 : size(g2avg,3)
                y(j) = g2avg(n,m,j)                                        ; % get g2 function to display
            end
            y = y(:)                                                       ; % create column vector of the g2 function
            % ---
            for j = 1 : size(g2avg,3)
                e(j) = g2avgErr(n,m,j)                                     ; % get the corresponding error to display
            end
            e = e(:)                                                       ; % create column vector of the error

            
            % =============================================================
            % --- create fits
            % =============================================================
            xdata = double(x(:))                                           ; % create column vector
            ydata = double(y(:))                                           ; % create column vector
            edata = double(e(:))                                           ; % create column vector of the error

                      
            if ( FIT1on == 1 )
                % --- parameters % limits for single exponential fit
                if (numel(xdata) > 3)
                    baseline = sum(ydata(end-2:end))                    ...
                             / numel(ydata(end-2:end))                     ; % take the average of the last 3 data points as baseline start parameter
                    contrast = sum(ydata(1:3)) / numel(ydata(1:3))      ...
                             - baseline                                    ; % take the average of the first 3 data points minus baseline as contrast start parameter
                    contrast = max(0.01, contrast)                         ; % make sure the contrast is positive
                    gamma    = 1 / xdata(floor(numel(xdata)/2))            ; % put the starting gamma value into the middle of the time axis
                else
                    baseline = 1.0                                         ; % emergency default baseline start parameter
                    contrast = 0.3                                         ; % emergency default contrast start parameter
                    gamma    = 1.0                                         ; % emergency default gamma start parameter
                end
                    
                start1 = [baseline contrast gamma]                         ; % baseline, contrast & gamma
                low1   = [baseline/2 contrast/5 gamma/1e03]                ; % define lower limits
                high1  = [baseline*2 contrast*5 gamma*1e03]                ; % define higher limits
                clear baseline contrast gamma                              ;
               
                % --- perform fit & calculated best fit at data points
                opt = optimset ('Display','off','MaxIter',250,          ...
                                'MaxFunEvals',1000,'TolX',1.0e-20)         ;
                [fit1,fit1_resnorm,fit1_residual,fit1_flag,fit1_info] = ...
                    lsqcurvefit (@singleExponent,start1,xdata,ydata,    ...
                                 low1,high1,opt)                           ;
                fit1data = singleExponent(fit1,xdata)                      ;
                
                sig1 = errorsigma(xdata,ydata,edata,fit1,0.01*fit1      ...
                                 ,ones(size(fit1)),@singleExponent,fit1data); % calculate sigmas of fitting parameter
                             
                % --- store some results (parameters,resnorm,...)
                fit1_results(n,m,1) = fit1(1)                              ;
                fit1_results(n,m,2) = fit1(2)                              ;
                fit1_results(n,m,3) = fit1(3)                              ;
                sig1_results(n,m,1) = sig1(1)                              ;
                sig1_results(n,m,2) = sig1(2)                              ;
                sig1_results(n,m,3) = sig1(3)                              ;
                gof1_results(n,m,1) = fit1_resnorm                         ;
                gof1_results(n,m,2) = fit1_flag                            ;
                
                % --- clear some fit values
                clear opt start1 low1 high1 fit1 fit1_info fit1_flag       ;
                clear fit1_resnorm fit1_residual sig1                      ;
                
            end
            
            
            if ( FIT2on == 1 )
                % --- parameters % limits for stretched exponential fit
                if (numel(xdata) > 3)
                    baseline = sum(ydata(end-2:end))                    ...
                             / numel(ydata(end-2:end))                     ; % take the average of the last 3 data points as baseline start parameter
                    contrast = sum(ydata(1:3)) / numel(ydata(1:3))      ...
                             - baseline                                    ; % take the average of the first 3 data points minus baseline as contrast start parameter
                    contrast = max(0.01, contrast)                         ; % make sure the contrast is positive
                    gamma    = 1 / xdata(floor(numel(xdata)/2))            ; % put the starting gamma value into the middle of the time axis
                else
                    baseline = 1.0                                         ; % emergency default baseline start parameter
                    contrast = 0.3                                         ; % emergency default contrast start parameter
                    gamma    = 1.0                                         ; % emergency default gamma start parameter
                end
                exponent = 1.0                                             ;
                
                start2 = [baseline contrast gamma exponent]                ;
                low2   = [baseline/2 contrast/5 gamma/1e03 exponent/3]     ;
                high2  = [baseline*2 contrast*5 gamma*1e03 exponent*3]     ;
                clear baseline contrast gamma exponent                     ;
               
                % --- perform fit & calculated best fit at data points
                opt = optimset ('Display','off','MaxIter',250,          ...
                                'MaxFunEvals',1000,'TolX',1.0e-20)         ;
                [fit2,fit2_resnorm,fit2_residual,fit2_flag,fit2_info] = ...
                    lsqcurvefit (@stretchedExponent,start2,xdata,ydata, ...
                                 low2,high2,opt)                           ;
                fit2data = stretchedExponent(fit2,xdata)                   ;

                sig2 = errorsigma(xdata,ydata,edata,fit2,0.01*fit2      ...
                                 ,ones(size(fit2)),@stretchedExponent,fit2data) ; % calculate sigmas of fitting parameter

                % --- store some results (parameters,resnorm,...)
                fit2_results(n,m,1) = fit2(1)                              ;
                fit2_results(n,m,2) = fit2(2)                              ;
                fit2_results(n,m,3) = fit2(3)                              ;
                fit2_results(n,m,4) = fit2(4)                              ;
                sig2_results(n,m,1) = sig2(1)                              ;
                sig2_results(n,m,2) = sig2(2)                              ;
                sig2_results(n,m,3) = sig2(3)                              ;
                sig2_results(n,m,4) = sig2(4)                              ;
                gof2_results(n,m,1) = fit2_resnorm                         ;
                gof2_results(n,m,2) = fit2_flag                            ;
                
                % --- clear some fit values
                clear opt start2 low2 high2 fit2 fit2_info fit2_flag       ;
                clear fit2_resnorm fit2_residual sig2                      ;

            end
            
            
            if ( CUSTOMFITon == 1 )
                
                % --- define starting parameters & lower and upper limits
                [cstart,clow,chigh] = customfitparameter(xdata,ydata)      ;
               
                % --- perform fit & calculated best fit at data points
                opt = optimset ('Display','off','MaxIter',250,          ...
                                'MaxFunEvals',1000,'TolX',1.0e-20)         ;
                [cfit,cfit_resnorm,cfit_residual,cfit_flag,cfit_info] = ...
                    lsqcurvefit (@customfit,cstart,xdata,ydata,         ...
                                 clow,chigh,opt)                           ;
                cfitdata = customfit(cfit,xdata)                           ;
                
                csig = errorsigma(xdata,ydata,edata,cfit,0.01*cfit      ...
                                 ,ones(size(cfit)),@customfit,cfitdata)    ; % calculate sigmas of fitting parameter
                
                % --- store some results (parameters,resnorm,...)
                for nparameter = 1 : numel(cfit)
                    cfit_results(n,m,nparameter) = cfit(nparameter)        ;
                    csig_results(n,m,nparameter) = csig(nparameter)        ;
                end
                cgof_results(n,m,1) = cfit_resnorm                         ;
                cgof_results(n,m,2) = cfit_flag                            ;
                
                % --- clear some fit values
                clear opt cstart clow chigh cfit cfit_info cfit_flag       ;
                clear cfit_resnorm cfit_residual csig                      ;

            end
            
            
            % =============================================================
            % --- plot correlation functions & selected fits
            % =============================================================
            figure(hFigSAR_CF(l))                                          ;
            subplot(3,3,i)                                                 ; % create subplot
            % ---
            semilogx(x,y,'ko','MarkerSize',5,'LineWidth',1.5)              ; % plot g2 function
            % ---
            hold on                                                        ;
            % --- plot errorbars
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            semilogx(x,yl,'kx')                                            ; % plot lower  values of the error
            semilogx(x,yu,'kx')                                            ; % plot higher values of the error
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                semilogx(xj,yj,'k-')                                       ; % plot vertical lines to connect loer & higher errors 
            end
            clear yl yu xj yj                                              ;
            % --- plot selected fits
            if ( CUSTOMFITon == 1 )
            plot(xdata,cfitdata,'g-','LineWidth',1.5)                      ; % plot the custom fit into the plot
            end
            if ( FIT2on == 1 )
            plot(xdata,fit2data,'r-','LineWidth',1.5)                      ; % plot the stretched exponential fit into the plot
            end
            if ( FIT1on == 1 )
            plot(xdata,fit1data,'b-','LineWidth',1.5)                      ; % plot the single exponential fit into the plot
            end
            % ---
            hold off                                                       ;
            
            
            % =============================================================
            % --- define the plot limits & create axis labels
            % =============================================================
            extender = 3                                                   ;
            x_min = min(x) / extender                                      ;
            x_max = max(x) * extender                                      ;
            xlim([x_min x_max])                                            ;
            % ---
            if ( isempty(isfinite(y)) == 0 )
%                 y_min = min(y(find(isfinite(y)))) - 0.05                   ;  
%                 y_max = max(y(find(isfinite(y)))) + 0.05                   ;  
                y_min = 0.95                                               ;  
                y_max = 1.65                                               ;  
                if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                    y_min = 0.98                                           ;
                    y_max = 1.02                                           ;
                end
                ylim([y_min y_max])                                        ;
            else
               y_min = 0.98                                                ;
               y_max = 1.02                                                ;
               ylim([y_min y_max])                                         ;
            end
            % ---
            xlabel('${\Delta}$t [s]','Interpreter','latex','FontSize',12)  ;
            ylabel('g2','FontSize',12)                                     ;
            
            
            % =============================================================
            % --- define positions of information text in the figures
            % =============================================================
            if (ccdimginfo.geometry == 0 )
                if ( ccdimginfo.dnophi == 1 )
                    xpos  = 10^(log10(x_min) +                          ...
                                0.45 * (log10(x_max)-log10(x_min)))        ;
                    ypos1 = y_min + 0.90 * (y_max - y_min)                 ;
                    str1 = {[sprintf('q=%3.3g',corr_info(n,m,1)),'{\AA}$^{-1}$']}      ;
                    text('Position',[xpos ypos1],'String',str1,         ...
                         'Interpreter','latex','FontSize',9)               ;
                else
                    xpos  = 10^(log10(x_min) +                          ...
                                0.45 * (log10(x_max)-log10(x_min)))        ;
                    ypos1 = y_min + 0.78 * (y_max - y_min)                 ;
                    str1(1) = {[sprintf('q=%3.3g'  ,corr_info(n,m,1)),'{\AA}$^{-1}$']} ;
                    str1(2) = {[sprintf('phi=%6.2f',corr_info(n,m,2)),'$^{\circ}$']}   ;
                    text('Position',[xpos ypos1],'String',str1,         ...
                         'Interpreter','latex','FontSize',9 )              ;
                end
            end
            if (ccdimginfo.geometry == 1 )
                if ( ccdimginfo.dnophi == 1 )
                    xpos  = 10^(log10(x_min) +                          ...
                                0.45 * (log10(x_max)-log10(x_min)))        ;
                    ypos1 = y_min + 0.90 * (y_max - y_min)                 ;
                    str1 = {[sprintf('q=%3.3g',corr_info(n,m,1)),'{\AA}$^{-1}$']}      ;
                    text('Position',[xpos ypos1],'String',str1,         ...
                         'Interpreter','latex','FontSize',9)               ;
                else
                    xpos  = 10^(log10(x_min) +                          ...
                                0.45 * (log10(x_max)-log10(x_min)))        ;
                    ypos1 = y_min + 0.78 * (y_max - y_min)                 ;
                    str1(1) = {[sprintf('q=%3.3g'  ,corr_info(n,m,1)),'{\AA}$^{-1}$']}     ;
                    str1(2) = {[sprintf('q$_{z}$=%3.3g',corr_info(n,m,2)),'{\AA}$^{-1}$']} ;
                    text('Position',[xpos ypos1],'String',str1,         ...
                         'Interpreter','latex','FontSize',9)               ;
                end
            end
            if ( FIT1on == 1 | FIT2on == 1 | CUSTOMFITon == 1 )
                xpos   = 10^(log10(x_min) +                             ...
                         0.45 * (log10(x_max)-log10(x_min)))               ;
                if ( ccdimginfo.dnophi == 1 )
                    ypos2  = y_min + 0.78 * (y_max - y_min)                ;
                    ypos3  = y_min + 0.66 * (y_max - y_min)                ;
                else
                    ypos2  = y_min + 0.66 * (y_max - y_min)                ;
                    ypos3  = y_min + 0.54 * (y_max - y_min)                ;
                end
                % ---
                if ( CUSTOMFITon == 1 )                                      % assume custom fit has highest priority to be displayed
                    str2 = '${\Gamma}$'                                    ;
                    str3 = sprintf('=%5.3gs$^{-1}$',cfit_results(n,m,3))   ;
                    str4 = [str2,str3]                                     ;
                    text('Position',[xpos ypos2],'String',str4,         ...
                         'Interpreter','latex','FontSize',9)               ;
                    % ---
                    if ( cgof_results(n,m,1) >= good )
                        str5 = sprintf('Fit not reliable!')                ;
                        text('Position',[xpos ypos3],'String',str5,     ...
                             'Interpreter','latex','FontSize',9)           ;
                    end
                elseif ( FIT1on == 1 )                                       % no custom fit --> single exponential fit results are displayed
                    str2 = '${\Gamma}$'                                    ;
                    str3 = sprintf('=%5.3gs$^{-1}$',fit1_results(n,m,3))   ;
                    str4 = [str2,str3]                                     ;
                    text('Position',[xpos ypos2],'String',str4,         ...
                         'Interpreter','latex','FontSize',9)               ;
                    % ---
                    if ( gof1_results(n,m,1) >= good )
                        str5 = sprintf('Fit not reliable!')                ;
                        text('Position',[xpos ypos3],'String',str5,     ...
                             'Interpreter','latex','FontSize',9)           ;
                    end
                elseif ( FIT2on == 1 )                                       % no custom or single exponential fit --> stretched exponential fitting are displayed
                    str2 = '${\Gamma}$'                                    ;
                    str3 = sprintf('=%5.3gs$^{-1}$',fit2_results(n,m,3))   ;
                    str4 = [str2,str3]                                     ;
                    text('Position',[xpos ypos2],'String',str4,         ...
                         'Interpreter','latex','FontSize',9)               ;
                    % ---
                    if ( gof2_results(n,m,1) >= good )
                        str5 = sprintf('Fit not reliable!')                ;
                        text('Position',[xpos ypos3],'String',str5,     ...
                             'Interpreter','latex','FontSize',9)           ;
                    end
                end
            end
            
            
            % =============================================================
            % --- define plot axes layout
            % =============================================================
            set(gca,'XScale','log','YScale','linear')                      ;
            set(gca,'XTickMode','manual','TickLength',[0.025 0.025])       ;
            set(gca,'XTick',[1e-9 1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2    ...
                             1e-1 1e-0 1e+1 1e+2 1e+3 1e+4 1e+5 1e+6])     ; % all thinkable time decades for XPCS
            set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')    ;
            set(gca,'YMinorTick','off','YGrid','off','YMinorGrid','off')   ;
            set(gca,'FontSize',10,'Linewidth',1.5)                         ;
            textobj = findobj('type','text')                               ;
            set(textobj,'FontUnits','points','FontSize',9)                 ;
            pause(0.010)                                                   ;
            
            
            % =============================================================
            % --- create ASCII files of correlation functions
            % =============================================================
            if ( ASCIIon == 1 )
                filetitle = sprintf([ccdimginfo.name                    ...
                                    ,sprintf('g2_%04i_average.ASCII',k)])  ; % define filename
                filename  = fullfile(ccdimginfo.savepath,filetitle)        ; % build full file name
                out       = fopen(filename,'w')                            ;
                % ---
                fprintf(out,'# File name      :    %s\n',filename)         ;
                fprintf(out,'# G2 averaged at :      \n')                  ;
                if (ccdimginfo.geometry == 0 )
                    if ( ccdimginfo.dnophi == 1 )
                        fprintf(out,'# q   = %3.3g^{-1}     \n',       ...
                                corr_info(n,m,1))                          ;
                    else
                        fprintf(out,'# q   = %3.3g^{-1} \n',           ...
                                corr_info(n,m,1))                          ;
                        fprintf(out,'# phi = %4.3g      \n',           ...
                                corr_info(n,m,2))                          ;
                    end
                end
                if (ccdimginfo.geometry == 1 )
                    if ( ccdimginfo.dnophi == 1 )
                        fprintf(out,'# q   = %3.3g^{-1}     \n',       ...
                                corr_info(n,m,1))                          ;
                    else
                        fprintf(out,'# q   = %3.3g^{-1}     \n',       ...
                                corr_info(n,m,1))                          ;
                        fprintf(out,'# q_z = %3.3g^{-1}     \n',       ...
                                corr_info(n,m,2))                          ;
                    end
                end
                fprintf(out,'# G2 averaged over %6i pixels.\n',         ...
                        corr_info(n,m,3))                                  ;
                fprintf(out,'# \n')                                        ;
                fprintf(out,'# \n')                                        ;
                % ---
                str1   = '#t[s]        meas. g2      error      '          ;
                output = [x(:) y(:) e(:) ]                                 ;
                str2   = '%10.2e %13.4e %13.4e'                            ;
                if ( FIT1on == 1 )
                    fprintf(out,'# Parameters of single exponential fit    : \n')  ;
                    fprintf(out,'# Baseline : %6.3f +- %6.3f \n'        ...
                               ,fit1_results(n,m,1),sig1_results(n,m,1))   ;
                    fprintf(out,'# Contrast : %6.3f +- %6.3f \n'        ...
                               ,fit1_results(n,m,2),sig1_results(n,m,2))   ;
                    fprintf(out,'# Gamma    : %11.3e +- %11.3es^{-1} \n'...
                               ,fit1_results(n,m,3),sig1_results(n,m,3))   ;
                    fprintf(out,'# \n')                                    ;
                    % ---
                    fprintf(out,'# Quality of single exponential fit   : \n') ;
                    fprintf(out,'# Norm of residuals                   : %7.4f\n',gof1_results(n,m,1));
                    fprintf(out,'# \n')                                    ;
                    fprintf(out,'# \n')                                    ;
                    % ---
                    str1     = [str1,'   Fit1       ']                     ;
                    output   = [output fit1data(:)]                        ;
                    str2     = [str2,'%14.4e']                             ;
                end
                if ( FIT2on == 1 )
                    fprintf(out,'# Parameters of stretched exponential fit  : \n') ;
                    fprintf(out,'# Baseline : %6.3f +- %6.3f \n'        ...
                               ,fit2_results(n,m,1),sig2_results(n,m,1))   ;
                    fprintf(out,'# Contrast : %6.3f +- %6.3f \n'        ...
                               ,fit2_results(n,m,2),sig2_results(n,m,2))   ;
                    fprintf(out,'# Gamma    : %11.3e +- %11.3es^{-1} \n'...
                               ,fit2_results(n,m,3),sig2_results(n,m,3))   ;
                    fprintf(out,'# Stretching Exponent : %6.3f +- %6.3f \n' ...
                               ,fit2_results(n,m,4),sig2_results(n,m,4))   ;
                    fprintf(out,'# \n')                                    ;
                    % ---
                    fprintf(out,'# Quality of stretched exponential fit : \n') ;
                    fprintf(out,'# Norm of residuals                   : %7.4f\n',gof2_results(n,m,1));
                    fprintf(out,'# \n')                                    ;
                    fprintf(out,'# \n')                                    ;
                    % ---
                    str1     = [str1,'   Fit2       ']                     ;
                    output   = [output fit2data(:)]                        ;
                    str2     = [str2,'%14.4e']                             ;
                end
                if ( CUSTOMFITon == 1 )
                    fprintf(out,'# Parameters of custom fit  : \n')               ;
                    fprintf(out,'# Baseline             : %6.3f +- %6.3f \n'            ...
                               ,cfit_results(n,m,1),csig_results(n,m,1))                   ;
                    fprintf(out,'# Contrast             : %6.3f +- %6.3f \n'            ...
                               ,cfit_results(n,m,2),csig_results(n,m,2))                   ;
                    fprintf(out,'# Gamma                : %11.3e +- %11.3es^{-1}\n'     ...
                               ,cfit_results(n,m,3),csig_results(n,m,3))                   ;
                    if ( size(cfit_results,3) > 3 )
                        for nparameter = 4 : size(cfit_results,3)
                            fprintf(out,'# Custom Parameter(%02i) : %11.3e +- %11.3e \n'                          ...
                                       ,nparameter,cfit_results(n,m,nparameter),csig_results(n,m,nparameter))      ;
                        end
                    end
                    fprintf(out,'# \n')                                    ;
                    % ---
                    fprintf(out,'# Quality of custom fit : \n')            ;
                    fprintf(out,'# Norm of residuals                   : %7.4f\n',cgof_results(n,m,1));
                    fprintf(out,'# \n')                                    ;
                    fprintf(out,'# \n')                                    ;
                    % ---
                    str1     = [str1,'   Customfit  ']                     ;
                    output   = [output cfitdata(:)]                        ;
                    str2     = [str2,'%14.4e']                             ;
                end
                str1 = [str1,'\n']                                         ;
                str2 = [str2,'\n']                                         ;
                % ---
                fprintf(out,str1)                                          ;
                fprintf(out,str2,output')                                  ;
                % ---
                fclose(out)                                                ;
                clear out filename filetitle str1 str2 output              ;
                pause(0.010)                                               ;
            end
            
            
        end
    end
    % ---
    % --- option to add a supertitle to the multiplot
    % ---
    if ( SUPERTITLEon == 1 )
        dataCollectionTime=[' - collected ',ccdimginfo.start_time{b2average(1)}] ;
        superTitleString  =[figureTitleLineName,dataCollectionTime]              ;
        STh               = supertitle(superTitleString)                         ;
    end
    % ---
    % --- print option
    % ---
    if ( PNGon == 1 )
        set(gcf,'PaperPositionMode','auto')                                ;
        figtitle = sprintf([ccdimginfo.name,                            ...
                            sprintf('acf_multiplot_%03i',l),'.png'])       ; % define filename figure
        figname  = fullfile(ccdimginfo.savepath,figtitle)                  ; % build full file name
        print(fignumber,'-dpng','-r600','-opengl',figname)                 ; % create png file of plot
        pause(0.010)                                                       ;
    end
    % ---
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                     ,fignumber,etime(t2,t1))                                   ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{strB}],'HorizontalAlignment','left')             ;
    pause(0.010)                                                           ;
    clear t t1 t2 strA strB                                                ;
    % ---
end
% ---
figLeft   = startx                                                         ;
figHeight = starty                                                         ;
% ---
clear multiplotnumber extender singleExponent stretchedExponent usermask    ;
clear str1 str2 str3 str4 str5 xpos ypos1 ypos2 ypos3 textobj              ;
clear g2avg xdata ydata fit1data fit2data startx starty                    ;
clear i j k l n m x x_min x_max y y_min y_max                              ;
pause(0.010)                                                               ;


% =========================================================================
% --- create ASCII files summarizing the fit results
% =========================================================================
if ( ASCIIon == 1 && ( FIT1on == 1 | FIT2on == 1 | CUSTOMFITon == 1 ) )      % if any fit was done and ASCII output is wanted
    filetitle = [ccdimginfo.name,'average_fitresults.ASCII']               ; % define filename
    filename  = fullfile(ccdimginfo.savepath,filetitle)                    ; % build full file name
    out       = fopen(filename,'w')                                        ;
    % ---
    fprintf(out,'# File name      :    %s\n',filename)                     ;
    fprintf(out,'# \n')                                                    ;
    % --- create header line
    str1   = '#  no.     '                                                 ;
    if ( ccdimginfo.geometry == 0 )
        if ( ccdimginfo.dnophi == 1 )
            str1     = [str1,'   q_p           pixels    ']                ;
        else 
            str1     = [str1,'   phi[deg]      q_p           pixels    ']  ;
        end
    end
    if ( ccdimginfo.geometry == 1 )
        if ( ccdimginfo.dnophi == 1 )
            str1     = [str1,'   q_p           pixels    ']                ;
        else
            str1     = [str1,'   q_z           q_p           pixels    ']  ;
        end
    end
    % --- create data for the starting columns (#,phi/q_z,q_p,pixels)
    k  = 0                                                                 ;
    for m = 1 : ccdimginfo.dnophi
        for n = 1 : ccdimginfo.dnoq
            k = k + 1                                                      ;
            c1(k) = k                                                      ; % running index
            c2(k) = corr_info(n,m,2)                                       ; % phi/q_z value
            c3(k) = corr_info(n,m,1)                                       ; % q parallel value
            c4(k) = corr_info(n,m,3)                                       ; % number of averaged pixels
        end
    end
    if ( ccdimginfo.dnophi == 1 )
        output = [c1(:) c3(:) c4(:)]                                       ;
        str2   = '%10.2e %13.3e %13.3e'                                    ;
    else
        output = [c1(:) c2(:) c3(:) c4(:)]                                 ;
        str2   = '%10.2e %13.3e %13.3e %13.3e'                             ;
    end
    clear c1 c2 c3 c4                                                      ;
    % ---
    if ( FIT1on == 1 )
        k  = 0                                                             ;
        for m = 1 : ccdimginfo.dnophi
            for n = 1 : ccdimginfo.dnoq
                k = k + 1                                                  ;
                c5a(k) = fit1_results(n,m,1)                               ; % fitting value for baseline
                c5b(k) = sig1_results(n,m,1)                               ; % error of fitting value for baseline
                c6a(k) = fit1_results(n,m,2)                               ; % fitting value for contrast
                c6b(k) = sig1_results(n,m,2)                               ; % error of fitting value for contrast
                c7a(k) = fit1_results(n,m,3)                               ; % fitting value for Gamma
                c7b(k) = sig1_results(n,m,3)                               ; % error of fitting value for Gamma
                c8 (k) = gof1_results(n,m,1)                               ; % quality of fit
            end
        end
        % ---
        str1     = [str1,'   Baseline      Error      ']                   ;
        str1     = [str1,'   Contrast      Error      ']                   ;
        str1     = [str1,'   Gamma         Error      ']                   ;
        str1     = [str1,'   Quality   ']                                  ;
        output   = [output c5a(:) c5b(:) c6a(:) c6b(:) c7a(:) c7b(:) c8(:)] ;
        str2     = [str2,'%13.3e %13.3e %13.3e %13.3e %13.3e %13.3e %13.3e'];
        clear c5a c5b c6a c6b c7a c7b c8                                    ;
    end
    % ---
    if ( FIT2on == 1 )
        k  = 0                                                             ;
        for m = 1 : ccdimginfo.dnophi
            for n = 1 : ccdimginfo.dnoq
                k = k + 1                                                  ;
                c9a (k) = fit2_results(n,m,1)                              ; % fitting value for baseline
                c9b (k) = sig2_results(n,m,1)                              ; % error of fitting value for baseline
                c10a(k) = fit2_results(n,m,2)                              ; % fitting value for contrast
                c10b(k) = sig2_results(n,m,2)                              ; % error of fitting value for contrast
                c11a(k) = fit2_results(n,m,3)                              ; % fitting value for Gamma
                c11b(k) = sig2_results(n,m,3)                              ; % error of fitting value for Gamma
                c12a(k) = fit2_results(n,m,4)                              ; % fitting value for stretching exponent
                c12b(k) = sig2_results(n,m,4)                              ; % error of fitting value for stretching exponent
                c13 (k) = gof2_results(n,m,1)                              ; % quality of fit
            end
        end
        % ---
        str1     = [str1,'   Baseline      Error      ']                   ;
        str1     = [str1,'   Contrast      Error      ']                   ;
        str1     = [str1,'   Gamma         Error      ']                   ;
        str1     = [str1,'   str. exp.     Error      ']                   ;
        str1     = [str1,'   Quality   ']                                  ;
        output   = [output c9a(:) c9b(:) c10a(:) c10b(:) c11a(:) c11b(:) c12a(:) c12b(:) c13(:)] ;
        str2     = [str2,'%13.3e %13.3e %13.3e %13.3e %13.3e %13.3e %13.3e %13.3e %13.3e']       ;
        clear c9a c9b c10a c10b c11a c11b c12a c12b c13                                          ;
    end
    % ---
    if ( CUSTOMFITon == 1 )
        k  = 0                                                             ;
        for m = 1 : ccdimginfo.dnophi
            for n = 1 : ccdimginfo.dnoq
                k = k + 1                                                  ;
                c14a(k) = cfit_results(n,m,1)                              ; % fitting value for baseline
                c14b(k) = csig_results(n,m,1)                              ; % error of fitting value for baseline
                c15a(k) = cfit_results(n,m,2)                              ; % fitting value for contrast
                c15b(k) = csig_results(n,m,2)                              ; % error of fitting value for contrast
                c16a(k) = cfit_results(n,m,3)                              ; % fitting value for Gamma
                c16b(k) = csig_results(n,m,3)                              ; % error of fitting value for Gamma
            end
        end
        % ---
        str1     = [str1,'   Baseline      Error      ']                   ;
        str1     = [str1,'   Contrast      Error      ']                   ;
        str1     = [str1,'   Gamma         Error      ']                   ;
        output   = [output c14a(:) c14b(:) c15a(:) c15b(:) c16a(:) c16b(:)];
        str2     = [str2,'%13.3e %13.3e %13.3e %13.3e %13.3e %13.3e']      ;
        clear c14a c14b c15a c15b c16a c16b
        % ---
        % --- additional parameters from the customfit
        % ---
        if ( size(cfit_results,3) > 3 )
             for nparameter = 4 : size(cfit_results,3)
                 k  = 0                                                    ;
                 for m = 1 : ccdimginfo.dnophi
                     for n = 1 : ccdimginfo.dnoq
                         k = k + 1                                         ;
                         cx(k) = cfit_results(n,m,nparameter)              ; % next parameter from the custom fit
                         cy(k) = csig_results(n,m,nparameter)              ; % error of next parameter from the custom fit
                     end
                 end
                 % ---
                 strx     = sprintf('   CP%02i      ',nparameter)          ; 
                 stry     = sprintf('   ECP%02i     ',nparameter)          ; 
                 str1     = [str1,strx,stry]                               ;
                 output   = [output cx(:) cy(:)]                           ;
                 str2     = [str2,'%13.3e %13.3e']                         ;
                 clear cx cy strx stry                                     ;
             end
        end
        % ---
        k  = 0                                                             ;
        for m = 1 : ccdimginfo.dnophi
            for n = 1 : ccdimginfo.dnoq
                k = k + 1                                                  ;
                c17(k) = cgof_results(n,m,1)                               ; % quality of fit
            end
        end
        % ---
        str1     = [str1,'   Quality   ']                                  ;
        output   = [output c17(:)]                                         ;
        str2     = [str2,' %13.3e']                                        ;
        clear c17                                                          ;
    end
    % --- 
    str1 = [str1,'\n']                                                     ; % add cariage return to header line
    str2 = [str2,'\n']                                                     ; % add cariage return to fitting data
    % ---
    fprintf(out,str1)                                                      ; % print header line
    fprintf(out,str2,output')                                              ; % print fit data
    % ---
    fclose(out)                                                            ; % close ascii file
    clear out filename filetitle str1 str2 output                          ; % clear remaining variables
    pause(0.010)                                                           ;
end


% =========================================================================
% --- Plot of single exponential fitting results
% =========================================================================
if ( FIT1on == 1 )                                                           % makes only sense if single exponential fitting is on
    for m = 1 : ccdimginfo.dnophi                                            % loop over all dynamic phi's
        fignumber = fignumber + 1                                          ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')     ;
        pause(0.01)                                                        ;
        t1 = clock                                                         ;
        % ---
        hFigSAR_Fit1 = figure('Name',[figureTitleLineName,' - single exponential'] ...
                            ,'Tag','SAR_fit1results')                                 ;
        clf; cla                                                           ;
        % ---
        startx = figLeft   + m * xFigShift                                 ; % position of the left border of the figure in pixel
        starty = figHeight + m * yFigShift                                 ; % position of the lower border of the figure in pixel
        while ( (starty + yFigSize + yFigMenuSize) > yScreenSize )
            starty = starty - yFigShift                                    ;
        end
        set(gcf,'Position',[startx starty xFigSize yFigSize])              ;
    
        % =================================================================
        % --- prepare data for subplot(2,2,1)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ; % get q values
                y(n) = fit1_results(n,m,2)                                 ; % get contrast values
                e(n) = sig1_results(n,m,2)                                 ; % get error of contrast values
                z(n) = gof1_results(n,m,1)                                 ; % get norm of residuals of fit1
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit1)                                                ;
        subplot(2,2,1)                                                     ; % contrast vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('Contrast','FontSize',14)                                   ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.00                                               ;
                y_max = 0.10                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.00                                                   ;
            y_max = 0.10                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(2,2,2)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ; % get q values
                y(n) = fit1_results(n,m,1)                                 ; % get baseline values
                e(n) = sig1_results(n,m,1)                                 ; % get baseline values
                z(n) = gof1_results(n,m,1)                                 ; % get norm of residuals of fit1
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit1)                                                ;
        subplot(2,2,2)                                                     ; % baseline vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('Baseline','FontSize',14)                                   ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.98                                               ;
                y_max = 1.02                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.98                                                   ;
            y_max = 1.02                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(2,2,4)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ; % get q values
                y(n) = 1 / fit1_results(n,m,3)                             ; % get tau = 1/Gamma values
                e(n) = 1 / fit1_results(n,m,3)^2 * sig1_results(n,m,3)     ; % get error for tau = 1/Gamma values
                z(n) = gof1_results(n,m,1)                                 ; % get norm of residuals of fit1
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit1)                                                ;
        subplot(2,2,4)                                                     ; % tau = 1/Gamma vs. q (loglog plot)
        cla                                                                ;
        loglog(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                    ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x(yl>0),yl(yl>0),'kx')                                    ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                if ( yl(j) > 0 )
                    xj = [x(j),x(j),x(j)]                                  ;
                    yj = [yl(j),y(j),yu(j)]                                ;
                    plot(xj,yj,'k-')                                       ; % connect lower & higher errorbar values
                else
                    xj = [x(j),x(j)]                                       ;
                    yj = [y(j),yu(j)]                                      ;
                    plot(xj,yj,'k-')                                       ; % connect higher errorbar values                    
                end
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('${\tau} = 1/{\Gamma}$ [s]','Interpreter','latex','FontSize',14) ;
        % --- define plot limits
        extender = 1.2                                                     ;
        % ---
        if ( floor(log10(max(x))) - floor(log10(min(x))) == 0 )
            x_min = 10^(floor(log10(min(x)))) / extender                   ;
            x_max = 10^(ceil (log10(max(x)))) * extender                   ;
        else
            if( floor(log10(max(x))) - floor(log10(min(x))) == 1 )
                if (   abs(log10(min(x)) - floor(log10(min(x))))        ...
                    <= abs(log10(max(x)) - ceil (log10(max(x)))) )
                    x_min = 10^(floor(log10(min(x)))) / extender           ;
                    x_max = max(x) * extender                              ;
                else
                    x_min = min(x) / extender                              ;
                    x_max = 10^(ceil (log10(max(x)))) * extender           ;
                end
            else
                x_min = min(x) / extender                                  ;
                x_max = max(x) * extender                                  ;
            end
        end
        if ( isempty(x_min) == 0 && isempty(x_max) == 0 ) 
            xlim([x_min x_max])                                            ;
        else
            xlim([0.1 10])                                                 ;
        end
        % ---
        if ( floor(log10(max(y))) - floor(log10(min(y))) == 0 )
            y_min = 10^(floor(log10(min(y)))) / extender                   ;
            y_max = 10^(ceil (log10(max(y)))) * extender                   ;
        else
            if( floor(log10(max(y))) - floor(log10(min(y))) == 1 )
                if (   abs(log10(min(y)) - floor(log10(min(y))))        ...
                    <= abs(log10(max(y)) - ceil (log10(max(y)))) )
                    y_min = 10^(floor(log10(min(y)))) / extender           ;
                    y_max = max(y) * extender                              ;
                else
                    y_min = min(y) / extender                              ;
                    y_max = 10^(ceil (log10(max(y)))) * extender           ;
                end
            else
                y_min = min(y) / extender                                  ;
                y_max = max(y) * extender                                  ;
            end
        end
        if ( isempty(y_min) == 0 && isempty(y_max) == 0 ) 
            ylim([y_min y_max])                                            ;
        else
            ylim([0.1 10])                                                 ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','sw')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','sw')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                               ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'XTickMode','manual','TickLength',[0.025 0.025])           ;
        set(gca,'XTick',[1e-9 1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2        ...
                         1e-1 1e-0 1e+1 1e+2 1e+3 1e+4 1e+5 1e+6])         ; % all thinkable q decades for XPCS
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                    ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        % ---
        % --- option to add a supertitle to the multiplot
        % ---
        if ( SUPERTITLEon == 1 )
            dataCollectionTime = [' - collected '                       ...
                                 ,ccdimginfo.start_time{b2average(1)}]     ;
            superTitleString   = [figureTitleLineName,dataCollectionTime]  ;
            STh                = supertitle(superTitleString)              ;
        end
        
        % --- print option
        if ( PNGon == 1 )
            set(gcf,'PaperPositionMode','auto')                            ;
            figtitle = sprintf([ccdimginfo.name,                        ...
                        sprintf('acf_fit1result_%03i',m),'.png'])          ; % define filename figure
            figname  = fullfile(ccdimginfo.savepath,figtitle)              ; % build full file name
            print(fignumber,'-dpng','-r600','-opengl',figname)             ; % create png file of plot
        end
        pause(0)                                                           ;
        % ---
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                        ,fignumber,etime(t2,t1))                                    ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{strB}],'HorizontalAlignment','left')         ;
        pause(0.01)                                                        ;
        clear t t1 t2 strA strB                                            ;
        % ---
    end
    % ---
    figLeft   = startx                                                     ;
    figHeight = starty                                                     ;
    % ---
    clear usermask                                                         ;
    clear n m x x_min x_max y y_min y_max z startx starty                  ;
    clear fit1_results gof1_results                                        ;
end


% =========================================================================
% --- Plot of stretched exponential fitting results
% =========================================================================
if ( FIT2on == 1 )                                                           % makes only sense if stretched exponential fitting is on
    for m = 1 : ccdimginfo.dnophi                                            % loop over all dynamic phi's
        fignumber = fignumber + 1                                          ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')     ;
        pause(0.01)                                                        ;
        t1 = clock                                                         ;
        % ---
        hFigSAR_Fit2 = figure('Name',[figureTitleLineName,' - stretched exponential'] ...
                            ,'Tag','SAR_fit2results')                                    ;
        clf; cla                                                           ;
        % ---
        startx = figLeft   + m * xFigShift                                 ; % position of the left border of the figure in pixel
        starty = figHeight + m * yFigShift                                 ; % position of the lower border of the figure in pixel
        while ( (starty + yFigSize + yFigMenuSize) > yScreenSize )
            starty = starty - yFigShift                                    ;
        end
        set(gcf,'Position',[startx starty xFigSize yFigSize])              ;
    
        % =================================================================
        % --- prepare data for subplot(2,2,1)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = fit2_results(n,m,2)                                 ;
                e(n) = sig2_results(n,m,2)                                 ;
                z(n) = gof2_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit2)                                                ;
        subplot(2,2,1)                                                     ; % contrast vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('Contrast','FontSize',14)                                   ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.00                                               ;
                y_max = 0.10                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.00                                                   ;
            y_max = 0.10                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','off','XGrid','off','XMinorGrid','off')       ;
        set(gca,'YMinorTick','off','YGrid','off','YMinorGrid','off')       ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(2,2,2)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = fit2_results(n,m,1)                                 ;
                e(n) = sig2_results(n,m,1)                                 ;
                z(n) = gof2_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit2)                                                ;
        subplot(2,2,2)                                                     ; % baseline vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('Baseline','FontSize',14)                                   ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.98                                               ;
                y_max = 1.02                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.98                                                   ;
            y_max = 1.02                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','off','XGrid','off','XMinorGrid','off')       ;
        set(gca,'YMinorTick','off','YGrid','off','YMinorGrid','off')       ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(2,2,3)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = fit2_results(n,m,4)                                 ;
                e(n) = sig2_results(n,m,4)                                 ;
                z(n) = gof2_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit2)                                                ;
        subplot(2,2,3)                                                     ; % stretching exponent vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('Stretching Exponent','FontSize',14)                        ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.98                                               ;
                y_max = 1.02                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.98                                                   ;
            y_max = 1.02                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','off','XGrid','off','XMinorGrid','off')       ;
        set(gca,'YMinorTick','off','YGrid','off','YMinorGrid','off')       ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(2,2,4)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = 1 / fit2_results(n,m,3)                             ;
                e(n) = 1 / fit2_results(n,m,3)^2 * sig2_results(n,m,3)     ;
                z(n) = gof2_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_Fit2)                                                ;
        subplot(2,2,4)                                                     ; % tau = 1/Gamma vs. q (loglog plot)
        cla                                                                ;
        loglog(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                    ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x(yl>0),yl(yl>0),'kx')                                    ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                if ( yl(j) > 0 )
                    xj = [x(j),x(j),x(j)]                                  ;
                    yj = [yl(j),y(j),yu(j)]                                ;
                    plot(xj,yj,'k-')                                       ; % connect lower & higher errorbar values
                else
                    xj = [x(j),x(j)]                                       ;
                    yj = [y(j),yu(j)]                                      ;
                    plot(xj,yj,'k-')                                       ; % connect higher errorbar values                    
                end
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',14)     ;
        ylabel('${\tau} = 1/{\Gamma}$ [s]','Interpreter','latex','FontSize',14) ;
        % --- define plot limits
        extender = 1.2                                                     ;
        % ---
        if ( floor(log10(max(x))) - floor(log10(min(x))) == 0 )
            x_min = 10^(floor(log10(min(x)))) / extender                   ;
            x_max = 10^(ceil (log10(max(x)))) * extender                   ;
        else
            if( floor(log10(max(x))) - floor(log10(min(x))) == 1 )
                if (   abs(log10(min(x)) - floor(log10(min(x))))        ...
                    <= abs(log10(max(x)) - ceil (log10(max(x)))) )
                    x_min = 10^(floor(log10(min(x)))) / extender           ;
                    x_max = max(x) * extender                              ;
                else
                    x_min = min(x) / extender                              ;
                    x_max = 10^(ceil (log10(max(x)))) * extender           ;
                end
            else
                x_min = min(x) / extender                                  ;
                x_max = max(x) * extender                                  ;
            end
        end
        if ( isempty(x_min) == 0 && isempty(x_max) == 0 ) 
            xlim([x_min x_max])                                            ;
        else
            xlim([0.1 10])                                                 ;
        end
        % ---
        if ( floor(log10(max(y))) - floor(log10(min(y))) == 0 )
            y_min = 10^(floor(log10(min(y)))) / extender                   ;
            y_max = 10^(ceil (log10(max(y)))) * extender                   ;
        else
            if( floor(log10(max(y))) - floor(log10(min(y))) == 1 )
                if (   abs(log10(min(y)) - floor(log10(min(y))))        ...
                    <= abs(log10(max(y)) - ceil (log10(max(y)))) )
                    y_min = 10^(floor(log10(min(y)))) / extender           ;
                    y_max = max(y) * extender                              ;
                else
                    y_min = min(y) / extender                              ;
                    y_max = 10^(ceil (log10(max(y)))) * extender           ;
                end
            else
                y_min = min(y) / extender                                  ;
                y_max = max(y) * extender                                  ;
            end
        end
        if ( isempty(y_min) == 0 && isempty(y_max) == 0 ) 
            ylim([y_min y_max])                                            ;
        else
            ylim([0.1 10])                                                 ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','sw')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','sw')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'XTickMode','manual','TickLength',[0.025 0.025])           ;
        set(gca,'XTick',[1e-9 1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2        ...
                         1e-1 1e-0 1e+1 1e+2 1e+3 1e+4 1e+5 1e+6])         ; % all thinkable q decades for XPCS
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',12,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,12/14)                                   ;
            set(lhandle,'Fontsize',10)                                     ;
        end
        % ---
        pause(0)                                                           ;
    
    
        % ---
        % --- option to add a supertitle to the multiplot
        % ---
        if ( SUPERTITLEon == 1 )
            dataCollectionTime = [' - collected '                       ...
                                 ,ccdimginfo.start_time{b2average(1)}]     ;
            superTitleString   = [figureTitleLineName,dataCollectionTime]  ;
            STh                = supertitle(superTitleString)              ;
        end
        
        % ---
        if ( PNGon == 1 )
            set(gcf,'PaperPositionMode','auto')                            ;
            figtitle = sprintf([ccdimginfo.name,                        ...
                        sprintf('acf_fit2result_%03i',m),'.png'])          ; % define filename figure
            figname  = fullfile(ccdimginfo.savepath,figtitle)              ; % build full file name
            print(fignumber,'-dpng','-r600','-opengl',figname)             ; % create png file of plot
        end
        pause(0)                                                           ;
        % ---
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                        ,fignumber,etime(t2,t1))                                    ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{strB}],'HorizontalAlignment','left')         ;
        pause(0.01)                                                        ;
        clear t t1 t2 strA strB                                            ;
        % ---
    end
    % ---
    figLeft   = startx                                                     ;
    figHeight = starty                                                     ;
    % ---
    clear usermask                                                         ;
    clear n m x x_min x_max y y_min y_max z startx starty                  ;
    clear fit2_results gof2_results                                        ;
end


% =========================================================================
% --- Plot of custom fit results (up to 9 parameters are shown!)
% =========================================================================
if ( CUSTOMFITon == 1 )                                                           % makes only sense if single exponential fitting is on
    for m = 1 : ccdimginfo.dnophi                                            % loop over all dynamic phi's
        fignumber = fignumber + 1                                          ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')     ;
        pause(0.01)                                                        ;
        t1 = clock                                                         ;
        % ---
        hFigSAR_CFit = figure('Name',[figureTitleLineName,' - custom fit results'] ...
                            ,'Tag','SAR_customfitresults')                            ;
        clf; cla                                                           ;
        % ---
        startx = figLeft   + m * xFigShift                                 ; % position of the left border of the figure in pixel
        starty = figHeight + m * yFigShift                                 ; % position of the lower border of the figure in pixel
        while ( (starty + yFigSize + yFigMenuSize) > yScreenSize )
            starty = starty - yFigShift                                    ;
        end
        set(gcf,'Position',[startx starty xFigSize yFigSize])              ;
    
        % =================================================================
        % --- prepare data for subplot(3,3,1)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = cfit_results(n,m,2)                                 ;
                e(n) = csig_results(n,m,2)                                 ;
                z(n) = cgof_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---        
        figure(hFigSAR_CFit)                                                ;
        subplot(3,3,1)                                                     ; % contrast vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)     ;
        ylabel('Contrast','Interpreter','latex','FontSize',12)             ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.00                                               ;
                y_max = 0.10                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.00                                                   ;
            y_max = 0.10                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',10,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,9/12)                                    ;
            set(lhandle,'Fontsize',8)                                      ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(3,3,2)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = cfit_results(n,m,1)                                 ;
                e(n) = csig_results(n,m,1)                                 ;
                z(n) = cgof_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_CFit)                                                ;
        subplot(3,3,2)                                                     ; % baseline vs. q (linear plot)
        cla                                                                ;
        plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                      ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x,yl,'kx')                                                ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                xj = [x(j),x(j),x(j)]                                      ;
                yj = [yl(j),y(j),yu(j)]                                    ;
                plot(xj,yj,'k-')                                           ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)     ;
        ylabel('Baseline','Interpreter','latex','FontSize',12)             ;
        % --- define plot limits
        if ( isempty(isfinite(y)) == 0 )
            y_min = min(y(find(isfinite(y)))) - 0.02                       ;
            y_max = max(y(find(isfinite(y)))) + 0.02                       ;
            if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                y_min = 0.98                                               ;
                y_max = 1.02                                               ;
            end
            ylim([y_min y_max])                                            ;
        else
            y_min = 0.98                                                   ;
            y_max = 1.02                                                   ;
            ylim([y_min y_max])                                            ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','ne')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','ne')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                                ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'TickLength',[0.025 0.025])                                ;
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',10,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,9/12)                                    ;
            set(lhandle,'Fontsize',8)                                      ;
        end
        % ---
        pause(0)                                                           ;
    
        
        % =================================================================
        % --- prepare data for subplot(3,3,3)
        % =================================================================
        x = []; x1 = []; y = []; y1 = []; e = []; e1 = []; z = []; z1 = [] ; % initialization 
        for n = 1 : ccdimginfo.dnoq
            if ( corr_info(n,m,3) > 0 )                                      % check if dynamic q phi partition contained good pixels
                x(n) = corr_info(n,m,1)                                    ;
                y(n) = 1 / cfit_results(n,m,3)                             ;
                e(n) = 1 / cfit_results(n,m,3)^2 * csig_results(n,m,3)     ;
                z(n) = cgof_results(n,m,1)                                 ;
            else
                x(n) = 0                                                   ; % dummy value
                y(n) = 0                                                   ; % dummy value
                e(n) = 0                                                   ; % dummy value
                z(n) = 1                                                   ; % set norm of residuals in empty q phi partition to one
            end
        end
        x = x(:)                                                           ; % create column vectors
        y = y(:)                                                           ; % create column vectors
        e = e(:)                                                           ; % create column vectors
        z = z(:)                                                           ; % create column vectors
        % --- divide results into good fits & bad fits
        x1 = x(z >  good)                                                  ;
        y1 = y(z >  good)                                                  ;
        e1 = e(z >  good)                                                  ;
        z1 = z(z >  good)                                                  ;
        x  = x(z <= good)                                                  ;
        y  = y(z <= good)                                                  ;
        e  = e(z <= good)                                                  ;
        z  = z(z <= good)                                                  ;
        % ---
        figure(hFigSAR_CFit)                                                ;
        subplot(3,3,3)                                                     ; % tau = 1/Gamma vs. q (loglog plot)
        cla                                                                ;
        loglog(x,y,'ko','MarkerSize',6,'LineWidth',1.5)                    ;
        % ---
        if ( numel(x) > 0 )
            hold on                                                        ;
            yl = y - e                                                     ;
            yu = y + e                                                     ;
            plot(x(yl>0),yl(yl>0),'kx')                                    ; % plot lower  errorbar values
            plot(x,yu,'kx')                                                ; % plot higher errorbar values
            for j = 1 : numel(x)
                if ( yl(j) > 0 )
                    xj = [x(j),x(j),x(j)]                                  ;
                    yj = [yl(j),y(j),yu(j)]                                ;
                    plot(xj,yj,'k-')                                       ; % connect lower & higher errorbar values
                else
                    xj = [x(j),x(j)]                                       ;
                    yj = [y(j),yu(j)]                                      ;
                    plot(xj,yj,'k-')                                       ; % connect higher errorbar values                    
                end
            end
            hold off                                                       ;
            clear yl yu xj yj                                              ;
        end
        % ---
        xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)     ;
        ylabel('${\tau} = 1/{\Gamma}$ [s]','Interpreter','latex','FontSize',12) ;
        % --- define plot limits
        extender = 1.2                                                     ;
        % ---
        if ( floor(log10(max(x))) - floor(log10(min(x))) == 0 )
            x_min = 10^(floor(log10(min(x)))) / extender                   ;
            x_max = 10^(ceil (log10(max(x)))) * extender                   ;
        else
            if( floor(log10(max(x))) - floor(log10(min(x))) == 1 )
                if (   abs(log10(min(x)) - floor(log10(min(x))))        ...
                    <= abs(log10(max(x)) - ceil (log10(max(x)))) )
                    x_min = 10^(floor(log10(min(x)))) / extender           ;
                    x_max = max(x) * extender                              ;
                else
                    x_min = min(x) / extender                              ;
                    x_max = 10^(ceil (log10(max(x)))) * extender           ;
                end
            else
                x_min = min(x) / extender                                  ;
                x_max = max(x) * extender                                  ;
            end
        end
        if ( isempty(x_min) == 0 && isempty(x_max) == 0 ) 
            xlim([x_min x_max])                                            ;
        else
            xlim([0.1 10])                                                 ;
        end
        % ---
        if ( floor(log10(max(y))) - floor(log10(min(y))) == 0 )
            y_min = 10^(floor(log10(min(y)))) / extender                   ;
            y_max = 10^(ceil (log10(max(y)))) * extender                   ;
        else
            if( floor(log10(max(y))) - floor(log10(min(y))) == 1 )
                if (   abs(log10(min(y)) - floor(log10(min(y))))        ...
                    <= abs(log10(max(y)) - ceil (log10(max(y)))) )
                    y_min = 10^(floor(log10(min(y)))) / extender           ;
                    y_max = max(y) * extender                              ;
                else
                    y_min = min(y) / extender                              ;
                    y_max = 10^(ceil (log10(max(y)))) * extender           ;
                end
            else
                y_min = min(y) / extender                                  ;
                y_max = max(y) * extender                                  ;
            end
        end
        if ( isempty(y_min) == 0 && isempty(y_max) == 0 ) 
            ylim([y_min y_max])                                            ;
        else
            ylim([0.1 10])                                                 ;
        end
        % ---
        if ( BADon == 1 && numel(x1) > 0 )
            hold on                                                        ;
            plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)                ; % plot them
            % ---
            y1l = y1 - e1                                                  ;
            y1u = y1 + e1                                                  ;
            plot(x1,y1l,'kx')                                              ; % plot lower  errorbar values
            plot(x1,y1u,'kx')                                              ; % plot higher errorbar values
            for j = 1 : numel(x1)
                x1j = [x1(j),x1(j),x1(j)]                                  ;         
                y1j = [y1l(j),y1(j),y1u(j)]                                ;
                plot(x1j,y1j,'k-')                                         ; % connect lower & higher errorbar values
            end
            hold off                                                       ;
            clear y1l y1u x1j y1j                                          ;
        end
        % --- create legend (not perfect yet)
        if ( BADon == 1 && LEGENDon == 1 )
            goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)]         ;
            badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)]         ;
            if ( numel(z1) > 0 )                                             % if there bad fitting results
                lhandle = legend(goodstr,badstr,'Location','sw')           ; % create legend entrees for good and bad
            else
                lhandle = legend(goodstr,'Location','sw')                  ; % create legend entrees for good and bad               
            end
            legend('boxon')                                               ;
        end
        % ---
        clear x1 y1 z1                                                     ;
        % --- define plot layout
        set(gca,'XTickMode','manual','TickLength',[0.025 0.025])           ;
        set(gca,'XTick',[1e-9 1e-8 1e-7 1e-6 1e-5 1e-4 1e-3 1e-2        ...
                         1e-1 1e-0 1e+1 1e+2 1e+3 1e+4 1e+5 1e+6])         ; % all thinkable q decades for XPCS
        set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off')        ;
        set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off')        ;
        set(gca,'FontSize',10,'Linewidth',1.5)                             ;
        % ---
        if ( BADon == 1 && LEGENDon == 1 )
%             resize_legend(lhandle,9/12)                                    ;
            set(lhandle,'Fontsize',8)                                      ;
        end
        % ---
        pause(0)                                                           ;

        
        % =================================================================
        % --- prepare data for subplot(3,3,i) with 4<=i<=9
        % =================================================================
        if ( size(cfit_results,3) > 3 )
            for nparameter = 4 : min(9,size(cfit_results,3))
                % ---
                x = []; x1 = []; y = []; y1 = []                           ; % initialization
                e = []; e1 = []; z = []; z1 = []                           ; % initialization 
                for n = 1 : ccdimginfo.dnoq
                    if ( corr_info(n,m,3) > 0 )                              % check if dynamic q phi partition contained good pixels
                        x(n) = corr_info(n,m,1)                            ;
                        y(n) = cfit_results(n,m,nparameter)                ;
                        e(n) = csig_results(n,m,nparameter)                ;
                        z(n) = cgof_results(n,m,1)                         ;
                    else
                        x(n) = 0                                           ; % dummy value
                        y(n) = 0                                           ; % dummy value
                        e(n) = 0                                           ; % dummy value
                        z(n) = 1                                           ; % set norm of residuals in empty q phi partition to one
                    end
                end
                x = x(:)                                                   ; % create column vectors
                y = y(:)                                                   ; % create column vectors
                e = e(:)                                                   ; % create column vectors
                z = z(:)                                                   ; % create column vectors
                % --- divide results into good fits & bad fits
                x1 = x(z >  good)                                          ;
                y1 = y(z >  good)                                          ;
                e1 = e(z >  good)                                          ;
                z1 = z(z >  good)                                          ;
                x  = x(z <= good)                                          ;
                y  = y(z <= good)                                          ;
                e  = e(z <= good)                                          ;
                z  = z(z <= good)                                          ;
                % ---
                figure(hFigSAR_CFit)                                        ;
                subplot(3,3,nparameter)                                    ;
                cla                                                        ;
                plot(x,y,'ko','MarkerSize',6,'LineWidth',1.5)              ;
                % ---
                if ( numel(x) > 0 )
                    hold on                                                ;
                    yl = y - e                                             ;
                    yu = y + e                                             ;
                    plot(x,yl,'kx')                                        ; % plot lower  errorbar values
                    plot(x,yu,'kx')                                        ; % plot higher errorbar values
                    for j = 1 : numel(x)
                        xj = [x(j),x(j),x(j)]                              ;
                        yj = [yl(j),y(j),yu(j)]                            ;
                        plot(xj,yj,'k-')                                   ; % connect lower & higher errorbar values
                    end
                    hold off                                               ;
                    clear yl yu xj yj                                      ;
                end
                % ---
                xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)         ;
                ylabel(sprintf('Custom fit parameter #%0i2',nparameter),'FontSize',12) ;
                % --- define plot limits
                if ( isempty(isfinite(y)) == 0 )
                    y_min = min(y(find(isfinite(y)))) - 0.02               ;
                    y_max = max(y(find(isfinite(y)))) + 0.02               ;
                    if ( isempty(y_min) == 1 | isempty(y_max) == 1 )
                        y_min = 0.00                                       ;
                        y_max = 0.10                                       ;
                    end
                    ylim([y_min y_max])                                    ;
                else
                    y_min = 0.00                                           ;
                    y_max = 0.10                                           ;
                    ylim([y_min y_max])                                    ;
                end
                % ---
                if ( BADon == 1 && numel(x1) > 0 )
                    hold on                                                ;
                    plot(x1,y1,'k+','MarkerSize',6,'LineWidth',1.5)        ; % plot them
                    % ---
                    y1l = y1 - e1                                          ;
                    y1u = y1 + e1                                          ;
                    plot(x1,y1l,'kx')                                      ; % plot lower  errorbar values
                    plot(x1,y1u,'kx')                                      ; % plot higher errorbar values
                    for j = 1 : numel(x1)
                        x1j = [x1(j),x1(j),x1(j)]                          ;         
                        y1j = [y1l(j),y1(j),y1u(j)]                        ;
                        plot(x1j,y1j,'k-')                                 ; % connect lower & higher errorbar values
                    end
                    hold off                                               ;
                    clear y1l y1u x1j y1j                                  ;
                end
                % --- create legend (not perfect yet)
                if ( BADon == 1 && LEGENDon == 1 )
                    goodstr = ['{\chi}^{2} {\leq} ',sprintf('%4.3f',good)] ;
                    badstr  = ['{\chi}^{2} > '     ,sprintf('%4.3f',good)] ;
                    if ( numel(z1) > 0 )                                             % if there bad fitting results
                        lhandle = legend(goodstr,badstr,'Location','ne')   ; % create legend entrees for good and bad
                    else
                        lhandle = legend(goodstr,'Location','ne')          ; % create legend entrees for good and bad               
                    end
                    legend('boxon')                                        ;
                end
                % ---
                clear x1 y1 z1                                             ;
                % --- define plot layout
                set(gca,'TickLength',[0.025 0.025])                        ;
                set(gca,'XMinorTick','on','XGrid','off','XMinorGrid','off');
                set(gca,'YMinorTick','on','YGrid','off','YMinorGrid','off');
                set(gca,'FontSize',10,'Linewidth',1.5)                     ;
                % ---
                if ( BADon == 1 && LEGENDon == 1 )
%                     resize_legend(lhandle,9/12)                            ;
                    set(lhandle,'Fontsize',8)                              ;
                end
                % ---
                pause(0)                                                   ;
            end        
        end

        
        % ---
        % --- option to add a supertitle to the multiplot
        % ---
        if ( SUPERTITLEon == 1 )
            dataCollectionTime = [' - collected '                       ...
                                 ,ccdimginfo.start_time{b2average(1)}]     ;
            superTitleString   = [figureTitleLineName,dataCollectionTime]  ;
            STh                = supertitle(superTitleString)              ;
        end
        
        % --- print option
        if ( PNGon == 1 )
            set(gcf,'PaperPositionMode','auto')                            ;
            figtitle = sprintf([ccdimginfo.name,                        ...
                        sprintf('acf_cfitresult_%03i',m),'.png'])          ; % define filename figure
            figname  = fullfile(ccdimginfo.savepath,figtitle)              ; % build full file name
            print(fignumber,'-dpng','-r600','-opengl',figname)             ; % create png file of plot
        end
        pause(0)                                                           ;
        % ---
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                        ,fignumber,etime(t2,t1))                                    ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{strB}],'HorizontalAlignment','left')         ;
        pause(0.01)                                                        ;
        clear t t1 t2 strA strB                                            ;
        % ---
    end
    % ---
    figLeft   = startx                                                     ;
    figHeight = starty                                                     ;
    % ---
    clear usermask                                                         ;
    clear n m x x_min x_max y y_min y_max z startx starty                  ;
    clear cfit_results cgof_results good                                   ;
end


% =========================================================================
% --- Waterfall plot of up to 25 correlation functions as a function of q
% --- plot not working yet 
% =========================================================================
WATERFALL = 0                                                              ; % not done for averaging different batches yet
if ( WATERFALLon == 1 )
    fignumber = fignumber + 1                                              ;
    % ---
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strA   = sprintf('   Creating figure #%03i of AVERAGES ...',fignumber) ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';strA}],'HorizontalAlignment','left')         ;
    pause(0.01)                                                            ;
    t1 = clock                                                             ;
    % ---
    hFigSAR_WF = figure('Name',[figureTitleLineName,' - waterfall plot'] ...
          ,'Tag','SAR_waterfall')                                          ;
    clf; cla                                                               ;
    % ---
    figLeft   = figLeft   + xFigShift                                      ; % position of the left border of the figure in pixel
    figHeight = figHeight + yFigShift                                      ; % position of the lower border of the figure in pixel
    if ( (figHeight + yFigSize + yFigMenuSize) > yScreenSize )
        figHeight = figHeight - yFigShift                                  ;
    end
    set(gcf,'Position',[figLeft figHeight xFigSize yFigSize])              ;
    % ---
    t = squeeze(ccdimginfo.result.delay{nBatch})                           ;
    qmap = ccdimginfo.qmap(R1:R2,C1:C2)                                    ;
    for n = 1 : ccdimginfo.dnoq
        for m = 1 : ccdimginfo.dnophi
            q_info(n,m)   = mean(qmap(ccdimginfo.dmask{n,m}))              ;
        end
    end
    clear qmap                                                             ;
    q = squeeze(mean(q_info,2))                                            ;
    % ---
    g2avg = ccdimginfo.result.g2avg{nBatch}                                ; % get averaged g2 functions
    acf   = squeeze(mean(g2avg,2))                                         ; % average over phi partitions
    % ---
    lastq = 25                                                             ;
    if ( ccdimginfo.dnoq < lastq ) 
        lastq = ccdimginfo.dnoq                                            ;
    end
    % ---
    figure(hFigSAR_WF)                                                      ;
    waterfall(t,q(1:lastq),acf(1:lastq,:))                                 ;
    % ---
    xlim([min(t),max(t)])                                                  ;
    ylim([min(q(1:lastq)),max(q(1:lastq))])                                ;
    zlim([min(min(acf(1:lastq,:))) - 0.02 ,                             ...
          max(max(acf(1:lastq,:))) + 0.02])                                ;
    % ---
    colormap(winter)                                                       ;
    set(gca,'XScale','log','ZScale','lin')                                 ;
    % ---
    xlabel('{\Delta}t [s]')                                                ;
    ylabel('Q [{\AA}$^{-1}$]','Interpreter','latex')                       ;
    zlabel('g2')                                                           ;
    view([35,20])                                                          ;
    pause(0)                                                               ;
    % ---
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                 'xpcsmain_EditMessage'),'string')                         ;
    strB   = sprintf('   Finish figure #%03i of AVERAGES in %05.2f seconds.' ...
                     ,fignumber,etime(t2,t1))                                   ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{strB}],'HorizontalAlignment','left')             ;
    pause(0.01)                                                            ;
    clear t t1 t2 strA strB                                                ;
    % ---
    if ( PNGon == 1 )
        set(gcf,'PaperPositionMode','auto')                                ;
        figtitle = [ccdimginfo.name,'acf_waterfall']                       ; % define filename figure
        figname  = fullfile(ccdimginfo.savepath,figtitle)                  ; % build full file name
        print(fignumber,'-dpng','-r600','-opengl',figname)                 ; % create png file of plot
        pause(0)                                                           ;
    end
    % ---
    clear g2avg acf t q q_info n m lastq                                   ;
end


% =========================================================================
% --- plot the intermediate scattering function g1
% =========================================================================
if ( PLOTG1on == 1 && ASCIIon == 1 && FIT1on == 1 )
    filetitle = sprintf([ccdimginfo.name, sprintf('fitresults.ASCII')])    ;
    file = fullfile(ccdimginfo.savepath,filetitle)                         ;
    b= loadascii(file)                                                     ;
    % ---
    multiplotnumber = ceil(size(b.data,1)/9)                               ; % calculate how many multiplots are created
    % ---
    for l = 1 : multiplotnumber
        % ---
        figuretag = sprintf('SAR_intermediatescatteringfigure_%02i',l)     ;
        hFigSAR_G1(l) = figure('Name',[figureTitleLineName,' - ',figuretag] ...
                               ,'Tag',figuretag)                           ;
        clf; cla                                                           ;
        % ---
        startx = figLeft   + l * xFigShift                                 ;
        starty = figHeight + l * yFigShift                                 ;
        while ( (starty + yFigSize + yFigMenuSize) > yScreenSize )
            starty = starty - yFigShift                                    ;
        end
        set(gcf,'Position',[startx starty xFigSize yFigSize])              ;
         % ---
        for i = 1 : 9
            k = i + (l-1) * 9                                              ;
            if ( k <= size(b.data,1) )                                       
                filetitle = sprintf([ccdimginfo.name                    ...
                                    ,sprintf('g2_%04i_average.ASCII',k)])  ;   
                file = fullfile(ccdimginfo.savepath,filetitle)             ;
                a = loadascii(file)                                        ; % load the ASCII file and store in a
        

                % =========================================================
                % --- data preparation 
                % =========================================================
                % --- to plot now f = sqrt((g2-baseline)/contrast) vs time
                % --- first, one needs to assign the data
                x = a.data(:,1)                                            ; % the delays are store in column 1
                y = a.data(:,2)                                            ; % the measured g2 is stored in column 2
                x = x(:); y = y(:)                                         ; % transform x & y to column vectors
                % --- get the fit parameter
                contrast = b.data(k,6)                                     ; % it's the fourth g2 & contrast values are in column 6
                baseline = b.data(k,4)                                     ; % it's the fourth g2 & baseline values are in column 4
                qvalue   = b.data(k,2)                                     ; % get the average q value
                % ---
                y      = sqrt((y-baseline)/contrast)                       ; % calculate f
                y_imag = imag(y)                                           ; % calculate imaginary part of f
                % ---
                Imags     = find(y_imag ~= 0)                              ; % find positions of (g2-baseline) < 0
                if ( numel(Imags) ~= 0 )
                    firstImag = Imags(1)                                   ; % find position of the first value of (g2-baseline) < 0
                else
                    firstImag = numel(y) +1                                ; % find position of the first value of (g2-baseline) < 0    
                end


                % =========================================================
                % --- create the plots in the figure
                % =========================================================
                subplot(3,3,i)                                             ;
                if ( numel(x(1:firstImag-1)) > 1/5 * numel(x) )              % at a minimum every 5th point has to be real
                    plot(x(1:firstImag-1),y(1:firstImag-1),'ko-')          ; % plot only up values with (g2-baseline) > 0
                    % ---
                    xlabel('${\Delta}$t [s]','Interpreter','latex'      ...
                          ,'FontSize',12)                                  ; % original labels are store in a.columnName
                    ylabel('g1','Interpreter','latex','FontSize',12)       ;
                    % ---
                    xpos = min(x(1:firstImag-1))                            ...
                         + 0.45*(max(x(1:firstImag-1))-min(x(1:firstImag-1)))  ;    
                    ypos = min(y(1:firstImag-1))                            ...
                         + 0.90*(max(y(1:firstImag-1))-min(y(1:firstImag-1)))  ;   
                    str = [sprintf('q =%3.3g',qvalue),'{\AA}$^{-1}$']     ;
                    text('Position',[xpos ypos],'String',str,           ...
                         'Interpreter','latex','FontSize',10)              ;
                end
            end
        end
        % ---
        % --- option to add a supertitle to the multiplot
        % ---
        if ( SUPERTITLEon == 1 )
            dataCollectionTime = [' - collected '                       ...
                                 ,ccdimginfo.start_time{b2average(1)}]     ;
            superTitleString   = [figureTitleLineName,dataCollectionTime]  ;
            STh                = supertitle(superTitleString)              ;
        end
    end
    clear Imags firstImag y_imag qvalue contrast baseline b a x y          ;
    clear filetitle file figuretag multiplotnumber                         ;
end


% =========================================================================
% --- clear remaining variables
% =========================================================================
clear corr_info extender msgstr figLeft figHeight fignumber fignum2 nBatch ;  
if ( PNGon == 1 )
    clear figname figtitle                                                 ;
end


end
% =========================================================================
% --- End of plots results from the dynamic analysis
% =========================================================================


% =========================================================================
% --- set pushbutton status back
% =========================================================================
hPushbuttonLoad      = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonLoad')                   ;
hPushbuttonBatchinfo = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonBatchinfo')              ;
hPushbuttonAnalysis  = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonAnalysis')               ;
hPushbuttonSystem    = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonSystem')                 ;
hPushbuttonRun       = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonRun')                    ;
hPushbuttonStop      = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonStop')                   ;
% ---
set([hPushbuttonLoad,hPushbuttonBatchinfo,hPushbuttonAnalysis,          ...
     hPushbuttonSystem,hPushbuttonRun],'Enable','on')                      ;
set( hPushbuttonStop,'enable','off')                                       ;


% =========================================================================
% --- function to resize the font size of a legend
% =========================================================================
function  resize_legend (hL, ResizeFact)
% ---
% --- RESIZE_LEGEND - Changes LEGEND fontsize while maintaining proper
% --- alignment between the legend's text strings and line symbols
% --- & the 'Interpreter' to 'Tex'
%
% --- Syntax:  resize_legend(hL, ResizeFact);
%
% --- Inputs:  hL           Legend axes handle
% ---          ResizeFact   Factor by which to resize the legend.
%
% --- Output:  none
%
% --- Example: hL = legend(h,'string1','string2',...);
% ---          resize_legend(hL,2) ; %Make fontsize twice bigger
% ---

if nargin ~= 2
    error('Two input arguments are required')                              ;
end
p    = get(hL,'position')                                                  ;
p(3) = p(3)*ResizeFact                                                     ;
p(4) = p(4)*ResizeFact                                                     ;
set(hL,'position', p)                                                      ;

ht   = findobj( get(hL,'children'), 'type', 'text')                        ;
set(ht,'FontSize',get(ht(1),'FontSize')*ResizeFact)                        ;
set(ht,'Interpreter','tex')                                                ;


% =========================================================================
% --- function for single exponential fit
% =========================================================================
function F = singleExponent(x,xdata)
% ---
% --- x(1) : Baseline
% --- x(2) : Contrast
% --- x(3) : Gamma = 1/tau
% ---
F = x(1) + x(2) * exp(- 2 * x(3) * xdata)                                  ; % this assumes a homodyne detection scheme


% =========================================================================
% --- function for stretched exponential fit
% =========================================================================
function F = stretchedExponent(x,xdata)
% ---
% --- x(1) : Baseline
% --- x(2) : Contrast
% --- x(3) : Gamma = 1/tau
% --- x(4) : Stretching Exponent
% ---
F = x(1) + x(2) * exp( - 2 * (x(3) * xdata).^x(4) )                        ; % this assumes a homodyne detection scheme


% =========================================================================
% --- functions to calculate errorbars of the fitting parameter
% =========================================================================
function sig = errorsigma(x,y,err,p,dp,free,func,f)
% =========================================================================
%
% --- ERRORSIGMA computes the sigma for each fitting parameter value
% --- call this function after a fit is performed
% --- [sig,covp,corp,r2,rv,f] = errorsigma(x,y,wt,p,dp,free,func,f)
% --- free is 0 for fixed parameters, 1 for variable parameters
% --- a good choice for dp = 0.01 * p
%
% =========================================================================

% =========================================================================
% --- input preparation
% =========================================================================
x            = x(:)                                                        ; % x data
y            = y(:)                                                        ; % y data
err          = err(:)                                                      ; % error of data
selected     = find(err)                                                   ;
wt           = zeros(size(y))                                              ; % create weighting vector
wt(selected) = 1 ./ err(selected)                                          ;
wt           = wt(:)                                                       ;

m            = length(x)                                                   ; % number of data points
p            = p(:)                                                        ; % parameters of final fit
free         = free(:)                                                     ; % vector showing which parameters are varying
dp           = dp(:)                                                       ; % a vector telling how much to vary the parameter

if ( nargin <= 8 ) 
    f = []                                                                 ;
end
if ( isempty(f) || (length(f) ~= m) )
    f = feval(func,p,x)                                                    ;
end
f = f(:)                                                                   ;

index = find( ~isnan(x) & ~isnan(y) & ~isnan(wt)& ~isnan(f)             ...
            & ~isinf(f) & ~isinf(y) & ~isinf(wt) & wt)                     ;
        
x  = x(index)                                                              ;
y  = y(index)                                                              ;
wt = wt(index)                                                             ;
f  = f(index)                                                              ;
m  = length(y)                                                             ;
dp = dp .* free                                                            ;

sig  = []                                                                  ;
covp = []                                                                  ;
corp = []                                                                  ;
r2   = []                                                                  ;
rv   = []                                                                  ;
msav = []                                                                  ;

% =========================================================================
% --- data reduction for data sets above 1000 points
% =========================================================================
nreduce = 1000                                                             ;
if ( m > nreduce )                                                           % try to avoid crash if too many points
%     disp('Reducing data for statistics to 1000 points')                    ;
    xsav = x                                                               ;
    ysav = y                                                               ;
    fsav = f                                                               ;
    wsav = wt                                                              ;
    k    = m/nreduce                                                       ;
    k    = floor(k*(1:nreduce))                                            ;
    k    = k(find(k >= 1 & k <= m))                                        ;
    % ---
    x    = x(k)                                                            ;
    y    = y(k)                                                            ;
    f    = f(k)                                                            ;
    if (length(wt))
        wt = wt(k)                                                         ;
    end
    msav = m                                                               ;
    m    = nreduce                                                         ;
end
clear nreduce                                                              ;

% =========================================================================
% --- calculate variance COV matrix & correlation matrix of parameters
% --- & reevaluate the Jacobian at optimal values
% --- The following section is Ray Muzic's estimate for covariance 
% --- and correlation assuming covariance of data is a diagonal matrix 
% --- proportional to diag(1/wt.^2).
% --- cov matrix of data estimate is from 
% --- Bard Equation 7-5-13 and Row 1 of Table 5.1
% =========================================================================
jac = partialderivatives(x,f,p,dp,func)                                    ;
msk = find(dp ~= 0)                                                        ;
n   = length(msk)                                                          ; % reduce n to equal number of estimated parameters
jac = jac(:, msk)                                                          ; % use only fitted parameters

Qinv  = diag(wt.*wt)                                                       ;
Q     = diag((0*wt+1)./(wt.^2))                                            ;
resid = y-f                                                                ; %unweighted residuals

jtgjinv = []                                                               ;
jtgjinv = pinv(jac'*Qinv*jac)                                              ;
if ~isempty(jtgjinv)
    covp = resid' * Qinv * resid / (m-n) * jtgjinv                         ;
    d    = sqrt(abs(diag(covp)))                                           ;
    corp = covp./(d*d')                                                    ;
else
    disp('Warning : Couldn''t compute squared Jacobian inverse matrix!')   ;
    covp = zeros(n,n)                                                      ;
    corp = ones(n,n)                                                       ;
end

stdp = sqrt(diag(covp))                                                    ;

if ~isempty(msav)                                                            % restoring data
    x  = xsav                                                              ;
    y  = ysav                                                              ;
    f  = fsav                                                              ;
    m  = msav                                                              ;
    wt = wsav                                                              ;
end

j   = 1                                                                    ;
sig = zeros(size(p))                                                       ;
for i = 1 : length(stdp)
    while free(j)==0
        j=j+1                                                              ;
    end
    sig(j) = stdp(i)                                                       ;
    j = j + 1                                                              ;
end

r  = corrcoef(y.*wt,f.*wt)                                                 ;
r2 = r.*r'                                                                 ;
rv = sum(((f-y).*wt).^2/length(y))                                         ;


% =========================================================================
% --- subfunction
% =========================================================================
function df_dp = partialderivatives(x,f,p,dp,func)
% =========================================================================
%
% --- y = partialderivatives(x,y,p,dp,func)
% --- Returns the partial derivatives of function 'func'.
% --- 'x'(vect) is x axis values, 'y' is y values,
% --- 'p' and 'dp' are parameters and their variation.
% --- 'func' is the function concerned (y=func(x,p)).
% --- Output 'df_dp' is a vector (or matrix) of partials varying of 'dp'
%
% =========================================================================
[nr,nc] = size(x)                                                          ;
if ( nr < nc )
    x = x'                                                                 ;
end
p0    = p                                                                  ; % save init params
df_dp = zeros(length(x), length(p))                                        ;
i = find((abs(dp) <= 1e-12) | ~isfinite(dp) )                              ; % these parameters should not vary
if ( ~isempty(i) )
    dp(i) = 0                                                              ;
end
for i = 1 : length(p)
    if ( dp(i) ~= 0 )
        p(i) = p(i) + dp(i)                                                ;
        t = feval(func,p,x)                                                ;
        if ( length(t) ~= length(f) )
%             disp('Warn : fix length problem in jacobian')                  ;
            t = t(1:length(x))                                             ;
        end
        df_dp(:,i) = (t(:)-f(:)) / dp(i)                                   ;
        p = p0                                                             ;
    end
end
if (nr < nc)
    df_dp = df_dp'                                                         ;
end


% ---
% EOF
