function main
% ---
% --- Main function to perform the g2 calculation in 'real' time
% ---
% --- by MS 10162006


% ---
testfile = 'D:\testdata\Aerogel_fullframe_transmission\Aero_XR6_6_00001-00110.imm' ;
istart   = 1                                                               ;
iend     = 100                                                             ;
dstart   = 101                                                             ;
dend     = 110                                                             ;
repeats  = 2                                                               ;
% --- Dark image
for j = dstart : dend
    f = openmultiimm(testfile,j)                                           ;
    if ( j == dstart )
        Dark = zeros(size(f.imm(101:1100,101:1100)))                       ; 
    end
    Dark = Dark + f.imm(101:1100,101:1100)                                 ;
end
Dark = Dark / (dend-dstart+1)                                              ;

% --- create delays and maxLevel
dpl = 4                                                                    ;
delay = finddelays(repeats*(iend-istart+1),dpl,1)                          ;
if ( numel(delay) <= 7 )
    maxLevel = 1                                                           ;
else
    maxLevel = ceil((numel(delay)-7)/4) + 1                                ;           
end

% =========================================================================
% --- main loop
% =========================================================================
STOPon = 0                                                                 ;     
i      = 0                                                                 ;
% ---
while ( i < repeats * (iend-istart+1) || STOPon == 1 )
    
    % =====================================================================
    % --- modify loop counter, check for stop & update display
    % =====================================================================    
    i = i + 1                                                              ; % image counter (start @ 1)
    if ( mod(i,10) == 0 )
        disp(sprintf('Calculating at image #%6i',i))                       ;
    end
    
    % =====================================================================
    % --- load images, do background and lld
    % =====================================================================
%     Img = ones(100,1000) + 0.2 * rand(100,1000)                            ;
    f = openmultiimm(testfile,ceil(i/repeats))                             ;
    if ( i < repeats * (iend-istart+1) )
        g = openmultiimm(testfile,ceil(i/repeats)+1)                       ;
    else
        g = f                                                              ;
    end
    partf = 1/repeats *(repeats - mod(i-1,repeats))                        ;
    partg = 1 - partf                                                      ;
    Img = ( partf * f.imm(101:1100,101:1100)                            ...
          + partg * g.imm(101:1100,101:1100) ) - Dark                      ;
    Img(Img <= 15) = 1e-09                                                 ;
    
    % =====================================================================
    % --- initialization
    % =====================================================================
    if ( i == 1 )                                                            % initialization (only once)
        % ---
        avgImg  = zeros(size(Img,1),size(Img,2),'single')                  ; % initialize the average image
        % --- Level 1
        L01Img = zeros(size(Img,1),size(Img,2),2*dpl,'single')             ;
        % --- All higher Levels
        for k = 2 : maxLevel
            LImg = zeros(size(Img,1),size(Img,2),dpl+1,maxLevel-1,'single'); % initialize Image array
            LC   = zeros(maxLevel-1,1)                                     ; % initialize level counter
        end
        % ---
        G2 = zeros(size(Img,1),size(Img,2),(maxLevel+1)*dpl-1,'single')    ; % initialize the G2 tensor
        GC = zeros((maxLevel+1)*dpl-1,1)                                   ; % counter for the G2 sums
    end
    
    % =====================================================================
    % --- create average image
    % =====================================================================
    avgImg = avgImg + Img                                                  ;
    
    % =====================================================================
    % --- normalize image by average value in a partition
    % =====================================================================
    partImg = runmean(runmean(Img,25,2,'mirroredge'),25,1,'mirroredge')    ; % calculate the partition averaged image
    partImg(partImg<1e-09) = 1e-09                                         ; % remove zeros
    Img = Img ./ partImg                                                   ; % normalize image by the partition averaged image
    
    % =====================================================================
    % --- create & update image stacks for each level
    % =====================================================================
    
    % --- Level 1
    L01Img(:,:,mod(i-1,2*dpl)+1) = Img(:,:)                                ; % replace the old with the new image in the level 1 stack
    
    % --- All higher levels    
    for k = 2 : maxLevel
        if (mod(i,2^(k-1)) == 0 && i >= 2^(k-1) )
            if ( i == 2^(k-1) )
                LC(k-1) = 0                                                ; % start level counter
            end
            LC(k-1) = LC(k-1) + 1                                          ;
            if ( k > 2 )
                LImg(:,:,mod(LC(k-1)-1,dpl+1)+1,k-1) = 1/2*(LImg(:,:,mod(LC(k-2)-1,dpl+1)+1,k-2) ...
                                                           +LImg(:,:,mod(LC(k-2)-2,dpl+1)+1,k-2))   ;
            elseif ( k == 2 )
                LImg(:,:,mod(LC(k-1)-1,dpl+1)+1,k-1) = 1/2*(L01Img(:,:,mod(i-1,2*dpl)+1) ...
                                                           +L01Img(:,:,mod(i-2,2*dpl)+1))   ;
            end
        end
    end
    
    
    % =====================================================================
    % --- create g2 calculation
    % =====================================================================
    
    % --- Level 1
    for l = 1 : 2*dpl-1
        if ( i >= l+1 )
            G2(:,:,l) = G2(:,:,l) + L01Img(:,:,mod(i-1  ,2*dpl)+1)      ...
                                 .* L01Img(:,:,mod(i-1-l,2*dpl)+1)         ; % latest image times an 'l' times older image
            GC(l) = GC(l) + 1                                              ;
        end
    end

    % --- All higher Levels
    for k = 2 : maxLevel
        for l = 1 : dpl
            n = k*dpl + l-1                                                ; % delay number
            
            if (  mod(i,2^(k-1)) == mod(2^k+(l-1)*2^(k-1),2^(k-1))      ...
               && i >= 2^k+(l-1)*2^(k-1) )
                
                if ( i == 2^k+(l-1)*2^(k-1) )
                    GC(n) = 0                                             ; % start level counter
                end
                GC(n) = GC(n) + 1                                         ;
                G2(:,:,n) = G2(:,:,n)                 ...
                          + LImg(:,:,mod(GC(n)-1  ,dpl+1)+1,k-1)       ...
                         .* LImg(:,:,mod(GC(n)-1+l,dpl+1)+1,k-1)          ; % latest image times an 'l' times older image
            end
            
        end
    end

end


% =========================================================================
% --- some control figures
% =========================================================================
figure(1), imagesc(avgImg/i),set(gca,'YDir','normal')
% ---
for m = 1 : size(G2,3)
    if ( GC(m) ~= 0 )
        G2(:,:,m) = G2(:,:,m) / GC(m)                                      ; % normalize G2 by the corresponding G2 counter
    end
end
% ---
avgG2 = squeeze(mean(mean(G2,2),1))                                        ; % just average each full G2 map
figure(2), semilogx(delay,avgG2(1:numel(delay)),'bo-')                     ;


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;

% ---
% EOF
