function mainanalysis(nBatch)
%
% Perform the static  & dynamic analysis
%
% the results of this routine are stored in a structure 'ccdimginfo.result'
%
% Michael Sprung
% $Revision: 1.0b $  $Date: 2005/02/23 $
% $Revision: 1.1  $  $Date: 2005/04/24 $ check normalizations
% $Revision: 1.2  $  $Date: 2005/08/29 $ recheck normalizations
% $Revision: 1.3  $  $Date: 2005/09/12 $ add minor improvements
% $Revision: 1.4  $  $Date: 2005/09/15 $ make only static analysis possible
% $Revision: 1.5  $  $Date: 2005/12/13 $ normalize intensities properly
% $Revision: 1.6  $  $Date: 2006/07/28 $ corrected flaw in finding
%                                        saturated pixels


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0)                                                                   ;


% =========================================================================
% --- define test switches
% =========================================================================
storeG2Tensoron = 0                                                        ; % switch if the full g2 tensor is stored or not (only offered for reflection geometry)
saveSetup       = 0                                                        ; % switch to store setup information or not
pause(0)                                                                   ;


% =========================================================================
% --- create data and dark indices
% =========================================================================
dataIndex = ccdimginfo.ndata0todo(nBatch):ccdimginfo.ndataendtodo(nBatch)  ; % data index
darkIndex = ccdimginfo.ndark0todo(nBatch):ccdimginfo.ndarkendtodo(nBatch)  ; % dark index
pause(0)                                                                   ;


% =========================================================================
% --- try to create saving subdirectory 'result' (needed if the analysis
% --- button was never pressed, e.g in batchmode)
% =========================================================================
warning('off','MATLAB:MKDIR:DirectoryExists')                              ;
dummy = mkdir(ccdimginfo.savepath)                                         ; % try to create directory to store the results
if ( dummy == 0 )                                                            % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'            ...
                    ;'   Try to save in Matlab XPCSGUI folder!'}]       ...
        ,'HorizontalAlignment','left')                                     ;
    pause(0.001)                                                           ;
    % ---
    ccdimginfo.savepath    = fullfile(pwd,'result')                        ; % try to create a subdirectory in the Matlab XPCSGUI folder
    dummy = mkdir(ccdimginfo.savepath)                                     ;
    if ( dummy == 0 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',...
            [msgstr;{'   Failed creating saving directory !'            ...
                    ;'   Please check write permissions!'}]             ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        return                                                             ;
    end
end
clear dummy                                                                ;
warning('on','MATLAB:MKDIR:DirectoryExists')                               ;
pause(0)                                                                   ;


% =========================================================================
% --- check ccdimginfo.result substructure
% --- clear old results of selected batches 
% --- save the 'new' setup information to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save the setup information !'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================

% --- check size off all result fields and change them if neccessary
if isfield(ccdimginfo,'result') 
    if (  isfield(ccdimginfo.result,'analysistime')                     ...
       && numel(ccdimginfo.result.analysistime)  <numel(ccdimginfo.preset))
        ccdimginfo.result.analysistime{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'totalIntensity')                   ...
       && numel(ccdimginfo.result.totalIntensity)<numel(ccdimginfo.preset))
        ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []    ;
    end
    if (  isfield(ccdimginfo.result,'aIt')                              ...
       && numel(ccdimginfo.result.aIt)           <numel(ccdimginfo.preset))
        ccdimginfo.result.aIt{numel(ccdimginfo.preset)} = []            ;
    end
    if (  isfield(ccdimginfo.result,'Iqphi')                            ...
       && numel(ccdimginfo.result.Iqphi)         <numel(ccdimginfo.preset))
        ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2')                               ...
       && numel(ccdimginfo.result.g2)            <numel(ccdimginfo.preset))
        ccdimginfo.result.g2{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'g2avg')                            ...
       && numel(ccdimginfo.result.g2avg)         <numel(ccdimginfo.preset)) 
        ccdimginfo.result.g2avg{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2avgErr')                         ... 
       && numel(ccdimginfo.result.g2avgErr)      <numel(ccdimginfo.preset))
        ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'goodPixels')                       ...
       && numel(ccdimginfo.result.goodPixels)    <numel(ccdimginfo.preset))
        ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)} = []        ;
    end
    if (  isfield(ccdimginfo.result,'NEPhoton')                         ...
       && numel(ccdimginfo.result.NEPhoton)      <numel(ccdimginfo.preset))
        ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'delay')                            ...
       && numel(ccdimginfo.result.delay)         <numel(ccdimginfo.preset))
        ccdimginfo.result.delay{numel(ccdimginfo.preset)} = []             ;
    end
else
    ccdimginfo.result.analysistime{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []        ;
    ccdimginfo.result.aIt{numel(ccdimginfo.preset)}            = []        ;
    ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.g2avg{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)}     = []        ;
    ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.delay{numel(ccdimginfo.preset)}          = []        ;
end
    
% --- reset result fields for the batch to be analyzed
ccdimginfo.result.analysistime{nBatch}   = []                              ; % clear any previous time stamp
ccdimginfo.result.totalIntensity{nBatch} = []                              ; % clear the total intensity per slice
ccdimginfo.result.aIt{nBatch}            = []                              ; % clear averaged image
ccdimginfo.result.Iqphi{nBatch}          = []                              ; % clear Iqphi vector
ccdimginfo.result.g2{nBatch}             = []                              ; % clear g2 tensor
ccdimginfo.result.g2avg{nBatch}          = []                              ; % clear averaged g2's
ccdimginfo.result.g2avgErr{nBatch}       = []                              ; % clear the error of the averaged g2's
ccdimginfo.result.goodPixels{nBatch}     = []                              ; % clear the number of good pixels in a dynamic q phi partition
ccdimginfo.result.NEPhoton{nBatch}       = []                              ; % clear the indix of pixels which received not enough photons
ccdimginfo.result.delay{nBatch}          = []                              ; % clear vector with delay times
    
% --- store result field in result & remove result field the ccdimginfo
result     = ccdimginfo.result                                             ;
ccdimginfo = rmfield(ccdimginfo,'result')                                  ;
if ( isfield(ccdimginfo,'testimg') )
    ccdimginfo = rmfield(ccdimginfo,'testimg')                             ;
end
    
% --- save the setup information
if ( saveSetup == 1 )
    nobatch  = sprintf('batch%03i_',nBatch)                                ;
    savefile = sprintf([ccdimginfo.name,nobatch,'setup.mat'])              ; % define filename for binary output
    savefull = fullfile(ccdimginfo.savepath,savefile)                      ; % build full file name
    save(savefull,'ccdimginfo')                                            ; % store the setup information
    clear nobatch                                                          ;
end

% --- restore result field in ccdimginfo & remove result
ccdimginfo.result = result                                                 ;
clear result                                                               ;
    
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving setup information',            ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;


% =========================================================================
% --- check if a flatfield correction is needed
% =========================================================================
if ( ccdimginfo.flatfield == 1 )
    [flatfield] = getflatfield(ccdimginfo)                                 ;
else
    flatfield = []                                                         ;
end
pause(0)                                                                   ;


% =========================================================================
% --- if needed, apply blemish file to ccdimginfo.usermask
% =========================================================================
if ( ccdimginfo.blemish == 1 )
    [blemish] = getblemish(ccdimginfo)                                     ;
    ccdimginfo.usermask = ccdimginfo.usermask .* blemish                   ;
end
pause(0)                                                                   ;


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1 = ccdimginfo.maskroi(1,2)                                               ; % row    start of ROI
R2 = ccdimginfo.maskroi(2,2)                                               ; % row    end   of ROI
C1 = ccdimginfo.maskroi(1,1)                                               ; % column start of ROI
C2 = ccdimginfo.maskroi(2,1)                                               ; % column end   of ROI


% =========================================================================
% --- create averaged dark image 'sumDark'
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Create dark image & dark infos!'}]        ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;

sumDark    = zeros( ccdimginfo.row_end - ccdimginfo.row_beg + 1         ...
                  , ccdimginfo.col_end - ccdimginfo.col_beg + 1,'single')  ; % initialize dark image as single
% ---
for i = 1:length(darkIndex)
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        f = openfile(ccdimginfo.imagefile{nBatch},darkIndex(i))            ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark = sumDark + f.imm                                      ; % add dark images
        else
            sumDark = sumDark + f.imm .* flatfield                         ; % add flatfield corrected dark images
        end
    else
        f = openfile(ccdimginfo.darkfile{nBatch},darkIndex(i))              ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark = sumDark + f.imm                                      ; % add dark images
        else
            sumDark = sumDark + f.imm .* flatfield                         ; % add flatfield corrected dark images
        end
    end
end

if ( numel(darkIndex) > 0 )
    sumDark     = sumDark / numel(darkIndex)                               ; % calculate averaged dark image
end
% ---
sumDarkMean = single( sum(sumDark(:)) / numel(sumDark) )                   ; % calculate mean value of averaged dark image
sumDarkStd  = single( std(sumDark(:)) )                                    ; % calculate standard deviation of averaged dark image
% ---
satLevel = single(ccdimginfo.saturation * ones(size(sumDark)) - sumDark)   ; % calculate new saturation values including dark values
% ---
clear darkIndex f                                                          ;
pause(0)                                                                   ;
% ---
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish creating the dark image',             ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;


% =========================================================================
% =========================================================================
% =========================================================================
% --- Main loop
% =========================================================================
% =========================================================================
% =========================================================================


% =========================================================================
% --- initialization
% =========================================================================
dpl   = ccdimginfo.dpl                                                     ;
delay = finddelays(length(dataIndex),dpl,1)                                ;
dt    = mean(diff(immelapsed(ccdimginfo.imagefile{nBatch}               ...
                            ,dataIndex(1),dataIndex(end))))                ;
delay = delay * dt                                                         ;
if ( numel(delay) <= 7 )
    maxLevel = 1                                                           ;
else
    maxLevel = ceil((numel(delay)-7)/4) + 1                                ;           
end
% ---
avgImg = zeros(ccdimginfo.row_end-ccdimginfo.row_beg+1                  ...
              ,ccdimginfo.col_end-ccdimginfo.col_beg+1                  ...
              ,'single')                                                   ; % initialize the average image
% --- Level 1
L01Img = zeros(R2-R1+1,C2-C1+1,2*dpl,'single')                             ; % initialize 1st level image array
% --- All higher Levels
for k = 2 : maxLevel
    LImg = zeros(R2-R1+1,C2-C1+1,dpl+1,maxLevel-1,'single')                ; % initialize higher level image arrays
    LC   = zeros(maxLevel-1,1)                                             ; % initialize level counter
end
% ---
G2 = zeros(R2-R1+1,C2-C1+1,(maxLevel+1)*dpl-1,'single')                    ; % initialize the G2 tensor
GC = zeros((maxLevel+1)*dpl-1,1)                                           ; % counter for the G2 sums
    


% =========================================================================
% --- start of data file import
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start to import data !'}]                 ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================

notzero = 1.0e-09                                                          ; % create a variable which is almost zero to prevent 'Divide by Zero' errors
for i = 1 : length(dataIndex)                                                % loop over all frames

    % =====================================================================
    % --- show progress message
    % =====================================================================
    if ( i == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('\r   Import image no. %04i of %04i images'    ...
                         ,i,length(dataIndex))                             ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    if ( length(dataIndex) < 1000 && (floor(i/10)-i/10+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                    'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Import image no. %04i of %04i images',     ...
                         i,length(dataIndex))                              ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end
    if ( length(dataIndex) >= 1000 && (floor(i/100)-i/100+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                    'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Import image no. %04i of %04i images',     ...
                        i,length(dataIndex))                              ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end


    % =====================================================================
    % --- load data, remove dark image & flatfield correction
    % =====================================================================
    f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i))                ; % load image file
    if ( isempty(flatfield) == 1 )                                           % no flatfield correction needed
        trueData = f.imm - sumDark                                         ; %  do backgroud subtraction
    else
        trueData = f.imm .* flatfield  - sumDark                           ; % perform flatfield correction and do backgroud subtraction
    end
    clear f    

    % =====================================================================
    % --- do lower level discrimination (LLD)
    % =====================================================================
    if (ccdimginfo.lld <= 0)
        trueData(trueData <= abs(ccdimginfo.lld)) = notzero                ; % reset all values below the lower level discriminator to ~zero
    end
    if (ccdimginfo.lld > 0)
        trueData(trueData <= ccdimginfo.lld * sumDarkStd) = notzero    ; % reset all values below the lower level discriminator to ~zero
    end
        

    % =====================================================================
    % --- create avgImg
    % =====================================================================
    avgImg = avgImg + trueData                                             ; % add images to the averaged image (after lower level discrimination)
    
    % =====================================================================
    % --- 1) find saturated pixels
    % --- 2) find pixels with not enough photons NEPhoton
    % --- 3) integrate the intensity of each image
    % --- 4) create the static q phi partition average image
    % --- 5) perform standard normalization
    % =====================================================================
    if ( i == 1 )
        saturatedPixels = cell(length(dataIndex),1)                        ;
    end
    S(:,:) = trueData(R1:R2,C1:C2)                                         ; % take dark subtracted values of ROI of each slice
    T(:,:) = satLevel(R1:R2,C1:C2)                                         ; % take saturation levels of ROI of each slice
    indices = find(S > T)                                                  ; % get the indices of saturated pixels
    saturatedPixels{i,1} = indices                                         ; % store the indices of saturated pixels in ROI units
    % ---
    if ( i == 1 )                                                            % these things need to be done only once
        minPhoton   = ceil(0.030 * length(dataIndex))                      ; % intensity must be found in at least 3% of the slices/frames
        dummy       = zeros(size(S,1),size(S,2)) + minPhoton               ; % create a slice sized matrix to check if enough photons hit a pixel
        NEPhoton    = find(ccdimginfo.usermask(R1:R2,C1:C2))               ; % start with all unmasked pixels in ROI
        clear minPhoton                                                    ;
    end
    ind1                  = find(S(NEPhoton))                              ; % find non zeros on a slice
    dummy(NEPhoton(ind1)) = dummy(NEPhoton(ind1)) - 1                      ; % reduce the value of dummy by one at pixels where one photon hit
    ind2                  = find(dummy(NEPhoton) > 0)                      ; % remaining pixels with not enough photons
    NEPhoton              = NEPhoton(ind2)                                 ; % reduce the size of the 'not enough photon counter'
    clear ind1 ind2                                                        ;
    % ---
    if ( i == 1 )
        totalIntensity = zeros(length(dataIndex),1,'single')               ;
    end
    S_ROI = S(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                       ;
    T_ROI = T(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                       ;
    totalIntensity(i,1)  = sum(S_ROI(S_ROI <= T_ROI) )                  ...
                         / ccdimginfo.efficiency                        ...
                         / ccdimginfo.adupphot                          ...
                         / ccdimginfo.preset(nBatch)                       ; % calculate the intensity [photons/sec] of the unmasked, unsaturated ROI
    clear S_ROI T_ROI                                                      ;
    % ---
    if ( i == 1 )
        goodsqphipartitions = ones(ccdimginfo.snoq,ccdimginfo.snophi)      ;
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                if ( isempty(ccdimginfo.smask{n,m}) )
                    goodsqphipartitions(n,m) = 0                           ; % count a partition as bad if it does not contain pixels
                end
            end
        end
    end
    % ---
    aIq     = zeros(ccdimginfo.snoq,ccdimginfo.snophi,'single')            ;
    partImg = zeros(R2-R1+1,C2-C1+1,'single')                              ;
    for n = 1 : ccdimginfo.snoq
        for m = 1 : ccdimginfo.snophi
            if ( numel(ccdimginfo.smask{n,m}) > 0 )
                Snm = S(ccdimginfo.smask{n,m})                             ; % take only (unmasked) pixels from one static q phi partition
                Tnm = T(ccdimginfo.smask{n,m})                             ; % take only (unmasked) pixels from one static q phi partition
                if (numel(Snm(Snm <= Tnm)) > 0)
                    aIq(n,m) = mean(Snm(Snm <= Tnm))                       ; % store static partition average
                    if ( goodsqphipartitions(n,m) == 1 && aIq(n,m) ~= 0 )
                        partImg(ccdimginfo.smask{n,m})= aIq(n,m)           ; % partImg : only not saturated unmasked pixels are averaged
                    else
                        goodsqphipartitions(n,m) = 0                       ; % mark this partition as bad if aIq = 0 
                    end
                end
            end
        end
    end
    partImg(partImg<=notzero) = notzero                                    ;
    clear Snm Tnm S T                                                      ;
    % ---
    Img = trueData(R1:R2,C1:C2) ./ partImg                                 ; % normalize image by the partition averaged image
    clear trueData partImg                                                 ;

    
    
    
    % =====================================================================
    % --- create & update image stacks for each level
    % =====================================================================
    
    % --- Level 1
    L01Img(:,:,mod(i-1,2*dpl)+1) = Img(:,:)                                ; % replace the old with the new image in the level 1 stack
    clear Img                                                              ;
    
    % --- All higher levels    
    for k = 2 : maxLevel
        if (mod(i,2^(k-1)) == 0 && i >= 2^(k-1) )
            if ( i == 2^(k-1) )
                LC(k-1) = 0                                                ; % start level counter
            end
            LC(k-1) = LC(k-1) + 1                                          ;
            if ( k > 2 )
                LImg(:,:,mod(LC(k-1)-1,dpl+1)+1,k-1) = 1/2*(LImg(:,:,mod(LC(k-2)-1,dpl+1)+1,k-2) ...
                                                           +LImg(:,:,mod(LC(k-2)-2,dpl+1)+1,k-2))   ;
            elseif ( k == 2 )
                LImg(:,:,mod(LC(k-1)-1,dpl+1)+1,k-1) = 1/2*(L01Img(:,:,mod(i-1,2*dpl)+1) ...
                                                           +L01Img(:,:,mod(i-2,2*dpl)+1))   ;
            end
        end
    end
    
    
    % =====================================================================
    % --- create g2 calculation
    % =====================================================================
    
    % --- Level 1
    for l = 1 : 2*dpl-1
        if ( i >= l+1 )
            G2(:,:,l) = G2(:,:,l) + L01Img(:,:,mod(i-1  ,2*dpl)+1)      ...
                                 .* L01Img(:,:,mod(i-1-l,2*dpl)+1)         ; % latest image times an 'l' times older image
            GC(l) = GC(l) + 1                                              ;
        end
    end

    % --- All higher Levels
    for k = 2 : maxLevel
        for l = 1 : dpl
            n = k*dpl + l-1                                                ; % delay number
            
            if (  mod(i,2^(k-1)) == mod(2^k+(l-1)*2^(k-1),2^(k-1))      ...
               && i >= 2^k+(l-1)*2^(k-1) )
                
                if ( i == 2^k+(l-1)*2^(k-1) )
                    GC(n) = 0                                             ; % start level counter
                end
                GC(n) = GC(n) + 1                                         ;
                G2(:,:,n) = G2(:,:,n)                 ...
                          + LImg(:,:,mod(GC(n)-1  ,dpl+1)+1,k-1)       ...
                         .* LImg(:,:,mod(GC(n)-1+l,dpl+1)+1,k-1)          ; % latest image times an 'l' times older image
            end
            
        end
    end
    
end    
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish data preparation in ',                ...
    num2str(t),' seconds.']}],'HorizontalAlignment','left')                ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of main loop
% =========================================================================



% =========================================================================
% --- create SPMask: mask to exclude saturated pixels in usermask
% --- create aIt   : time averaged intensity map <I(q,t)>_t (only once)
% --- create Iqphi : averaged intensity of static partitions of aIt
% --- mask all ~good static q phi partitions (see goodsqphipartitions) 
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start creating time averages'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
SPMask = ones(R2-R1+1,C2-C1+1,'single')                                    ; % initialize SPMask as ones (no saturated pixels) on ROI
for i = 1:length(dataIndex)                                                  % loop over all frames
    indices         = saturatedPixels{i,1}                                 ;
    SPMask(indices) = 0                                                    ; % set the Mask for all saturated pixels to zero
end
ccdimginfo.usermask(R1:R2,C1:C2) = SPMask.*ccdimginfo.usermask(R1:R2,C1:C2); % incoorperate the saturated pixel mask into the user mask
clear indices saturatedPixels SPMask                                       ;
% ---
dummy = ccdimginfo.usermask(R1:R2,C1:C2)                                   ; % take only ROI of usermask
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( goodsqphipartitions(n,m) == 0 )
            dummy(ccdimginfo.smask{n,m}) = 0                               ; % mask every pixel of a bad static q phi partition                        
        end
    end
end
ccdimginfo.usermask(R1:R2,C1:C2) = dummy(:,:)                              ; % store the corrected usermask
clear dummy                                                                ;
% ---
aIt               = zeros(size(ccdimginfo.qmap,1)                       ...
                         ,size(ccdimginfo.qmap,2),'single')                ; % initialize aIt (time averaged of dynamic image)
aIt(R1:R2,C1:C2)  = avgImg(R1:R2,C1:C2) / length(dataIndex)                ;
aIt(aIt<=notzero) = notzero                                                ; % reset values below ~zero in aIt to ~zero
% ---
Iqphi = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                           ; % initialize Iqphi
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                              % check if static q phi partition contains pixels
            dummy1 = aIt(R1:R2,C1:C2)                                      ; % take only ROI of aIt
            dummy2 = ccdimginfo.usermask(R1:R2,C1:C2)                      ; % take only ROI of usermask
            Inm    = dummy1(ccdimginfo.smask{n,m})                         ; % get values of aIt for all pixels of a static q phi partition
            Mnm    = dummy2(ccdimginfo.smask{n,m})                         ; % get values of usermask (with SPMask) for all pixels of a static q phi partition
            if ( numel(Inm(Mnm==1)) > 0 )
                Iqphi(n,m) = mean(Inm(Mnm==1))                             ; % calculate the average intensity in the averaged static q phi partitions Iqphi(sq,sphi)
            end
        end
    end
end
Iqphi(Iqphi<=notzero) = notzero                                            ;
clear Inm Mnm dummy1 dummy2                                                ;
% --- try to calculate absolute intensities
if ( numel(ccdimginfo.dpix) > 1 )                                            % pixel size is different in x & y direction
    dpix_x = ccdimginfo.dpix(1)                                            ;    
    dpix_y = ccdimginfo.dpix(2)                                            ;
else                                                                         % pixel size is identical in x & y direction
    dpix_x = ccdimginfo.dpix                                               ;
    dpix_y = dpix_x                                                        ;
end
Iqphi = Iqphi / ccdimginfo.efficiency / ccdimginfo.adupphot             ...
      / ccdimginfo.preset(nBatch)                                          ; % this can always be calculated (unit: [photons/sec])
ccdimginfo.normalization = 0                                               ; % standard normalization case for both geometries (unit: [photons/sec])
% ---
if ( ccdimginfo.geometry == 0 )                                              % in transmission mode special normalization are possiple
    Iqphi = Iqphi / ( dpix_x/ccdimginfo.rr * dpix_y/ccdimginfo.rr )        ; % divide by the solid angle; unit is still [photons/sec]
    ccdimginfo.normalization = 1                                           ; % normalization case set to one [photon/sec]
    if ( ccdimginfo.tflux == 1 )                                             % if the transmitted flux was measured, than 'beam_i' & 'ring_i_sample' are known 
        Iqphi = Iqphi / ccdimginfo.beam_i(nBatch)                       ...
              / ccdimginfo.ring_i(nBatch)                               ...
              * ccdimginfo.ring_i_sample(nBatch)                           ; % Iqphi can be normalized by transmitted flux (unit: dimensionless)
        ccdimginfo.normalization = 2                                       ; % normalization case set to two [no dimension]
    end
    if (  ccdimginfo.normalization == 2                                 ...
       && ccdimginfo.thicknessflag > 0 )                                     % further normalization might be possible (two cases)
        % --- case 3a
        if (  ccdimginfo.tflux == 1 && ccdimginfo.iflux == 1            ...
          && ccdimginfo.thicknessflag == 2 )                                % if the transmitted & incident flux were measured & the attenuation length is known
            Iqphi = Iqphi / ( 0.1 * ccdimginfo.atten_length )           ...  % divide by the attenuation length (convert to [cm])
                  / log( (ccdimginfo.beam_i_vacuum(nBatch)              ...
                         /ccdimginfo.ring_i_vacuum(nBatch))             ...
                       / (ccdimginfo.beam_i(nBatch)                     ...
                         /ccdimginfo.ring_i_sample(nBatch))  )             ; % divide by log(incident flux / transmitted flux)
            ccdimginfo.normalization = 3                                   ; % normalization case set to three (dimension: [1/cm])
        end
        % --- case 3b (check only if not case 3a)
        if (  ccdimginfo.normalization == 2                             ...
           && ccdimginfo.thicknessflag == 1 )                                % sample thickness in [mm] is known
             Iqphi = Iqphi / ( 0.1 * ccdimginfo.thickness )                ; % divide by sample thickness (converted to [cm]) 
             ccdimginfo.normalization = 3                                  ; % normalization case set to three (dimension: [1/cm])
        end
    end
else                                                                          % reflection geometry 
    if ( ccdimginfo.iflux == 1 )
        Iqphi = Iqphi / ccdimginfo.beam_i_vacuum(nBatch)                ...
              / ccdimginfo.ring_i(nBatch)                               ...
              * ccdimginfo.ring_i_vacuum(nBatch)                           ; % Iqphi can be normalized by incident flux (unit: dimensionless)  
        ccdimginfo.normalization = 4                                       ; % normalization case set to four (no dimension)
    end
end
clear dpix_x dpix_y                                                        ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
   'String',[msgstr;{['   Finish creating time averages',               ...
   ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')          ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of creating time averaged intensity maps <I(q,t)>_t
% =========================================================================


% =========================================================================
% --- create averaged correlation function of dynamic q phi partitions,
% --- calculate standard deviations & estimate the number of valid g2's
% =========================================================================
if ( ccdimginfo.analysistype == 1 )
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Calculate averaged g2 functions!'}]   ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================

    g2avg    = ones(ccdimginfo.dnoq,ccdimginfo.dnophi,size(G2,3),'single') ; % initialize the averaged g2 tensor
    g2avgErr =zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,size(G2,3),'single') ; % initialize the tensor for the error of the averaged g2 tensor
    % ---
    goodPixels       = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,'single')   ; % initialize the matrix to store the number of good pixels in a q phi partition
    g2mask           = ccdimginfo.usermask(R1:R2,C1:C2)                    ; % create a mask for the correlation functions
    g2mask(NEPhoton) = 0                                                   ; % mask 'not enough photon' pixels on g2mask
    % ---
    for k = 1 : size(G2,3)                                                   % loop over all delays
        g2dummy1 = G2(:,:,k)                                               ; % take one delay of the correlation tensor
        g2dummy1(g2mask==0) = -1000                                        ; % set masked pixels to -1000
        % ---
        for n = 1 : ccdimginfo.dnoq
            for m = 1 : ccdimginfo.dnophi
                g2dummy2 = g2dummy1(ccdimginfo.dmask{n,m})                 ; % take only values of one dynamic q phi partition
                if ( numel(g2dummy2(g2dummy2~=-1000)) > 0 )
                    g2avg(n,m,k)    = mean(g2dummy2(g2dummy2~=-1000))      ; % average only valid entrees of the g2 tensor
                    if ( k == 1 )
                        goodPixels(n,m) =                               ...
                            numel(g2dummy2(g2dummy2~=-1000))               ; % get the number of good pixels in a dynamic q phi partition
                    end
                    g2avgErr(n,m,k) = single( sqrt(1/goodPixels(n,m))   ...
                                    * std(g2dummy2(g2dummy2~=-1000)))      ; % calculate the error of the valid g2 functions (Std of the Mean!)
                else
                    g2avg(n,m,k)    = 1                                    ;
                    g2avgErr(n,m,k) = 0                                    ;
                    if ( k == 1 )
                        goodPixels(n,m) = 0                                ;
                    end
                end
            end
        end
        % ---
    end
    % ---
    clear g2dummy1 g2dummy2 k n m                                          ;

    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,                                  ...
                         'Tag','xpcsmain_EditMessage'),'string')           ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish averaging g2 functions',          ...
        ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')     ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
end


% =========================================================================
% --- store the correlation functions to the output structure
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Assign results to output structure!'}]    ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% ---

ccdimginfo.result.totalIntensity{nBatch}     = totalIntensity              ; % store the totalIntensity on a slice/frame
ccdimginfo.result.aIt{nBatch}                = aIt                         ; % store the averaged image
ccdimginfo.result.Iqphi{nBatch}              = Iqphi                       ; % store the averaged intensity in the static q phi partitions of the averaged image
if ( ccdimginfo.analysistype == 1 )
    if ( ccdimginfo.kinetics == 1 && storeG2Tensoron == 1 )
        ccdimginfo.result.g2{nBatch}         = G2                          ; % store g2 tensor (for each pixel) only in kinetic mode (full frame too big!!!)
    else
        ccdimginfo.result.g2{nBatch}         = []                          ;
    end
    ccdimginfo.result.g2avg{nBatch}          = g2avg                       ; % store the averaged g2 functions for each dynamic q phi partition
    ccdimginfo.result.g2avgErr{nBatch}       = g2avgErr                    ; % store the error of the averaged g2 functions
    ccdimginfo.result.goodPixels{nBatch}     = goodPixels                  ; % store the number of valid g2 functions in each dynamic q phi partition
    ccdimginfo.result.NEPhoton{nBatch}       = NEPhoton                    ; % store the indix of pixels which received not enough photons
    ccdimginfo.result.delay{nBatch}          = delay                       ; % store the delay vector
end
clear totalIntensity sumDark aIt g2final g2avg goodPixels NEPhoton delay   ;
% ---
analysistime = fix(clock)                                                  ;
ccdimginfo.result.analysistime{nBatch} =                                ...
    sprintf('%04i-%02i-%02i %02i:%02i:%02i',analysistime(1)             ...  % years
           ,analysistime(2),analysistime(3),analysistime(4)             ...  % months, days, hours
           ,analysistime(5),analysistime(6) )                              ; % minutes, seconds
clear analysistime                                                         ;

% ---
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish assigning results',                   ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;


% =========================================================================
% --- save ccdimginfo to figure
% =========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;


% =========================================================================
% --- save results to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save result file to disk!'}]              ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% ---

savefile = sprintf([ccdimginfo.name,'result.mat'])                         ; % define filename for binary output
savefull = fullfile(ccdimginfo.savepath,savefile)                          ; % build full file name
save(savefull,'ccdimginfo')                                                ; % store the main structure

% ---
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving result file',                  ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;

                
% ---
% EOF
