% =========================================================================
% --- initialization
% =========================================================================
avgImg   = zeros(rows,cols,'single')                                       ; % initialize the average image (full size)

% --- 
dpl      = 4                                                               ; % define the delays per level
noframes = 200                                                             ; % define the number of frames to be anaylzed
delay    = finddelays(noframes,dpl,1)                                      ; % find possible delays
dt       = 1.000                                                           ; % defien the time between frames
delay    = delay * dt                                                      ; % convert delays to times

% --- calculate the maximum level of delays 
if ( numel(delay) <= 2*dpl-1 )
    maxLevel = 1                                                           ;
else
    maxLevel = ceil((numel(delay)-(2*dpl-1))/dpl) + 1                      ;           
end

% --- Image stacks for all Levels
LImg = zeros(rows,cols,2*dpl,maxLevel,'single')                            ; % initialize image arrays (ROI size) for all levels
LC   = zeros(maxLevel,1)                                                   ; % initialize level counter

% --- G2 tensor
G2 = zeros(rows,cols,(maxLevel+1)*dpl-1,'single')                          ; % initialize the G2 tensor (ROI size)
GC = zeros((maxLevel+1)*dpl-1,1)                                           ; % counter for the G2 sums
    

% =========================================================================
% --- start of data file import (loop over all frames)
% =========================================================================
notzero = 1.0e-09                                                          ; % create a variable which is almost zero to prevent 'Divide by Zero' errors
for i = 1 : noframes                                                         % loop over all frames


    % =====================================================================
    % --- get one image, remove dark image & flatfield correction
    % =====================================================================

    % =====================================================================
    % --- do lower level discrimination (LLD)
    % =====================================================================
        
    % =====================================================================
    % --- create avgImg & 
    % =====================================================================
    avgImg = avgImg + Img                                                  ; % add images to the averaged image (after lower level discrimination)
    
    % =====================================================================
    % --- create & update image stacks for each level
    % =====================================================================
    % --- Level 1
    LImg(:,:,mod(i-1,2*dpl)+1,1) = Img(:,:)                                ; % replace the oldest image in the level 1 stack with the new one
    LC(1) = LC(1) + 1                                                      ; % increase level 1 counter
    clear Img                                                              ;
    % --- All higher levels    
    for k = 2 : maxLevel
        if ( mod(i,2^(k-1)) == 0 )
            LImg(:,:,mod(LC(k),2*dpl)+1,k) = 1/2                        ...
                    * ( LImg(:,:,mod(LC(k-1)-2,2*dpl)+1,k-1)            ...
                      + LImg(:,:,mod(LC(k-1)-1,2*dpl)+1,k-1) )             ; % average (k-1) level images into level k
            LC(k) = LC(k) + 1                                              ; % increase level k counter
        end
    end
    
    
    % =====================================================================
    % --- create g2 calculation
    % =====================================================================
    % --- Level 1
    for l = 1 : 2*dpl-1                                                  % loop over level 1 delays
        if ( i >= l+1 )
            G2(:,:,l) = G2(:,:,l) + LImg(:,:,mod(i-1  ,2*dpl)+1,1)  ...
                                 .* LImg(:,:,mod(i-1-l,2*dpl)+1,1)     ; % latest image times an 'l' times older image
            GC(l)     = GC(l) + 1                                      ; % increase G2 counter
        end
    end
    % --- All higher Levels
    for k = 2 : maxLevel                                                 % loop over all higher levels
        for l = 1 : dpl                                                  % loop over all delays of level k
            n   = k*dpl + l-1                                          ; % delay number
            g2n = dpl * 2^(k-1) + l * 2^(k-1)                          ; % 1st change to add to the delay n of g2
            if (  mod(i,2^(k-1)) == mod(g2n,2^(k-1)) && i >= g2n )
                G2(:,:,n) = G2(:,:,n)                               ...
                          + LImg(:,:,mod(GC(n)+dpl+l-1,2*dpl)+1,k)  ...
                         .* LImg(:,:,mod(GC(n)        ,2*dpl)+1,k)     ; % latest image times an 'l' times older image
                GC(n)     = GC(n) + 1                                  ; % increase G2 counter
            end
        end
    end
    
end
% =========================================================================
% --- end of main loop
% =========================================================================


% --- create normalized averaged image (full size)
aIt               = avgImg / noframes                                      ; % calculated aIt (normalized average of images)
aIt(aIt<=notzero) = notzero                                                ; % reset values below ~zero in aIt to ~zero
clear avgImg noframes                                                      ;

% --- calculate the partition averaged 'averaged image'
Iqphi     = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                       ; % initialize Iqphi
partIqphi = zeros(rows,cols,'single')                                      ; % initialize partIqphi (ROI sized imaged with partition averaged values)
dummy1    = aIt                                                            ; % take only ROI of aIt
dummy2    = ccdimginfo.usermask                                            ; % take only ROI of usermask
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                              % check if static q phi partition contains pixels
            Inm    = dummy1(ccdimginfo.smask{n,m})                         ; % get values of aIt for all pixels of a static q phi partition
            Mnm    = dummy2(ccdimginfo.smask{n,m})                         ; % get values of usermask (with SPMask) for all pixels of a static q phi partition
            if ( numel(Inm(Mnm==1)) > 0 )
                Iqphi(n,m) = mean(Inm(Mnm==1))                             ; % calculate the average intensity in the averaged static q phi partitions Iqphi(sq,sphi)
                partIqphi(ccdimginfo.smask{n,m}) = Iqphi(n,m)              ; % fill the partition averaged image
            end
        end
    end
end
Iqphi(Iqphi<=notzero)         = notzero                                    ;
partIqphi(partIqphi<=notzero) = notzero                                    ;
clear Inm Mnm dummy1 dummy2                                                ;

% ---
for k = 1 : size(G2,3)
    if ( GC(k) ~= 0 )
        G2(:,:,k) = G2(:,:,k) / GC(k) ./ (partIqphi(:,:)).^2               ; % normalize G2 by the corresponding G2 counter & the partition average
    end
end
clear GC partIqphi                                                         ;

% --- average over dynamic partitions
