function NX_structure
% --- NX_structure Calculate the structure factor of a concentrated
% --- solution of Latex spheres in glycerol assuming the cross section of
% --- a dilute solution contains a constant structure factor
%
% --- Michael Sprung
% --- $Revision: 1.0 $  $Date: 2006/08/015 $
% ---


%==========================================================================
% --- get the names of the ascii files of the cross sections of the 
% --- concentrated & dilute solutions of latex sheres in glycerol
% --- and load them
%==========================================================================
currentPath = pwd                                                          ;
% --- dillute solution
[filenameA, filepathA] = uigetfile(                                     ...
    {'*_IQ.ASCII', 'ASCII Files (*.ASCII)'},                            ...
     'Select I(Q) ASCII Files of the dilute solution',                  ...
     'MultiSelect', 'off')                                                 ;
fileA = fullfile(filepathA,filenameA)                                      ; % the whole filename

% --- start to look for the 2nd file in the 1st files directory
str = ['cd ','''',filepathA,'''']                                          ;
eval(str)                                                                  ;

% --- concentrated solution
[filenameB, filepathB] = uigetfile(                                     ...
    {'*_IQ.ASCII', 'ASCII Files (*.ASCII)'},                            ...
     'Select I(Q) ASCII Files of the concentrated solution',            ...
     'MultiSelect', 'off')                                                 ;
fileB = fullfile(filepathB,filenameB)                                      ; % the whole filename

% --- go back to m-file directory
str = ['cd ','''',currentPath,'''']                                        ;
eval(str)                                                                  ;

% --- load data
A = loadascii(fileA)                                                       ;
B = loadascii(fileB)                                                       ;


%==========================================================================
% --- data preparation
%==========================================================================
qd   = A.data(:,1)                                                         ;
Id   = A.data(:,2)                                                         ;

qc   = B.data(:,1)                                                         ;
Ic   = B.data(:,2)                                                         ;

minq = 0.0020                                                              ;
qd   = qd(qd>max(min(qc),minq))                                            ;
Id   = Id(qd>max(min(qc),minq))                                            ;

maxq = 0.011                                                               ;
q    = qd(qd<min(maxq,max(qc)))                                            ;
Ici  = interp1(qc,Ic,q)                                                    ; % get interpolated intensities
Idi  = Id(qd<min(maxq,max(qc)))                                            ; % use only identical part of Id

SFc  = Ici ./ Idi                                                          ; % calculate structure facotr


%==========================================================================
% --- figure creation
%==========================================================================
delete(findall(0,'Tag','SFvsQ1'))                                          ;
hFigSF_Q1 = figure('Name','structure factor vs. q','Tag','SFvsQ1')         ;
figure(hFigSF_Q1)                                                          ;
loglog(q,Idi,'ro-',q,Ici,'bo-')                                            ;
xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)             ;
ylabel('Intensity  [photons/sec]','FontSize',12)                           ;        

delete(findall(0,'Tag','SFvsQ2'))                                          ;
hFigSF_Q2 = figure('Name','structure factor vs. q','Tag','SFvsQ2')         ;
figure(hFigSF_Q2)                                                          ;
plot(q,SFc,'ro-')                                                          ;
xlim([0 max(q)])                                                           ;
spreadSFc = max(SFc(:)) - min(SFc(:))                                      ;
ylim([max(0,min(SFc)-0.1*spreadSFc) max(SFc)+0.1*spreadSFc])               ;
xlabel('Q [{\AA}$^{-1}$]','Interpreter','latex','FontSize',12)             ;
ylabel('Structure Factor','FontSize',12)                                   ;       


%==========================================================================
% --- ASCII output
%==========================================================================
SF_out = [q,SFc]                                                           ;
savingname = fullfile(filepathB,'structure_factor.txt')                    ;
save(savingname,'SF_out','-ascii','-double','-tabs')                       ;

%==========================================================================
% --- clear variables
%==========================================================================
clear str filenameA filepathA filenameB filepathB fileA fileB              ;


% ---
% EOF