function test_combineg2

% =========================================================================
% --- define some parameter and switches
% =========================================================================
FITon    = 1                                                               ;
FIGUREon = 1                                                               ;
niter    = 100                                                             ;


% =========================================================================
% --- loop to test combineg2
% =========================================================================
for j = 1 : niter
    % --- 1st data set
    x1 = logspace(-2.3,-0.3,40)                                            ;
    y1 = 0.4*exp(-2*0.1*x1)                                                ; % contrast : 0.40
    y1 = y1 + 1.40                                                         ; % baseline : 1.40
    e1 = 0.06  * ones(size(x1))                                            ; % error    : +- 0.03
    y1 = y1 + e1 .* (rand(size(y1))-0.5)                                   ;
    % --- 2nd data set
    x2 = logspace(-1,1,40)                                                 ;
    y2 = (0.35+0.50*(rand-0.5))*exp(-2*0.1*x2)                              ; % contrast : 0.35 +- 0.25
    y2 = y2 + (1.20+0.80*(rand-0.5))                                       ; % baseline : 1.20 +- 0.40
    e2 = 0.06 * ones(size(x2))                                             ; % error    : +- 0.03
    y2 = y2 + e2 .* (rand(size(y2))-0.5)                                   ;
    % --- 3rd data set
    x3 = logspace(0.3,2.3,40)                                              ;
    y3 = 0.3*exp(-2*0.1*x3)                                                ; % contrast : 0.30
    y3 = y3 + 1.00                                                         ; % baseline : 1.00
    e3 = 0.06 * ones(size(x3))                                             ; % error    : +- 0.03
    y3 = y3 + e3 .*(rand(size(y3))-0.5)                                    ;
    % ---
    x1 = x1(:); x2 = x2(:); x3 = x3(:)                                     ;
    y1 = y1(:); y2 = y2(:); y3 = y3(:)                                     ;
    e1 = e1(:); e2 = e2(:); e3 = e3(:)                                     ;
    % --- figure of raw data
    if ( FIGUREon == 1 )
        figure(100)                                                        ;
        subplot(2,1,1)                                                     ;
        semilogx(x1,y1,'ro',x2,y2,'bo',x3,y3,'go')                         ;
    end
    % --- prepare data for combineg2
%     x       = {x1;x2;x3}                                                   ; % data meaningfull on linear x axis
    x       = {log10(x1);log10(x2);log10(x3)}                              ; % data meaningfull on logarithmic x axis
    y       = {y1;y2;y3}                                                   ;
    e       = {e1;e2;e3}                                                   ;
    % --- call combineg2
%     [x,y,e]   = combineg2(x,y,e,'low2high')                                ;
    [x,y,e]   = combineg2(x,y,e,'high2low')                                ;
    % --- entangle combineg2 output
%     x4      = x{1,1}                                                       ; % data meaningfull on linear x axis
%     x5      = x{2,1}                                                       ; % data meaningfull on linear x axis
%     x6      = x{3,1}                                                       ; % data meaningfull on linear x axis
    x4      = 10.^x{1,1}                                                   ; % data meaningfull on logarithmic x axis
    x5      = 10.^x{2,1}                                                   ; % data meaningfull on logarithmic x axis
    x6      = 10.^x{3,1}                                                   ; % data meaningfull on logarithmic x axis
    y4      = y{1,1}                                                       ;
    y5      = y{2,1}                                                       ;
    y6      = y{3,1}                                                       ;
    e4      = e{1,1}                                                       ;
    e5      = e{2,1}                                                       ;
    e6      = e{3,1}                                                       ;
    % --- figure of merged data
    if ( FIGUREon == 1 )
        figure(100)                                                        ;
        subplot(2,1,2)                                                     ;
        semilogx(x4,y4,'ro',x5,y5,'bo',x6,y6,'go')                         ;
    end

    
    if ( FITon ==1 )
        % =================================================================
        % --- define fitting function, start parameter and [dp niter tol]
        % =================================================================
        xc       = [x4;x5;x6]                                              ;
        yc       = [y4;y5;y6]                                              ;
        ec       = [y4;y5;y6]                                              ;
        func     = @singleExponent                                         ; % handle to fitting function
        free     = [1 1 1]                                                 ; % define which parameters to vary
        pin      = [mean(yc(end-6:end)) mean(yc(1:6))-mean(yc(end-6:end)) 1.0]     ; % define start parameter [Baseline Contrast Gamma]
        fcp      = [1e-01 1e03 1e-09]                                      ; % [dp max. iteration tol]
        % =================================================================
        % --- call Levenberg-Marquart algorithm 
        % --- start parameters are very constraint 
        % =================================================================
        [p,sig,f,covp,corp,r2,rv]=levenbergfit(xc,yc,ec,pin,free,func,fcp) ;
        % =================================================================
        % --- collect fitting parameter
        % =================================================================
        if ( j == 1)
            Baseline = zeros(niter,1)                                      ;
            Contrast = zeros(niter,1)                                      ;
            Gamma    = zeros(niter,1)                                      ;            
        end
        Baseline(j) = p(1)                                                 ;
        Contrast(j) = p(2)                                                 ;
        Gamma(j)    = p(3)                                                 ;
    end
    
end

if ( FITon == 1 )
    Baseline = Baseline'                                                   ;
    Contrast = Contrast'                                                   ;
    Gamma    = Gamma'                                                      ;
    % ---
    meanBaseline = mean(Baseline)                                          ;
    meanContrast = mean(Contrast)                                          ;
    meanGamma    = mean(Gamma)                                             ;
    stdBaseline  = std(Baseline)                                           ;
    stdContrast  = std(Contrast)                                           ; 
    stdGamma     = std(Gamma)                                              ;
    % ---
    BaselineInfo = [meanBaseline,stdBaseline]                                  
    ContrastInfo = [meanContrast,stdContrast]                                  
    GammaInfo    = [meanGamma   ,stdGamma]                                     
    % ---
    Params     = [Baseline,Contrast,Gamma]                                 ;
    % ---
    meanParams = zeros(size(Params))                                       ;
    stdParams  = zeros(size(Params))                                       ;
    % ---
    meanParams(:,1) = meanBaseline                                         ;
    meanParams(:,2) = meanContrast                                         ;
    meanParams(:,3) = meanGamma                                            ;
    % ---
    stdParams(:,1)  = stdBaseline                                          ;
    stdParams(:,2)  = stdContrast                                          ;
    stdParams(:,3)  = stdGamma                                             ;
    % ---
    outliers        = abs(Params - meanParams) > 3 * stdParams             ;
    sum(outliers) 
end


% =========================================================================
% --- define fitting function
% =========================================================================
function y = singleExponent(x,p)
% --- y = gauss(x,p) 
% --- p = [Baseline Contrast Gamma]

y=p(1)+p(2)*exp(-2.0*p(3)*x)                                               ;


% ---
% EOF
