function uncompress_batch(nBatch)
%
% converts a batch of compressed data into an uncompressed '.imm' multifile 
%
% Before starting this routine please start 'xpcsgui' and load the
% batchinfofile
%
% Michael Sprung
% $Revision: 1.0b $  $Date: 2005/12/05 $
%


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0)                                                                   ;


% =========================================================================
% --- start progress messages
% =========================================================================
ta = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
msgnew = sprintf('\r   Starting to uncompress the batch!')                 ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
pause(0.005)                                                               ; % wait for 5ms to display message


% =========================================================================
% --- try to create directory to store uncompressed data
% =========================================================================
CDataDir  = ccdimginfo.imgPath                                             ;
UCDataDir = [CDataDir,'_ucp']                                              ; 
[dummy1 dummy2 dummy3] = mkdir(UCDataDir)                                  ; % try to create uncompressed data directory
if ( dummy1 == 0 )                                                           % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'}],         ...
        'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    return
end
clear dummy1 dummy2 dummy3                                                 ;
pause(0)                                                                   ;


% =========================================================================
% --- assign name of compressed data of Batch# nBatch to 'CFileName'
% =========================================================================
CFileName = ccdimginfo.imagefile{nBatch,1}                                 ;


% =========================================================================
% --- create the file name for the uncompressed data multifile
% =========================================================================
frameBeg = sprintf('%04i',ccdimginfo.ndata0(nBatch))                       ;
frameEnd = sprintf('%04i',ccdimginfo.ndataend(nBatch))                     ;
UCFileName = fullfile(UCDataDir,[ccdimginfo.name,'ucp_',                ...
                                 frameBeg,'-',frameEnd,'.imm'])            ;


% =========================================================================
% --- copy the batchinfo file to the uncompressed data directory
% --- modify the 'name' & 'child' parameter of the UC batchinfo file
% =========================================================================
[pathstr,name,ext,versn] = fileparts(ccdimginfo.batchinfoFile)             ; % get batchinfo file parts
UCBatchinfoFile          = fullfile(UCDataDir,[name,ext])                  ; % create full name for the uncompressed batchinfo file

[Cfid,message] = fopen(ccdimginfo.batchinfoFile)                           ; % open file for modifications
if ( Cfid == -1 )                                                            % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(Cfid)                                                            ;
    return                                                                 ;
end
[UCfid,message] = fopen(UCBatchinfoFile,'w+')                              ; % open file for modifications
if ( UCfid == -1 )                                                           % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(UCfid)                                                          ;
    return                                                                 ;
end

k = 0                                                                      ;
while ( feof(Cfid) == 0 )
    k = k + 1                                                              ; % line number counter
    scanline      = fgetl(Cfid)                                            ;
    equal_pos     = find(scanline == '=')                                  ;
    left_str      = scanline(1:equal_pos-1)                                ;
    dummy         = left_str                                               ;
    left_str(findstr(left_str,' '))=''                                     ;
    if ( strcmp(left_str,'name') == 1 || strcmp(left_str,'child') == 1 )
       LineStart  = dummy                                                  ;
       right_str  = scanline(equal_pos+1:end)                              ;
       LineEnd    = right_str                                              ; 
       LineEndNew = [LineEnd(1:end-1),'ucp_',LineEnd(end)]                 ;
       LineNew    = [LineStart,'=',LineEndNew]                             ;
       fprintf(UCfid,[LineNew,'\n'])                                       ;
    else
       fprintf(UCfid,[scanline,'\n'])                                      ;
    end
end
clear LineStart LineEnd LineEndNew LineNew

fclose(Cfid)                                                               ; % close file
fclose(UCfid)                                                              ; % close file
                                 
                                 
% =========================================================================
% --- open the compressed data file and create uncompressed data file
% =========================================================================
[Cfid,message] = fopen(CFileName)                                          ;        
if ( Cfid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(Cfid)                                                           ;
    return                                                                 ;
end
[UCfid,message] = fopen(UCFileName,'w+')                                   ;        
if ( UCfid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(UCfid)                                                          ;
    return                                                                 ;
end


% =========================================================================
% --- get image size
% =========================================================================
fseek(Cfid,108,'bof')                                                      ;
UCcols  = fread(Cfid,1 ,'int')                                             ;
UCrows  = fread(Cfid,1 ,'int')                                             ;
immSize = 2 * UCcols * UCrows + 1024                                       ; % 2 bytes per pixel + 1024 Byte header



% =========================================================================
% --- start of loop over all frames
% =========================================================================
CPos = 0                                                                   ;
FilesInBatch = ccdimginfo.ndataend(nBatch) - ccdimginfo.ndata0(nBatch) + 1 ;
for i = 1 : ccdimginfo.ndataend(nBatch) - ccdimginfo.ndata0(nBatch) + 1
    

    % =====================================================================
    % --- show progress message
    % =====================================================================
    if ( i == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('\r   Convert image no. %04i of %04i images'   ...
                         ,i,FilesInBatch)                                  ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    if ( FilesInBatch < 1000 && (floor(i/10)-i/10+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Convert image no. %04i of %04i images',    ...
                         i,FilesInBatch)                                   ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end
    if ( FilesInBatch >= 1000 && (floor(i/100)-i/100+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Convert image no. %04i of %04i images',    ...
                         i,FilesInBatch)                                   ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end


    % --- start to read and write headers
    fseek(Cfid ,CPos+(i-1)*1024,'bof')                                     ; % go to the next header start of the compressed file
    fseek(UCfid,(i-1)*immSize,'bof')                                       ; % go to the next header start of the uncompressed file

    % ---
    magic = fread(Cfid,1 ,'*int')                                          ;
    fwrite(UCfid,magic   ,'int')                                           ;
    clear magic                                                            ;
    % ---
    noversion = fread(Cfid,1 ,'*int')                                      ;
    fwrite(UCfid,noversion-1   ,'int')                                     ; % change the version flag to uncompressed files
    clear noversion                                                        ;
    % ---
    datestr = fread(Cfid,32,'*char')                                       ;
    fwrite(UCfid,datestr   ,'char')                                        ;
    clear datestr                                                          ;
    % ---
    prefix = fread(Cfid,16,'*char')                                        ;
    fwrite(UCfid,prefix   ,'char')                                         ;
    clear prefix                                                           ;
    % ---
    number = fread(Cfid,1 ,'*int')                                         ;
    fwrite(UCfid,number   ,'int')                                          ;
    clear number                                                           ;
    % ---
    suffix = fread(Cfid,16,'*char')                                        ;
    fwrite(UCfid,suffix   ,'char')                                         ;
    clear suffix                                                           ;
    % ---
    monitor = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,monitor   ,'int')                                         ;
    clear monitor                                                          ;
    % ---
    shutter = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,shutter   ,'int')                                         ;
    clear shutter                                                          ;
    % ---
    row_beg = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,row_beg   ,'int')                                         ;
    clear row_beg                                                          ;
    % ---
    row_end = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,row_end   ,'int')                                         ;
    clear row_end                                                          ;
    % ---
    col_beg = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,col_beg   ,'int')                                         ;
    clear col_beg                                                          ;
    % ---
    col_end = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,col_end   ,'int')                                         ;
    clear col_end                                                          ;
    % ---
    row_bin = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,row_bin   ,'int')                                         ;
    clear row_bin                                                          ;
    % ---
    col_bin = fread(Cfid,1 ,'*int')                                        ;
    fwrite(UCfid,col_bin   ,'int')                                         ;
    clear col_bin                                                          ;
    % ---
    cols = fread(Cfid,1 ,'*int')                                           ;
    fwrite(UCfid,cols   ,'int')                                            ;
    clear cols                                                             ;
    % ---
    rows = fread(Cfid,1 ,'*int')                                           ;
    fwrite(UCfid,rows   ,'int')                                            ;
    clear rows                                                             ;
    % ---
    bytes = fread(Cfid,1 ,'*int')                                          ;
    fwrite(UCfid,bytes   ,'int')                                           ;
    clear bytes                                                            ;
    % ---
    kinetics = fread(Cfid,1 ,'*int')                                       ;
    fwrite(UCfid,kinetics   ,'int')                                        ;
    clear kinetics                                                         ;
    % ---
    kinwinsize = fread(Cfid,1 ,'*int')                                     ;
    fwrite(UCfid,kinwinsize   ,'int')                                      ;
    clear kinwinsize                                                       ;
    % ---
    elapsed = fread(Cfid,1 ,'*double')                                     ;
    fwrite(UCfid,elapsed   ,'double')                                      ;
    clear elapsed                                                          ;
    % ---
    preset = fread(Cfid,1 ,'*double')                                      ;
    fwrite(UCfid,preset   ,'double')                                       ;
    clear preset                                                           ;
    % ---
    topup = fread(Cfid,1 ,'*int')                                          ;
    fwrite(UCfid,topup   ,'int')                                           ;
    clear topup                                                            ;
    % ---
    inject = fread(Cfid,1 ,'*int')                                         ;
    fwrite(UCfid,inject   ,'int')                                          ;
    clear inject                                                           ;
    % ---
    dlen = fread(Cfid,1 ,'*int')                                           ;
    fwrite(UCfid,dlen   ,'int')                                            ;
    CData =  double(dlen)                                                  ;
    clear dlen                                                             ;
    % ---
    pdbs_count = fread(Cfid,1 ,'*int')                                     ;
    fwrite(UCfid,pdbs_count   ,'int')                                      ;
    clear pdbs_count                                                       ;
    % ---
    buffer_number = fread(Cfid,1 ,'*uint32')                               ;
    fwrite(UCfid,buffer_number   ,'uint32')                                ;
    clear buffer_number                                                    ;
    % ---
    systick = fread(Cfid,1 ,'*uint32')                                     ;
    fwrite(UCfid,systick   ,'uint32')                                      ;
    clear systick                                                          ;
    % ---
    pv1 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv1   ,'char')                                            ;
    clear pv1                                                              ;
    % ---
    pv1VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv1VAL   ,'float')                                        ;
    clear pv1VAL                                                           ;
    % ---
    pv2 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv2   ,'char')                                            ;
    clear pv2                                                              ;
    % ---
    pv2VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv2VAL   ,'float')                                        ;
    clear pv2VAL                                                           ;
    % ---
    pv3 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv3   ,'char')                                            ;
    clear pv3                                                              ;
    % ---
    pv3VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv3VAL   ,'float')                                        ;
    clear pv3VAL                                                           ;
    % ---
    pv4 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv4   ,'char')                                            ;
    clear pv4                                                              ;
    % ---
    pv4VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv4VAL   ,'float')                                        ;
    clear pv4VAL                                                           ;
    % ---
    pv5 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv5   ,'char')                                            ;
    clear pv5                                                              ;
    % ---
    pv5VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv5VAL   ,'float')                                        ;
    clear pv5VAL                                                           ;
    % ---
    pv6 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv6   ,'char')                                            ;
    clear pv6                                                              ;
    % ---
    pv6VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv6VAL   ,'float')                                        ;
    clear pv6VAL                                                           ;
    % ---
    pv6 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv6   ,'char')                                            ;
    clear pv6                                                              ;
    % ---
    pv7VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv7VAL   ,'float')                                        ;
    clear pv7VAL                                                           ;
    % ---
    pv8 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv8   ,'char')                                            ;
    clear pv8                                                              ;
    % ---
    pv8VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv8VAL   ,'float')                                        ;
    clear pv8VAL                                                           ;
    % ---
    pv9 = fread(Cfid,40,'*char')                                           ;
    fwrite(UCfid,pv9   ,'char')                                            ;
    clear pv9                                                              ;
    % ---
    pv9VAL = fread(Cfid,1 ,'*float')                                       ;
    fwrite(UCfid,pv9VAL   ,'float')                                        ;
    clear pv9VAL                                                           ;
    % ---
    pv10 = fread(Cfid,40,'*char')                                          ;
    fwrite(UCfid,pv10   ,'char')                                           ;
    clear pv10                                                             ;
    % ---
    pv10VAL = fread(Cfid,1 ,'*float')                                      ;
    fwrite(UCfid,pv10VAL   ,'float')                                       ;
    clear pv10VAL                                                          ;
    % ---
    if ( i == 1)
        CurrentFilePosition = ftell(UCfid)                                 ;
        MissingHeaderBytes  = 1024 - CurrentFilePosition + (i-1)*immSize   ;
    end
    if ( MissingHeaderBytes > 0 )
        for j = 1 : MissingHeaderBytes
            fwrite(UCfid,0   ,'uint8')                                     ;
        end
    end
    % --- finish to read and write headers

    
    
    % --- read the compressed data
    PixelNumber = floor(CData/8)                                           ;
    
    % --- fast code
    fseek(Cfid ,CPos+i*1024,'bof')                                         ; % go to the next data start of the compressed file
    data       = fread(Cfid,2*PixelNumber,'uint32')                        ; % read the full data block 
    datavalues = data(2:2:end)                                             ; % create vector with intensity values
    PixelIndex = data(1:2:end-1) + 1                                       ; % create vector with pixel indices
    PixelValue = single(uread(uwrite(datavalues,'uint32')               ...
                             ,PixelNumber,'float32'))'                     ; % convert data values to binary to float32 to single
                         
%     % --- slow code                     
%     fseek(Cfid ,CPos+i*1024,'bof')                                         ; % go to the next data start of the compressed file
%     PixelIndex = fread(Cfid,PixelNumber,'uint32',4) + 1                    ;
%     fseek(Cfid ,CPos+i*1024 + 4,'bof')                                     ; % go to the next data start of the compressed file
%     PixelValue = fread(Cfid,PixelNumber,'float=>single',4)                 ;  
    
    % --- create uncompressed data
    UCData = zeros(UCcols,UCrows)                                          ;
    UCData(PixelIndex) = PixelValue                                        ;
    
    % --- write uncompressed data
    fseek(UCfid,(i-1)*immSize+1024,'bof')                                  ;
    fwrite(UCfid,UCData,'ushort')                                          ;

    % --- calculate CPos for the next image
    CPos  = CPos + CData                                                   ; % calculate the next position of a hearder start 


end
% =========================================================================
% --- end of loop over all frames
% =========================================================================


% =========================================================================
% --- close the compressed data file and uncompressed data file
% =========================================================================
fclose(Cfid)                                                               ;
fclose(UCfid)                                                              ;


% =========================================================================
% --- finish message
% =========================================================================
tb = etime(clock,ta)                                                       ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
msgnew = sprintf(['\r   Finish to uncompress the batch in '             ...
                 ,num2str(tb),' seconds.'])                                ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
           ,'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
pause(0.005)                                                               ; % wait for 5ms to display message


% =========================================================================
% --- load the uncompressed batchinfo file
% =========================================================================
loadbatchinfo(UCBatchinfoFile)                                             ;


% ---
% EOF
