%UREAD  Read binary data from uint8 vector.
%   A = UREAD(S,SIZE,PRECISION) reads binary data from the
%   uint8 vector S and writes it into matrix A.
%   
%   The SIZE argument is optional; if not specified, one value is read
%   in.  This behavior is different than FREAD where all the values are
%   read to end of file.  If specified, size must be positive integer.
%   
%   The PRECISION argument is a string that specifies the format
%   of the data to be read. It commonly contains a datatype specifier
%   like 'int' or 'float' followed by an integer giving the size in
%   bits.  Any of the following strings may be used.  If not specified,
%   the default is 'uchar'.
%   
%       MATLAB     Description
%       'uchar'    unsigned character,  8 bits.
%       'schar'    signed character,  8 bits.
%       'int8'     integer, 8 bits.
%       'int16'    integer, 16 bits.
%       'int32'    integer, 32 bits.
%       'int64'    integer, 64 bits.
%       'uint8'    unsigned integer, 8 bits.
%       'uint16    unsigned integer, 16 bits.
%       'uint32'   unsigned integer, 32 bits.
%       'uint64'   unsigned integer, 64 bits.
%       'single'   floating point, 32 bits.
%       'float32'  floating point, 32 bits.
%       'double'   floating point, 64 bits.
%       'float64'  floating point, 64 bits.
%
%   For example,
%
%       c = uwrite(magic(5), 'double');
%       s = uread(c, 5, 'double');
%
%   UWRITE will create a uint8 vector c of length 200 with the binary
%   representation of the magic(5) array.
%   UREAD will read back the first column and return a double vector.
%
%
%   See also UWRITE, FREAD, FWRITE.
%
%   Copyright (c) 2002 Sridhar Anandakrishnan, sak@essc.psu.edu

