function mainanalysis_kinetics(nBatch)
%
% Perform the static  & dynamic analysis for data in kinetics mode
%
% the results of this routine are stored in a structure 'ccdimginfo.result'
%
% Michael Sprung
% $Revision: 1.0b $Date: 2005/02/23 $
% $Revision: 1.1  $Date: 2005/04/24 $ check normalizations
% $Revision: 1.2  $Date: 2005/08/29 $ recheck normalizations
% $Revision: 1.3  $Date: 2005/09/12 $ add minor improvements
% $Revision: 1.4  $Date: 2005/09/15 $ make only static analysis possible
% $Revision: 1.5  $Date: 2005/12/13 $ normalize intensities properly
% $Revision: 1.6  $Date: 2006/07/28 $ corrected flaw in finding
%                                        saturated pixels
% $Revision: 2.0  $Date: 2006/11/08 $ specialized for kinetics mode, 
%                                     only symmetric normalization

% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish getting application data
% =========================================================================


% =========================================================================
% --- define test switches
% =========================================================================
storeTensoron = 0                                                          ; % switch to store G2, IF & IP tensors storage
saveSetup     = 0                                                          ; % switch to store setup information
pause(0.001)                                                               ;
% =========================================================================
% --- finish defining test switches
% =========================================================================


% =========================================================================
% --- create data and dark indices
% =========================================================================
dataIndex = ccdimginfo.ndata0todo(nBatch):ccdimginfo.ndataendtodo(nBatch)  ; % data index
darkIndex = ccdimginfo.ndark0todo(nBatch):ccdimginfo.ndarkendtodo(nBatch)  ; % dark index
pause(0.001)                                                               ;
% =========================================================================
% --- finish creating data and dark indices
% =========================================================================


% =========================================================================
% --- try to create saving subdirectory 'result' (needed if the analysis
% --- button was never pressed, e.g in batchmode)
% =========================================================================
warning('off','MATLAB:MKDIR:DirectoryExists')                              ;
dummy = mkdir(ccdimginfo.savepath)                                         ; % try to create directory to store the results
if ( dummy == 0 )                                                            % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'            ...
                    ;'   Try to save in Matlab XPCSGUI folder!'}]       ...
        ,'HorizontalAlignment','left')                                     ;
    pause(0.001)                                                           ;
    % ---
    ccdimginfo.savepath    = fullfile(pwd,'result')                        ; % try to create a subdirectory in the Matlab XPCSGUI folder
    dummy = mkdir(ccdimginfo.savepath)                                     ;
    if ( dummy == 0 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',...
            [msgstr;{'   Failed creating saving directory !'            ...
                    ;'   Please check write permissions!'}]             ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        return                                                             ;
    end
end
clear dummy                                                                ;
warning('on','MATLAB:MKDIR:DirectoryExists')                               ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish trying to create saving subdirectory 'result'
% =========================================================================


% =========================================================================
% --- check ccdimginfo.result substructure
% --- clear old results of selected batches 
% --- save the 'new' setup information to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save the setup information !'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
% --- check size off all result fields and change them if neccessary
if isfield(ccdimginfo,'result') 
    if (  isfield(ccdimginfo.result,'analysistime')                     ...
       && numel(ccdimginfo.result.analysistime)  <numel(ccdimginfo.preset))
        ccdimginfo.result.analysistime{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'totalIntensity')                   ...
       && numel(ccdimginfo.result.totalIntensity)<numel(ccdimginfo.preset))
        ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []    ;
    end
    if (  isfield(ccdimginfo.result,'framespacing')                     ...
       && numel(ccdimginfo.result.framespacing)  <numel(ccdimginfo.preset))
        ccdimginfo.result.framespacing{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'aIt')                              ...
       && numel(ccdimginfo.result.aIt)           <numel(ccdimginfo.preset))
        ccdimginfo.result.aIt{numel(ccdimginfo.preset)} = []               ;
    end
    if (  isfield(ccdimginfo.result,'Iqphi')                            ...
       && numel(ccdimginfo.result.Iqphi)         <numel(ccdimginfo.preset))
        ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'G2')                               ...
       && numel(ccdimginfo.result.G2)            <numel(ccdimginfo.preset))
        ccdimginfo.result.G2{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'IF')                               ...
       && numel(ccdimginfo.result.IF)            <numel(ccdimginfo.preset))
        ccdimginfo.result.IF{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'IP')                               ...
       && numel(ccdimginfo.result.IP)            <numel(ccdimginfo.preset))
        ccdimginfo.result.IP{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'g2avg')                            ...
       && numel(ccdimginfo.result.g2avg)         <numel(ccdimginfo.preset)) 
        ccdimginfo.result.g2avg{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2avgErr')                         ... 
       && numel(ccdimginfo.result.g2avgErr)      <numel(ccdimginfo.preset))
        ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'goodPixels')                       ...
       && numel(ccdimginfo.result.goodPixels)    <numel(ccdimginfo.preset))
        ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)} = []        ;
    end
    if (  isfield(ccdimginfo.result,'NEPhoton')                         ...
       && numel(ccdimginfo.result.NEPhoton)      <numel(ccdimginfo.preset))
        ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'delay')                            ...
       && numel(ccdimginfo.result.delay)         <numel(ccdimginfo.preset))
        ccdimginfo.result.delay{numel(ccdimginfo.preset)} = []             ;
    end
else
    ccdimginfo.result.analysistime{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []        ;
    ccdimginfo.result.framespacing{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.aIt{numel(ccdimginfo.preset)}            = []        ;
    ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.G2{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.IF{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.IP{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.g2avg{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)}     = []        ;
    ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.delay{numel(ccdimginfo.preset)}          = []        ;
end
% --- reset result fields for the batch to be analyzed
ccdimginfo.result.analysistime{nBatch}   = []                              ; % clear any previous time stamp
ccdimginfo.result.totalIntensity{nBatch} = []                              ; % clear the total intensity per slice
ccdimginfo.result.framespacing{nBatch}   = []                              ; % clear the average time between frames
ccdimginfo.result.aIt{nBatch}            = []                              ; % clear averaged image
ccdimginfo.result.Iqphi{nBatch}          = []                              ; % clear Iqphi vector
ccdimginfo.result.G2{nBatch}             = []                              ; % clear G2 tensor
ccdimginfo.result.IF{nBatch}             = []                              ; % clear IF tensor
ccdimginfo.result.IP{nBatch}             = []                              ; % clear IP tensor
ccdimginfo.result.g2avg{nBatch}          = []                              ; % clear averaged g2's
ccdimginfo.result.g2avgErr{nBatch}       = []                              ; % clear the error of the averaged g2's
ccdimginfo.result.goodPixels{nBatch}     = []                              ; % clear the number of good pixels in a dynamic q phi partition
ccdimginfo.result.NEPhoton{nBatch}       = []                              ; % clear the indix of pixels which received not enough photons
ccdimginfo.result.delay{nBatch}          = []                              ; % clear vector with delay times
% --- store result field in result & remove result field the ccdimginfo
result     = ccdimginfo.result                                             ;
ccdimginfo = rmfield(ccdimginfo,'result')                                  ;
if ( isfield(ccdimginfo,'testimg') )
    ccdimginfo = rmfield(ccdimginfo,'testimg')                             ;
end
% --- save the setup information
if ( saveSetup == 1 )
    nobatch  = sprintf('batch%03i_',nBatch)                                ;
    savefile = sprintf([ccdimginfo.name,nobatch,'setup.mat'])              ; % define filename for binary output
    savefull = fullfile(ccdimginfo.savepath,savefile)                      ; % build full file name
    save(savefull,'ccdimginfo')                                            ; % store the setup information
    clear nobatch                                                          ;
end
% --- restore result field in ccdimginfo & remove result
ccdimginfo.result = result                                                 ;
clear result                                                               ;    
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving setup information',            ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish checking ccdimginfo.result substructure, clearing old results
% --- & saving the 'new' setup information to disk
% =========================================================================


% =========================================================================
% --- check if a flatfield correction is needed
% --- if needed, apply blemish file to ccdimginfo.usermask
% =========================================================================
if ( ccdimginfo.flatfield == 1 )
    [flatfield] = getflatfield(ccdimginfo)                                 ;
else
    flatfield = []                                                         ;
end
if ( ccdimginfo.blemish == 1 )
    [blemish] = getblemish(ccdimginfo)                                     ;
    ccdimginfo.usermask = ccdimginfo.usermask .* blemish                   ;
    clear blemish                                                          ;
end
pause(0.001)                                                               ;
% =========================================================================
% --- finish blemish and flatfield implementation
% =========================================================================


% =========================================================================
% --- check if data is stored in a compressed multifile
% =========================================================================
SByte = []                                                                 ;
if ( ccdimginfo.compressedmulti(nBatch) == 1 )
    if ( isempty(ccdimginfo.SBCM{nBatch}) == 1 )
        SByte = indexcompressedmultiimm(ccdimginfo.imagefile{nBatch})      ; % try to get starting byte position of indices
    else
        SByte = ccdimginfo.SBCM{nBatch}                                    ; % had already been created
    end
end
% =========================================================================
% --- finish checking if data is stored in a compressed multifile
% =========================================================================


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1      = ccdimginfo.maskroi(1,2)                                          ; % row    start of slice or frame ROI
R2      = ccdimginfo.maskroi(2,2)                                          ; % row    end   of slice or frame ROI
C1      = ccdimginfo.maskroi(1,1)                                          ; % column start of slice or frame ROI
C2      = ccdimginfo.maskroi(2,1)                                          ; % column end   of slice or frame ROI
% ---
nFrames = length(dataIndex)                                                ; % number of data frames
nSlices = ccdimginfo.lastslice-ccdimginfo.firstslice+1                     ; % number of good slices per frame
% ---
rows    = ccdimginfo.row_end - ccdimginfo.row_beg + 1                      ; % number of rows of full frame 
cols    = ccdimginfo.col_end - ccdimginfo.col_beg + 1                      ; % number of rows of full frame 
% =========================================================================
% --- finish defining some abbreviations
% =========================================================================


% =========================================================================
% --- create averaged dark image 'sumDark'
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Create dark image & dark infos!'}]        ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
sumDark    = zeros(rows,cols,'single')                                     ; % initialize dark image as single
if ( ccdimginfo.lld > 0 )
   sumDarkRMS = zeros(rows,cols,'single')                                  ; % initialize dark rms image as single
end
% ---
for i = 1 : length(darkIndex)
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        f = openfile(ccdimginfo.imagefile{nBatch},darkIndex(i))            ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark rms images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark rms images
            end
        end
    else
        f = openfile(ccdimginfo.darkfile{nBatch},darkIndex(i))             ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark images
            end
        end
    end
end
% ---
if ( numel(darkIndex) > 0 )
    sumDark     = sumDark    / numel(darkIndex)                            ; % calculate averaged dark image
    if ( ccdimginfo.lld > 0 )
        sumDarkRMS  = sumDarkRMS / numel(darkIndex) - sumDark.^2           ; % calculate averaged dark rms image
    end
end
clear darkIndex f                                                          ;
% ---
satLevel  = single(ccdimginfo.saturation * ones(size(sumDark)) - sumDark)  ; % calculate new saturation values including dark values
% ---
DImg      = satLevel(R1:R2,C1:C2)                                          ; % take saturation levels of ROI
DImg_Mask = DImg(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                    ; % take saturation levels of unmasked pixels in ROI
DImgNM    = cell(ccdimginfo.snoq,ccdimginfo.snophi)                        ;
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( numel(ccdimginfo.smask{n,m}) > 0 )
            DImgNM{n,m} = DImg(ccdimginfo.smask{n,m})                      ; % take only (unmasked) pixels from one static q phi partition
        end
    end
end
clear satLevel                                                             ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish creating the dark image',             ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of creating dark averages
% =========================================================================


% =========================================================================
% --- initialize 'colLow' & 'colHigh' for data sectioning (memorycheck_roi)
% =========================================================================
colLow  = zeros(ccdimginfo.noDataSection,1)                                ;
colHigh = zeros(ccdimginfo.noDataSection,1)                                ;
for noS = 1 : ccdimginfo.noDataSection
    colLow (noS) = ccdimginfo.dataSections(noS)                            ; % create vector with starting column positions for data sectioning
    if (noS < ccdimginfo.noDataSection)
        colHigh(noS) = ccdimginfo.dataSections(noS+1) - 1                  ; % create vector with ending column positions for data sectioning
    else
        colHigh(noS) = ccdimginfo.dataSections(noS+1)                      ; % set the last value of 'colHigh' to the last value of dataSections
    end
end
pause(0.001)                                                               ;
% =========================================================================
% --- finish initializing 'colLow' & 'colHigh' for data sectioning
% =========================================================================


% =========================================================================
% --- Main loop - Over one dataSection at a time
% =========================================================================
notzero = 1.0e-09                                                          ; % define 'notzero' to prevent 'Divide by Zero' errors
for noS = 1 : ccdimginfo.noDataSection

    % =====================================================================
    % --- define shortcuts 'A' & 'B'
    % =====================================================================
    A = colLow (noS)                                                       ; % starting column of this section with respect to ROI
    B = colHigh(noS)                                                       ; % ending   column of this section with respect to ROI
    pause(0.001)                                                           ;
    % =====================================================================
    % --- end of defining shortcuts 'A' & 'B'
    % =====================================================================


    % =====================================================================
    % --- define dpl, slicedpl, framespacing & slicespacing
    % --- calculate number of delays 'nDelay'
    % =====================================================================
    if ( noS == 1 )
        framespacings = immelapsed(ccdimginfo.imagefile{nBatch}         ...
                                  ,dataIndex(1),dataIndex(end) )           ; % get the times between the frames
        framespacing  = mean(diff(framespacings))                          ; % calculate the average time between frames      
        dpl           = ccdimginfo.dpl                                     ;
        clear framespacings                                                ;
        % ---
        slicespacing  = ccdimginfo.preset(nBatch)                          ;
        slicedpl      = ccdimginfo.slicedpl                                ;
        % ---
        if (nSlices > 1)
            slicedelays       = finddelays(nSlices,slicedpl,1)             ;
        else
            slicedelays = []                                               ;
        end
        framedelays = finddelays(nFrames,dpl,1)                            ;
        if (nSlices > 1)
            totaldelays = [slicedelays, framedelays]                       ;
        else
            totaldelays = framedelays                                      ;
        end
        nDelay = length(totaldelays)                                       ;
        clear slicedelays framedelays totaldelays                          ;
    end
    % =====================================================================
    % --- end of define dpl, slicedpl, framespacing & slicespacing
    % =====================================================================


    % =====================================================================
    % --- some initializations (largest tensor first !!!)
    % --- some tensors need to be created only once  !!!
    % =====================================================================
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Initialize large tensors !'}]         ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        IQTT    = zeros(R2-R1+1,B-A+1,nFrames,nSlices,'single') + notzero  ; % intensity tensor I(x,y,t_frame,t_slice)
        if ( noS == 1 )                                                      % tensor is small enough to be calculated only once
            G2  = zeros(R2-R1+1,C2-C1+1,nDelay,'single') + notzero         ; % initialize G2 tensor
            IF  = zeros(R2-R1+1,C2-C1+1,nDelay,'single') + notzero         ; % initialize IF (intensities future) tensor
            IP  = zeros(R2-R1+1,C2-C1+1,nDelay,'single') + notzero         ; % initialize IP (intensities past)   tensor
        end
    end
    trueFrame = zeros(rows,cols,'single')                                  ; % initialize trueFrame (background subtracted data) as single
    if (noS == 1)
        sumFrames = zeros(rows,cols,'single')                              ; % initialize sumFrames (time average of all images) as single
    end
    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,                                  ...
                         'Tag','xpcsmain_EditMessage'),'string')           ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish initializing large tensors',      ...
        ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')     ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
    % =====================================================================
    % --- end of some initializations (largest tensor first !!!)
    % =====================================================================
    

    % =====================================================================
    % --- loop over all images
    % =====================================================================
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Start to import data !'}]             ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================
    for i = 1 : nFrames                                                      % loop over all frames

        % =================================================================
        % --- show progress message
        % =================================================================
        if ( i == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('\r   Import image no. %05i of %05i images'...
                             ,i,nFrames)                                   ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
            pause(0.001)                                                   ; % wait for 1ms to display message
        end
        if ( nFrames <= 400 && (floor(i/25)-i/25+1) == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('   Import image no. %05i of %05i images', ...
                             i,nFrames)                                    ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
                pause(0.001)                                               ; % wait for 1ms to display message
        end
        if ( nFrames > 400 && nFrames <= 1000 && floor(i/50)-i/50+1 == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('   Import image no. %05i of %05i images', ...
                             i,nFrames)                                    ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
                pause(0.001)                                               ; % wait for 1ms to display message
        end
        if ( nFrames > 1000 && (floor(i/100)-i/100+1) == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('   Import image no. %05i of %05i images', ...
                             i,nFrames)                                    ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
                pause(0.001)                                               ; % wait for 1ms to display message
        end
        % =================================================================
        % --- end of showing progress message
        % =================================================================


        % =================================================================
        % --- load data, remove dark image & flatfield correction
        % =================================================================
        if ( ~isempty(SByte) == 1 )
            f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i)      ...
                        ,SByte(dataIndex(i)-ccdimginfo.ndata0(nBatch)+1))  ; % load image file with pointer           
        else
            f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i))        ; % load image file
        end
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            trueFrame = f.imm - sumDark                                    ; %  do backgroud subtraction
        else
            trueFrame = f.imm .* flatfield  - sumDark                      ; % perform flatfield correction and do backgroud subtraction
        end
        clear f                                                            ;
        % =================================================================
        % --- end of loading data, removing dark & flatfield correction
        % =================================================================
        

        % =================================================================
        % --- lower level discrimination (LLD)
        % =================================================================
        if (ccdimginfo.lld < 0)
            trueFrame(trueFrame <= abs(ccdimginfo.lld)) = notzero          ; % reset all values below the lower level discriminator to ~zero
        end
        if (ccdimginfo.lld > 0)
            trueFrame(trueFrame <= ccdimginfo.lld * sumDarkRMS) = notzero  ; % reset all values below the lower level discriminator to ~zero
        end
        % =================================================================
        % --- end of lower level discrimination (LLD)
        % =================================================================
        

        % =================================================================
        % --- create sumFrames (needed only once)
        % =================================================================
        if (noS == 1)
            sumFrames = sumFrames + trueFrame                              ; % add images to sumFrames (after lower level discrimination)
        end
        % =================================================================
        % --- end of creating sumFrames
        % =================================================================


        % =================================================================
        % --- loop over slices
        % =================================================================
        for j = 1 : nSlices                                                  % loop over good slices only
            
            k          = (j-1) + ccdimginfo.firstslice                     ; % shift the counter in a way that ccdimginfo.firstslice == 1
            sliceStart = ccdimginfo.sliceinfo(k,1) + R1 - 1                ; % get the sliceStart position for each slice
            sliceEnd   = ccdimginfo.sliceinfo(k,1) + R2 - 1                ; % get the sliceEnd   position for each slice

            % =============================================================
            % --- Image examination:
            % --- find saturated pixels (only once)
            % --- find pixels with not enough photons NEPhoton (only once)
            % --- integrate the intensity of each slices (only once)
            % =============================================================
            if ( noS == 1 )
                % --- get ROI of each slice from the frame
                S(:,:) = trueFrame(sliceStart:sliceEnd,C1:C2)              ; % take dark subtracted values of ROI of each slice
                % --- calculate the total intensitiy per slice
                if ( i == 1 && j == 1 )
                    totalIntensity = zeros(nFrames,nSlices,'single')       ;
                end
                S_ROI = S(ccdimginfo.usermask(R1:R2,C1:C2) == 1)           ; % get unmasked pixels
                totalIntensity(i,j)  = sum(S_ROI(S_ROI <= DImg_Mask) )  ...
                                     / ccdimginfo.efficiency            ...
                                     / ccdimginfo.adupphot              ...
                                     / ccdimginfo.preset(nBatch)           ; % calculate the intensity [photons/sec] of the unmasked, unsaturated ROI
                clear S_ROI                                                ;
                % --- find saturated pixels on the slice
                if ( i == 1 && j == 1 )
                    saturatedPixels = cell(nFrames,nSlices)                ;
                end
                indices = find(S > DImg)                                   ; % get the indices of saturated pixels
                saturatedPixels{i,j} = indices                             ; % store the indices of saturated pixels in ROI units
                % ---
                if ( ccdimginfo.analysistype == 1 )
                    % --- find pixels, which recieved less than 2 photons 
                    if ( i == 1 && j == 1 )                                  % these things need to be done only once
                        dummy    = zeros(size(S)) + 2                      ; % create a slice sized matrix to check if 2 photons hit a pixel
                        NEPhoton = find(ccdimginfo.usermask(R1:R2,C1:C2))  ; % start with all unmasked pixels in ROI
                    end
                    ind1                  = find(S(NEPhoton))              ; % find non zeros on a slice
                    dummy(NEPhoton(ind1)) = dummy(NEPhoton(ind1)) - 1      ; % reduce the value of dummy by one at pixels where one photon hit
                    ind2                  = find(dummy(NEPhoton) > 0)      ; % remaining pixels with not enough photons
                    NEPhoton              = NEPhoton(ind2)                 ; % reduce the size of the 'not enough photon counter'
                    clear ind1 ind2                                        ;
                    % --- mark all static partitions with pixels as good
                    goodsqphipartitions = ones(ccdimginfo.snoq,ccdimginfo.snophi);
                    for n = 1 : ccdimginfo.snoq
                        for m = 1 : ccdimginfo.snophi
                            if ( isempty(ccdimginfo.smask{n,m}) )
                                goodsqphipartitions(n,m) = 0               ; % count a partition as bad if it does not contain pixels
                            end
                        end
                    end
                end
            end
            % =============================================================
            % --- end of image examination
            % =============================================================

            
            % =============================================================
            % --- create the 4-D data tensor IQTT(x,y,t1,t2)
            % =============================================================
            if (  ccdimginfo.analysistype == 1 )
                IQTT(:,:,i,j) = trueFrame(sliceStart:sliceEnd,A:B)         ;
            end
            % =============================================================
            % --- end of creating the 4-D data tensor IQTT(x,y,t1,t2)
            % =============================================================


        end
        % =================================================================
        % --- end of loop over slices
        % =================================================================
        
        
    end
    clear trueFrame S dummy                                                ;
    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish data preparation in ',            ...
        num2str(t),' seconds.']}],'HorizontalAlignment','left')            ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
    % =====================================================================
    % --- end of loop over frames
    % =====================================================================

    
    % =====================================================================
    % --- start of the multi-tau calculation
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
            ,'String',[msgstr;{' ';'   Main call of multi-tau'}]        ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;
        % =================================================================
        maxsize              = 50                                          ; % number of MB of data send to 'rawg2' at a time
        sizematrix           = size(IQTT)                                  ;
        [cLow,cHigh,noparts] = checksize(sizematrix,maxsize)               ;
        clear sizematrix maxsize                                           ;
        % =================================================================
        % --- loop over data chunks
        % =================================================================
        for j = 1 : noparts

            % =============================================================
            % --- show progress message
            % =============================================================
            if ( j ~= 1 )
                msgstr = get(findall(hFigXPCSMain,'Tag',                ...
                             'xpcsmain_EditMessage'),'string')             ;
                msgnew =                                                ...
                    sprintf('   Correlate no. %02i of %02i data sets'   ...
                           ,j,noparts)                                     ;
                set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'), ...
                    'String',[msgstr;{msgnew}],                         ...
                    'HorizontalAlignment','left')                          ;
                pause(0.001)                                               ; % wait for 1ms to display message
            else
                msgstr = get(findall(hFigXPCSMain,'Tag',                ...
                             'xpcsmain_EditMessage'),'string')             ;
                msgnew =                                                ...
                    sprintf('\r   Correlate no. %02i of %02i data sets' ...
                           ,j,noparts)                                     ;
                set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'), ...
                    'String',[msgstr;{msgnew}],                         ...
                    'HorizontalAlignment','left')                          ;
                pause(0.001)                                               ; % wait for 1ms to display message
            end
            % =============================================================
            % --- end of showing progress message
            % =============================================================

            
            % =============================================================
            % --- main call of 'SNG2'
            % =============================================================
            [delay,G2part,IFpart,IPpart] =                              ...
                SNG2( IQTT(:,cLow(j):cHigh(j),:,:)                      ...
                    , dpl, framespacing, slicedpl, slicespacing)           ; % main call of multi-tau routine
            % =============================================================
            % --- end of main call of 'rawg2'
            % =============================================================

            
            % =============================================================
            % --- set the g2 tensor of a section together
            % =============================================================
            G2(:,A-C1+cLow(j):A-C1+cHigh(j),:) = G2part(:,:,:)                       ; % build the G2 tensor of a section
            IF(:,A-C1+cLow(j):A-C1+cHigh(j),:) = IFpart(:,:,:)                       ; % build the IF tensor of a section
            IP(:,A-C1+cLow(j):A-C1+cHigh(j),:) = IPpart(:,:,:)                       ; % build the IP tensor of a section
            % =============================================================
            % --- end of setting the g2 tensor of a section together
            % =============================================================

        end
        clear IQTT j cLow cHigh noparts G2part IFpart IPpart               ;
        % =================================================================
        % --- end of loop over data chunks
        % =================================================================
        
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish calling multi-tau ',          ...
            'in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')  ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end
    % =====================================================================
    % ------------ end of the main call of the multi-tau routine ----------
    % =====================================================================


    % =====================================================================
    % --- create SPMask: mask to exclude saturated pixels in usermask
    % --- mask all ~good static q phi partitions (see goodsqphipartitions) 
    % --- create aIt   : time averaged intensity map <I(q,t)>_t (only once)
    % --- create Iqphi : averaged intensity of static partitions of aIt
    % =====================================================================
    if ( noS == 1 )
        % =================================================================
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start creating time averages'}]   ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % =================================================================
        SPMask = ones(R2-R1+1,C2-C1+1,'single')                            ; % initialize SPMask as ones (no saturated pixels) on ROI
        for i = 1 : nFrames                                                  % loop over all frames
            for j = 1 : nSlices                                              % loop over good slices only
                indices         = saturatedPixels{i,j}                     ;
                SPMask(indices) = 0                                        ; % set the Mask for all saturated pixels to zero
            end
        end
        ccdimginfo.usermask(R1:R2,C1:C2) = SPMask .*                    ...
                                           ccdimginfo.usermask(R1:R2,C1:C2); % incoorperate the saturated pixel mask into the user mask
        clear indices saturatedPixels SPMask                               ;
        % ---
        if ( ccdimginfo.analysistype == 1 )
            dummy = ccdimginfo.usermask(R1:R2,C1:C2)                       ; % take only ROI of usermask
            for n = 1 : ccdimginfo.snoq
                for m = 1 : ccdimginfo.snophi
                    if ( goodsqphipartitions(n,m) == 0 )
                        dummy(ccdimginfo.smask{n,m}) = 0                   ; % mask every pixel of a bad static q phi partition                        
                    end
                end
            end
            ccdimginfo.usermask(R1:R2,C1:C2) = dummy(:,:)                  ; % store the corrected usermask
            clear dummy                                                    ;
        end
        % ---
        aIt       = zeros(size(ccdimginfo.qmap),'single')                  ; % initialize aIt (time averaged of dynamic image)
        sumFrames = sumFrames / nFrames                                    ; % calculate averaged data image (frame level, dynamic case)
        for j = 1 : nSlices                                                  % loop over good slices only
            k          = (j-1) + ccdimginfo.firstslice                     ; % shift the counter in a way that ccdimginfo.firstslice == 1
            sliceStart = ccdimginfo.sliceinfo(k,1) + R1 - 1                ; % get the sliceStart position for each slice
            sliceEnd   = ccdimginfo.sliceinfo(k,1) + R2 - 1                ; % get the sliceEnd   position for each slice
            % ---
            aIt(R1:R2,C1:C2) = aIt(R1:R2,C1:C2)                         ...
                             + sumFrames(sliceStart:sliceEnd,C1:C2)        ; % sum ROI of all good slices
        end
        aIt = aIt / nSlices                                                ; % normalize aIt by dividing with the number of good slices
        aIt(aIt<=notzero) = notzero                                        ; % reset values below ~zero in aIt to ~zero
        % ---
        Iqphi = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                   ; % initialize Iqphi
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                      % check if static q phi partition contains pixels
                    dummy1 = aIt(R1:R2,C1:C2)                              ; % take only ROI of aIt
                    dummy2 = ccdimginfo.usermask(R1:R2,C1:C2)              ; % take only ROI of usermask
                    Inm    = dummy1(ccdimginfo.smask{n,m})                 ; % get values of aIt for all pixels of a static q phi partition
                    Mnm    = dummy2(ccdimginfo.smask{n,m})                 ; % get values of usermask (with SPMask) for all pixels of a static q phi partition
                    if ( numel(Inm(Mnm==1)) > 0 )
                        Iqphi(n,m) = mean(Inm(Mnm==1))                     ; % calculate the average intensity in the averaged static q phi partitions Iqphi(sq,sphi)
                    end
                end
            end
        end
        Iqphi(Iqphi<=notzero) = notzero                                    ;
        clear Inm Mnm dummy1 dummy2                                        ;
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish creating time averages',      ...
            ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left') ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
        % =================================================================
    end
    clear i j k n m sliceStart sliceEnd sumFrames                          ;
    % =====================================================================
    % --- end of creating SPMask
    % --- end of masking all 'bad' static partitions 
    % --- end of creating aIt
    % --- end of creating Iqphi
    % =====================================================================

    
    % =====================================================================
    % --- try to calculate absolute intensities
    % =====================================================================
    if ( noS == 1 )
        if ( numel(ccdimginfo.dpix) > 1 )                                    % pixel size is different in x & y direction
            dpix_x = ccdimginfo.dpix(1)                                    ;
            dpix_y = ccdimginfo.dpix(2)                                    ;
        else                                                                 % pixel size is identical in x & y direction
            dpix_x = ccdimginfo.dpix                                       ;
            dpix_y = dpix_x                                                ;
        end
        Iqphi = Iqphi / ccdimginfo.efficiency / ccdimginfo.adupphot     ...
              / ccdimginfo.preset(nBatch)                                  ; % this can always be calculated (unit: [photons/sec])
        ccdimginfo.normalization = 0                                       ; % standard normalization case for both geometries (unit: [photons/sec])
        % ---
        if ( ccdimginfo.geometry == 0 )                                      % in transmission mode special normalization are possiple
            Iqphi = Iqphi / ( dpix_x/ccdimginfo.rr * dpix_y/ccdimginfo.rr ); % divide by the solid angle; unit is still [photons/sec]
            ccdimginfo.normalization = 1                                   ; % normalization case set to one [photon/sec]
            if ( ccdimginfo.tflux == 1 )                                     % if the transmitted flux was measured, than 'beam_i' & 'ring_i_sample' are known 
                Iqphi = Iqphi / ccdimginfo.beam_i(nBatch)               ...
                      / ccdimginfo.ring_i(nBatch)                       ...
                      * ccdimginfo.ring_i_sample(nBatch)                   ; % Iqphi can be normalized by transmitted flux (unit: dimensionless)
                ccdimginfo.normalization = 2                               ; % normalization case set to two [no dimension]
            end
            if (  ccdimginfo.normalization == 2                         ...
               && ccdimginfo.thicknessflag > 0 )                              % further normalization might be possible (two cases)
                % --- case 3a
                if (  ccdimginfo.tflux == 1 && ccdimginfo.iflux == 1    ...
                   && ccdimginfo.thicknessflag == 2 )                        % if the transmitted & incident flux were measured & the attenuation length is known
                    Iqphi = Iqphi / ( 0.1 * ccdimginfo.atten_length )   ...  % divide by the attenuation length (convert to [cm])
                          / log( (ccdimginfo.beam_i_vacuum(nBatch)      ...
                                 /ccdimginfo.ring_i_vacuum(nBatch))     ...
                               / (ccdimginfo.beam_i(nBatch)             ...
                                 /ccdimginfo.ring_i_sample(nBatch))  )     ; % divide by log(incident flux / transmitted flux)
                    ccdimginfo.normalization = 3                           ; % normalization case set to three (dimension: [1/cm])
                end
                % --- case 3b (check only if not case 3a)
                if (  ccdimginfo.normalization == 2                     ...
                   && ccdimginfo.thicknessflag == 1 )                        % sample thickness in [mm] is known
                    Iqphi = Iqphi / ( 0.1 * ccdimginfo.thickness )         ; % divide by sample thickness (converted to [cm]) 
                    ccdimginfo.normalization = 3                           ; % normalization case set to three (dimension: [1/cm])
                end
            end
        else                                                                 % reflection geometry 
            if ( ccdimginfo.iflux == 1 )
                Iqphi = Iqphi / ccdimginfo.beam_i_vacuum(nBatch)        ...
                      / ccdimginfo.ring_i(nBatch)                       ...
                      * ccdimginfo.ring_i_vacuum(nBatch)                   ; % Iqphi can be normalized by incident flux (unit: dimensionless)  
                ccdimginfo.normalization = 4                               ; % normalization case set to four (no dimension)
           end
        end
        % ---
        clear dpix_x dpix_y                                                ;
    end
    % =====================================================================
    % --- end of trying to calculate absolute intensities
    % =====================================================================
    
end
clear A B flatfield dataIndex goodsqphipartitions                          ; % clear the leftover large tensors
clear DImg DImgNM DImg_Mask SByte                                          ;
% =========================================================================
% --- End of loop over all sections
% =========================================================================


% =========================================================================
% --- create averaged correlation function of dynamic q phi partitions,
% --- calculate standard deviations & estimate the number of valid g2's
% =========================================================================
if ( ccdimginfo.analysistype == 1 )
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Calculate averaged g2 functions!'}]   ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================
    g2avg    =  ones(ccdimginfo.dnoq,ccdimginfo.dnophi,nDelay,'single')    ; % initialize the averaged g2 tensor
    g2avgErr = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,nDelay,'single')    ; % initialize the tensor for the error of the averaged g2 tensor
    % ---
    goodPixels       = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,'single')   ; % initialize the matrix to store the number of good pixels in a q phi partition
    g2mask           = ccdimginfo.usermask(R1:R2,C1:C2)                    ; % create a mask for the correlation functions
    g2mask(NEPhoton) = 0                                                   ; % mask 'not enough photon' pixels on g2mask
    % ---
    for k = 1 : nDelay                                                       % loop over all delays
        dummyIF   = IF(:,:,k)                                              ; % take delay k of IF
        dummyIP   = IP(:,:,k)                                              ; % take delay k of IF
        partIFk   = zeros(size(dummyIF)) + notzero                         ; % create partIFk matrix
        partIPk   = zeros(size(dummyIP)) + notzero                         ; % create partIPk matrix
        IFqphi    = zeros(ccdimginfo.snoq,ccdimginfo.snophi)               ; % initialize IFqphi
        IPqphi    = zeros(ccdimginfo.snoq,ccdimginfo.snophi)               ; % initialize IPqphi
        % ---
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                      % check if static q phi partition contains pixels
                    IFnm   = dummyIF(ccdimginfo.smask{n,m})                ; % get values of aIt for all pixels of a static q phi partition
                    IPnm   = dummyIP(ccdimginfo.smask{n,m})                ; % get values of aIt for all pixels of a static q phi partition
                    Mnm    = g2mask(ccdimginfo.smask{n,m})                 ; % get values of usermask for all pixels of a static q phi partition
                    if ( numel(IFnm(Mnm==1)) > 0 )
                        IFqphi(n,m) = mean(IFnm(Mnm==1))                   ; % calculate the average intensity in one partition in IF(k)
                        partIFk(ccdimginfo.smask{n,m}) = IFqphi(n,m)       ; % fill the partition averaged image
                    end
                    if ( numel(IPnm(Mnm==1)) > 0 )
                        IPqphi(n,m) = mean(IPnm(Mnm==1))                   ; % calculate the average intensity in one partition in IP(k)
                        partIPk(ccdimginfo.smask{n,m}) = IPqphi(n,m)       ; % fill the partition averaged image
                    end
                end
            end
        end
        % ---
        g2dummy1            = G2(:,:,k) ./ (partIFk .* partIPk)            ; % symmetric normalize G2(k) by the partition averages
        g2dummy1(g2mask==0) = -1000                                        ; % set masked pixels to -1000
        % ---
        for n = 1 : ccdimginfo.dnoq                                          % loop over dynamic q   partitions
            for m = 1 : ccdimginfo.dnophi                                    % loop over dynamic phi partitions
                g2dummy2 = g2dummy1(ccdimginfo.dmask{n,m})                 ; % take only values of one dynamic q phi partition
                if ( numel(g2dummy2(g2dummy2~=-1000)) > 0 )
                    g2avg(n,m,k)    = mean(g2dummy2(g2dummy2~=-1000))      ; % average only valid entrees of the g2 tensor
                    if ( k == 1 )
                        goodPixels(n,m) =                               ...
                            numel(g2dummy2(g2dummy2~=-1000))               ; % get the number of good pixels in a dynamic q phi partition
                    end
                    g2avgErr(n,m,k) = single( sqrt(1/goodPixels(n,m))    ...
                                    * std(g2dummy2(g2dummy2~=-1000)))      ; % calculate the error of the valid g2 functions (Std of the Mean!)
                else
                    g2avg(n,m,k)    = 1                                    ;
                    g2avgErr(n,m,k) = 0                                    ;
                    if ( k == 1 )
                        goodPixels(n,m) = 0                                ;
                    end
                end
            end
        end
        % ---
    end
    % ---
    clear IFqphi IPqphi partIFk partIPk dummyIF dummyIP Mnm IFnm IPnm      ;
    clear g2dummy1 g2dummy2 g2mask k n m                                   ;
    clear C1 C2 R1 R2                                                      ;
    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,                                  ...
                         'Tag','xpcsmain_EditMessage'),'string')           ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish averaging g2 functions',          ...
        ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')     ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
end
% =========================================================================
% --- finish creating averaged correlation function of dynamic partitions,
% --- calculating standard deviations & estimating the number of valid g2's
% =========================================================================


% =========================================================================
% --- store the correlation functions to the output structure
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Assign results to output structure!'}]    ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
ccdimginfo.result.totalIntensity{nBatch}     = totalIntensity              ; % store the totalIntensity on a slice/frame
ccdimginfo.result.framespacing{nBatch}       = framespacing                ; % store the time between frames
ccdimginfo.result.aIt{nBatch}                = aIt                         ; % store the averaged image
ccdimginfo.result.Iqphi{nBatch}              = Iqphi                       ; % store the averaged intensity in the static q phi partitions of the averaged image
if ( ccdimginfo.analysistype == 1 )
    if ( storeTensoron == 1 )
        ccdimginfo.result.G2{nBatch}         = G2                          ; % store g2 tensor (for each pixel) only in kinetic mode (full frame too big!!!)
        ccdimginfo.result.IF{nBatch}         = IF                          ; % store g2 tensor (for each pixel) only in kinetic mode (full frame too big!!!)
        ccdimginfo.result.IP{nBatch}         = IP                          ; % store g2 tensor (for each pixel) only in kinetic mode (full frame too big!!!)
    else
        ccdimginfo.result.G2{nBatch}         = []                          ;
        ccdimginfo.result.IF{nBatch}         = []                          ;
        ccdimginfo.result.IF{nBatch}         = []                          ;
    end
    ccdimginfo.result.g2avg{nBatch}          = g2avg                       ; % store the averaged g2 functions for each dynamic q phi partition
    ccdimginfo.result.g2avgErr{nBatch}       = g2avgErr                    ; % store the error of the averaged g2 functions
    ccdimginfo.result.goodPixels{nBatch}     = goodPixels                  ; % store the number of valid g2 functions in each dynamic q phi partition
    ccdimginfo.result.NEPhoton{nBatch}       = NEPhoton                    ; % store the indix of pixels which received not enough photons
    ccdimginfo.result.delay{nBatch}          = delay                       ; % store the delay vector
end
clear Iqphi g2avgErr
clear totalIntensity sumDark aIt G2 IF IP g2avg goodPixels NEPhoton delay  ;
% ---
analysistime = fix(clock)                                                  ;
ccdimginfo.result.analysistime{nBatch} =                                ...
    sprintf('%04i-%02i-%02i %02i:%02i:%02i',analysistime(1)             ...  % years
           ,analysistime(2),analysistime(3),analysistime(4)             ...  % months, days, hours
           ,analysistime(5),analysistime(6) )                              ; % minutes, seconds
clear analysistime                                                         ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish assigning results',                   ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish storing the results into the application data
% =========================================================================


% =========================================================================
% --- save application data 'ccdimginfo' to main figure
% =========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;
% =========================================================================
% --- finish saving application data 'ccdimginfo' to main figure
% =========================================================================


% =========================================================================
% --- save results to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save result file to disk!'}]              ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
savefile = sprintf([ccdimginfo.name,'result.mat'])                         ; % define filename for binary output
savefull = fullfile(ccdimginfo.savepath,savefile)                          ; % build full file name
save(savefull,'ccdimginfo')                                                ; % store the main structure
clear savefile savefull ccdimginfo                                         ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving result file',                  ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish saving results to disk
% =========================================================================


% =========================================================================
% --- clear leftover variables
% =========================================================================
clear colLow colHigh smooth storeTensoron slicespacing slicedpl saveSetup  ;
clear notzero noS nBatch msgstr msgnew rows cols nFrames nSlices           ;
clear dpl framespacing nDelay hFigXPCSMain                                 ;
whos
% =========================================================================
% --- finish clearing leftover variables
% =========================================================================


% =========================================================================
% --- subfunction checksize
% =========================================================================
function [cLow,cHigh,noparts] = checksize(sizematrix,maxsize)
% ---
% --- This functions checks in how many parts the data tensor should be
% --- split before it is sent to the multi-tau correlation routine and 
% --- creates the vectors colLow and colHigh with information,
% --- where to split it.
% ---
% --- The input variable should be the result of sizematrix = size(Matrix)
% ---

% totalsize = 2/1.048576e6 * prod(sizematrix)                              ; % size of the correlation tensor in MB (for 'uint16')
totalsize = 4/1.048576e6 * prod(sizematrix)                                ; % size of the correlation tensor in MB (for 'single')
% --- calculate the number of data sections needed
if (totalsize > maxsize)
    noparts = ceil(totalsize/maxsize)                                      ;
else
    noparts = 1                                                            ;
end
clear totalsize                                                            ;

% --- create vector of column positions to split the data
i                = 1 : noparts + 1                                         ;
colsections      = (i-1)*ceil(sizematrix(2)/noparts) + 1                   ;
colsections(end) = sizematrix(2)                                           ;
clear i                                                                    ;

% --- some initializations for the data sectioning
cLow  = zeros(noparts,1)                                                   ;
cHigh = zeros(noparts,1)                                                   ;
for j = 1 : noparts
   cLow (j) = colsections(j)                                               ;
   cHigh(j) = colsections(j+1) - 1                                         ;
end
cHigh(end) = sizematrix(2)                                                 ;
clear colsections                                                          ;
% =========================================================================
% --- end of subfunction checksize
% =========================================================================


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;
% =========================================================================
% --- end of subfunction finddelays
% =========================================================================


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;
% =========================================================================
% --- end of subfunction indexofdelay
% =========================================================================


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;
% =========================================================================
% --- end of subfunction levelofdelay
% =========================================================================


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;
% =========================================================================
% --- end of subfunction delayofindex
% =========================================================================


% ---
% EOF
