function newbatchinfo
%
% Creates a new 'batchinfo' file after all batches of a compressed data set 
% have been converted into uncompressed '.imm' multifiles by YORICK 
%
% Before starting this routine please start 'xpcsgui' and load the
% batchinfofile of the compressed data
% 
% This routine will copy the old 'batchinfo' to '*.oldbatchinfo' 
% and modify the '*.batchinfo' file accordingly.
% Finally, it will load the new 'batchinfo' file so that one can continue
% with the analysis right away.
%
% Michael Sprung
% $Revision: 1.0 $  $Date: 2006/06/14 $
%


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0)                                                                   ;


% =========================================================================
% --- copy the batchinfo file to the uncompressed data directory
% --- modify the 'name' & 'child' parameter of the UC batchinfo file
% =========================================================================
[pathstr,name] = fileparts(ccdimginfo.batchinfoFile)                       ; % get batchinfo file parts
OldBatchinfofile         = fullfile(pathstr,[name '.oldbatchinfo'])        ;
clear pathstr name                                                         ;

dummy  = copyfile(ccdimginfo.batchinfoFile,OldBatchinfofile)               ;
if ( dummy == 0 )                                                            % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed to copy batchinfo file!'}],             ...
        'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    return
end
clear dummy                                                                ;


[Cfid,message] = fopen(OldBatchinfofile)                                   ; % open the old batchinfo file to read it
if ( Cfid == -1 )                                                            % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(Cfid)                                                           ;
    return                                                                 ;
end
[UCfid,message] = fopen(ccdimginfo.batchinfoFile,'w+')                     ; % open the batchinfo file to write modifications
if ( UCfid == -1 )                                                           % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(UCfid)                                                          ;
    return                                                                 ;
end

k = 0                                                                      ;
while ( feof(Cfid) == 0 )
    k = k + 1                                                              ; % line number counter
    scanline      = fgetl(Cfid)                                            ;
    equal_pos     = find(scanline == '=')                                  ;
    left_str      = scanline(1:equal_pos-1)                                ;
    dummy         = left_str                                               ;
    left_str(findstr(left_str,' '))=''                                     ;
    if ( strcmp(left_str,'name') == 1 || strcmp(left_str,'child'))
       LineStart  = dummy                                                  ;
       right_str  = scanline(equal_pos+1:end)                              ;
       LineEnd    = right_str                                              ; 
       LineEndNew = [LineEnd(1:end-1),'ucp_',LineEnd(end)]                 ;
       LineNew    = [LineStart,'=',LineEndNew]                             ;
       fprintf(UCfid,[LineNew,'\n'])                                       ;
    else
       fprintf(UCfid,[scanline,'\n'])                                      ;
    end
end
clear LineStart LineEnd LineEndNew LineNew

fclose(Cfid)                                                               ; % close file
fclose(UCfid)                                                              ; % close file
                                 

% =========================================================================
% --- load the uncompressed batchinfo file
% =========================================================================
loadbatchinfo(ccdimginfo.batchinfoFile)                                    ;


% ---
% EOF
