function [cstart,clow,chigh] = customfitparameter(xdata,ydata)
% ---
% --- General comment:
% --- A great problem in fitting is a good choice of starting parameters
% --- & to define sensible limits for these parameters. The success
% --- probability to have a converging fit is often determined by this
% --- choice. So be carefull!!!
% ---
% --- User supplied custom fit parameter called from 'showresult.m'
% --- 
% --- The fit function needs a set of starting fitting parameters.
% --- It works best if also a lower and upper limit for the fitting
% --- parameters is defined. Please make sure that the number of parameters
% --- match to the custom fit function!!!
% ---
% --- Standard parameter (used in all g2 fit functions):
% ---
% --- x(1) : Baseline
% --- x(2) : Contrast
% --- x(3) : Gamma = 1/tau
% ---
% ---
% --- Custom parameter (used only for custom g2 fit functions):
% ---
% --- x(4) : ...
% --- x(5) : ...
% --- ...
% ---
% --- The data can often supply some information about how the fitting
% --- parameters should be chosen. Below is the parameter choice for the
% --- single exponential fit listed.
% ---
% --- Example : single exponential fit 
% ---
% --- 'F = x(1) + x(2) * exp( - 2 * x(3) * xdata )'
% ---

% ---
% --- please comment this code if you enter custom fitting parameter 
% ---
     if (numel(xdata) > 3)
          baseline = sum(ydata(end-2:end)) / numel(ydata(end-2:end))       ; % take the average of the last 3 data points as baseline start parameter
          contrast = sum(ydata(1:3)) / numel(ydata(1:3)) - baseline        ; % take the average of the first 3 data points minus baseline as contrast start parameter
          contrast = max(0.05, contrast)                                   ; % make sure the contrast is positive and atleast 5%
          gamma    = 1 / xdata(floor(numel(xdata)/2))                      ; % put the starting gamma value into the middle of the time axis
     else
          baseline = 1.0                                                   ; % emergency default baseline start parameter
          contrast = 0.3                                                   ; % emergency default contrast start parameter
          gamma    = 1.0                                                   ; % emergency default gamma start parameter
     end
     cstart = [baseline contrast gamma]                                    ; % starting fitting parameters
     clow   = [baseline/2 contrast/5 gamma/1e03]                           ; % define lower limits
     chigh  = [baseline*2 contrast*5 gamma*1e03]                           ; % define higher limits
     clear baseline contrast gamma                                         ;


% ---
% --- please put the code for the custom parameter below
% ---



% ---
% EOF
