function [ccdimginfo] = detectorinfo(ccdimginfo)
% 
% DETECTORINFO Load some detector related parameters to ccdimginfo
%
% Michael Sprung
% $Revision: 1.0 $Date: 2005/09/13 $
% $Revision: 2.0 $Date: 2006/11/16 $
% 
% Correction files like blemish & flatfield are included now
% Correction files like distortion & parasitic are still not enabled 
%


% =========================================================================
% --- Detector 1 : Direct Detection CCD in slow mode
% =========================================================================
if ( ccdimginfo.detector == 1 )
    ccdimginfo.ccdHardwareColSize = 1242                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1152                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.0225                                 ; % pixel size in mm
    ccdimginfo.saturation         = 65535                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 476                                    ; % adu per photon @ 7.6keV
    ccdimginfo.efficiency         = 0.28                                   ; % efficiency @ 7.6keV
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.blemish            = 1                                      ; % uses blemish file   (see 'getblemish.m')
    ccdimginfo.flatfield          = 1                                      ; % uses flatfiled file (see 'getflatfield.m')
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-1",  "collectimg_long"


% =========================================================================
% --- Detector 2 : Direct Detection CCD in fast mode
% =========================================================================
if ( ccdimginfo.detector == 2 )
    ccdimginfo.ccdHardwareColSize = 1242                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1152                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.0225                                 ; % pixel size in mm
    ccdimginfo.saturation         = 4095                                   ; % saturation count in one pixel
    ccdimginfo.adupphot           = 90                                     ; % adu per photon @ 6.6keV
    ccdimginfo.efficiency         = 0.45                                   ; % efficiency @ 6.6keV
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.blemish            = 1                                      ;
    ccdimginfo.flatfield          = 1                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-2",  "collectimg"


% =========================================================================
% --- Detector 3 : Capillary Tapered Phosphor CCD 1st Harmonic
% =========================================================================
if ( ccdimginfo.detector == 3 )
    ccdimginfo.ccdHardwareColSize = 1242                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1152                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.054                                  ; % pixel size in mm
    ccdimginfo.saturation         = 65535                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 0.755                                  ; % adu per photon
    ccdimginfo.efficiency         = 1.00                                   ;
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.distortion         = ''                                     ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 1                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
%  value,defaults,"rdfunc-3",     "collectimg_long"
%  value,defaults,"distortion-3", detectordirectory+"meshes.b"
   

% =========================================================================
% --- Detector 4 : Capillary Tapered Phosphor CCD 3rd Harmonic
% =========================================================================
if ( ccdimginfo.detector == 4 )
    ccdimginfo.ccdHardwareColSize = 1242                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1152                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 3                                      ;
    ccdimginfo.dpix               = 0.054                                  ; % pixel size in mm
    ccdimginfo.saturation         = 65535                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 2.265                                  ; % adu per photon
    ccdimginfo.efficiency         = 0.39                                   ;
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.distortion         = ''                                     ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 1                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
%  value,defaults,"rdfunc-4",     "collectimg_long"
%  value,defaults,"distortion-4", detectordirectory+"meshes.b"



% =========================================================================
% --- Detector 5 : DALSA
% =========================================================================
if ( ccdimginfo.detector == 5 )
    ccdimginfo.ccdHardwareColSize = 1024                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1024                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = 1                                      ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.014                                  ; % pixel size in mm
    ccdimginfo.saturation         = 4095                                   ; % saturation count in one pixel
    ccdimginfo.adupphot           = 75.6                                   ; % adu per photon
    ccdimginfo.efficiency         = 0.40                                   ;
    ccdimginfo.gain               = 4                                      ;
    ccdimginfo.blemish            = 1                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-5",  "collectimg"


% =========================================================================
% --- Detector 6 : SMD CCD 1st Harmonic four times gain
% =========================================================================
if ( ccdimginfo.detector == 6 )
    ccdimginfo.ccdHardwareColSize = 1024                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1024                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          =  1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.014                                  ; % pixel size in mm
    ccdimginfo.saturation         = 4095                                   ; % saturation count in one pixel
    ccdimginfo.adupphot           = 75.6                                   ; % adu per photon [18.9 * 4 (Gain)] no '/gain' in normalization
    ccdimginfo.efficiency         = 0.40                                   ;
    ccdimginfo.gain               = 4                                      ;
    ccdimginfo.blemish            = 1                                      ;
    ccdimginfo.flatfield          = 1                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-6",    "collectimg"


% =========================================================================
% --- Detector 7 : Other (to be determined)
% =========================================================================
if ( ccdimginfo.detector == 7 )
    ccdimginfo.ccdHardwareColSize = 1340                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1300                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.02                                   ; % pixel size in mm
    ccdimginfo.saturation         = 65535                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 1                                      ; % adu per photon
    ccdimginfo.efficiency         = 1                                      ;
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-7",  "collectimg_long"


% =========================================================================
% --- Detector 8 : PI-LCX 1300x1340
% =========================================================================
if ( ccdimginfo.detector == 8 )
    ccdimginfo.ccdHardwareColSize = 1340                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1300                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          = -1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.020                                  ; % pixel size in mm
    ccdimginfo.saturation         = 65535                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 830                                    ; % adu per photon
    ccdimginfo.efficiency         = 0.550                                  ;
    ccdimginfo.gain               = 1                                      ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-8",  "collectimg_long"


% =========================================================================
% --- Detector 9 : CMOS
% =========================================================================
if ( ccdimginfo.detector == 9 )
    ccdimginfo.ccdHardwareColSize = 1280                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1024                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          =  1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = 0.0067                                 ; % pixel size in mm
    ccdimginfo.saturation         = 4095                                   ; % saturation count in one pixel
    ccdimginfo.adupphot           = 276                                    ; % adu per photon (139 : 1.88 ; 276 : 2.92)
    ccdimginfo.efficiency         = 0.083                                  ;
    ccdimginfo.gain               = 2.92                                   ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-9",  "collectimg_long"


% =========================================================================
% --- Detector 10 : CMOS at 47deg .tilt
% =========================================================================
if ( ccdimginfo.detector == 10 )
    ccdimginfo.ccdHardwareColSize = 1280                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1024                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          =  1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = [0.0067,0.00455]                       ; % pixel size in mm
    ccdimginfo.saturation         = 4095                                   ; % saturation count in one pixel
    ccdimginfo.adupphot           = 268                                    ; % adu per photon (161 : 1.88 ; 268 : 2.92)
    ccdimginfo.efficiency         = 0.0117                                 ;
    ccdimginfo.gain               = 2.92                                   ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-10",  "collectimg_long"



% =========================================================================
% --- Detector 11 : TBD
% =========================================================================
if ( ccdimginfo.detector == 11 )
    ccdimginfo.ccdHardwareColSize = 1340                                   ; % define ccd hardware col size
    ccdimginfo.ccdHardwareRowSize = 1300                                   ; % define ccd hardware row size
    ccdimginfo.ccdxsense          = -1                                     ;
    ccdimginfo.ccdzsense          =  1                                     ;
    ccdimginfo.harmonic           = 1                                      ;
    ccdimginfo.dpix               = [0.0067,0.00455]                       ; % pixel size in mm
    ccdimginfo.saturation         = 32765                                  ; % saturation count in one pixel
    ccdimginfo.adupphot           = 200                                    ; % adu per photon
    ccdimginfo.efficiency         = 0.043                                  ;
    ccdimginfo.gain               = 1.88                                   ;
    ccdimginfo.blemish            = 0                                      ;
    ccdimginfo.flatfield          = 0                                      ;
    ccdimginfo.distortion         = 0                                      ;
    ccdimginfo.parasitic          = 0                                      ;
end
% value,defaults,"rdfunc-11",  "collectimg_long"

% ---
% EOF
