function framespacing = framespacingFromDelayVector(nBatch)
% --- This is a subfunction called from 'showaverageresult.m'.
% --- It retrieves the time difference between frames from the
% --- delay vector of the result structure. This function was needed
% --- to remove a call from 'showaverageresult' to look at the data
% --- directly. The function is now able to work with just the result
% --- structure.
% ---
% --- by MS 20060228
% ---

% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;


% =========================================================================
% --- define some parameters from result structure
% =========================================================================
nslices  = ccdimginfo.lastslice - ccdimginfo.firstslice + 1                ; % get the number of slices
slicedpl = ccdimginfo.slicedpl                                             ;
mindelay = 1                                                               ;

% =========================================================================
% --- get number of delays from the slices
% =========================================================================
if (nslices > 1)
    slicedelays = finddelays(nslices,slicedpl,mindelay)                    ;
    nslicedelay = length(slicedelays)                                      ;
else
    nslicedelay = 0                                                        ;
end


% =========================================================================
% --- calculate the framespacing
% =========================================================================
delayVector  = ccdimginfo.result.delay{nBatch}                             ;
framespacing = delayVector(nslicedelay+2) - delayVector(nslicedelay+1)     ; % get framespacing


% =========================================================================
% --- clear variables
% =========================================================================
clear nslices slicedpl mindelay slicedelays nslicedelay delayVector        ;
clear hFigXPCSMain ccdimginfo                                              ;

% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;


% ---
% EOF
