function compressedmulti = iscompressedmultiimm(varargin)
% ---
% --- ISCOMPRESSEDMULTIIMM checks if file is a compressed multi file 
% ---
% --- USAGE : COMPRESSEDMULTI = ISCOMPRESSEDMULTIIMM(FILENAME)
% ---
% --- Input Argument:
% --- FILENAME : image file name
% ---
% --- Output Argument:
% --- COMPRESSEDMULTI : flag to see if file is compressed (0 : no|1 : yes)
% ---
% --- Michael Sprung
% --- $Revision: 1.0 $Date: 2006/11/01 $
% ---


% =========================================================================
% --- input file preparation
% =========================================================================
file            = varargin{1}                                              ;
compressedmulti = -1                                                       ; % initialize compressedmulti flag


% =========================================================================
% --- open file
% =========================================================================
[fid,message] = fopen(file)                                                ;        
if ( fid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    return                                                                 ;
end


% =========================================================================
% --- check for mode, compression & immversion
% =========================================================================
findDash        = findstr(file,'-')                                        ;
% ---
fseek(fid,0,'bof')                                                         ;
modeflag        = fread(fid,1 ,'int')                                      ;
% ---
fseek(fid,4,'bof')                                                         ;
compressionflag = fread(fid,1 ,'int')                                      ;
% ---
fseek(fid,616,'bof')                                                       ;
immversionflag  = fread(fid,1 ,'int')                                      ;
% --- check for multifile format
if ( immversionflag >= 11 && modeflag ~= 2 )
    fclose(fid)                                                            ; % not a multifile
    return                                                                 ;
elseif ( immversionflag < 11 && isempty(findDash) == 1 )
    fclose(fid)                                                            ; % not a multifile by file naming convention
    return                                                                 ;    
end
% --- check for compression
compressedmulti = 0                                                        ; % assume uncompressed multi file
if ( compressionflag == 65540 && isempty(findstr(file,'_ucp_')) )            % condition for SMD legacy code
    compressedmulti = 1                                                    ;
elseif ( compressionflag == 6 && immversionflag >= 11 )                      % condition for compressed data from Imageserver program
    compressedmulti = 1                                                    ;
end


% ---
% EOF