function mainanalysis(nBatch)
%
% Perform the static  & dynamic analysis
%
% the results of this routine are stored in a structure 'ccdimginfo.result'
%
% Michael Sprung
% $Revision: 1.0b $  $Date: 2005/02/23 $
% $Revision: 1.1  $  $Date: 2005/04/24 $ check normalizations
% $Revision: 1.2  $  $Date: 2005/08/29 $ recheck normalizations
% $Revision: 1.3  $  $Date: 2005/09/12 $ add minor improvements
% $Revision: 1.4  $  $Date: 2005/09/15 $ make only static analysis possible
% $Revision: 1.5  $  $Date: 2005/12/13 $ normalize intensities properly
% $Revision: 1.6  $  $Date: 2006/07/28 $ corrected flaw in finding
%                                        saturated pixels


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish getting application data
% =========================================================================


% =========================================================================
% --- define test switches
% =========================================================================
Method          = 0                                                        ; % switch to use < I(q,t) >_q (Method == 0) or a Gaussian filter (Method == 1)
ccdimginfo.ssn  = 1                                                        ; % switch for standard or symmetric normalization

% the long time scale correction creates problems if the sample is static !
longTerm        = 0                                                        ; % switch of the long time correction part of the calculation
% if ( ccdimginfo.detector == 5 || ccdimginfo.detector == 6 )
%     longTerm = 1                                                             % switch only for the SMD camera
% end
smooth          = 0                                                        ; % switch of the smoothing part of the calculation

storeG2Tensoron = 0                                                        ; % switch of full g2 tensor storage (only offered for reflection geometry)
saveSetup       = 0                                                        ; % switch to store setup information or not
pause(0.001)                                                               ;
% =========================================================================
% --- finish defining test switches
% =========================================================================


% =========================================================================
% --- create data and dark indices
% =========================================================================
dataIndex = ccdimginfo.ndata0todo(nBatch):ccdimginfo.ndataendtodo(nBatch)  ; % data index
darkIndex = ccdimginfo.ndark0todo(nBatch):ccdimginfo.ndarkendtodo(nBatch)  ; % dark index
pause(0.001)                                                               ;
% =========================================================================
% --- finish creating data and dark indices
% =========================================================================


% =========================================================================
% --- try to create saving subdirectory 'result' (needed if the analysis
% --- button was never pressed, e.g in batchmode)
% =========================================================================
warning('off','MATLAB:MKDIR:DirectoryExists')                              ;
dummy = mkdir(ccdimginfo.savepath)                                         ; % try to create directory to store the results
if ( dummy == 0 )                                                            % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'            ...
                    ;'   Try to save in Matlab XPCSGUI folder!'}]       ...
        ,'HorizontalAlignment','left')                                     ;
    pause(0.001)                                                           ;
    % ---
    ccdimginfo.savepath    = fullfile(pwd,'result')                        ; % try to create a subdirectory in the Matlab XPCSGUI folder
    dummy = mkdir(ccdimginfo.savepath)                                     ;
    if ( dummy == 0 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',...
            [msgstr;{'   Failed creating saving directory !'            ...
                    ;'   Please check write permissions!'}]             ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        return                                                             ;
    end
end
clear dummy                                                                ;
warning('on','MATLAB:MKDIR:DirectoryExists')                               ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish trying to create saving subdirectory 'result'
% =========================================================================


% =========================================================================
% --- check ccdimginfo.result substructure
% --- clear old results of selected batches 
% --- save the 'new' setup information to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save the setup information !'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
% --- check size off all result fields and change them if neccessary
if isfield(ccdimginfo,'result') 
    if (  isfield(ccdimginfo.result,'analysistime')                     ...
       && numel(ccdimginfo.result.analysistime)  <numel(ccdimginfo.preset))
        ccdimginfo.result.analysistime{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'totalIntensity')                   ...
       && numel(ccdimginfo.result.totalIntensity)<numel(ccdimginfo.preset))
        ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []    ;
    end
    if (  isfield(ccdimginfo.result,'aIt')                              ...
       && numel(ccdimginfo.result.aIt)           <numel(ccdimginfo.preset))
        ccdimginfo.result.aIt{numel(ccdimginfo.preset)} = []            ;
    end
    if (  isfield(ccdimginfo.result,'Iqphi')                            ...
       && numel(ccdimginfo.result.Iqphi)         <numel(ccdimginfo.preset))
        ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2')                               ...
       && numel(ccdimginfo.result.g2)            <numel(ccdimginfo.preset))
        ccdimginfo.result.g2{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'g2avg')                            ...
       && numel(ccdimginfo.result.g2avg)         <numel(ccdimginfo.preset)) 
        ccdimginfo.result.g2avg{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2avgErr')                         ... 
       && numel(ccdimginfo.result.g2avgErr)      <numel(ccdimginfo.preset))
        ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'goodPixels')                       ...
       && numel(ccdimginfo.result.goodPixels)    <numel(ccdimginfo.preset))
        ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)} = []        ;
    end
    if (  isfield(ccdimginfo.result,'NEPhoton')                         ...
       && numel(ccdimginfo.result.NEPhoton)      <numel(ccdimginfo.preset))
        ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'delay')                            ...
       && numel(ccdimginfo.result.delay)         <numel(ccdimginfo.preset))
        ccdimginfo.result.delay{numel(ccdimginfo.preset)} = []             ;
    end
else
    ccdimginfo.result.analysistime{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []        ;
    ccdimginfo.result.aIt{numel(ccdimginfo.preset)}            = []        ;
    ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.g2avg{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)}     = []        ;
    ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.delay{numel(ccdimginfo.preset)}          = []        ;
end
% --- reset result fields for the batch to be analyzed
ccdimginfo.result.analysistime{nBatch}   = []                              ; % clear any previous time stamp
ccdimginfo.result.totalIntensity{nBatch} = []                              ; % clear the total intensity per slice
ccdimginfo.result.aIt{nBatch}            = []                              ; % clear averaged image
ccdimginfo.result.Iqphi{nBatch}          = []                              ; % clear Iqphi vector
ccdimginfo.result.g2{nBatch}             = []                              ; % clear g2 tensor
ccdimginfo.result.g2avg{nBatch}          = []                              ; % clear averaged g2's
ccdimginfo.result.g2avgErr{nBatch}       = []                              ; % clear the error of the averaged g2's
ccdimginfo.result.goodPixels{nBatch}     = []                              ; % clear the number of good pixels in a dynamic q phi partition
ccdimginfo.result.NEPhoton{nBatch}       = []                              ; % clear the indix of pixels which received not enough photons
ccdimginfo.result.delay{nBatch}          = []                              ; % clear vector with delay times
% --- store result field in result & remove result field the ccdimginfo
result     = ccdimginfo.result                                             ;
ccdimginfo = rmfield(ccdimginfo,'result')                                  ;
if ( isfield(ccdimginfo,'testimg') )
    ccdimginfo = rmfield(ccdimginfo,'testimg')                             ;
end
% --- save the setup information
if ( saveSetup == 1 )
    nobatch  = sprintf('batch%03i_',nBatch)                                ;
    savefile = sprintf([ccdimginfo.name,nobatch,'setup.mat'])              ; % define filename for binary output
    savefull = fullfile(ccdimginfo.savepath,savefile)                      ; % build full file name
    save(savefull,'ccdimginfo')                                            ; % store the setup information
    clear nobatch                                                          ;
end
% --- restore result field in ccdimginfo & remove result
ccdimginfo.result = result                                                 ;
clear result                                                               ;    
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving setup information',            ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish checking ccdimginfo.result substructure, clearing old results
% --- & saving the 'new' setup information to disk
% =========================================================================


% =========================================================================
% --- check if a flatfield correction is needed
% --- if needed, apply blemish file to ccdimginfo.usermask
% =========================================================================
if ( ccdimginfo.flatfield == 1 )
    [flatfield] = getflatfield(ccdimginfo)                                 ;
else
    flatfield = []                                                         ;
end
if ( ccdimginfo.blemish == 1 )
    [blemish] = getblemish(ccdimginfo)                                     ;
    ccdimginfo.usermask = ccdimginfo.usermask .* blemish                   ;
    clear blemish                                                          ;
end
pause(0.001)                                                               ;
% =========================================================================
% --- finish blemish and flatfield implementation
% =========================================================================


% =========================================================================
% --- check if data is stored in a compressed multifile
% =========================================================================
SByte = []                                                                 ;
if ( ccdimginfo.compressedmulti(nBatch) == 1 )
    if ( isempty(ccdimginfo.SBCM{nBatch}) == 1 )
        SByte = indexcompressedmultiimm(ccdimginfo.imagefile{nBatch})      ; % try to get starting byte position of indices
    else
        SByte = ccdimginfo.SBCM{nBatch}                                    ; % had already been created
    end
end
% =========================================================================
% --- finish checking if data is stored in a compressed multifile
% =========================================================================


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1 = ccdimginfo.maskroi(1,2)                                               ; % row    start of ROI
R2 = ccdimginfo.maskroi(2,2)                                               ; % row    end   of ROI
C1 = ccdimginfo.maskroi(1,1)                                               ; % column start of ROI
C2 = ccdimginfo.maskroi(2,1)                                               ; % column end   of ROI
% =========================================================================
% --- finish defining some abbreviations
% =========================================================================


% =========================================================================
% --- create averaged dark image 'sumDark'
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Create dark image & dark infos!'}]        ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
sumDark    = zeros( ccdimginfo.row_end - ccdimginfo.row_beg + 1         ...
                  , ccdimginfo.col_end - ccdimginfo.col_beg + 1,'single')  ; % initialize dark image as single
if ( ccdimginfo.lld > 0 )
   sumDarkRMS = zeros(ccdimginfo.row_end - ccdimginfo.row_beg + 1       ...
                     ,ccdimginfo.col_end - ccdimginfo.col_beg + 1,'single'); % initialize dark rms image as single
end
% ---
for i = 1:length(darkIndex)
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        f = openfile(ccdimginfo.imagefile{nBatch},darkIndex(i))            ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark rms images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark rms images
            end
        end
    else
        f = openfile(ccdimginfo.darkfile{nBatch},darkIndex(i))             ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark images
            end
        end
    end
end
% ---
if ( numel(darkIndex) > 0 )
    sumDark     = sumDark    / numel(darkIndex)                            ; % calculate averaged dark image
    if ( ccdimginfo.lld > 0 )
        sumDarkRMS  = sumDarkRMS / numel(darkIndex) - sumDark.^2           ; % calculate averaged dark rms image
    end
end
clear darkIndex f                                                          ;
% ---
satLevel  = single(ccdimginfo.saturation * ones(size(sumDark)) - sumDark)  ; % calculate new saturation values including dark values
DImg      = satLevel(R1:R2,C1:C2)                                          ; % take saturation levels of ROI
DImg_Mask = DImg(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                    ; % take saturation levels of unmasked pixels in ROI
% ---
DImgNM    = cell(ccdimginfo.snoq,ccdimginfo.snophi)                        ;
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( numel(ccdimginfo.smask{n,m}) > 0 )
            DImgNM{n,m} = DImg(ccdimginfo.smask{n,m})                      ; % take only (unmasked) pixels from one static q phi partition
        end
    end
end
clear satLevel                                                             ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish creating the dark image',             ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of creating dark averages
% =========================================================================


% =========================================================================
% --- initialize 'colLow' & 'colHigh' for data sectioning (memorycheck_roi)
% =========================================================================
colLow  = zeros(ccdimginfo.noDataSection,1)                                ;
colHigh = zeros(ccdimginfo.noDataSection,1)                                ;
for noS = 1 : ccdimginfo.noDataSection
    colLow (noS) = ccdimginfo.dataSections(noS)                            ; % create vector with starting column positions for data sectioning
    if (noS < ccdimginfo.noDataSection)
        colHigh(noS) = ccdimginfo.dataSections(noS+1) - 1                  ; % create vector with ending column positions for data sectioning
    else
        colHigh(noS) = ccdimginfo.dataSections(noS+1)                      ; % set the last value of 'colHigh' to the last value of dataSections
    end
end
pause(0.001)                                                               ;
% =========================================================================
% --- finish initializing 'colLow' & 'colHigh' for data sectioning
% =========================================================================


% =========================================================================
% --- Main loop - Over one dataSection at a time
% =========================================================================
for noS = 1 : ccdimginfo.noDataSection

    % =====================================================================
    % --- define shortcuts 'A' & 'B'
    % =====================================================================
    A = colLow (noS)                                                       ;
    B = colHigh(noS)                                                       ;
    pause(0.001)                                                           ;


    % =====================================================================
    % --- some initializations (largest tensor first !!!)
    % --- some tensors need to be created only once  !!!
    % =====================================================================
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Initialize large tensors !'}]         ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % ---
    if ( ccdimginfo.analysistype == 1 && ccdimginfo.kinetics == 0)           % for full frame mode
        % ---
        IQTT = zeros(R2-R1+1                                            ...
                    ,B-A+1                                              ...
                    ,length(dataIndex)                                  ...
                    ,1                                                  ...
                    ,'single')                                             ; % intensity tensor I(x,y,t_frame,1)
        % ---
        if ( noS == 1 && Method == 0 )                                       % tensor is small enough to be calculated only once
            aIq = zeros(ccdimginfo.snoq                                 ...
                       ,ccdimginfo.snophi                               ...
                       ,length(dataIndex)                               ...
                       ,1                                               ...
                       ,'single')                                          ; % initialize intensity average over static q phi partitions
        end
        % ---
    end
    % ---
    if ( ccdimginfo.analysistype == 1 && ccdimginfo.kinetics == 1)           % for kinetic mode
        % ---
        IQTT = zeros(R2-R1+1                                            ...
                    ,B-A+1                                              ...
                    ,length(dataIndex)                                  ...
                    ,ccdimginfo.lastslice-ccdimginfo.firstslice+1       ...
                    ,'single')                                             ; % intensity tensor I(x,y,t_frame,t_slice)
        % ---
        if ( noS == 1 && Method == 0 )                                       % tensor is small enough to be calculated only once
            aIq = zeros(ccdimginfo.snoq                                 ...
                       ,ccdimginfo.snophi                               ...
                       ,length(dataIndex)                               ...
                       ,ccdimginfo.lastslice-ccdimginfo.firstslice+1    ...
                       ,'single')                                          ; % initialize intensity average over static q phi partitions for each slice
        end
        % ---
    end
    % ---
    trueData = zeros(ccdimginfo.row_end-ccdimginfo.row_beg+1            ...
                    ,ccdimginfo.col_end-ccdimginfo.col_beg+1            ...
                    ,'single')                                             ; % initialize trueData (background subtracted data) as single
    % ---
    if (noS == 1)                                                            % tensors are small enough to be calculated only once
        trueDynamic = zeros(ccdimginfo.row_end-ccdimginfo.row_beg+1     ...
                           ,ccdimginfo.col_end-ccdimginfo.col_beg+1     ...
                           ,'single')                                      ; % initialize trueDynamic (time average of all images) as single
    end
    % ---
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,                                  ...
                         'Tag','xpcsmain_EditMessage'),'string')           ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish initializing large tensors',      ...
        ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')     ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
    

    % =====================================================================
    % --- start of data file import
    % =====================================================================
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Start to import data !'}]             ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================

    notzero = 1.0e-09                                                      ; % create a variable which is almost zero to prevent 'Divide by Zero' errors

    for i = 1:length(dataIndex)                                              % loop over all frames

        % =================================================================
        % --- show progress message
        % =================================================================
        if ( i == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('\r   Import image no. %04i of %04i images'...
                             ,i,length(dataIndex))                         ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
            pause(0.001)                                                   ; % wait for 1ms to display message
        end
        if ( length(dataIndex) < 1000 && (floor(i/10)-i/10+1) == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('   Import image no. %04i of %04i images', ...
                             i,length(dataIndex))                          ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
                pause(0.001)                                               ; % wait for 1ms to display message
        end
        if ( length(dataIndex) >= 1000 && (floor(i/100)-i/100+1) == 1 )
            msgstr = get(findall(hFigXPCSMain,'Tag',                    ...
                         'xpcsmain_EditMessage'),'string')                 ;
            msgnew = sprintf('   Import image no. %04i of %04i images', ...
                             i,length(dataIndex))                          ;
            set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),     ...
                'String',[msgstr;{msgnew}],'HorizontalAlignment','left')   ;
                pause(0.001)                                               ; % wait for 1ms to display message
        end


        % =================================================================
        % --- load data, remove dark image & flatfield correction
        % =================================================================
        if ( ~isempty(SByte) == 1 )
            f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i)      ...
                        ,SByte(dataIndex(i)-ccdimginfo.ndata0(nBatch)+1))  ; % load image file with pointer           
        else
            f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i))        ; % load image file
        end
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            trueData = f.imm - sumDark                                     ; %  do backgroud subtraction
        else
            trueData = f.imm .* flatfield  - sumDark                       ; % perform flatfield correction and do backgroud subtraction
        end
        

        % =================================================================
        % --- do lower level discrimination (LLD)
        % =================================================================
        if (ccdimginfo.lld < 0)
            trueData(trueData <= abs(ccdimginfo.lld)) = notzero            ; % reset all values below the lower level discriminator to ~zero
        end
        if (ccdimginfo.lld > 0)
            trueData(trueData <= ccdimginfo.lld * sumDarkRMS) = notzero    ; % reset all values below the lower level discriminator to ~zero
        end
        

        % =================================================================
        % --- create trueDynamic (needed only once)
        % =================================================================
        if (noS == 1)
            trueDynamic = trueDynamic + trueData                           ; % add images to trueDynamic (after lower level discrimination)
        end


        % =================================================================
        % --- create data tensor for the correlation routine
        % =================================================================
        for j = 1 : (ccdimginfo.lastslice - ccdimginfo.firstslice + 1)       % loop over good slices only
            if ( ccdimginfo.kinetics == 0 )
                sliceStart = R1                                            ; % in full frame / roi mode use this as slice start
                sliceEnd   = R2                                            ; % in full frame / roi mode use this as slice end
            else
                k          = (j-1) + ccdimginfo.firstslice                 ; % shift the counter in a way that ccdimginfo.firstslice == 1
                sliceStart = ccdimginfo.sliceinfo(k,1) + R1 - 1            ; % get the sliceStart position for each slice
                sliceEnd   = ccdimginfo.sliceinfo(k,1) + R2 - 1            ; % get the sliceEnd   position for each slice
            end


            % =============================================================
            % --- create the 4-D data tensor IQTT(x,y,t1,t2)
            % =============================================================
            if (  ccdimginfo.analysistype == 1 )
                IQTT(:,:,i,j) = trueData(sliceStart:sliceEnd,A:B)          ;
            end


            % =============================================================
            % --- find saturated pixels (only once)
            % --- find pixels with not enough photons NEPhoton (only once)
            % --- integrate the intensity of each slices (only once)
            % =============================================================
            if ( noS == 1 )
                if ( i == 1 && j == 1 )
                    saturatedPixels = cell(length(dataIndex)            ...
                            ,ccdimginfo.lastslice-ccdimginfo.firstslice+1) ;
                end
                S(:,:) = trueData(sliceStart:sliceEnd,C1:C2)               ; % take dark subtracted values of ROI of each slice
                indices = find(S > DImg)                                   ; % get the indices of saturated pixels
                saturatedPixels{i,j} = indices                             ; % store the indices of saturated pixels in ROI units
                % ---
                if ( i == 1 && j == 1 )                                      % these things need to be done only once
                    minPhoton   = ceil(0.030 * length(dataIndex))          ; % intensity must be found in at least 3% of the slices/frames
                    dummy       = zeros(size(S,1),size(S,2)) + minPhoton   ; % create a slice sized matrix to check if enough photons hit a pixel
                    NEPhoton    = find(ccdimginfo.usermask(R1:R2,C1:C2))   ; % start with all unmasked pixels in ROI
                    clear minPhoton                                        ;
                end
                ind1                  = find(S(NEPhoton))                  ; % find non zeros on a slice
                dummy(NEPhoton(ind1)) = dummy(NEPhoton(ind1)) - 1          ; % reduce the value of dummy by one at pixels where one photon hit
                ind2                  = find(dummy(NEPhoton) > 0)          ; % remaining pixels with not enough photons
                NEPhoton              = NEPhoton(ind2)                     ; % reduce the size of the 'not enough photon counter'
                clear ind1 ind2                                            ;
                % ---
                if ( i == 1 && j == 1 )
                    totalIntensity = zeros(length(dataIndex)            ...
                        ,ccdimginfo.lastslice-ccdimginfo.firstslice+1   ...
                        ,'single')                                         ;
                end
                S_ROI = S(ccdimginfo.usermask(R1:R2,C1:C2) == 1)           ;
                totalIntensity(i,j)  = sum(S_ROI(S_ROI <= DImg_Mask) )  ...
                                     / ccdimginfo.efficiency            ...
                                     / ccdimginfo.adupphot              ...
                                     / ccdimginfo.preset(nBatch)           ; % calculate the intensity [photons/sec] of the unmasked, unsaturated ROI
                clear S_ROI                                                ;
            end


            % =============================================================
            % --- create static q phi partition average tensor (only once)
            % =============================================================
            if ( ccdimginfo.analysistype == 1 && noS == 1 && Method == 0 )
                for n = 1 : ccdimginfo.snoq
                    for m = 1 : ccdimginfo.snophi
                        if ( numel(ccdimginfo.smask{n,m}) > 0 )
                            Snm = S(ccdimginfo.smask{n,m})                 ; % take only (unmasked) pixels from one static q phi partition
                            if (numel(Snm(Snm <= DImgNM{n,m})) > 0)
                                aIq(n,m,i,j)= mean(Snm(Snm <= DImgNM{n,m})); % aIq(sq,sphi,t1,t2) ; only not saturated unmasked pixels are averaged
                            end
                        end
                    end
                end
                clear Snm                                                  ;
            end


        end
    end
    % ---
    if ( noS == 1 )
        clear S dummy                                                      ;
    end
    if ( ccdimginfo.analysistype == 1 && noS == 1 && Method == 0 )
        % ---
        goodsqphipartitions = ones(ccdimginfo.snoq,ccdimginfo.snophi)      ;
        % ---
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                if ( isempty(ccdimginfo.smask{n,m}) )
                    goodsqphipartitions(n,m) = 0                           ; % count a partition as bad if it does not contain pixels
                end
            end
        end
        % ---
        [nsq,msphi,i0,j0] = ind2sub(size(aIq),find(~aIq))                  ;
        if ( numel(nsq) >= 1 )
           for l = 1 : numel(nsq)
               goodsqphipartitions(nsq(l),msphi(l)) = 0                    ; % set the goodsqphipartition value to zero if the static average is zero at any time 
           end
        end
        clear l nsq msphi i0 j0                                            ;
        % ---
        aIq(~aIq) = notzero                                                ; % reset aIq in static q phi partitions with no unmasked unsaturated pixel
    end
    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish data preparation in ',            ...
        num2str(t),' seconds.']}],'HorizontalAlignment','left')            ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
    % =====================================================================
    % --- end of data file import
    % =====================================================================


    % =====================================================================
    % --- create SPMask: mask to exclude saturated pixels in usermask
    % --- create aIt   : time averaged intensity map <I(q,t)>_t (only once)
    % --- create Iqphi : averaged intensity of static partitions of aIt
    % --- mask all ~good static q phi partitions (see goodsqphipartitions) 
    % =====================================================================
    if ( noS == 1 )
        % =================================================================
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start creating time averages'}]   ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % =================================================================
        SPMask = ones(R2-R1+1,C2-C1+1,'single')                            ; % initialize SPMask as ones (no saturated pixels) on ROI
        for i = 1:length(dataIndex)                                          % loop over all frames
            for j = 1 : (ccdimginfo.lastslice - ccdimginfo.firstslice + 1)   % loop over good slices only
                indices         = saturatedPixels{i,j}                     ;
                SPMask(indices) = 0                                        ; % set the Mask for all saturated pixels to zero
            end
        end
        ccdimginfo.usermask(R1:R2,C1:C2) = SPMask .*                    ...
                                           ccdimginfo.usermask(R1:R2,C1:C2); % incoorperate the saturated pixel mask into the user mask
        clear indices saturatedPixels SPMask                               ;
        % ---
        if ( ccdimginfo.analysistype == 1 && Method == 0 )
            dummy = ccdimginfo.usermask(R1:R2,C1:C2)                       ; % take only ROI of usermask
            for n = 1 : ccdimginfo.snoq
                for m = 1 : ccdimginfo.snophi
                    if ( goodsqphipartitions(n,m) == 0 )
                        dummy(ccdimginfo.smask{n,m}) = 0                   ; % mask every pixel of a bad static q phi partition                        
                    end
                end
            end
            ccdimginfo.usermask(R1:R2,C1:C2) = dummy(:,:)                  ; % store the corrected usermask
            clear dummy                                                    ;
        end
        % ---
        trueDynamic = trueDynamic / length(dataIndex)                      ; % calculate averaged data image (frame level, dynamic case)
        % ---
        aIt         = zeros(size(ccdimginfo.qmap,1)                     ...
                           ,size(ccdimginfo.qmap,2),'single')              ; % initialize aIt (time averaged of dynamic image)
        % ---
        for j = 1 : (ccdimginfo.lastslice - ccdimginfo.firstslice + 1)       % loop over good slices only
            if ( ccdimginfo.kinetics == 0 )
                sliceStart = R1                                            ; % in full frame / roi mode use this as slice start
                sliceEnd   = R2                                            ; % in full frame / roi mode use this as slice end
            else
                k          = (j-1) + ccdimginfo.firstslice                 ; % shift the counter in a way that ccdimginfo.firstslice == 1
                sliceStart = ccdimginfo.sliceinfo(k,1) + R1 - 1            ; % get the sliceStart position for each slice
                sliceEnd   = ccdimginfo.sliceinfo(k,1) + R2 - 1            ; % get the sliceEnd   position for each slice
            end
            % ---
            aIt(R1:R2,C1:C2) = aIt(R1:R2,C1:C2)                         ...
                             + trueDynamic(sliceStart:sliceEnd,C1:C2)      ; % sum ROI of all good slices
        end
        % ---
        aIt = aIt / (ccdimginfo.lastslice-ccdimginfo.firstslice+1)         ; % normalize aIt by dividing with the number of good slices
        aIt(aIt<=notzero) = notzero                                        ; % reset values below ~zero in aIt to ~zero
        % ---
        Iqphi = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                   ; % initialize Iqphi
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                      % check if static q phi partition contains pixels
                    dummy1 = aIt(R1:R2,C1:C2)                              ; % take only ROI of aIt
                    dummy2 = ccdimginfo.usermask(R1:R2,C1:C2)              ; % take only ROI of usermask
                    Inm    = dummy1(ccdimginfo.smask{n,m})                 ; % get values of aIt for all pixels of a static q phi partition
                    Mnm    = dummy2(ccdimginfo.smask{n,m})                 ; % get values of usermask (with SPMask) for all pixels of a static q phi partition
                    if ( numel(Inm(Mnm==1)) > 0 )
                        Iqphi(n,m) = mean(Inm(Mnm==1))                     ; % calculate the average intensity in the averaged static q phi partitions Iqphi(sq,sphi)
                    end
                end
            end
        end
        Iqphi(Iqphi<=notzero) = notzero                                    ;
        clear Inm Mnm dummy1 dummy2                                        ;
        % --- try to calculate absolute intensities
        if ( numel(ccdimginfo.dpix) > 1 )                                    % pixel size is different in x & y direction
            dpix_x = ccdimginfo.dpix(1)                                    ;
            dpix_y = ccdimginfo.dpix(2)                                    ;
        else                                                                 % pixel size is identical in x & y direction
            dpix_x = ccdimginfo.dpix                                       ;
            dpix_y = dpix_x                                                ;
        end
        Iqphi = Iqphi / ccdimginfo.efficiency / ccdimginfo.adupphot     ...
              / ccdimginfo.preset(nBatch)                                  ; % this can always be calculated (unit: [photons/sec])
        ccdimginfo.normalization = 0                                       ; % standard normalization case for both geometries (unit: [photons/sec])
        % ---
        if ( ccdimginfo.geometry == 0 )                                      % in transmission mode special normalization are possiple
            Iqphi = Iqphi / ( dpix_x/ccdimginfo.rr * dpix_y/ccdimginfo.rr ); % divide by the solid angle; unit is still [photons/sec]
            ccdimginfo.normalization = 1                                   ; % normalization case set to one [photon/sec]
            if ( ccdimginfo.tflux == 1 )                                     % if the transmitted flux was measured, than 'beam_i' & 'ring_i_sample' are known 
                Iqphi = Iqphi / ccdimginfo.beam_i(nBatch)               ...
                      / ccdimginfo.ring_i(nBatch)                       ...
                      * ccdimginfo.ring_i_sample(nBatch)                   ; % Iqphi can be normalized by transmitted flux (unit: dimensionless)
                ccdimginfo.normalization = 2                               ; % normalization case set to two [no dimension]
            end
            if (  ccdimginfo.normalization == 2                         ...
               && ccdimginfo.thicknessflag > 0 )                              % further normalization might be possible (two cases)
                % --- case 3a
                if (  ccdimginfo.tflux == 1 && ccdimginfo.iflux == 1    ...
                   && ccdimginfo.thicknessflag == 2 )                        % if the transmitted & incident flux were measured & the attenuation length is known
                    Iqphi = Iqphi / ( 0.1 * ccdimginfo.atten_length )   ...  % divide by the attenuation length (convert to [cm])
                          / log( (ccdimginfo.beam_i_vacuum(nBatch)      ...
                                 /ccdimginfo.ring_i_vacuum(nBatch))     ...
                               / (ccdimginfo.beam_i(nBatch)             ...
                                 /ccdimginfo.ring_i_sample(nBatch))  )     ; % divide by log(incident flux / transmitted flux)
                    ccdimginfo.normalization = 3                           ; % normalization case set to three (dimension: [1/cm])
                end
                % --- case 3b (check only if not case 3a)
                if (  ccdimginfo.normalization == 2                     ...
                   && ccdimginfo.thicknessflag == 1 )                        % sample thickness in [mm] is known
                    Iqphi = Iqphi / ( 0.1 * ccdimginfo.thickness )         ; % divide by sample thickness (converted to [cm]) 
                    ccdimginfo.normalization = 3                           ; % normalization case set to three (dimension: [1/cm])
                end
            end
        else                                                                 % reflection geometry 
            if ( ccdimginfo.iflux == 1 )
                Iqphi = Iqphi / ccdimginfo.beam_i_vacuum(nBatch)        ...
                      / ccdimginfo.ring_i(nBatch)                       ...
                      * ccdimginfo.ring_i_vacuum(nBatch)                   ; % Iqphi can be normalized by incident flux (unit: dimensionless)  
                ccdimginfo.normalization = 4                               ; % normalization case set to four (no dimension)
           end
        end
        % ---
        clear dpix_x dpix_y                                                ;
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish creating time averages',      ...
            ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left') ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
        % =================================================================
    end
    clear i j k n m sliceStart sliceEnd f trueData trueDynamic             ;
    % =====================================================================
    % --- end of creating time averaged intensity maps <I(q,t)>_t
    % =====================================================================



    % =====================================================================
    % --- setup the multi-tau correlation routine 'rawg2'
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start setting up multi-tau'}]     ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % =================================================================
        message          = 0                                               ; % define message
        mindelay         = 1                                               ; % define mindelay
        avgfirst         = 0                                               ; % define average first
        % ---
        if (ccdimginfo.kinetics == 0)
            slicespacing  = 0                                              ;
            slicedpl      = ccdimginfo.slicedpl                            ;
            framespacings = immelapsed(ccdimginfo.imagefile{nBatch}     ...
                                      ,dataIndex(1),dataIndex(end) )       ;
            framespacing  = mean(diff(framespacings))                      ;
            % ---
            dpl           = ccdimginfo.dpl                                 ;
        else
            slicespacing  = ccdimginfo.preset(nBatch)                      ;
            slicedpl      = ccdimginfo.slicedpl                            ;
            framespacings = immelapsed(ccdimginfo.imagefile{nBatch}     ...
                                      ,dataIndex(1),dataIndex(end) )       ;
            framespacing  = mean(diff(framespacings))                      ;
            dpl           = ccdimginfo.dpl                                 ;
        end
        clear framespacings                                                ;
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish setting up multi-tau',        ...
            ' routine in ',num2str(t),' seconds.']}],                   ...
            'HorizontalAlignment','left')                                  ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end


    % =====================================================================
    % --- start of the setup of the standard normalization
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 && ccdimginfo.ssn == 0)
        % =================================================================
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start standard normalization'}]   ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % =================================================================
        if ( noS == 1)
            aIqt = zeros(size(ccdimginfo.qmap,1)                        ...
                        ,size(ccdimginfo.qmap,2),'single')                 ; % initialize aIqt = < <I(q,t)>_q >_t : time average of q phi average of IQTT
        end


        % =================================================================
        % --- Method == 0
        % =================================================================
        if ( Method == 0)
            for i = 1 : size(IQTT,3)                                         % loop over frames
                for j = 1: size(IQTT,4)                                      % loop over slices
                    % ---
                    AIq = zeros(size(ccdimginfo.qmap,1)                 ...
                               ,size(ccdimginfo.qmap,2),'single')          ; % initialize AIq to extend aIq from q phi partition size to a full slice size
                    % ---
                    dummy = zeros(R2-R1+1,C2-C1+1)                         ;
                    for n = 1 : ccdimginfo.snoq
                        for m = 1 : ccdimginfo.snophi
                            if ( goodsqphipartitions(n,m) == 1 )
                                dummy(ccdimginfo.smask{n,m}) = aIq(n,m,i,j); % assign every pixel of a good static q phi partition its mean value
                            end
                        end
                    end
                    AIq(R1:R2,C1:C2) = dummy(:,:)                          ;
                    clear dummy                                            ;
                    % ---
                    if ( noS == 1 )
                        aIqt = aIqt + AIq                                  ; % create the time average of the static q phi partition averages
                    end
                    % ---
                    dummy                 = AIq(R1:R2,A:B)                 ;
                    dummy(dummy<=notzero) = -1                             ; % check for zeros and reset them to -1 (easy to find later)
                    AIq(R1:R2,A:B)        = dummy(:,:)                     ;
                    clear dummy                                            ;
                    % ---
                    IQTT(:,:,i,j)   = IQTT(:,:,i,j) ./ AIq(R1:R2,A:B)      ; % normalize data by AIq to create: I(q,t) = I(q,t) / <I(q,t)>_q
                    % ---
                end
            end
            clear i j n m AIq                                              ;
            % ---
            if ( noS == 1)
                aIqt = aIqt / ( size(IQTT,3) * size(IQTT,4) )              ; % normalize aIqt (divide by # frames times # slices)
                aIqt(aIqt<=notzero) = notzero                              ; % reset values below ~zero in IQTT to ~zero
            end
            % ---
        end


        % =================================================================
        % --- Method == 1
        % =================================================================
        if (Method == 1)
            gfilter = gaussianfilter(9,9)                                  ; % create 9 by 9 pixel filter for 'conv2'
            % ---
            for i = 1 : size(IQTT,3)
                for j = 1: size(IQTT,4)
                    % ---
                    normIq  = convolve2(IQTT(:,:,i,j),gfilter,'reflect')   ; % calculate 9 by 9 pixel averaged image
                    % ---
                    aIqt(R1:R2,A:B) = aIqt(R1:R2,A:B) + normIq(:,:)        ; % add normIq's of all times to aIqt
                    % ---
                    normIq(normIq<=notzero) = -1                           ; % check for zeros and reset them to -1 (easy to find later)
                    IQTT(:,:,i,j) = IQTT(:,:,i,j) ./ normIq(:,:)           ; % normalize data by normaIq to create: I(q,t) = I(q,t) / <I(q,t)>_q
                    % ---
               end
            end
            % ---
            aIqt(:,A:B) = aIqt(:,A:B) / ( size(IQTT,3) * size(IQTT,4) )    ; % normalize aIqt (divide by # frames times # slices)
            % ---
            dummy                 = aIqt(:,A:B)                            ;
            dummy(dummy<=notzero) = notzero                                ;
            aIqt(:,A:B)           = dummy(:,:)                             ;
            clear dummy
            % ---
            clear i j normIq gfilter                                       ;
        end


        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish standard normalization',      ...
            ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left') ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
        % =================================================================
    end
    % =====================================================================
    % --- end of the setup of the standard normalization
    % =====================================================================



    % =====================================================================
    % ---------- start of the main call of the multi-tau routine ----------
    % ---   This call is independent of the method and runs always !!!  ---
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
            ,'String',[msgstr;{' ';'   Main call of multi-tau'}]        ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;

        % =================================================================
        % --- Reset negative values in IQTT to ~zero
        % =================================================================
        IQTT(IQTT<=notzero) = notzero                                      ; 
            
        % =================================================================
        % --- calculate how to split the data tensor in even smaller parts
        % =================================================================
        maxsize              = 50                                          ; % number of MB of data send to 'rawg2' at a time
        sizematrix           = size(IQTT)                                  ;
        [cLow,cHigh,noparts] = checksize(sizematrix,maxsize)               ;
        clear sizematrix maxsize                                           ;

        % =================================================================
        % --- loop over data chunks
        % =================================================================
        for j = 1 : noparts

            % =============================================================
            % --- show progress message
            % =============================================================
            if ( j ~= 1 )
                msgstr = get(findall(hFigXPCSMain,'Tag',                ...
                             'xpcsmain_EditMessage'),'string')             ;
                msgnew =                                                ...
                    sprintf('   Correlate no. %02i of %02i data sets'   ...
                           ,j,noparts)                                     ;
                set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'), ...
                    'String',[msgstr;{msgnew}],                         ...
                    'HorizontalAlignment','left')                          ;
                pause(0.001)                                               ; % wait for 1ms to display message
            else
                msgstr = get(findall(hFigXPCSMain,'Tag',                ...
                             'xpcsmain_EditMessage'),'string')             ;
                msgnew =                                                ...
                    sprintf('\r   Correlate no. %02i of %02i data sets' ...
                           ,j,noparts)                                     ;
                set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'), ...
                    'String',[msgstr;{msgnew}],                         ...
                    'HorizontalAlignment','left')                          ;
                pause(0.001)                                               ; % wait for 1ms to display message
            end
            % ---
            [delay,g2part] = rawg2( IQTT(:,cLow(j):cHigh(j),:,:),dpl    ...
                                  , framespacing, slicedpl, slicespacing...
                                  , avgfirst, mindelay, message)           ; % main call of multi-tau routine
            % ---
            if (j == 1 && noS == 1)
                g2 = ones(size(ccdimginfo.qmap,1)                       ...
                         ,size(ccdimginfo.qmap,2),size(g2part,3),'single') ; % initialize g2 ( Any g2 structure should be initialized as ones !!! )
            end
            % ---
            g2(R1:R2,A-1+cLow(j):A-1+cHigh(j),:) = g2part(:,:,:)           ; % build the full g2 tensor of a section
            % ---
        end
        clear j cLow cHigh noparts g2part
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish calling multi-tau ',          ...
            'in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')  ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end
    % =====================================================================
    % ------------ end of the main call of the multi-tau routine ----------
    % =====================================================================


    % =====================================================================
    % ----------------------- try to clear IQTT ---------------------------
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 && ccdimginfo.ssn == 0)
        clear IQTT                                                         ; % try to use the first chance to delete the large data tensor from memory
    end


    % =====================================================================
    % --- start of the symmetric normalization main routine
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 && ccdimginfo.ssn == 1)
        % =================================================================
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
            ,'String',[msgstr;{' ';'   Start symmetric normalization'}] ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;
        % =================================================================


        % =================================================================
        % --- Setup
        % =================================================================
        avgfirst = 1                                                       ; % set the average first switch == 1
        if ( noS == 1)
            aIqt = zeros(size(ccdimginfo.qmap,1)                        ...
                        ,size(ccdimginfo.qmap,2),'single')                 ; % initialize aIqt = < <I(q,t)>_q >_t : the time average of the q average of IQTT
        end


        % =================================================================
        % --- Method == 0
        % =================================================================
        if (Method == 0)
            % ---
            clear IQTT                                                     ; % the large data tensor is no longer used for this Method !!!
            % ---
            if ( noS == 1)
                % =========================================================
                % ------- here the multi-tau routine is called again ------
                % =========================================================
                [delayqphi,g2qphi] = rawg2(aIq,dpl,framespacing,slicedpl...
                                          ,slicespacing,avgfirst        ...
                                          ,mindelay,message)               ; % calculate ( < <I(q,t)>_q >_t_left * < <I(q,t)>_q >_t_right )
                % =========================================================
                % =========================================================
                clear delayqphi                                            ;
                % ---
                g2sn    = ones(size(ccdimginfo.qmap,1)                  ...
                              ,size(ccdimginfo.qmap,2)                  ...
                              ,size(g2qphi,3),'single')                    ; % initialize g2sn (short for: g2_symmetric_normalization)
                % ---
                for i = 1 : size(g2qphi,3)                                   % loop over all delays
                    g2dummy = ones(R2-R1+1,C2-C1+1,'single')               ; % initialize g2dummy
                    for n = 1 : ccdimginfo.snoq
                        for m = 1 : ccdimginfo.snophi
                            if ( goodsqphipartitions(n,m) == 1 )
                                g2dummy(ccdimginfo.smask{n,m}) =        ...
                                    g2qphi(n,m,i)                          ; % assign the g2qphi value to every pixel of good static q phi partitions for each delay
                            end
                        end
                    end
                    g2dummy(g2dummy<=notzero) = notzero                    ;
                    g2sn(R1:R2,C1:C2,i) = g2dummy(:,:)                     ; % create the g2sn tensor
                    clear g2dummy
                end
                clear i n m g2qphi                                         ;
                % ---
                dummy = zeros(R2-R1+1,C2-C1+1,'single')                    ;
                for n = 1 : ccdimginfo.snoq
                    for m = 1 : ccdimginfo.snophi
                        if ( goodsqphipartitions(n,m) == 1 )
                            dummy(ccdimginfo.smask{n,m})                ...
                                = mean(mean(aIq(n,m,:,:),4),3)             ; % assign the time averaged value to each pixel of a static q phi partition
                        end
                    end
                end
                dummy(dummy<=notzero) = notzero                            ;
                aIqt(R1:R2,C1:C2) = dummy(:,:)                             ;
                clear dummy                                                ;
                % ---
            end
            % ---
            g2(R1:R2,A:B,:) = g2(R1:R2,A:B,:) ./ g2sn(R1:R2,A:B,:)         ; % calculate the g2 tensor of a section
            % ---
        end


        % =================================================================
        % --- Method == 1
        % =================================================================
        if (Method == 1)
            % ---
            gfilter = gaussianfilter(9,9)                                  ; % create 9 by 9 pixel filter for 'convolve2'
            for i = 1 : size(IQTT,3)
                for j = 1: size(IQTT,4)
                    IQTT(:,:,i,j)   = convolve2(IQTT(:,:,i,j)           ...
                                               ,gfilter,'reflect')         ; % smooth images of data tensor with 9 by 9 pixel filter
                    aIqt(R1:R2,A:B) = aIqt(R1:R2,A:B) + IQTT(:,:,i,j)      ; % add q averaged image of all times to aIqt
                end
            end
            aIqt(R1:R2,A:B) = aIqt(R1:R2,A:B) / (size(IQTT,3)*size(IQTT,4)); % normalize aIqt
            % ---
            dummy = aIqt(R1:R2,A:B)                                        ;
            dummy(dummy<=notzero) = notzero                                ;
            aIqt(R1:R2,A:B) = dummy                                        ;
            clear dummy i j gfilter                                        ;
            % ---
            % --- calculate how to split the data tensor in smaller parts
            % ---
            maxsize              = 100                                     ; % use only 100 MB of data for the correlation routine
            sizematrix           = size(IQTT)                              ; % vector containing the dimension of IQTT
            [cLow,cHigh,noparts] = checksize(sizematrix,maxsize)           ;
            clear sizematrix maxsize                                       ;
            % ---
            IQTT(IQTT<=notzero) = notzero                                  ; % reset IQTT values
            % ---
            for j = 1 : noparts
                if ( j == 1 && noS == 1 )
                     g2sn = ones(size(ccdimginfo.qmap,1)                ...
                                ,size(ccdimginfo.qmap,2)                ...
                                ,size(g2,3),'single')                      ; % initialize g2sn (stands for: g2_symmetric_normalization)
                end
                % =========================================================
                % ------- here the multi-tau routine is called again ------
                % =========================================================
                [delay_sn,g2sn(R1:R2,A-1+cLow(j):A-1+cHigh(j),:)]       ...
                              = rawg2(IQTT(:,cLow(j):cHigh(j),:,:)      ...
                                     ,dpl,framespacing,slicedpl         ...
                                     ,slicespacing,avgfirst             ...
                                     ,mindelay,message)                    ; % calculate ( < <I(q,t)>_q >_t_left * < <I(q,t)>_q >_t_right )
                % =========================================================
                % =========================================================
            end
            clear IQTT                                                     ; % delete the large data tensor
            clear j cLow cHigh noparts delay_sn                            ;
            % ---
            dummy                 = g2sn(R1:R2,A:B,:)                      ;
            dummy(dummy<=notzero) = notzero                                ;
            g2sn(R1:R2,A:B,:)     = dummy(:,:)                             ;
            clear dummy                                                    ;
            % ---
            g2(R1:R2,A:B,:) = g2(R1:R2,A:B,:) ./ g2sn(R1:R2,A:B,:)         ; % calculate the the g2 tensor of a section
            % ---
        end


        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish symmetric normalization',     ...
            ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left') ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
        % =================================================================

    end
    % =====================================================================
    % --- end of the symmetric normalization main routine
    % =====================================================================


    % =====================================================================
    % --- create complete correlation tensor 'g2final'
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
            ,'String',[msgstr;{' ';'   Start creating g2 Tensor'}]      ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;
        % =================================================================
        if ( noS == 1)
            g2final = ones( size(ccdimginfo.qmap,1)                     ...
                          , size(ccdimginfo.qmap,2), size(g2,3),'single')  ; % initialize the complete correlation tensor g2final
        end
        g2final(R1:R2,A:B,:) = g2(R1:R2,A:B,:)                             ; % set the complete correlation tensor together
        clear g2                                                           ;
        % =================================================================
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish creating g2 Tensor in ',      ...
            num2str(t),' seconds.']}],'HorizontalAlignment','left')        ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end
    

    % =====================================================================
    % --- multiply g2final with [ < <I(q,t)>_q >_t / <I(q,t)>_t ]^2
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 && longTerm == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start long times correction'}]    ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % ---
        warning off MATLAB:DivideByZero                                    ;
        for j = 1 : size(g2final,3)
            g2final(R1:R2,A:B,j) = g2final(R1:R2,A:B,j)                 ...
                            .* (aIqt(R1:R2,A:B).^2 ./ aIt(R1:R2,A:B).^2)   ; % multiply g2final with [ < <I(q,t)>_q >_t / <I(q,t)>_t ]^2
        end
        warning on MATLAB:DivideByZero                                     ;
        % ---
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish long time scale ',            ...
            'correction in ',num2str(t),' seconds.']}],                 ...
            'HorizontalAlignment','left')  ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end


    % =====================================================================
    % --- normalization smoothing (not working yet)
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 && smooth == 1 )
        t1 = clock                                                         ;
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
           ,'String',[msgstr;{' ';'   Start smoothing'}]                ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        % ---
        aIq_norm             = mean(mean(aIq,4),3)                         ; % time averaged aIq
        aIq_norm(aIq_norm<1) = 1                                           ;
        aIq_normimg          = ones(size(ccdimginfo.qmap,1)             ...
                                   ,size(ccdimginfo.qmap,2),'single')      ; % initialize aIq_normimg
        % ---
        dummy = zeros(R2-R1+1,C2-C1+1,'single')                            ;
        for n = 1 : ccdimginfo.snoq
            for m = 1 : ccdimginfo.snophi
                dummy(ccdimginfo.smask{n,m}) = aIq_norm(n,m)               ; % expand aIq_norm to full image size
            end
        end
        aIq_normimg(R1:R2,C1:C2) = dummy(:,:)                              ;
        clear n m aIq_norm dummy                                           ;
        % ---
        normfac(R1:R2,C1:C2) = aIt(R1:R2,C1:C2) ./ aIq_normimg(R1:R2,C1:C2);
        % ---
        normfac(normfac<1) = 1                                             ;
        for j = 1 : size(g2final,3)
            g2final(R1:R2,C1:C2,j) = g2final(R1:R2,C1:C2,j)             ...
                                  ./ normfac(R1:R2,C1:C2)                  ; % normalize the correlation functions
        end
        clear j normfac
        % ---
        t2 = clock                                                         ;
        t  = etime(t2,t1)                                                  ;
        % ---
        msgstr = get(findall(hFigXPCSMain,                              ...
                             'Tag','xpcsmain_EditMessage'),'string')       ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{['   Finish smoothing in ',               ...
            num2str(t),' seconds.']}],'HorizontalAlignment','left')        ;
        clear t t1 t2                                                      ;
        pause(0.001)                                                       ;
    end

end
clear Method A B g2sn aIqt aIq flatfield dataIndex goodsqphipartitions     ; % clear the leftover large tensors
clear DImg DImgNM DImg_Mask SByte                                          ;
% =========================================================================
% --- End of main loop
% =========================================================================


% =========================================================================
% --- create averaged correlation function of dynamic q phi partitions,
% --- calculate standard deviations & estimate the number of valid g2's
% =========================================================================
if ( ccdimginfo.analysistype == 1 )
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
       ,'String',[msgstr;{' ';'   Calculate averaged g2 functions!'}]   ...
       ,'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    % =====================================================================
    g2avg    = ones(ccdimginfo.dnoq,ccdimginfo.dnophi,                  ...
                    size(g2final,3),'single')                              ; % initialize the averaged g2 tensor
    g2avgErr =zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,                  ...
                    size(g2final,3),'single')                              ; % initialize the tensor for the error of the averaged g2 tensor
    % ---
    goodPixels       = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,'single')   ; % initialize the matrix to store the number of good pixels in a q phi partition
    g2mask           = ccdimginfo.usermask(R1:R2,C1:C2)                    ; % create a mask for the correlation functions
    g2mask(NEPhoton) = 0                                                   ; % mask 'not enough photon' pixels on g2mask
    % ---
    for k = 1 : size(g2final,3)                                              % loop over all delays
        g2dummy1 = g2final(R1:R2,C1:C2,k)                                  ; % take one delay of the correlation tensor
        g2dummy1(g2mask==0) = -1000                                        ; % set masked pixels to -1000
        % ---
        for n = 1 : ccdimginfo.dnoq
            for m = 1 : ccdimginfo.dnophi
                g2dummy2 = g2dummy1(ccdimginfo.dmask{n,m})                 ; % take only values of one dynamic q phi partition
                if ( numel(g2dummy2(g2dummy2~=-1000)) > 0 )
                    g2avg(n,m,k)    = mean(g2dummy2(g2dummy2~=-1000))      ; % average only valid entrees of the g2 tensor
                    if ( k == 1 )
                        goodPixels(n,m) =                               ...
                            numel(g2dummy2(g2dummy2~=-1000))               ; % get the number of good pixels in a dynamic q phi partition
                    end
                    g2avgErr(n,m,k) = single( sqrt(1/goodPixels(n,m)) * ...
                        std(g2dummy2(g2dummy2~=-1000)))                    ; % calculate the error of the valid g2 functions (Std of the Mean!)
                else
                    g2avg(n,m,k)    = 1                                    ;
                    g2avgErr(n,m,k) = 0                                    ;
                    if ( k == 1 )
                        goodPixels(n,m) = 0                                ;
                    end
                end
            end
        end
        % ---
    end
    % ---
    clear g2dummy1 g2dummy2 k n m                                          ;
    clear C1 C2 R1 R2 g2mask
    % =====================================================================
    t2 = clock                                                             ;
    t  = etime(t2,t1)                                                      ;
    msgstr = get(findall(hFigXPCSMain,                                  ...
                         'Tag','xpcsmain_EditMessage'),'string')           ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish averaging g2 functions',          ...
        ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')     ;
    clear t t1 t2                                                          ;
    pause(0.001)                                                           ;
end
% =========================================================================
% --- finish creating averaged correlation function of dynamic partitions,
% --- calculating standard deviations & estimating the number of valid g2's
% =========================================================================


% =========================================================================
% --- store the correlation functions to the output structure
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Assign results to output structure!'}]    ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
ccdimginfo.result.totalIntensity{nBatch}     = totalIntensity              ; % store the totalIntensity on a slice/frame
ccdimginfo.result.aIt{nBatch}                = aIt                         ; % store the averaged image
ccdimginfo.result.Iqphi{nBatch}              = Iqphi                       ; % store the averaged intensity in the static q phi partitions of the averaged image
if ( ccdimginfo.analysistype == 1 )
    if ( ccdimginfo.kinetics == 1 && storeG2Tensoron == 1 )
        ccdimginfo.result.g2{nBatch}         = g2final                     ; % store g2 tensor (for each pixel) only in kinetic mode (full frame too big!!!)
    else
        ccdimginfo.result.g2{nBatch}         = []                          ;
    end
    ccdimginfo.result.g2avg{nBatch}          = g2avg                       ; % store the averaged g2 functions for each dynamic q phi partition
    ccdimginfo.result.g2avgErr{nBatch}       = g2avgErr                    ; % store the error of the averaged g2 functions
    ccdimginfo.result.goodPixels{nBatch}     = goodPixels                  ; % store the number of valid g2 functions in each dynamic q phi partition
    ccdimginfo.result.NEPhoton{nBatch}       = NEPhoton                    ; % store the indix of pixels which received not enough photons
    ccdimginfo.result.delay{nBatch}          = delay                       ; % store the delay vector
end
clear Iqphi g2avgErr
clear totalIntensity sumDark aIt g2final g2avg goodPixels NEPhoton delay   ;
% ---
analysistime = fix(clock)                                                  ;
ccdimginfo.result.analysistime{nBatch} =                                ...
    sprintf('%04i-%02i-%02i %02i:%02i:%02i',analysistime(1)             ...  % years
           ,analysistime(2),analysistime(3),analysistime(4)             ...  % months, days, hours
           ,analysistime(5),analysistime(6) )                              ; % minutes, seconds
clear analysistime                                                         ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish assigning results',                   ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish storing the results into the application data
% =========================================================================


% =========================================================================
% --- save application data 'ccdimginfo' to main figure
% =========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;
% =========================================================================
% --- finish saving application data 'ccdimginfo' to main figure
% =========================================================================


% =========================================================================
% --- save results to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save result file to disk!'}]              ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
savefile = sprintf([ccdimginfo.name,'result.mat'])                         ; % define filename for binary output
savefull = fullfile(ccdimginfo.savepath,savefile)                          ; % build full file name
save(savefull,'ccdimginfo')                                                ; % store the main structure
clear savefile savefull ccdimginfo                                         ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving result file',                  ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish saving results to disk
% =========================================================================

% =========================================================================
% --- clear leftover variables
% =========================================================================
clear colLow colHigh smooth storeG2Tensoron slicespacing slicedpl saveSetup;
clear notzero noS nBatch msgstr msgnew mindelay message longTerm           ;
clear avgfirst dpl framespacing hFigXPCSMain                               ;
% whos
% =========================================================================
% --- finish clearing leftover variables
% =========================================================================


% =========================================================================
% --- subfunction checksize
% =========================================================================
function [cLow,cHigh,noparts] = checksize(sizematrix,maxsize)
% ---
% --- This functions checks in how many parts the data tensor should be
% --- split before it is sent to the multi-tau correlation routine and 
% --- creates the vectors colLow and colHigh with information,
% --- where to split it.
% ---
% --- The input variable should be the result of sizematrix = size(Matrix)
% ---

% totalsize = 2/1.048576e6 * prod(sizematrix)                              ; % size of the correlation tensor in MB (for 'uint16')
totalsize = 4/1.048576e6 * prod(sizematrix)                                ; % size of the correlation tensor in MB (for 'single')
% --- calculate the number of data sections needed
if (totalsize > maxsize)
    noparts = ceil(totalsize/maxsize)                                      ;
else
    noparts = 1                                                            ;
end
clear totalsize                                                            ;

% --- create vector of column positions to split the data
i                = 1 : noparts + 1                                         ;
colsections      = (i-1)*ceil(sizematrix(2)/noparts) + 1                   ;
colsections(end) = sizematrix(2)                                           ;
clear i                                                                    ;

% --- some initializations for the data sectioning
cLow  = zeros(noparts,1)                                                   ;
cHigh = zeros(noparts,1)                                                   ;
for j = 1 : noparts
   cLow (j) = colsections(j)                                               ;
   cHigh(j) = colsections(j+1) - 1                                         ;
end
cHigh(end) = sizematrix(2)                                                 ;
clear colsections                                                          ;


% =========================================================================
% --- subfunction gaussianfilter
% =========================================================================
function gfilter = gaussianfilter(hsize,sigma)
% ---
% --- h = gaussianfilter(hsize,sigma) returns a rotationally symmetric 
% --- Gaussian lowpass filter of size hsize with standard deviation sigma
% --- (positive). hsize should be a scalar, i.e. h is a square matrix and
% --- sigma should also be a scalar.
% ---

if ( mod(hsize,2) == 0 )
    hsize = hsize + 1                                                      ;
end
if ( hsize <= 3 )
    hsize = 3                                                              ;
end
% ---
centerpoint = (hsize+1) / 2                                                ;
hg = zeros(hsize,hsize)                                                    ;
i = 1 : hsize                                                              ;
for j = 1 : hsize
        hg(:,j) = exp(-((i-centerpoint).^2+(j-centerpoint)^2)/(2*sigma^2)) ;
end
gfilter = hg / sum(sum(hg,2),1)                                            ;


% ---
% EOF
