function mainanalysis_cluster(nBatch)
%
% Perform static & dynamic (G2) analysis
%
% the results of this routine are stored in a structure 'ccdimginfo.result'
%
% Michael Sprung
% $Revision: 1.0 Date: 2005/02/23 $
% $Revision: 1.1 Date: 2005/04/24 $ check normalizations
% $Revision: 1.2 Date: 2005/08/29 $ recheck normalizations
% $Revision: 1.3 Date: 2005/09/12 $ add minor improvements
% $Revision: 1.4 Date: 2005/09/15 $ make only static analysis possible
% $Revision: 1.5 Date: 2005/12/13 $ normalize intensities properly
% $Revision: 1.6 Date: 2006/07/28 $ corrected flaw in finding
%                                        saturated pixels
% $Revision: 2.0 Date: 2006/10/31 $ modified to start cluster analysis
%                                        


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0.001)                                                               ;


% =========================================================================
% --- define directory of the X client   
% =========================================================================
currentdir = pwd                                                           ; % store the current directory
exceeddir  = 'C:\Program Files\Hummingbird\Connectivity\10.00\Exceed\'     ; % store the directory of the X client



% =========================================================================
% --- define test switches
% =========================================================================
storeTensoron = 0                                                          ; % switch if the full g2 tensor is stored or not
saveSetup     = 0                                                          ; % switch to store setup information or not
pause(0.001)                                                               ;


% =========================================================================
% --- create data and dark indices
% =========================================================================
dataIndex = ccdimginfo.ndata0todo(nBatch):ccdimginfo.ndataendtodo(nBatch)  ; % data index
darkIndex = ccdimginfo.ndark0todo(nBatch):ccdimginfo.ndarkendtodo(nBatch)  ; % dark index
pause(0.001)                                                               ;


% =========================================================================
% --- try to create saving subdirectory 'result' (needed if the analysis
% --- button was never pressed, e.g in batchmode)
% =========================================================================
warning('off','MATLAB:MKDIR:DirectoryExists')                              ;
dummy = mkdir(ccdimginfo.savepath)                                         ; % try to create directory to store the results
if ( dummy == 0 )                                                            % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'            ...
                    ;'   Try to save in Matlab XPCSGUI folder!'}]       ...
        ,'HorizontalAlignment','left')                                     ;
    pause(0.001)                                                           ;
    % ---
    ccdimginfo.savepath    = fullfile(pwd,'result')                        ; % try to create a subdirectory in the Matlab XPCSGUI folder
    dummy = mkdir(ccdimginfo.savepath)                                     ;
    if ( dummy == 0 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',...
            [msgstr;{'   Failed creating saving directory !'            ...
                    ;'   Please check write permissions!'}]             ...
           ,'HorizontalAlignment','left')                                  ;
        pause(0.001)                                                       ;
        return                                                             ;
    end
end
clear dummy                                                                ;
warning('on','MATLAB:MKDIR:DirectoryExists')                               ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish trying to create saving subdirectory 'result'
% =========================================================================


% =========================================================================
% --- check ccdimginfo.result substructure
% --- clear old results of selected batches 
% --- save the 'new' setup information to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save the setup information !'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
% --- check size off all result fields and change them if neccessary
if isfield(ccdimginfo,'result') 
    if (  isfield(ccdimginfo.result,'analysistime')                     ...
       && numel(ccdimginfo.result.analysistime)  <numel(ccdimginfo.preset))
        ccdimginfo.result.analysistime{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'totalIntensity')                   ...
       && numel(ccdimginfo.result.totalIntensity)<numel(ccdimginfo.preset))
        ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []    ;
    end
    if (  isfield(ccdimginfo.result,'framespacing')                     ...
       && numel(ccdimginfo.result.framespacing)  <numel(ccdimginfo.preset))
        ccdimginfo.result.framespacing{numel(ccdimginfo.preset)} = []      ;
    end
    if (  isfield(ccdimginfo.result,'aIt')                              ...
       && numel(ccdimginfo.result.aIt)           <numel(ccdimginfo.preset))
        ccdimginfo.result.aIt{numel(ccdimginfo.preset)} = []            ;
    end
    if (  isfield(ccdimginfo.result,'Iqphi')                            ...
       && numel(ccdimginfo.result.Iqphi)         <numel(ccdimginfo.preset))
        ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'G2')                               ...
       && numel(ccdimginfo.result.G2)            <numel(ccdimginfo.preset))
        ccdimginfo.result.G2{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'IF')                               ...
       && numel(ccdimginfo.result.IF)            <numel(ccdimginfo.preset))
        ccdimginfo.result.IF{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'IP')                               ...
       && numel(ccdimginfo.result.IP)            <numel(ccdimginfo.preset))
        ccdimginfo.result.IP{numel(ccdimginfo.preset)} = []                ;
    end
    if (  isfield(ccdimginfo.result,'g2avg')                            ...
       && numel(ccdimginfo.result.g2avg)         <numel(ccdimginfo.preset)) 
        ccdimginfo.result.g2avg{numel(ccdimginfo.preset)} = []             ;
    end
    if (  isfield(ccdimginfo.result,'g2avgErr')                         ... 
       && numel(ccdimginfo.result.g2avgErr)      <numel(ccdimginfo.preset))
        ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'goodPixels')                       ...
       && numel(ccdimginfo.result.goodPixels)    <numel(ccdimginfo.preset))
        ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)} = []        ;
    end
    if (  isfield(ccdimginfo.result,'NEPhoton')                         ...
       && numel(ccdimginfo.result.NEPhoton)      <numel(ccdimginfo.preset))
        ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)} = []          ;
    end
    if (  isfield(ccdimginfo.result,'delay')                            ...
       && numel(ccdimginfo.result.delay)         <numel(ccdimginfo.preset))
        ccdimginfo.result.delay{numel(ccdimginfo.preset)} = []             ;
    end
else
    ccdimginfo.result.analysistime{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.totalIntensity{numel(ccdimginfo.preset)} = []        ;
    ccdimginfo.result.framespacing{numel(ccdimginfo.preset)}   = []        ;
    ccdimginfo.result.aIt{numel(ccdimginfo.preset)}            = []        ;
    ccdimginfo.result.Iqphi{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.G2{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.IF{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.IP{numel(ccdimginfo.preset)}             = []        ;
    ccdimginfo.result.g2avg{numel(ccdimginfo.preset)}          = []        ;
    ccdimginfo.result.g2avgErr{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.goodPixels{numel(ccdimginfo.preset)}     = []        ;
    ccdimginfo.result.NEPhoton{numel(ccdimginfo.preset)}       = []        ;
    ccdimginfo.result.delay{numel(ccdimginfo.preset)}          = []        ;
end
% --- reset result fields for the batch to be analyzed
ccdimginfo.result.analysistime{nBatch}   = []                              ; % clear any previous time stamp
ccdimginfo.result.totalIntensity{nBatch} = []                              ; % clear the total intensity per slice
ccdimginfo.result.framespacing{nBatch}   = []                              ; % clear the total intensity per slice
ccdimginfo.result.aIt{nBatch}            = []                              ; % clear averaged image
ccdimginfo.result.Iqphi{nBatch}          = []                              ; % clear Iqphi vector
ccdimginfo.result.G2{nBatch}             = []                              ; % clear G2 tensor
ccdimginfo.result.IF{nBatch}             = []                              ; % clear IF (intensities future) tensor
ccdimginfo.result.IP{nBatch}             = []                              ; % clear IP (intensities past)   tensor
ccdimginfo.result.g2avg{nBatch}          = []                              ; % clear averaged g2's
ccdimginfo.result.g2avgErr{nBatch}       = []                              ; % clear the error of the averaged g2's
ccdimginfo.result.goodPixels{nBatch}     = []                              ; % clear the number of good pixels in a dynamic q phi partition
ccdimginfo.result.NEPhoton{nBatch}       = []                              ; % clear the indix of pixels which received not enough photons
ccdimginfo.result.delay{nBatch}          = []                              ; % clear vector with delay times
% --- store result field in result & remove result field the ccdimginfo
result     = ccdimginfo.result                                             ;
ccdimginfo = rmfield(ccdimginfo,'result')                                  ;
if ( isfield(ccdimginfo,'testimg') )
    ccdimginfo = rmfield(ccdimginfo,'testimg')                             ;
end
% --- save the setup information
if ( saveSetup == 1 )
    nobatch  = sprintf('batch%03i_',nBatch)                                ;
    savefile = sprintf([ccdimginfo.name,nobatch,'setup.mat'])              ; % define filename for binary output
    savefull = fullfile(ccdimginfo.savepath,savefile)                      ; % build full file name
    save(savefull,'ccdimginfo')                                            ; % store the setup information
    clear nobatch                                                          ;
end
% --- restore result field in ccdimginfo & remove result
ccdimginfo.result = result                                                 ;
clear result                                                               ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving setup information',            ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- finish checking ccdimginfo.result substructure, clearing old results
% --- & saving the 'new' setup information to disk
% =========================================================================


% =========================================================================
% --- check if a flatfield correction is needed
% --- if needed, apply blemish file to ccdimginfo.usermask
% =========================================================================
if ( ccdimginfo.flatfield == 1 )
    flatfield = getflatfield(ccdimginfo)                                   ;
else
    flatfield = []                                                         ;
end
if ( ccdimginfo.blemish == 1 )
    blemish = getblemish(ccdimginfo)                                       ;
    ccdimginfo.usermask = ccdimginfo.usermask .* blemish                   ;
    clear blemish                                                          ;
end
pause(0.001)                                                               ;
% =========================================================================
% --- finish blemish and flatfield implementation
% =========================================================================


% =========================================================================
% --- check if data is stored in a compressed multifile
% =========================================================================
SByte = []                                                                 ;
if ( ccdimginfo.compressedmulti(nBatch) == 1 )
    if ( isempty(ccdimginfo.SBCM{nBatch}) == 1 )
        SByte = indexcompressedmultiimm(ccdimginfo.imagefile{nBatch})      ; % try to get starting byte position of indices
    else
        SByte = ccdimginfo.SBCM{nBatch}                                    ; % had already been created
    end
end
% =========================================================================
% --- finish checking if data is stored in a compressed multifile
% =========================================================================


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1 = ccdimginfo.maskroi(1,2)                                               ; % row    start of ROI
R2 = ccdimginfo.maskroi(2,2)                                               ; % row    end   of ROI
C1 = ccdimginfo.maskroi(1,1)                                               ; % column start of ROI
C2 = ccdimginfo.maskroi(2,1)                                               ; % column end   of ROI
% =========================================================================
% --- finish defining some abbreviations
% =========================================================================


% =========================================================================
% --- create averaged dark image 'sumDark'
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Create dark image & dark infos!'}]        ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
sumDark    = zeros( ccdimginfo.row_end - ccdimginfo.row_beg + 1         ...
                  , ccdimginfo.col_end - ccdimginfo.col_beg + 1,'single')  ; % initialize dark image as single
if ( ccdimginfo.lld > 0 )
   sumDarkRMS = zeros(ccdimginfo.row_end - ccdimginfo.row_beg + 1       ...
                     ,ccdimginfo.col_end - ccdimginfo.col_beg + 1,'single'); % initialize dark rms image as single
end
% ---
for i = 1:length(darkIndex)
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        f = openfile(ccdimginfo.imagefile{nBatch},darkIndex(i))            ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark rms images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark rms images
            end
        end
    else
        f = openfile(ccdimginfo.darkfile{nBatch},darkIndex(i))             ; % load dark images
        if ( isempty(flatfield) == 1 )                                       % no flatfield correction needed
            sumDark    = sumDark    + f.imm                                ; % add dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + f.imm.^2                         ; % add dark images
            end
        else
            sumDark    = sumDark    +  f.imm .* flatfield                  ; % add flatfield corrected dark images
            if ( ccdimginfo.lld > 0 )
                sumDarkRMS = sumDarkRMS + (f.imm .* flatfield).^2          ; % add flatfield corrected dark images
            end
        end
    end
end
if ( numel(darkIndex) > 0 )
    sumDark     = sumDark    / numel(darkIndex)                            ; % calculate averaged dark image
    if ( ccdimginfo.lld > 0 )
        sumDarkRMS  = sumDarkRMS / numel(darkIndex) - sumDark.^2           ; % calculate averaged dark rms image
    end
end
clear darkIndex f                                                          ;
% ---
satLevel  = single(ccdimginfo.saturation * ones(size(sumDark)) - sumDark)  ; % calculate new saturation values including dark values
DImg      = satLevel(R1:R2,C1:C2)                                          ; % take saturation levels of ROI
DImg_Mask = DImg(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                    ; % take saturation levels of unmasked pixels in ROI
% ---
DImgNM    = cell(ccdimginfo.snoq,ccdimginfo.snophi)                        ;
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( numel(ccdimginfo.smask{n,m}) > 0 )
            DImgNM{n,m} = DImg(ccdimginfo.smask{n,m})                      ; % take only (unmasked) pixels from one static q phi partition
        end
    end
end
clear satLevel                                                             ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish creating the dark image',             ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of creating dark averages
% =========================================================================


% =========================================================================
% --- begin of cluster configuration & creating hdf files for analysis
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start cluster configuration!'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
% --- create dark image map
darkfile = fullfile(ccdimginfo.imgPath                                  ...
                   ,sprintf([ccdimginfo.name,'dark.mat']))                 ; % build full file name
dark     = uint16(ceil(sumDark))                                           ;
save(darkfile,'dark','-v7.3')                                              ; % save the dark image
clear dark darkfile                                                        ;
% --- create dark rms image map
if ( ccdimginfo.lld > 0 )
    darkrmsfile = fullfile(ccdimginfo.imgPath                           ...
                          ,sprintf([ccdimginfo.name,'darkrms.mat']))       ; % build full file name
    darkrms     = uint16(ceil(sumDarkRMS))                                 ;
    save(darkrmsfile,'darkrms','-v7.3')                                    ; % save the dark rms image
end
clear darkrms darkrmsfile                                                  ;
% --- create static partition map
staticfile = fullfile(ccdimginfo.imgPath                                ...
                     ,sprintf([ccdimginfo.name,'static.mat']))             ; % build full file name
static  = zeros(size(ccdimginfo.usermask),'uint16')                        ;
dummy   = zeros(R2-R1+1,C2-C1+1,'uint16')                                  ;
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( ~isempty(ccdimginfo.smask{n,m}) )
            dummy(ccdimginfo.smask{n,m}) = (n-1) * m + m                   ; % assign pixels of different partitions different values
        end
    end
end
static(R1:R2,C1:C2) = dummy                                                ;
save(staticfile,'static','-v7.3')                                          ; % save the static partition map
clear static staticfile dummy                                              ;
% --- create dynamic partition map
dynamicfile = fullfile(ccdimginfo.imgPath                               ...
                      ,sprintf([ccdimginfo.name,'dynamic.mat']))           ; % build full file name
dynamic = zeros(size(ccdimginfo.usermask),'uint16')                        ;
dummy   = zeros(R2-R1+1,C2-C1+1,'uint16')                                  ;
for n = 1 : ccdimginfo.dnoq
    for m = 1 : ccdimginfo.dnophi
        if ( ~isempty(ccdimginfo.dmask{n,m}) )
            dummy(ccdimginfo.dmask{n,m}) = (n-1) * m + m                   ; % assign pixels of different partitions different values
        end
    end
end
dynamic(R1:R2,C1:C2) = dummy                                               ;
save(dynamicfile,'dynamic','-v7.3')                                        ; % save the dynamic partition map
clear dynamic dynamicfile dummy                                            ;
% --- if neccessary, create flatfield map
if ( ~isempty(flatfield) == 1 )
    flatfieldfile = fullfile(ccdimginfo.imgPath                         ...
                            ,sprintf([ccdimginfo.name,'flatfield.mat']))   ; % build full file name
    save(flatfieldfile,'flatfield','-v7.3')                                ; % save the dynamic partition map
    clear flatfieldfile                                                    ;    
end
% ---
writeclustercfg(nBatch)                                                    ; % write the cluster configuration file
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish cluster configuration in ',           ...
    num2str(t),' seconds.']}],'HorizontalAlignment','left')                ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of cluster configuration & creating hdf files for analysis
% =========================================================================


% =========================================================================
% --- start cluster analysis & wait for its end 
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start cluster analysis!'}]                ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
% --- determine location of network disk
if ( strcmp(ccdimginfo.imgPath(1:3),'W:\') == 1 )
    networkstart = '//users-nfs/8-idi-a/'                                  ;
elseif ( strcmp(ccdimginfo.imgPath(1:3),'X:\') == 1 )
    networkstart = '//users-nfs/8-idi-b/'                                  ;
elseif ( strcmp(ccdimginfo.imgPath(1:3),'Y:\') == 1 )
    networkstart = '//users-nfs/8-idi-c/'                                  ;
elseif ( strcmp(ccdimginfo.imgPath(1:3),'Z:\') == 1 )
    networkstart = '//users-nfs/8-idi-d/'                                  ;
elseif ( strcmp(ccdimginfo.imgPath(1:3),'D:\') == 1 )
    networkstart = '//users-nfs/8-idi-a/'                                  ;
end
% --- determine image directory 
imagepath       = ccdimginfo.imgPath(4:end)                                ;
imagepathindex  = strfind(imagepath,'\')                                   ;
if ( ~isempty(imagepathindex) )
    imagepath(imagepathindex) = '/'                                        ;
end
if ( isempty(strfind(imagepath(end),'/')) )
    imagepath(end+1) = '/'                                                 ;
end
% --- 
configurationfile = ['''''',networkstart,imagepath                      ...
                    ,ccdimginfo.name,'cluster.cfg','''''']                 ; % assemble configuration file
startcluster      = ['!xstart.exe bixbite.xs '           ... 
                    ,'-c ''''startcluster(''',configurationfile,''')&''''']; % define external command
% ---
cd(exceeddir)                                                              ; % change into the X client directory
% eval(startcluster)                                                         ; % call external script to start cluster analysis
cd(currentdir)                                                             ; % come back to the old directory
% ---
clear networkstart imagepath imagepathindex startcluster configurationfile ;
clear currentdir exceeddir                                                 ;
% ---
clusterfinishfile = [ccdimginfo.imgPath filesep 'clusterresults' filesep...
                    'finish.txt']                                          ;
clusterfinish     = 1                                                      ;
while ( clusterfinish == 0 )
    clusterid = fopen(clusterfinishfile)                                   ; % try to open the clusterfinish file
    if ( clusterid == -1 )                                                   % pause 0.5 seconds if open fails
        pause(0.5)                                                         ;
        clear clusterid                                                    ;
    else
        fclose(clusterid)                                                  ; % close the clusterfinish file
        clusterfinish = 1                                                  ; % change the clusterfinish flag to one
    end
end
clear clusterfinish clusterfinishfile clusterid                            ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish cluster analysis in ',                ...
    num2str(t),' seconds.']}],'HorizontalAlignment','left')                ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of starting cluster analysis & waiting for its end 
% =========================================================================

% --- to be finished

% =========================================================================
% --- load & assemble cluster analysis results into ccdimginfo.result 
% =========================================================================


% =========================================================================
% --- end of loading & assembling cluster analysis results
% =========================================================================



% =========================================================================
% =========================================================================
% =========================================================================
% --- Main loop
% =========================================================================
% =========================================================================
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start the main calculation loop !'}]      ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================


% =========================================================================
% --- initialization
% =========================================================================
sumImg = zeros(ccdimginfo.row_end-ccdimginfo.row_beg+1                  ...
              ,ccdimginfo.col_end-ccdimginfo.col_beg+1,'single')           ; % initialize the summed image (full size)
% --- calculate average frame spacing
framespacing = mean(diff(immelapsed(ccdimginfo.imagefile{nBatch}        ...
                                   ,dataIndex(1),dataIndex(end))))         ; % average time between two frames
% --- get delay and maxLevel
if ( ccdimginfo.analysistype == 1 )
    dpl          = ccdimginfo.dpl                                          ;
    delay        = framespacing * finddelays(length(dataIndex),dpl,1)      ;
    if ( numel(delay) <= 2*dpl-1 )
        maxLevel = 1                                                       ;
    else
        maxLevel = ceil((numel(delay)-(2*dpl-1))/dpl) + 1                  ;           
    end
end
% --- initializations for the G2 calculation (only on ROI)
if ( ccdimginfo.analysistype == 1 )
    % --- Image stacks for all Levels
    LImg = zeros(R2-R1+1,C2-C1+1,2*dpl,maxLevel,'single')                  ; % initialize image arrays (ROI size) for all levels
    LC   = zeros(maxLevel,1)                                               ; % initialize level counter
    % --- G2 tensor & IF & IP + counter
    G2     = zeros(R2-R1+1,C2-C1+1,(maxLevel+1)*dpl-1,'single')            ; % initialize the G2  tensor (ROI size)
    IF     = zeros(R2-R1+1,C2-C1+1,(maxLevel+1)*dpl-1,'single')            ; % initialize the IF (intensities future) tensor (ROI size)
    IP     = zeros(R2-R1+1,C2-C1+1,(maxLevel+1)*dpl-1,'single')            ; % initialize the IP (intensities past)   tensor (ROI size)
    GC     = zeros((maxLevel+1)*dpl-1,1)                                   ; % counter for the G2 sums
end
    

% =========================================================================
% --- start of loop over all frames
% =========================================================================
notzero = 1.0e-09                                                          ; % create a variable which is almost zero to prevent 'Divide by Zero' errors
for i = 1 : length(dataIndex)                                                % loop over all frames


    % =====================================================================
    % --- show progress message
    % =====================================================================
    if ( i == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Process image no. %05i of %05i images'     ...
                         ,i,length(dataIndex))                             ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    if ( length(dataIndex) < 500 && (floor(i/25)-i/25+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                    'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Process image no. %05i of %05i images',    ...
                         i,length(dataIndex))                              ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end
    if (  length(dataIndex) >= 500 && length(dataIndex) < 1000          ...
       && (floor(i/50)-i/50+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                    'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Process image no. %05i of %05i images',    ...
                         i,length(dataIndex))                              ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end
    if ( length(dataIndex) >= 1000 && (floor(i/100)-i/100+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                       ...
                    'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Process image no. %05i of %05i images',   ...
                        i,length(dataIndex))                              ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    % =====================================================================
    % --- end of showing progress message
    % =====================================================================


    % =====================================================================
    % --- load data, remove dark image & flatfield correction
    % =====================================================================
    if ( ~isempty(SByte) == 1 )
        f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i)          ...
                    ,SByte(dataIndex(i)-ccdimginfo.ndata0(nBatch)+1))      ; % load image file with pointer           
    else
        f = openfile(ccdimginfo.imagefile{nBatch},dataIndex(i))            ; % load image file
    end
    if ( isempty(flatfield) == 1 )                                           % no flatfield correction needed
        trueData = f.imm - sumDark                                         ; %  do backgroud subtraction
    else
        trueData = f.imm .* flatfield  - sumDark                           ; % perform flatfield correction and do backgroud subtraction
    end
    clear f    
    % =====================================================================
    % --- end of loading data, removing dark image & flatfield correction
    % =====================================================================

    
    % =====================================================================
    % --- do lower level discrimination (LLD)
    % =====================================================================
    if (ccdimginfo.lld <= 0)
        trueData(trueData <= abs(ccdimginfo.lld)) = notzero                ; % reset all values below the lower level discriminator to ~zero
    end
    if (ccdimginfo.lld > 0)
        trueData(trueData <= ccdimginfo.lld * sumDarkRMS) = notzero        ; % reset all values below the lower level discriminator to ~zero
    end
    % =====================================================================
    % --- end of lower level discrimination (LLD)
    % =====================================================================
        

    % =====================================================================
    % --- create sumImg 
    % =====================================================================
    sumImg = sumImg + trueData                                             ; % add images to the averaged image (after lower level discrimination)
    % =====================================================================
    % --- end of creating sumImg 
    % =====================================================================

    
    % =====================================================================
    % --- Image examination:
    % --- 1) integrate the intensity of the ROI of each image
    % --- 2) find saturated pixels
    % --- 3) find pixels with not enough photons NEPhoton
    % --- 4) mark all static partitions with pixels as good (only once)
    % =====================================================================
    Img = trueData(R1:R2,C1:C2)                                            ; % take values of ROI
    clear trueData                                                         ;
    % --- calculate the integrated intensity per frame
    if ( i == 1 )
        totalIntensity = zeros(length(dataIndex),1,'single')               ;
    end
    Img_Mask = Img(ccdimginfo.usermask(R1:R2,C1:C2) == 1)                  ; % unmasked pixels on the ROI 
    totalIntensity(i,1)  = sum(Img_Mask(Img_Mask <= DImg_Mask) )        ...
                         / ccdimginfo.efficiency                        ...
                         / ccdimginfo.adupphot                          ...
                         / ccdimginfo.preset(nBatch)                       ; % calculate the intensity [photons/sec] of the unmasked, unsaturated ROI
    clear Img_Mask                                                         ;
    % --- find saturated pixels and store the indices
    if ( i == 1 )
        saturatedPixels = cell(length(dataIndex),1)                        ;
    end
    indices = find(Img > DImg)                                             ; % get the indices of saturated pixels in ROI units
    saturatedPixels{i,1} = indices                                         ; % store the indices of saturated pixels in ROI units
    % ---
    if ( ccdimginfo.analysistype == 1 )
        % --- find pixels, which recieved less than 2 photons
        if ( i == 1 )                                                        % these things need to be done only once
            dummy       = zeros(size(Img,1),size(Img,2)) + 2               ; % create a slice sized matrix to check if more than 2 photons hit a pixel
            NEPhoton    = find(ccdimginfo.usermask(R1:R2,C1:C2))           ; % start with all unmasked pixels in ROI
        end
        ind1                  = find(Img(NEPhoton))                        ; % find non zeros on a slice
        dummy(NEPhoton(ind1)) = dummy(NEPhoton(ind1)) - 1                  ; % reduce the value of dummy by one at pixels where one photon hit
        ind2                  = find(dummy(NEPhoton) > 0)                  ; % remaining pixels with not enough photons
        NEPhoton              = NEPhoton(ind2)                             ; % reduce the size of the 'not enough photon counter'
        clear ind1 ind2                                                    ;
        % --- mark all static partitions with pixels as good
        if ( i == 1 )
            goodsqphipartitions = ones(ccdimginfo.snoq,ccdimginfo.snophi)  ;
            for n = 1 : ccdimginfo.snoq
                for m = 1 : ccdimginfo.snophi
                    if ( isempty(ccdimginfo.smask{n,m}) )
                        goodsqphipartitions(n,m) = 0                       ; % count a partition as bad if it does not contain pixels
                    end
                end
            end
        end
    end
    % =====================================================================
    % --- end of image examination
    % =====================================================================

    
    % =====================================================================
    % --- update image stacks & perform multi-tau g2 calculation
    % =====================================================================
    if ( ccdimginfo.analysistype == 1 )
        
        % =================================================================
        % --- Level 1 
        % =================================================================
        % --- update image stacks
        LImg(:,:,mod(i-1,2*dpl)+1,1) = Img(:,:)                            ; % replace the oldest image in the level 1 stack with the new one
        LC(1) = LC(1) + 1                                                  ; % increase level 1 counter
        clear Img                                                          ; % clear the new image
        % --- perform multi-tau G2 calculation
        for l = 1 : 2*dpl-1                                                  % loop over all level 1 delays
            if ( i >= l+1 )
                G2(:,:,l) = G2(:,:,l) + LImg(:,:,mod(i-1  ,2*dpl)+1,1)  ...
                                     .* LImg(:,:,mod(i-1-l,2*dpl)+1,1)     ; % latest image times an 'l' times older image
                IF(:,:,l) = IF(:,:,l) + LImg(:,:,mod(i-1  ,2*dpl)+1,1)     ; % add to IF tensor
                IP(:,:,l) = IP(:,:,l) + LImg(:,:,mod(i-1-l,2*dpl)+1,1)     ; % add to IP tensor
                GC(l)     = GC(l) + 1                                      ; % increase G2 counter
            end
        end
        % =================================================================
        % --- end of Level 1 
        % =================================================================

        
        % =================================================================
        % --- All higher Levels
        % =================================================================
        for k = 2 : maxLevel                                                 % loop over all higher levels
            % --- update image stacks
            if ( mod(i,2^(k-1)) == 0 )
                LImg(:,:,mod(LC(k),2*dpl)+1,k) = 1/2                    ...
                        * ( LImg(:,:,mod(LC(k-1)-2,2*dpl)+1,k-1)        ...
                          + LImg(:,:,mod(LC(k-1)-1,2*dpl)+1,k-1) )         ; % average (k-1) level images into level k
                LC(k) = LC(k) + 1                                          ; % increase level k counter
            end
            % --- perform multi-tau G2 calculation
            for l = 1 : dpl                                                  % loop over all delays of level k
                n   = k*dpl + l-1                                          ; % delay number
                g2n = dpl * 2^(k-1) + l * 2^(k-1)                          ; % 1st change to add to the delay n of g2
                if (  mod(i,2^(k-1)) == mod(g2n,2^(k-1)) && i >= g2n )
                    G2(:,:,n) = G2(:,:,n)                               ...
                              + LImg(:,:,mod(GC(n)+dpl+l-1,2*dpl)+1,k)  ...
                             .* LImg(:,:,mod(GC(n)        ,2*dpl)+1,k)     ; % latest image times an 'l' times older image
                    IF(:,:,n) = IF(:,:,n)                               ...
                              + LImg(:,:,mod(GC(n)+dpl+l-1,2*dpl)+1,k)     ; % add to IF tensor
                    IP(:,:,n) = IP(:,:,n)                               ...
                              + LImg(:,:,mod(GC(n)        ,2*dpl)+1,k)     ; % add to IP tensor
                    GC(n)     = GC(n) + 1                                  ; % increase G2 counter
                end
            end
        end
        % =================================================================
        % --- end of all higher Levels
        % =================================================================
        
    end
    % =====================================================================
    % --- end of updating image stacks & performing multi-tau g2 calculation
    % =====================================================================
    
end
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish main calculation loop in ',           ...
    num2str(t),' seconds.']}],'HorizontalAlignment','left')                ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of main loop
% =========================================================================


% =========================================================================
% --- clear leftover main loop variables
% =========================================================================
clear sumDark DImg DImg_Mask DImgNM dummy SByte dpl maxLevel               ;
if ( ccdimginfo.lld > 0 )          
    clear sumDarkRMS                                                       ;
end
if ( ccdimginfo.analysistype == 1 )
    clear LImg LC g2n                                                      ;
end
if ( ~isempty(flatfield) == 1 )    
    clear flatfield                                                        ;
end
% =========================================================================
% --- end of clearing variables
% =========================================================================


% =========================================================================
% --- create SPMask: mask to exclude saturated pixels in usermask
% --- mask all ~good static q phi partitions (see goodsqphipartitions) 
% --- create aIt   : time averaged intensity map <I(q,t)>_t (only once)
% --- create Iqphi : averaged intensity of static partitions of aIt
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start creating time averages'}]           ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
SPMask = ones(R2-R1+1,C2-C1+1,'single')                                    ; % initialize SPMask as ones (no saturated pixels) on ROI
for i = 1:length(dataIndex)                                                  % loop over all frames
    indices         = saturatedPixels{i,1}                                 ;
    SPMask(indices) = 0                                                    ; % set the Mask for all saturated pixels to zero
end
ccdimginfo.usermask(R1:R2,C1:C2) = SPMask.*ccdimginfo.usermask(R1:R2,C1:C2); % incoorperate the saturated pixel mask into the user mask
clear indices saturatedPixels SPMask                                       ;
% ---
if ( ccdimginfo.analysistype == 1 )
    dummy = ccdimginfo.usermask(R1:R2,C1:C2)                               ; % take only ROI of usermask
    for n = 1 : ccdimginfo.snoq
        for m = 1 : ccdimginfo.snophi
            if ( goodsqphipartitions(n,m) == 0 )
                dummy(ccdimginfo.smask{n,m}) = 0                           ; % mask every pixel of a bad static q phi partition                        
            end
        end
    end
    ccdimginfo.usermask(R1:R2,C1:C2) = dummy(:,:)                          ; % store the corrected usermask
    clear dummy goodsqphipartitions                                        ;
end
% --- create normalized averaged image (full size)
aIt               = sumImg / length(dataIndex)                             ; % calculated aIt (normalized average of images)
aIt(aIt<=notzero) = notzero                                                ; % reset values below ~zero in aIt to ~zero
clear sumImg dataIndex                                                     ;
% --- create the partition averages of the 'averaged' image
Iqphi     = zeros(ccdimginfo.snoq,ccdimginfo.snophi)                       ; % initialize Iqphi
dummy1    = aIt(R1:R2,C1:C2)                                               ; % take only ROI of aIt
dummy2    = ccdimginfo.usermask(R1:R2,C1:C2)                               ; % take only ROI of usermask
for n = 1 : ccdimginfo.snoq
    for m = 1 : ccdimginfo.snophi
        if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                              % check if static q phi partition contains pixels
            Inm    = dummy1(ccdimginfo.smask{n,m})                         ; % get values of aIt for all pixels of a static q phi partition
            Mnm    = dummy2(ccdimginfo.smask{n,m})                         ; % get values of usermask (with SPMask) for all pixels of a static partition
            if ( numel(Inm(Mnm==1)) > 0 )
                Iqphi(n,m) = mean(Inm(Mnm==1))                             ; % calculate the average intensity in the averaged static partitions
            end
        end
    end
end
Iqphi(Iqphi<=notzero) = notzero                                            ;
clear Inm Mnm dummy1 dummy2                                                ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
   'String',[msgstr;{['   Finish creating time averages',               ...
   ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')          ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of creating time averaged intensity maps <I(q,t)>_t
% =========================================================================


% =========================================================================
% --- try to calculate absolute intensities
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Start calculating absolute intensities'}] ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
if ( numel(ccdimginfo.dpix) > 1 )                                            % pixel size is different in x & y direction
    dpix_x = ccdimginfo.dpix(1)                                            ;    
    dpix_y = ccdimginfo.dpix(2)                                            ;
else                                                                         % pixel size is identical in x & y direction
    dpix_x = ccdimginfo.dpix                                               ;
    dpix_y = dpix_x                                                        ;
end
Iqphi = Iqphi / ccdimginfo.efficiency / ccdimginfo.adupphot             ...
      / ccdimginfo.preset(nBatch)                                          ; % this can always be calculated (unit: [photons/sec])
ccdimginfo.normalization = 0                                               ; % standard normalization case for both geometries (unit: [photons/sec])
% ---
if ( ccdimginfo.geometry == 0 )                                              % in transmission mode special normalization are possiple
    Iqphi = Iqphi / ( dpix_x/ccdimginfo.rr * dpix_y/ccdimginfo.rr )        ; % divide by the solid angle; unit is still [photons/sec]
    ccdimginfo.normalization = 1                                           ; % normalization case set to one [photon/sec]
    if ( ccdimginfo.tflux == 1 )                                             % if the transmitted flux was measured, than 'beam_i' & 'ring_i_sample' are known 
        Iqphi = Iqphi / ccdimginfo.beam_i(nBatch)                       ...
              / ccdimginfo.ring_i(nBatch)                               ...
              * ccdimginfo.ring_i_sample(nBatch)                           ; % Iqphi can be normalized by transmitted flux (unit: dimensionless)
        ccdimginfo.normalization = 2                                       ; % normalization case set to two [no dimension]
    end
    if (  ccdimginfo.normalization == 2                                 ...
       && ccdimginfo.thicknessflag > 0 )                                     % further normalization might be possible (two cases)
        % --- case 3a
        if (  ccdimginfo.tflux == 1 && ccdimginfo.iflux == 1            ...
          && ccdimginfo.thicknessflag == 2 )                                % if the transmitted & incident flux were measured & the attenuation length is known
            Iqphi = Iqphi / ( 0.1 * ccdimginfo.atten_length )           ...  % divide by the attenuation length (convert to [cm])
                  / log( (ccdimginfo.beam_i_vacuum(nBatch)              ...
                         /ccdimginfo.ring_i_vacuum(nBatch))             ...
                       / (ccdimginfo.beam_i(nBatch)                     ...
                         /ccdimginfo.ring_i_sample(nBatch))  )             ; % divide by log(incident flux / transmitted flux)
            ccdimginfo.normalization = 3                                   ; % normalization case set to three (dimension: [1/cm])
        end
        % --- case 3b (check only if not case 3a)
        if (  ccdimginfo.normalization == 2                             ...
           && ccdimginfo.thicknessflag == 1 )                                % sample thickness in [mm] is known
             Iqphi = Iqphi / ( 0.1 * ccdimginfo.thickness )                ; % divide by sample thickness (converted to [cm]) 
             ccdimginfo.normalization = 3                                  ; % normalization case set to three (dimension: [1/cm])
        end
    end
else                                                                          % reflection geometry 
    if ( ccdimginfo.iflux == 1 )
        Iqphi = Iqphi / ccdimginfo.beam_i_vacuum(nBatch)                ...
              / ccdimginfo.ring_i(nBatch)                               ...
              * ccdimginfo.ring_i_vacuum(nBatch)                           ; % Iqphi can be normalized by incident flux (unit: dimensionless)  
        ccdimginfo.normalization = 4                                       ; % normalization case set to four (no dimension)
    end
end
clear dpix_x dpix_y                                                        ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
% ---
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
   'String',[msgstr;{['   Finish creating time averages',               ...
   ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')          ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of calculating absolute intensities
% =========================================================================


% =========================================================================
% --- create averaged correlation function of dynamic q phi partitions,
% --- calculate standard deviations & estimate the number of valid g2's
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Calculate averaged g2 functions!'}]       ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
if ( ccdimginfo.analysistype == 1 )
    g2avg   = ones(ccdimginfo.dnoq,ccdimginfo.dnophi,numel(delay),'single'); % initialize the averaged g2 tensor
    g2avgErr=zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,numel(delay),'single'); % initialize the tensor for the error of the averaged g2 tensor
    goodPixels       = zeros(ccdimginfo.dnoq,ccdimginfo.dnophi,'single')   ; % initialize the matrix to store the number of good pixels in a partition
    g2mask           = ccdimginfo.usermask(R1:R2,C1:C2)                    ; % create a mask for the correlation functions
    g2mask(NEPhoton) = 0                                                   ; % mask 'not enough photon' pixels on g2mask
    % ---
    for k = 1 : numel(delay)                                                 % loop over all delays
        if ( GC(k) ~= 0 )
            % ---
            G2(:,:,k) = G2(:,:,k) / GC(k)                                  ; % normalize delay k of G2 
            IF(:,:,k) = IF(:,:,k) / GC(k)                                  ; % normalize delay k of IF
            IP(:,:,k) = IP(:,:,k) / GC(k)                                  ; % normalize delay k of IP
            % ---
            dummyIF   = IF(:,:,k)                                          ; % take normalized delay k of IF
            dummyIP   = IP(:,:,k)                                          ; % take normalized delay k of IF
            % ---
            partIFk   = zeros(size(dummyIF))                               ; % create partIFk matrix
            partIPk   = zeros(size(dummyIP))                               ; % create partIPk matrix
            IFqphi    = zeros(ccdimginfo.snoq,ccdimginfo.snophi)           ; % initialize IFqphi
            IPqphi    = zeros(ccdimginfo.snoq,ccdimginfo.snophi)           ; % initialize IPqphi
            % ---
            for n = 1 : ccdimginfo.snoq
                for m = 1 : ccdimginfo.snophi
                    if ( ccdimginfo.smaskinfo(n,m,3) ~= 0 )                  % check if static q phi partition contains pixels
                        IFnm   = dummyIF(ccdimginfo.smask{n,m})            ; % get values of aIt for all pixels of a static q phi partition
                        IPnm   = dummyIP(ccdimginfo.smask{n,m})            ; % get values of aIt for all pixels of a static q phi partition
                        Mnm    = g2mask(ccdimginfo.smask{n,m})             ; % get values of usermask for all pixels of a static q phi partition
                        if ( numel(IFnm(Mnm==1)) > 0 )
                            IFqphi(n,m) = mean(IFnm(Mnm==1))               ; % calculate the average intensity in one partition in IF(k)
                            partIFk(ccdimginfo.smask{n,m}) = IFqphi(n,m)   ; % fill the partition averaged image
                        end
                        if ( numel(IPnm(Mnm==1)) > 0 )
                            IPqphi(n,m) = mean(IPnm(Mnm==1))               ; % calculate the average intensity in one partition in IP(k)
                            partIPk(ccdimginfo.smask{n,m}) = IPqphi(n,m)   ; % fill the partition averaged image
                        end
                    end
                end
            end
            % ---
            partIFk(partIFk<=notzero) = notzero                            ; % set zeros or negative values to notzero in partIFk
            partIPk(partIPk<=notzero) = notzero                            ; % set zeros or negative values to notzero in partIPk
            % ---
            g2dummy1            = G2(:,:,k) ./ (partIFk .* partIPk)        ; % symmetric normalize G2(k) by the partition averages
            g2dummy1(g2mask==0) = -1000                                    ; % set masked pixels to -1000
            % ---
            for n = 1 : ccdimginfo.dnoq                                      % loop over dynamic q   partitions
                for m = 1 : ccdimginfo.dnophi                                % loop over dynamic phi partitions
                    g2dummy2 = g2dummy1(ccdimginfo.dmask{n,m})             ; % take only values of one dynamic q phi partition
                    if ( numel(g2dummy2(g2dummy2~=-1000)) > 0 )
                        g2avg(n,m,k)    = mean(g2dummy2(g2dummy2~=-1000))  ; % average only valid entrees of the g2 tensor
                        if ( k == 1 )
                            goodPixels(n,m) =                           ...
                                numel(g2dummy2(g2dummy2~=-1000))           ; % get the number of good pixels in a dynamic q phi partition
                        end
                        g2avgErr(n,m,k) = single( sqrt(1/goodPixels(n,m))...
                                        * std(g2dummy2(g2dummy2~=-1000)))  ; % calculate the error of the valid g2 functions (Std of the Mean!)
                    else
                        g2avg(n,m,k)    = 1                                ;
                        g2avgErr(n,m,k) = 0                                ;
                        if ( k == 1 )
                            goodPixels(n,m) = 0                            ;
                        end
                    end
                end
            end
            % ---
        end
    end
    % ---
    clear GC  IFqphi IPqphi dummyIF dummyIP IFnm IPnm k n m                ;
    clear Mnm dummyF dummyP partIFk partIPk                                ;
    clear g2dummy1 g2dummy2 g2mask k n m                                   ;
end
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish averaging g2 functions',              ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of averaging g2 functions
% =========================================================================


% =========================================================================
% --- store the correlation functions to the output structure
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Assign results to output structure!'}]    ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
ccdimginfo.result.totalIntensity{nBatch}     = totalIntensity              ; % store the totalIntensity on a slice/frame
ccdimginfo.result.framespacing{nBatch}       = framespacing                ; % store the averaged time between frames
ccdimginfo.result.aIt{nBatch}                = aIt                         ; % store the averaged image aIt
ccdimginfo.result.Iqphi{nBatch}              = Iqphi                       ; % store the averaged intensity in the static q phi partitions of aIt
clear totalIntensity framespacing aIt Iqphi                                ;
% ---
if ( ccdimginfo.analysistype == 1 )
    if ( storeTensoron == 1 )
        ccdimginfo.result.G2{nBatch}         = G2                          ; % store G2 tensor (for each pixel)
        ccdimginfo.result.IF{nBatch}         = IF                          ; % store IF tensor (for each pixel)
        ccdimginfo.result.IP{nBatch}         = IP                          ; % store IP tensor (for each pixel)
    else
        ccdimginfo.result.G2{nBatch}         = []                          ;
        ccdimginfo.result.IF{nBatch}         = []                          ;
        ccdimginfo.result.IP{nBatch}         = []                          ;
    end
    ccdimginfo.result.g2avg{nBatch}          = g2avg                       ; % store the averaged g2 functions for each dynamic q phi partition
    ccdimginfo.result.g2avgErr{nBatch}       = g2avgErr                    ; % store the error of the averaged g2 functions
    ccdimginfo.result.goodPixels{nBatch}     = goodPixels                  ; % store the number of valid g2 functions in each dynamic q phi partition
    ccdimginfo.result.NEPhoton{nBatch}       = NEPhoton                    ; % store the indices of pixels which received less than 2 photons
    ccdimginfo.result.delay{nBatch}          = delay                       ; % store the delay vector
    clear G2 IF IP g2avg g2avgErr goodPixels NEPhoton delay                ;
end
% ---
analysistime = fix(clock)                                                  ;
ccdimginfo.result.analysistime{nBatch} =                                ...
    sprintf('%04i-%02i-%02i %02i:%02i:%02i',analysistime(1)             ...  % years
           ,analysistime(2),analysistime(3),analysistime(4)             ...  % months, days, hours
           ,analysistime(5),analysistime(6) )                              ; % minutes, seconds
clear analysistime                                                         ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish assigning results',                   ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of storing the results to the output structure
% =========================================================================


% =========================================================================
% --- save results to disk
% =========================================================================
t1 = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{' ';'   Save result file to disk!'}]              ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
% =========================================================================
savefile = sprintf([ccdimginfo.name,'result.mat'])                         ; % define filename for binary output
savefull = fullfile(ccdimginfo.savepath,savefile)                          ; % build full file name
save(savefull,'ccdimginfo')                                                ; % store the main structure
clear savefile savefull                                                    ;
% =========================================================================
t2 = clock                                                                 ;
t  = etime(t2,t1)                                                          ;
msgstr = get(findall(hFigXPCSMain,                                      ...
                     'Tag','xpcsmain_EditMessage'),'string')               ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   Finish saving result file',                  ...
    ' in ',num2str(t),' seconds.']}],'HorizontalAlignment','left')         ;
clear t t1 t2                                                              ;
pause(0.001)                                                               ;
% =========================================================================
% --- end of saving results to disk
% =========================================================================


% =========================================================================
% --- clear leftover vaiables
% =========================================================================
clear C1 C2 R1 R2 saveSetup storeTensoron notzero nBatch i l               ;
clear msgstr msgnew hFigXPCSMain ccdimginfo                                ;
whos
% =========================================================================
% --- finish clearing leftover vaiables
% =========================================================================


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;

                
% ---
% EOF
