function memorycheck(nBatch)                 
% ---
% --- calculate the number of steps of the main calculation routine due to
% --- memory restrictions
% ---
% --- noDataSection : number of data sections for calculation
% --- dataSections  : vector of column positions to split the data
%
% e.g.: 
% if the absolute size of the ccd chip is 1000 rows X 2000 columns
% and data is taken in an ROI of 301-700 x 501-1500 than the saved image
% size is 400 x 1000 pixels. The coordinates of the masked points are
% defined with respect to the saved image! If the usermask now selects
% only a small portion of the saved image than the maskroi is also defined
% in respect to the saved image! This is why dataSections starts at
% ccdimginfo.maskroi(1,1) and not necessarly at 1 like it was in the old
% memorycheck.
%
%
% Michael Sprung
% $Revision: 1.0 $  $Date: 2005/01/07 $
% $Revision: 1.1 $  $Date: 2005/11/22 $ try to make it possible to use a
% ROI feature defined by ccdimgino.maskroi which contains the coordinates
% of the lower left and upper right corner of the minimun rectangle which
% contains all unmasked pixels


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;


% =========================================================================
% --- create data index of actual batch
% =========================================================================
dataIndex = ccdimginfo.ndata0todo(nBatch):ccdimginfo.ndataendtodo(nBatch)  ;


% =========================================================================
% --- calculate how much of the data can be processed at one time 
% =========================================================================
if (ccdimginfo.analysistype == 0)                                            % don't worry for purely static analysis --> do calculation in 1 step
    ccdimginfo.noDataSection = 1                                           ;
    ccdimginfo.dataSections  = [ccdimginfo.col_beg,ccdimginfo.col_end]     ;
end
if (ccdimginfo.analysistype == 1)
    if ccdimginfo.kinetics == 0                                              % full frame or ROI mode
        TotalSize = 4/1.048576e6 * length(dataIndex)                    ...  % size of the correlation tensor in MB
                  * (ccdimginfo.maskroi(2,1)-ccdimginfo.maskroi(1,1)+1) ...  % number of rows (column position)
                  * (ccdimginfo.maskroi(2,2)-ccdimginfo.maskroi(1,2)+1)    ; % number of columns  (row position)
    end
    if ccdimginfo.kinetics == 1                                              % kinetic mode
        TotalSize = 4/1.048576e6 * length(dataIndex)                    ...  % size of the correlation tensor in MB
                  * (ccdimginfo.maskroi(2,1)-ccdimginfo.maskroi(1,1)+1) ...  % number of rows (column position)
                  * (ccdimginfo.lastslice - ccdimginfo.firstslice + 1)  ...  % number of good slices
                  * (ccdimginfo.maskroi(2,2)-ccdimginfo.maskroi(1,2)+1)    ; % number of columns  (row position)
    end
    if (TotalSize > ccdimginfo.memory)
        noDataSection = ceil(TotalSize/ccdimginfo.memory)                  ; % calculate the number of data sections needed
    else
        noDataSection = 1                                                  ;
    end
    clear TotalSize                                                        ;
    % --- create vector of column positions to split the data
    i            = 1 : noDataSection + 1                                   ; 
    dataSections = ccdimginfo.maskroi(1,1)                              ...
                 + (i-1) * ceil((ccdimginfo.maskroi(2,1)-ccdimginfo.maskroi(1,1)+1) / noDataSection) ;
    dataSections(end) = ccdimginfo.maskroi(2,1)                            ;
    clear i                                                                ;
    % --- assign parameters to the output structure
    ccdimginfo.noDataSection = noDataSection                               ;
    ccdimginfo.dataSections  = dataSections                                ;
    clear noDataSection dataSections                                       ;
end


% =========================================================================
% --- create output message
% =========================================================================
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
   ,'String',[msgstr;{['   The calculation is divided into '            ...
   ,num2str(ccdimginfo.noDataSection),' section(s).']}]                 ...
   ,'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;


% =========================================================================
% --- save ccdimginfo to figure
% =========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;


% ---
% EOF