function myfits_test

% =========================================================================
% --- define dummy data
% =========================================================================
x   = [1:0.05:100]                                                         ; % create some x vector
y0  = 1 + 3.0 * exp( - 0.5 * ((x-50)/8.0).^2 )                             ; % create an gaussian like y0 vector
y   = y0 + 7.5 * (0.5 - rand(size(x)))                                     ; % make the (data) y vector very noisy
err = 3.75 * ones(size(x))                                                 ; % calculate some error vector

% =========================================================================
% --- define fitting function, start parameter and [dp niter tol]
% =========================================================================
func     = @gauss                                                          ; % handle to gauss function below
free     = [1 1 1 1]                                                       ; % all parameters are free
% --- for levenberg algorithm [quiet constraint]
pinl     = [2 35 5 1]                                                      ; % start parameter for levenberg [amplitude position width background]
fcpl     = [1e-01 1e03 1e-09]                                              ; % [dp max. iteration tol]
% --- for simplex algorithm [testing the limits :-) ]
pins     = [1.0 1.0 1.0 1.0]                                               ; %  % start parameter for simplex [amplitude position width background]
fcps     = [1e+03 1e03 1e-09]                                              ; % [dp max. iteration tol]

% =========================================================================
% --- call Levenberg-Marquart algorithm 
% --- start parameters are very constraint 
% =========================================================================
% profile on; profile clear;

t1 = clock                                                                 ;
[p,sig,f,covp,corp,r2,rv] = levenbergfit(x,y,err,pinl,free,func,fcpl)      ;
t  = etime(clock,t1)                                                       ;

paramnames = {'Amplitude','Position','Width','Background'}                 ;
disp(sprintf('\rLevenberg-Marquart algorithm results'))                    ;
disp(sprintf('\rFit duration: %6.4f seconds',t))                           ;
disp(sprintf('Parameter name  Parameter value  Error of parameter value\r'));
for i = 1 : numel(p)
    disp(sprintf('%12s  %13.3f %13.3f', paramnames{i}, p(i), sig(i)))      ;
end
% --- calculate fitresults for plot
bestfitl = gauss(x,p)                                                      ;

% =========================================================================
% --- call Simplex algorithm
% --- start parameters are almost totally independent
% =========================================================================
t1 = clock                                                                 ;
[p,sig,f,v,fv,covp,corp,r2,rv] = simplexfit(x,y,err,pins,free,func,fcps)   ;
t  = etime(clock,t1)                                                       ;

paramnames = {'Amplitude','Position','Width','Background'}                 ;
disp(sprintf('\rSimplex algorithm results'))                               ;
disp(sprintf('\rFit duration: %6.4f seconds',t))                           ;
disp(sprintf('Parameter name  Parameter value  Error of parameter value\r'));
for i = 1 : numel(p)
    disp(sprintf('%12s  %13.3f %13.3f', paramnames{i}, p(i), sig(i)))      ;
end
% --- calculate fitresults for plot
bestfits = gauss(x,p)                                                      ;

% profreport('myfits_test');

% =========================================================================
% --- show demonstartion plot
% =========================================================================
delete(findall(0,'Tag','myfits_test'))                                     ;
figure('Name','Demonstration figure','Tag','myfits_test')                  ;
plot(x,y,'ko',x,y0,'k-',x,bestfitl,'b-',x,bestfits,'r-')                   ;


% =========================================================================
% --- define fitting function
% =========================================================================
function y = gauss(x,p)
% --- y = gauss(x,p) 
% --- p = [Amplitude Position Width Background]

y=p(4)+p(1)*exp(-0.5*((x-p(2))/p(3)).^2)                                   ;

% ---
% EOF
