function [dt,g2] = rawg2( IQTT, dpl, framespacing, slicedpl             ...
                        , slicespacing, avgfirst, mindelay , message )
% --- RAWG2   calculate the autocorrelation function in a multi-tau way
% ---
% --- structure of IQTT for a partition
% --- for kinetics        : [xpixel,ypixel, frame, slice]
% --- for full frame mode : [xpixel,ypixel, frame, 1]
% ---
% ---    - only IQTT is needed as a parameter
% ---    - the other parameter are optional
% ---      and the routine can use standard values
% ---
% ---
% --- Revision 2005/09/16 - include the possiblity of a display output
% --- for xpcsgui
% --- by MS


% =========================================================================
% --- get parameters from input tensor
% =========================================================================
nframes  = size(IQTT,3)                                                    ; % get the number of full frames
nslices  = size(IQTT,4)                                                    ; % get the number of slices


% =========================================================================
% --- check input parameter
% =========================================================================
if ( nargin == 8 && message == 1 )
    hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                         ; % if message display is on than look for XPCSGUI main window
    if ( isempty(hFigXPCSMain) == 0 )
        message = 0                                                        ; % set message to zero if the XPCSGUI main window is not found
    end
end
% ---
if (nargin == 7)
    message      = 0                                                       ; % if there is no message, define message == 0
end
% ---
if (nargin == 6)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
end
% ---
if (nargin == 5)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
    avgfirst     = 0                                                       ; % if there is no avgfirst defined, use standard normalization
end
% ---
if (nargin == 4)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
    avgfirst     = 0                                                       ; % if there is no avgfirst defined, use standard normalization
    slicespacing = 0.1                                                     ; % if the slicespacing is not defined set slicespacing == 0.1
end
% ---
if (nargin == 3)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
    avgfirst     = 0                                                       ; % if there is no avgfirst defined, use standard normalization
    slicespacing = 0.1                                                     ; % if the slicespacing is not defined set slicespacing == 1
    slicedpl     = 4                                                       ; % if the slicedpl is not defined set slicedpl == 4
end
% ---
if (nargin == 2)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
    avgfirst     = 0                                                       ; % if there is no avgfirst defined, use standard normalization
    slicespacing = 0.1                                                     ; % if the slicespacing is not defined set slicespacing == 1
    slicedpl     = 4                                                       ; % if the slicedpl is not defined set slicedpl == 4
    framespacing = 2 * nslices * slicespacing                              ; % if the framespacing is undefined set framespacing == 2*nslices*slicespacing
end
% ---
if (nargin == 1)
    message      = 0                                                       ; % if there is no message, define message == 0
    mindelay     = 1                                                       ; % if there is no mindelay, define mindelay == 1
    avgfirst     = 0                                                       ; % if there is no avgfirst defined, use standard normalization
    slicespacing = 0.1                                                     ; % if the slicespacing is not defined set slicespacing == 1
    slicedpl     = 4                                                       ; % if the slicedpl is not defined set slicedpl == 4
    framespacing = 2 * nslices * slicespacing                              ; % if the framespacing is undefined set framespacing == 2*nslices*slicespacing
    dpl          = 4                                                       ; % if the delays per level (dpl) is not defined use dpl == 4
end


% =========================================================================
% --- create delays and time vector
% =========================================================================
if (nslices > 1)
    slicedelays = finddelays(nslices,slicedpl,mindelay)                    ;
    nslicedelay = length(slicedelays)                                      ;
    sliceavg(:,:,:,1) = sum(IQTT(:,:,:,:),4) / nslices                     ; % create data tensor with only frame time spacing for later use
    clear j                                                                ;
else
    slicedelays = []                                                       ;
    nslicedelay = 0                                                        ;
end
% ---
framedelays = finddelays(nframes,dpl,1)                                    ;
% ---
if (nslices > 1)
    totaldelays = [slicedelays, framedelays]                               ;
else
    totaldelays = framedelays                                              ;
end
ntotaldelay = length(totaldelays)                                          ;
% ---
dt = zeros(1,length(totaldelays))                                          ; % initialize time vector
if (nslices > 1)
    dt(1:nslicedelay)     = totaldelays(1:nslicedelay)     * slicespacing  ; % for the slicedelays
    dt(nslicedelay+1:end) = totaldelays(nslicedelay+1:end) * framespacing  ; % for the framedelays
else
    dt = totaldelays * framespacing                                        ; % for the framedelays
end
% ---
clear slicedelays framedelays                                              ;


% =========================================================================
% --- initialize g2 tensor, create message string/switch
% =========================================================================
g2 = ones(size(IQTT,1),size(IQTT,2),ntotaldelay,'single')                  ; % Any g2 structure should be initialized as ones !!!

% --- create message string/switch
if (nslicedelay > 1)
    imgunits = 'slices'                                                    ;
else
    imgunits = 'frames'                                                    ;
end


% =========================================================================
% --- main loop
% =========================================================================
% warning off MATLAB:DivideByZero                                            ; % due to use of 'mean'
oldlevel   = 0                                                             ;
for k=1:ntotaldelay
    % ---
    if ( message == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),...
                     'string')                                             ;
        msgnew = sprintf('   Correlate delay no. %03i of %03i delays.'  ...
                        ,k,ntotaldelay)                                    ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    % ---
    if (strcmp(imgunits, 'slices') && (k > nslicedelay) )                    % check: if a switchover from slice averaging to frame averaging is needed
        oldlevel   = 0                                                     ;
        imgunits   = 'frames'                                              ;
        IQTT       = sliceavg                                              ; % assign the sliceavg to IQTT
        clear sliceavg                                                     ;
    end
    % ---
    % --- create delay, level & tau
    % ---
    thisdelay = totaldelays(k)                                             ;
    if strcmp(imgunits, 'slices')
        level  = levelofdelay(thisdelay, slicedpl)                         ; % find the level of the actual delay
    end
    if strcmp(imgunits, 'frames')
        level  = levelofdelay(thisdelay, dpl)                              ; % find the level of the actual delay
    end
    if (level < 2)
        tau = thisdelay                                                    ;
    else
        tau = int32(thisdelay / 2.^(level-1))                              ; % due to the ongoing data averaging the tau is not equal to the delay
    end
    % ---
    % --- slice calculation routine
    % ---
    if strcmp(imgunits, 'slices')
        % ---
        if (level > oldlevel && level > 1)                                   % check: if the new delay is on a higher level, than the data tensor can be averaged
            if ( mod(size(IQTT,4),2) == 0)                                   % check if the number of slices is even or odd
                lasteven = size(IQTT,4)                                    ; % calculate last even number
                lastodd  = lasteven -1                                     ; % calculate last usable odd  number
            else
                lasteven = size(IQTT,4) - 1                                ; % calculate last even number
                lastodd  = lasteven -1                                     ; % calculate last usable odd  number
            end
            IQTT(:,:,:,1:lasteven/2)     = ( IQTT(:,:,:,1:2:lastodd )   ...
                                           + IQTT(:,:,:,2:2:lasteven)) / 2 ; % average the data tensor
            IQTT(:,:,:,lasteven/2+1:end) = []                              ; % clear the useless part of the tensor
            clear lasteven lastodd                                         ;
        end
        % ---
        t = 1 : (size(IQTT,4)-tau)                                         ; % create time vector
        if ( avgfirst == 0 )
            g2(:,:,k) = mean(mean(IQTT(:,:,:,t) .* IQTT(:,:,:,t+tau),4),3) ; % calculate: <I(q,t) * I(q,t+tau)>_t
        else
            g2(:,:,k) = mean(mean( IQTT(:,:,:,t    ) ,4),3)             ...
                     .* mean(mean( IQTT(:,:,:,t+tau) ,4),3)                ; % calculate: <I(q,t)>_t_left * <I(q,t+tau)>_t_right
        end
        % ---
        oldlevel = level                                                   ; % redefine old level
    end
    % ---
    % --- frame calculation routine
    % ---
    if (strcmp(imgunits, 'frames'))
        % ---
        if (level > oldlevel && level > 1)                                   % check: if the new delay is on a higher level than the data tensor can be averaged
            if ( mod(size(IQTT,3),2) == 0)                                   % check if the number of frames is even or odd
                lasteven = size(IQTT,3)                                    ;
                lastodd  = lasteven -1                                     ;
            else
                lasteven = size(IQTT,3) - 1                                ;
                lastodd  = lasteven -1                                     ;
            end
            % ---
            IQTT(:,:,1:lasteven/2,1)     = ( IQTT(:,:,1:2:lastodd ,1)   ...
                                           + IQTT(:,:,2:2:lasteven,1)) / 2 ; % average the data tensor
            IQTT(:,:,lasteven/2+1:end,:) = []                              ; % clear the useless part of the tensor
            clear lasteven lastodd                                         ;
            % ---
        end
        % ---
        t = 1 : (size(IQTT,3)-tau)                                         ; % create time vector
        if ( avgfirst == 0)
            g2(:,:,k) = mean(IQTT(:,:,t,1)    .*      IQTT(:,:,t+tau,1),3) ; % calculate: <I(q,t) * I(q,t+tau)>_t
        else
            g2(:,:,k) = mean(IQTT(:,:,t,1),3) .* mean(IQTT(:,:,t+tau,1),3) ; % calculate: <I(q,t)>_t_left * <I(q,t+tau)>_t_right
        end
        % ---
        oldlevel = level                                                   ; % redefine old level
        % ---
    end
    % ---
end
% warning on MATLAB:DivideByZero                                             ; % due to use of 'mean'
% ---
clear IQTT dpl sizeMatrix framespacing avgfirst                            ;
clear imgunits k level oldlevel t tau thisdelay totaldelays mindelay       ;
clear slicedpl slicespacing ntotaldelay nframes nslices nslicedelay        ;
% ---
% =========================================================================
% --- end of main loop
% =========================================================================


% --- define some needed subfunctions for the multi-tau routine


% =========================================================================
% --- subfunction finddelays
% =========================================================================
function fDelays = finddelays(nframes,dpl,mindelay,maxdelay)
% ---
% ---
if (mindelay == 0)
    imin=1                                                                 ;
else
    imin=fix(ceil(indexofdelay(max(mindelay,0),dpl)))                      ;
end
% ---
imax = fix(indexofdelay(nframes,dpl)) - 1                                  ;
if nargin == 4
    imax = min(imax,fix(indexofdelay(maxdelay,dpl)))                       ;
end
% ---
fDelays = delayofindex(imin,imax,dpl)                                      ;


% =========================================================================
% --- subfunction indexofdelay
% =========================================================================
function iOfDelay = indexofdelay(delay,dpl)
% ---
level    = levelofdelay(delay,dpl)                                         ; % call levelofdelay
% ---
iOfDelay = zeros(1,length(delay))                                          ; % initialize iOfDelay
% ---
iOfDelay(delay<dpl)  = delay(delay<dpl) + 1                                ;
iOfDelay(delay>=dpl) = 1 + dpl * level(delay>=dpl)                      ...
             + (delay(delay>=dpl) - dpl * 2.^(level(delay>=dpl)-1))     ...
            ./ (2.^(level(delay>=dpl)-1))                                  ;


% =========================================================================
% --- subfunction levelofdelay
% =========================================================================
function lOfDelay = levelofdelay(delay,dpl)
% ---
lOfDelay = zeros(1,length(delay))                                          ; % initialize lOfDelay
% ---
lOfDelay(delay<dpl)  = 0                                                   ;
lOfDelay(delay>=dpl) = ceil(log((delay(delay>=dpl)+1)./dpl)/log(2))        ;


% =========================================================================
% --- subfunction delayofindex
% =========================================================================
function dOfIndex = delayofindex(imin,imax,dpl)
% ---
index    = imin:imax                                                       ; % create index vector
index    = index -1                                                        ; % start index vector from zero
% ---
level    = floor(index/dpl)                                                ; % initialize level vector
dOfIndex = zeros(1,length(index))                                          ; % initialize dOfIndex vector
% ---
dOfIndex(index<dpl)  = mod(index(index<dpl),dpl)                           ;
dOfIndex(index>=dpl) = 2.^(level(index>=dpl)-1)  ...
                    .* (dpl + mod(index(index>=dpl),dpl))                  ;


% =========================================================================
% --- subfunction delaymax
% =========================================================================
% function maxdelay = delaymax(nframes,dpl)
% ---
% level    = levelofdelay(nframes-1,dpl)                                     ; % call levelofdelay
% ---
% iOfLevel = 1:dpl                                                           ; % initialize index vector
% iOfLevel = (level .* dpl) + iOfLevel                                       ; % calculate indices of level
% dOfIndex = delayofindex(iOfLevel(1),iOfLevel(end),dpl)                     ; % get delays of level
% ---
% maxdelay = max(dOfIndex(dOfIndex<nframes))                                 ; % calculate maxdelay


% =========================================================================
% --- subfunction renumberframes
% =========================================================================
% function [ndata0,ndataend,ndata] = renumberframes(ndata0,ndataend,ndata)
% ---
% if (nargin < 3)
%     ndata = ndataend - ndata0 + 1                                          ;
% end
% ---
% if (length(ndata(:)) > 1)
%     ndataend      = cumsum(ndata)                                          ;
%     ndata0(1)     = 1                                                      ;
%     ndata0(2:end) = ndataend(1:length(ndataend(:))-1) + 1                  ;
% else
%     ndata0   = 1                                                           ;
%     ndataend = ndata                                                       ;
% end


% =========================================================================
% --- subfunction make_tlist
% =========================================================================
% function tList = make_tlist(startp,stopp,stepp)
% ---
% --- input:
% --- 1) vector of start points
% --- 2) vector of endpoints
% --- 3) step size (number) [optinal]
% ---
% if nargin < 3
%     stepp = 1                                                              ;
% end
% ---
% t = []                                                                     ;
% for i= 1 : length(startp(:))-1
%     k = startp(i):stepp:stopp(i)                                           ;
%     t = [t, k]                                                             ;
% end
% tList = t                                                                  ;


% ---
% EOF
