function runxpcs(varargin)
% RUNXPCS Start analyzing XPCS data. Called by xpcsgui main figure only.
%
% Zhang Jiang & Michael Sprung
% $Revision: 1.0 $Date: 2004/12/16 $
% $Revision: 1.1 $Date: 2005/09/09 $ by MS
% $Revision: 2.0 $Date: 2005/09/09 $ option to use real multi-tau
%                                    algorithms (memory saving) by MS
%


% =========================================================================
% --- Get Main figure handle & load application data if neccessary
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;

NewMethod = 0                                                              ;
if ( NewMethod == 1 )
    ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                   ;
    kinetics     = ccdimginfo.kinetics                                     ;
    clear ccdimginfo                                                       ;
end


% =========================================================================
% --- Set buttons on main figure
% =========================================================================
hPushbuttonLoad      = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonLoad')                   ;
hPushbuttonBatchinfo = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonBatchinfo')              ;
hPushbuttonAnalysis  = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonAnalysis')               ;
hPushbuttonSystem    = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonSystem')                 ;
hPushbuttonRun       = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonRun')                    ;
hPushbuttonStop      = findall(hFigXPCSMain,'Tag',                      ...
                              'xpcsmain_PushbuttonStop')                   ;
set([hPushbuttonLoad,hPushbuttonBatchinfo,hPushbuttonAnalysis,          ...
     hPushbuttonSystem,hPushbuttonRun],'Enable','off')                     ;
set( hPushbuttonStop,'enable','on')                                        ;


% =========================================================================
%                   Push analysis part below
% =========================================================================
% myprofiler                                                                 ;
td = clock                                                                 ; % get the starting time

% =========================================================================
% --- 1. map pixels and get sq,sphi,dq,dphi span
% =========================================================================
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{' ';'   Mapping each pixel of CCD image ...'}],   ...
    'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
ta = clock                                                                 ;
% =========================================================================
% myprofiler                                                                 ;
mapccdpixel                                                                ; % call of mapping function
getsdqphispan                                                              ; % call of q_phi_span function
% profreport('startroutines')                                                ;
% =========================================================================
tb = clock                                                                 ;
tc = etime(tb,ta)                                                          ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',        ...
    [msgstr;{['   Finish mapping each pixel of CCD image in ',          ...
     num2str(tc),' seconds.']}],'HorizontalAlignment','left')              ;
pause(0.001)                                                               ;
clear ta tb tc                                                             ;
% =========================================================================
% --- end of mapping pixels and getting sq,sphi,dq,dphi span
% =========================================================================


% =========================================================================
% --- 2. create q phi mask for static and dynamic analysis
% =========================================================================
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{' ';'   Creating the q phi partitions ...'}],     ...
    'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
ta = clock                                                                 ;
% =========================================================================
% myprofiler                                                                 ;
qphipartition                                                              ; % call of q_phi_partioning function
% profreport('partitioning')                                                 ;
% =========================================================================
tb = clock                                                                 ;
tc = etime(tb,ta)                                                          ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',        ...
    [msgstr;{['   Finish creating the q phi partitions in ',            ...
     num2str(tc),' seconds.']}],'HorizontalAlignment','left')              ;
pause(0.001)                                                               ;
clear ta tb tc                                                             ;
% =========================================================================
% --- end of creating q phi masks for static and dynamic analysis
% =========================================================================


% =========================================================================
% --- Start of loop of selected batches
% =========================================================================
[selectedBatches] = getbatches                                             ;
% ---
for n = 1 : length(selectedBatches) 
    
    nBatch = selectedBatches(n)                                            ;
    
    % =====================================================================
    % --- display which batch is analyzed
    % =====================================================================
    t1 = clock                                                             ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';['   Analyzing Batch No. :  ',            ...
         num2str(nBatch),' .']}],'HorizontalAlignment','left')             ;
    pause(0.001)                                                           ;
    % =====================================================================
    % --- end of displaying which batch is analyzed
    % =====================================================================

    
    % =====================================================================
    % --- 3. check memory conditions (only for old method)
    % =====================================================================
    if ( NewMethod == 0 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')          ...
            ,'String',[msgstr;{' ';'   Check memory conditions ...'}]   ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;
        % =================================================================
        memorycheck(nBatch)                                                ; % call of memory check function
        % =================================================================
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                                          'xpcsmain_EditMessage'),'string');
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')         ...
            ,'String',[msgstr;{'   Memory conditions are checked.'}]   ...
            ,'HorizontalAlignment','left')                                 ;
        pause(0.001)                                                       ;
    end
    % =====================================================================
    % --- end of checking memory conditions
    % =====================================================================


    % =====================================================================
    % --- 4. start the main analysis routine
    % =====================================================================
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';'   Start main analysis function ...'}],  ...
        'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    ta = clock                                                             ;
    % =====================================================================
%     myprofiler                                                             ;
    if ( NewMethod == 0 )
        mainanalysis(nBatch)                                               ; % call of main analysis function
%         mainanalysis_cluster(nBatch)                                       ; % call of main analysis function for cluster analysis
    elseif ( NewMethod == 1 )
        if ( kinetics == 0 )                                                 % full frame or ROI mode
            mainanalysis_fullframe(nBatch)                                 ; % call of main analysis function in fullframe mode        
        elseif ( kinetics == 1 )                                             % kinetics mode
            mainanalysis_kinetics(nBatch)                                  ; % call of main analysis function in kinetics mode
        end
    end
%     profreport('mainanalysis')                                             ;
    % =====================================================================
    tb = clock                                                             ;
    tc = etime(tb,ta)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish main analysis function in ',      ...
        num2str(tc),' seconds.']}],'HorizontalAlignment','left')           ;
    pause(0.001)                                                           ;
    clear ta tb tc                                                         ;
    % =====================================================================
    % --- end of the main analysis routine
    % =====================================================================


    % =====================================================================
    % --- 5. display some results
    % =====================================================================
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';'   Starting to display results ...'}],   ...
        'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    ta = clock                                                             ;
    % =====================================================================
%     myprofiler                                                             ;
    showresult(nBatch)                                                     ; % create some plots and png files
%     profreport('showresult') ;
    % =====================================================================
    tb = clock                                                             ;
    tc = etime(tb,ta)                                                      ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{['   Finish to show results in ',             ...
        num2str(tc),' seconds.']}],'HorizontalAlignment','left')           ;
    pause(0.001)                                                           ;
    clear ta tb tc                                                         ;
    % =====================================================================
    % --- end of displaying some results
    % =====================================================================

    
    % =====================================================================
    % --- compute the time to analyse the batch
    % =====================================================================
    t2 = etime(clock,t1)                                                   ;
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),             ...
        'String',[msgstr;{' ';['   Finish analyzing batch #',num2str(nBatch) ...
        ,' in ',num2str(t2),' seconds.']}],'HorizontalAlignment','left')   ;
    pause(0.001)                                                           ;
    clear t1 t2                                                            ;
    % =====================================================================
    % --- end of computing the time it took to analyse the batch
    % =====================================================================

    
end
clear nBatch kinetics NewMethod                                            ;
% =========================================================================
% --- End of loop of selected batches
% =========================================================================


% =========================================================================
% --- 6. create final message
% =========================================================================
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
dummy = num2str(length(selectedBatches))                                   ;
if ( dummy > 1 )
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
        ,'String',[msgstr;{' ';['   Finish analyzing '                  ...
        ,dummy,' batches.']}],'HorizontalAlignment','left')                ;
else
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')              ...
        ,'String',[msgstr;{' ';['   Finish analyzing '                  ...
        ,dummy,' batch.']}],'HorizontalAlignment','left')                  ;    
end
clear dummy selectedBatches                                                ;
pause(0.001)                                                               ;
% =========================================================================
te = clock                                                                 ;
tf = etime(te,td)                                                          ;
msgstr = get(findall(hFigXPCSMain,'Tag',                                ...
                                  'xpcsmain_EditMessage'),'string')        ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
    'String',[msgstr;{['   The analysis took ',                         ...
    num2str(tf),' seconds.']}],                                         ...
    'HorizontalAlignment','left')                                          ;
pause(0.001)                                                               ;
clear td te tf                                                             ;
% =========================================================================
% --- end of creating final message
% =========================================================================


% =========================================================================
% --- set pushbutton status back
% =========================================================================
set([hPushbuttonLoad,hPushbuttonBatchinfo,hPushbuttonAnalysis,          ...
     hPushbuttonSystem,hPushbuttonRun],'Enable','on')                      ;
set( hPushbuttonStop,'enable','off')                                       ;
% =========================================================================
% --- end of setting pushbutton status back
% =========================================================================


% =========================================================================
% profreport('runxpcs')                                                      ;
% =========================================================================


% ---
% EOF
