function showimage(varargin)
% SHOWIMAGE Function called by pushbutton of figure by viewinfo.m to
%   display average of images after correction to some batchinfo
%   parameters.
%
% Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/12/19 $
% $Revision: 1.1 $  $Date: 2006/08/23 $ MS fixed offset issue of specular
%                                       line


% =========================================================================
% --- get ccdimginfo
% =========================================================================
hFigXPCSMain     = findall(0,'Tag','xpcsmain_Fig')                         ;
hFigViewinfo     = findall(0,'Tag','viewinfo_Fig')                         ;
hFigShowImage    = findall(0,'Tag','showimage_fig1')                       ;   % display image
hFigShowImageInt = findall(0,'Tag','showimage_fig2')                       ;   % display intensity
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo');
else
    return;
end


% =========================================================================
% --- create figures if image figure or intensity figure does not exist
% =========================================================================
figureSize = [640 480];
if isempty(hFigShowImage)
    hFigShowImage = figure('BackingStore','off','Units','pixels',       ...
                           'DockControls','off',                        ...
                           'PaperOrient','portrait',                    ...
                           'IntegerHandle','off',                       ...
                           'NumberTitle','off','MenuBar','none',        ...
                           'Name','XPCS','color',[1 1 0.85],            ...
                           'Toolbar','none',                            ...
                           'WindowButtonMotionFcn',@imagefigure_WindowButtonMotionFcn,...
                           'Position',[300,225,figureSize],             ...
                           'Tag','showimage_fig1',                      ...
                           'UserData',[])                                  ;
    hToolbar = uitoolbar(hFigShowImage,'Tag','showimage_fig1_Toolbar')     ;
    iconToolbarZoom = load('zoom.mat')                                     ;
    hToolbarZoom = uitoggletool(hToolbar,                               ...
                                'CDATA',iconToolbarZoom.zoomCData,      ...
                                'Separator','on',                       ...
                                'TooltipString','Zoom',                 ...
                                'State','off',                          ...
                                'ClickedCallback',@toolbarZoomFcn,      ...
                                'Tag','toolbarZoom')                       ;
    iconToolbarPan = load('pan.mat')                                       ;
    hToolbarPan = uitoggletool(hToolbar,                                ...
                               'CDATA',iconToolbarPan.cdata,            ...
                               'TooltipString','Pan',                   ...
                               'ClickedCallback',@toolbarPanFcn,        ...
                               'Tag','toolbarPan')                         ;
    iconToolbarColormapeditor = load('colormapeditor.mat')                 ;
    hToolbarColormapeditor = uipushtool(hToolbar,                                ...
                                        'CDATA',iconToolbarColormapeditor.cdata, ...
                                        'TooltipString','Edit Colormap',         ...
                                        'ClickedCallback','colormapeditor;',     ...
                                        'Tag','toolbarColormapeditor')              ;
else
    figure(hFigShowImage)                                                  ;
end


% =========================================================================
% --- reset toolbar controls of image figure
% =========================================================================
resetToolbar(hFigShowImage)                                                ;


% =========================================================================
% --- if there is any change to the selected frames to display, 
% --- reset figure name and reload frames
% =========================================================================
if ~isempty(hFigViewinfo)
    selectedBatch = get(findall(hFigViewinfo,                           ...
                        'Tag','figviewinfo_PopupmenuBatchnumber'),'value') ;
else
    selectedBatch  = ccdimginfo.batchestodo(1)                             ;
end

dataIndex = ccdimginfo.ndata0todo(selectedBatch):ccdimginfo.ndataendtodo(selectedBatch);
darkIndex = ccdimginfo.ndark0todo(selectedBatch):ccdimginfo.ndarkendtodo(selectedBatch);

if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
    dataIndexToDisplay = dataIndex(1:min(5,length(dataIndex)))             ; % 5 data images
    darkIndexToDisplay = darkIndex(1:min(5,length(darkIndex)))             ; % 5 dark images
else
    dataIndexToDisplay = dataIndex(1:min(50,length(dataIndex)))            ; % 50 data images for SMD camera
    darkIndexToDisplay = darkIndex(1:min(50,length(darkIndex)))            ; % 50 dark images for SMD camera    
end

figureName = ['XPCS - Average Image of Frame '                          ...
             ,num2str(dataIndexToDisplay(1)),':'                        ...
             ,num2str(dataIndexToDisplay(end))]                            ;
         
if ( ~strcmp(get(hFigShowImage,'Name'),figureName) )
    set(hFigShowImage,'Name',figureName)                                   ;
    delete(findall(hFigShowImage,'Tag','showimage_slicelabel'))            ;
    delete(findall(hFigShowImage,'Tag','showimag_sliceseparationline'))    ;
    
    % --- try to find starting positions in compressedmulti files 
    if ( ccdimginfo.compressedmulti(selectedBatch) == 1 )
        if ( isempty(ccdimginfo.SBCM{selectedBatch}) == 1 )
            SByte = indexcompressedmultiimm(ccdimginfo.imagefile{selectedBatch}) ; % try to get starting byte position of indices
        else
            SByte = ccdimginfo.SBCM{selectedBatch}                         ; % had already been created
        end
    else
       SByte = []                                                          ; 
    end

    % --- get data images from image file
    sumData = 0                                                            ;
    for iDataIndexToDisplay = 1:length(dataIndexToDisplay)
        if ( ~isempty(SByte) == 1 )
            f = openfile(ccdimginfo.imagefile{selectedBatch}            ...
                        ,dataIndexToDisplay(iDataIndexToDisplay)        ...
                        ,SByte(dataIndexToDisplay(iDataIndexToDisplay)  ...
                              -ccdimginfo.ndata0(selectedBatch)+1))        ; % load image file with pointer           
        else
            f = openfile(ccdimginfo.imagefile{selectedBatch}            ...
                        ,dataIndexToDisplay(iDataIndexToDisplay))          ;
        end
        sumData = single(f.imm) + sumData                                  ;
    end
    % --- get dark images from image file
    sumDark = zeros(size(sumData))                                         ;
    for iDarkIndexToDisplay = 1:length(darkIndexToDisplay)
        if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
            f = openfile(ccdimginfo.imagefile{selectedBatch}            ...
                        ,darkIndexToDisplay(iDarkIndexToDisplay))          ;
        else
            f = openfile(ccdimginfo.darkfile{selectedBatch}             ...
                       ,darkIndexToDisplay(iDarkIndexToDisplay))           ;
        end
        sumDark = single(f.imm) + sumDark                                  ;
    end
       
    % --- calculate true data image by subtracting dark;
    % --- set negative values to zero 
    if ( length(darkIndexToDisplay) > 0 )
        trueData = sumData/length(dataIndexToDisplay)                   ...
                 - sumDark/length(darkIndexToDisplay)                      ;
    else
        trueData = sumData/length(dataIndexToDisplay)                      ;
    end
    trueData( trueData <= 0 ) = 0                                          ;
    
    % --- save trueData to ccdimginfo.testimg for defining mask and
    % --- displaying mask and partition later
    if ( ~isempty(SByte) == 1 )
        ccdimginfo.SBCM{selectedBatch} = SByte                             ;
    end
    ccdimginfo.testimg = trueData                                          ;        
    setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                       ;
    
    % --- activate figure and plot average of images
    figure(hFigShowImage)                                                  ;
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        hImm = imagesc(trueData,[0.9*min(trueData(:))                   ...
                                 1.1*(min(min(trueData(:))+1500 ,max(trueData(:))+1))]) ;
    else
        hImm = imagesc(trueData,[0.9*min(trueData(:))                   ...
                                 1.1*(min(min(trueData(:))+ 100 ,max(trueData(:))+1))]) ;
    end
    if (   ccdimginfo.geometry == 0 &&                                  ...
           max(size(trueData,1),size(trueData,2))                       ...
         / min(size(trueData,1),size(trueData,2)) <= 4/3 )
        axis image                                                         ;
    end
    set(hImm,'Tag','showimage_image')                                      ;
    uicontrol('Parent',hFigShowImage,'Style','Text',                    ...
                                     'BackgroundColor',[1 1 0.85],      ...
                                     'String','',                       ...
                                     'units','normalized',              ...
                                     'HorizontalAlignment','left',      ...
                                     'position',[0.05 0.01 0.5 0.05],   ...
                                     'Tag','mouseposition')                ;
    set(get(hImm,'Parent'),'YDir','normal')                                ;
    clear sumData sumDark trueData f figureName hImm                       ;
else
    figure(hFigShowImage)                                                  ;
end


% =========================================================================
% --- for kinetics mode, calculate offset & shifted pixels to plot
% --- correction line and total # of slices
% =========================================================================
if ccdimginfo.kinetics == 1
    shiftOffset = (ccdimginfo.slicetop-ccdimginfo.row_end)                 ;
    shiftedPixels = ccdimginfo.slicetop - shiftOffset                   ...
        - ccdimginfo.lastslice * ccdimginfo.kinwinsize                     ;
    totalSlice = floor(ccdimginfo.row_end/ccdimginfo.kinwinsize)           ;
end

% --- draw correction line for reflection geometry
if ccdimginfo.geometry == 1

    % =====================================================================
    % --- check if ccdimginfo.dpix (pixel size) is a scalar or a vector 
    % --- & assign values to dpix_x & dpix_y accordingly
    % =====================================================================
    if ( numel(ccdimginfo.dpix) > 1 )                                        % pixel size is different in x & y direction
        dpix_x = ccdimginfo.dpix(1)                                        ;
        dpix_y = ccdimginfo.dpix(2)                                        ;
    else                                                                     % pixel size is identical in x & y direction
        dpix_x = ccdimginfo.dpix                                           ;
        dpix_y = dpix_x                                                    ;
    end
    % =====================================================================
    % --- coordinates of DB & RB in pixel in respect to the position of the 
    % --- CCD during the measurement
    % =====================================================================
    xDBpix = ccdimginfo.x0    + ccdimginfo.ccdxsense * (ccdimginfo.ccdx - ccdimginfo.ccdx0)    / dpix_x ;
    yDBpix = ccdimginfo.y0    + ccdimginfo.ccdzsense * (ccdimginfo.ccdz - ccdimginfo.ccdz0)    / dpix_y ;
    xRBpix = ccdimginfo.xspec + ccdimginfo.ccdxsense * (ccdimginfo.ccdx - ccdimginfo.ccdxspec) / dpix_x ;
    yRBpix = ccdimginfo.yspec + ccdimginfo.ccdzsense * (ccdimginfo.ccdz - ccdimginfo.ccdzspec) / dpix_y ;
    % ---
    xDB2RB = (xRBpix - xDBpix) * dpix_x                                    ;
    yDB2RB = (yRBpix - yDBpix) * dpix_y                                    ;
    % =====================================================================
    % --- calculate the tilt angle from specular to beam0
    % =====================================================================
    if ( xDB2RB ~= 0 )
        tilt = atan( yDB2RB / xDB2RB )                                     ;
    else
        tilt = sign(yDB2RB) * pi / 2                                       ;
    end
    % =====================================================================
    % --- generate points on the specular streak at the image limits
    % =====================================================================
    xleft  = ccdimginfo.col_beg                                            ;
    yleft  = (xleft -xDBpix)*tan(tilt)+yDBpix                              ;
    xright = ccdimginfo.col_end                                            ;
    yright = (xright-xDBpix)*tan(tilt)+yDBpix                              ;
    % =====================================================================
    % --- shift yRBpix, xRBpix, yDBpix,xDBpix,xleft,yleft,xright,yright
    % --- if only part of of CCD is used to store images (ROI mode)
    % =====================================================================
    yRBpix = yRBpix - (ccdimginfo.row_beg-1)                               ;
    xRBpix = xRBpix - (ccdimginfo.col_beg-1)                               ;
    yDBpix = yDBpix - (ccdimginfo.row_beg-1)                               ;
    xDBpix = xDBpix - (ccdimginfo.col_beg-1)                               ;
    yleft  = yleft  - (ccdimginfo.row_beg-1)                               ;
    xleft  = xleft  - (ccdimginfo.col_beg-1)                               ;
    yright = yright - (ccdimginfo.row_beg-1)                               ;
    xright = xright - (ccdimginfo.col_beg-1)                               ;
    % =====================================================================
    % --- For kinetics mode, shift yRBpix, yDBpix, yleft
    % --- according to the last selected slice
    % =====================================================================
    if ( ccdimginfo.kinetics == 1 )
        yRBpix = yRBpix - shiftedPixels                                    ;
        yDBpix = yDBpix - shiftedPixels                                    ;
        yleft  = yleft  - shiftedPixels                                    ;
        yright = yright - shiftedPixels                                    ;
    end
    % =====================================================================
    % --- draw the correction lines for reflection geometry
    % =====================================================================
    delete(findall(hFigShowImage,'tag','showimage_line'))                  ;
    % ---
    xdata = [xleft,xright,xRBpix,xDBpix]                                   ;
    ydata = [yleft,yright,yRBpix,yDBpix]                                   ;
    [xdata,xindex] = sort(xdata)                                           ;
    ydata = ydata(xindex)                                                  ;
    % ---
    line('Parent',get(findall(hFigShowImage,'tag','showimage_image'),'Parent') ...
        ,'XData',xdata,'YData',ydata                                    ...
        ,'LineStyle','-','Color','r','Tag','showimage_line')               ;
end


% =========================================================================
% For kinetics mode: 
% 1. label all the slices and draw slice separation lines
% 2. create 2nd figure to show the average intensity of columns
% =========================================================================
if ccdimginfo.kinetics == 1
    % --- label all the slices and draw slice separation lines
    delete(findall(hFigShowImage,'Tag','showimage_slicelabel'));
    delete(findall(hFigShowImage,'Tag','showimag_sliceseparationline'));
    for iSlice = 1:totalSlice
        text('Parent',get(findall(hFigShowImage,'tag','showimage_image'),'Parent'),...
            'Position',[50 ccdimginfo.kinwinsize*(iSlice-0.5)+shiftOffset],...
            'String',num2str(iSlice),...
            'FontName','FixedWidth',...
            'FontWeight','normal',...
            'HorizontalAlignment','center',...
            'color',[1 1 1],...
            'Tag','showimage_slicelabel');
        line('Parent',get(findall(hFigShowImage,'tag','showimage_image'),'Parent'),...
            'XData',[0.5,ccdimginfo.col_end-ccdimginfo.col_beg+0.5],...
            'YData',[ccdimginfo.kinwinsize*iSlice+0.5+shiftOffset,ccdimginfo.kinwinsize*iSlice+0.5+shiftOffset],...
            'Tag','showimag_sliceseparationline',...
            'color','g');
    end
    % --- create figure if not exist, else refresh it
    if isempty(hFigShowImageInt)
        hFigShowImageInt = figure(...
        'BackingStore','off',...
        'Units','pixels',...
        'DockControls','off',...
        'PaperOrient','portrait',...
        'IntegerHandle','off',...
        'NumberTitle','off',...
        'MenuBar','none',...
        'Name','XPCS',...
        'color',[1 1 0.85],...
        'Toolbar','none',...
        'Position',[350,250,figureSize],...
        'Tag','showimage_fig2',...
        'UserData',[]);
    hAxes = axes('Parent',hFigShowImageInt,...
        'Units','normalized',...
        'Position',[0.13 0.11 0.775 0.815],...
        'Box','on',...
        'Tag','showimage_fig2_axes');
    set(hAxes,'XLim',[1 ccdimginfo.row_end-ccdimginfo.row_beg+1]);
    set(get(hAxes,'XLabel'),'String','Row Pixel Number');
    set(get(hAxes,'YLabel'),'String','ADU');
    hToolbar = uitoolbar(hFigShowImageInt,'Tag','showimage_fig2_Toolbar');
    iconToolbarZoom = load('zoom.mat');
    hToolbarZoom = uitoggletool(hToolbar,...
        'CDATA',iconToolbarZoom.zoomCData,...
        'Separator','on',...
        'TooltipString','Zoom',...
        'State','off',...
        'ClickedCallback',@toolbarZoomFcn,...
        'Tag','toolbarZoom');
    iconToolbarPan = load('pan.mat');
    hToolbarPan = uitoggletool(hToolbar,...
        'CDATA',iconToolbarPan.cdata,...
        'TooltipString','Pan',...
        'ClickedCallback',@toolbarPanFcn,...
        'Tag','toolbarPan');
    else
        figure(hFigShowImageInt);
        resetToolbar(hFigShowImageInt);
    end
    % --- calculate average intensity of colums and plot
    imageIntensity = get(findall(findall(0,'Tag','showimage_fig1'),'type','image'),'cdata');
    avgInt = sum(imageIntensity,2)/(ccdimginfo.col_end - ccdimginfo.col_beg + 1);
    delete(findall(hFigShowImageInt,'Tag','showimage_fig2_line'));  
    hLine = line('Parent',findall(hFigShowImageInt,'Tag','showimage_fig2_axes'),...
        'XData',[1:length(avgInt)],...
        'YData',avgInt,...
        'Tag','showimage_fig2_line');
    set(hFigShowImageInt,'Name',['XPCS - Average Intensity in Column of Frame ',num2str(dataIndexToDisplay(1)),':',num2str(dataIndexToDisplay(end))]);
    % --- label slice number 
    delete(findall(hFigShowImageInt,'Tag','showimage_fig2_slicelabel'));
    for iSlice = 1:totalSlice
        if iSlice < ccdimginfo.firstslice | iSlice > ccdimginfo.lastslice
            text('Parent',findall(hFigShowImageInt,'Tag','showimage_fig2_axes'),...
                'Position',[ccdimginfo.kinwinsize*(iSlice-0.5)+shiftOffset max(avgInt)/10],...
                'String',num2str(iSlice),...
                'FontName','FixedWidth',...
                'FontWeight','normal',...
                'HorizontalAlignment','center',...
                'color','r',...
                'BackgroundColor',[1.00 1.00 1.00],...
                'Tag','showimage_fig2_slicelabel');
        else
            text('Parent',findall(hFigShowImageInt,'Tag','showimage_fig2_axes'),...
                'Position',[ccdimginfo.kinwinsize*(iSlice-0.5)+shiftOffset max(avgInt)/10],...
                'String',num2str(iSlice),...
                'FontName','FixedWidth',...
                'FontWeight','normal',...
                'HorizontalAlignment','center',...
                'color','g',...
                'BackgroundColor',[1.00 1.00 1.00],...
                'Tag','showimage_fig2_slicelabel');
        end
    end
end


%==========================================================================
% --- show image figure in the front
%==========================================================================
figure(hFigShowImage)                                                      ;


%==========================================================================
% --- tracking mouse position
%==========================================================================
function imagefigure_WindowButtonMotionFcn(hObject,eventdata)
set(gcf,'selected','on')                                                   ;
pointPosition = get(gca,'CurrentPoint')                                    ;
XLim=get(gca,'XLim')                                                       ;
YLim=get(gca,'YLim')                                                       ;
XLimFlag=(pointPosition(1,1)>=XLim(1) & pointPosition(1,1)<=XLim(2))       ;
YLimFlag=(pointPosition(1,2)>=YLim(1) & pointPosition(1,2)<=YLim(2))       ;
cdata = get(findall(gcf,'type','image'),'cdata')                           ;
xpos = round(pointPosition(1,1))                                           ;
ypos = round(pointPosition(1,2))                                           ;
if (  xpos >= 1 && xpos <= size(cdata,2)                                ...
   && ypos >= 1 && ypos <= size(cdata,1)                                ...
   && XLimFlag == 1 && YLimFlag ==1 )
    set(gcf,'Pointer','crosshair')                                         ;
    set(findall(gcf,'Tag','mouseposition'),'string',['x=',num2str(xpos) ...
       ,', y=',num2str(ypos),', value=',num2str(cdata(ypos,xpos))])        ;
else
    set(gcf,'Pointer','arrow')                                             ;
end
clear XLim YLim XLimFlag YLimFlag                                          ;
clear pointPosition xpos ypos cdata                                        ;


%==========================================================================
% --- toolbar zoom callback
%==========================================================================
function toolbarZoomFcn(hObject,eventdata)
hFig = gcbf                                                                ;
hToolbarZoom = findall(hFig,'Tag','toolbarZoom')                           ;
hToolbarPan = findall(hFig,'Tag','toolbarPan')                             ;
zoom                                                                       ;
pan off                                                                    ;
set(hToolbarPan,'state','off')                                             ;


%==========================================================================
% --- toolbar pan callback
%==========================================================================
function toolbarPanFcn(hObject,eventdata)
hFig = gcbf                                                                ;
hToolbarZoom = findall(hFig,'Tag','toolbarZoom')                           ;
hToolbarPan = findall(hFig,'Tag','toolbarPan')                             ;
zoom off                                                                   ;
pan                                                                        ;
set(hToolbarZoom,'state','off')                                            ;


%==========================================================================
% --- reset toolbars
%==========================================================================
function resetToolbar(varargin)
hFigShowImage = varargin{1}                                                ;
hToolbarZoom = findall(hFigShowImage,'Tag','toolbarZoom')                  ;
hToolbarPan = findall(hFigShowImage,'Tag','toolbarPan')                    ;
set(hToolbarZoom,'state','off')                                            ;
set(hToolbarPan,'state','off')                                             ;
zoom(hFigShowImage,'off')                                                  ;
pan(hFigShowImage,'off')                                                   ;


% ---
% EOF
