function showmaskpartition(varargin)
% --- SHOWMASKPARTITION Called by viewanalysis gui only to show user defined
% --- mask and  partition.
% ---
% --- Zhang Jiang & Michael Sprung
% --- $Revision: 1.0 $  $Date: 2005/01/12 $
% --- $Revision: 2.0 $  $Date: 2006/09/29 $
% ---


% =========================================================================
% ---get application data
% =========================================================================
hFigXPCSMain     = findall(0,'Tag','xpcsmain_Fig')                         ;
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
ccdimginfo       = getappdata(hFigXPCSMain,'ccdimginfo')                   ;


% =========================================================================
% --- load image if ccdimginfo.testimg does not exist
% =========================================================================
if ( ~isfield(ccdimginfo,'testimg') )
    
    % --- determine the data and dark frames to be loaded
    displayBatchIndex = ccdimginfo.batchestodo(1)                          ;
    dataIndex = ccdimginfo.ndata0todo(displayBatchIndex)                ...       
              : ccdimginfo.ndataendtodo(displayBatchIndex)                 ;
    darkIndex = ccdimginfo.ndark0todo(displayBatchIndex)                ...
             : ccdimginfo.ndarkendtodo(displayBatchIndex)                  ;
    % ---     
    if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
        dataIndexToDisplay = dataIndex(1:min(5,length(dataIndex)))         ; % normally 5 data images
        darkIndexToDisplay = darkIndex(1:min(5,length(darkIndex)))         ; % normally 5 dark images
    else
        dataIndexToDisplay = dataIndex(1:min(50,length(dataIndex)))        ; % 50 data images for SMD camera
        darkIndexToDisplay = darkIndex(1:min(50,length(darkIndex)))        ; % 50 dark images for SMD camera    
    end
    
    % --- try to find starting positions in compressedmulti files 
    if ( ccdimginfo.compressedmulti(displayBatchIndex) == 1 )
        if ( isempty(ccdimginfo.SBCM{displayBatchIndex}) == 1 )
            SByte = indexcompressedmultiimm(ccdimginfo.imagefile{displayBatchIndex}) ; % try to get starting byte position of indices
        else
            SByte = ccdimginfo.SBCM{displayBatchIndex}                     ; % had already been created
        end
    else
       SByte = []                                                          ; 
    end

    % --- get data images from image file
    sumData = 0                                                            ;
    for iDataIndexToDisplay = 1:length(dataIndexToDisplay)
        if ( ~isempty(SByte) == 1 )
            f = openfile(ccdimginfo.imagefile{displayBatchIndex}        ...
                        ,dataIndexToDisplay(iDataIndexToDisplay)        ...
                        ,SByte(dataIndexToDisplay(iDataIndexToDisplay)  ...
                              -ccdimginfo.ndata0(displayBatchIndex)+1))   ;
        else
            f = openfile(ccdimginfo.imagefile{displayBatchIndex}        ...
                        ,dataIndexToDisplay(iDataIndexToDisplay))          ;            
        end
        sumData = single(f.imm) + sumData                                  ;
    end
    
    % --- get dark images from image file
    sumDark = 0                                                            ;
    for iDarkIndexToDisplay = 1:length(darkIndexToDisplay)
        if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
            f = openfile(ccdimginfo.imagefile{displayBatchIndex},        ...
                        darkIndexToDisplay(iDarkIndexToDisplay))           ;
        else
            f = openfile(ccdimginfo.darkfile{displayBatchIndex},         ...
                        darkIndexToDisplay(iDarkIndexToDisplay))           ;
        end
        sumDark = single(f.imm) + sumDark                                  ;
    end
    
    % --- calculate true data image by subtracting dark & applying lld
    if ( length(darkIndexToDisplay) > 0 )
        trueData = sumData/length(dataIndexToDisplay)                   ...
                 - sumDark/length(darkIndexToDisplay)                      ;
    else
        trueData = sumData/length(dataIndexToDisplay)                      ;
    end
    trueData( trueData <= abs(ccdimginfo.lld) ) = 0                        ;
    
    % --- save trueData to testimg
    if ( ~isempty(SByte) == 1 )
        ccdimginfo.SBCM{displayBatchIndex} = SByte                         ;
    end
    ccdimginfo.testimg = trueData                                          ;        
    setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                       ;
    clear sumData sumDark trueData f displayBatchIndex dataIndex darkIndex ;
    clear dataIndexToDisplay darkIndexToDisplay                            ;
    
end


% =========================================================================
% --- create figure layout for 'showmaskpartition_Fig'
% =========================================================================
delete(findall(0,'Tag','showmaskpartition_Fig'))                           ;
figureSize = [640 480]                                                     ;
% ---
hFigShowmaskpartition = figure('BackingStore','on','Units','pixels'     ...
    ,'DockControls','off','PaperOrient','portrait','IntegerHandle','off'...
    ,'NumberTitle','off','MenuBar','none'                               ...
    ,'Name','XPCS - Image with Mask & Partition'                        ...
    ,'WindowButtonMotionFcn',@showmaskpartition_WindowButtonMotionFcn   ...
    ,'color',[1 1 0.85],'Toolbar','none','Position',[300,225,figureSize]...
    ,'Tag','showmaskpartition_Fig','UserData',[])                          ;
% ---
hToolbar = uitoolbar(hFigShowmaskpartition,'Tag',...
                    'showmaskpartition_Toolbar')                           ;
iconToolbarZoom = load('zoom.mat')                                         ;
hToolbarZoom = uitoggletool(hToolbar,'CDATA',iconToolbarZoom.zoomCData  ...
                           ,'Separator','on','TooltipString','Zoom'     ...
                           ,'State','off','Tag','toolbarZoom'           ...
                           ,'ClickedCallback',@toolbarZoomFcn)             ;
iconToolbarPan = load('pan.mat')                                           ;
hToolbarPan = uitoggletool(hToolbar,'CDATA',iconToolbarPan.cdata        ...
                          ,'TooltipString','Pan','Tag','toolbarPan'     ...
                          ,'ClickedCallback',@toolbarPanFcn)               ;
iconToolbarColormapeditor = load('colormapeditor.mat')                     ;
hToolbarColormapeditor = uipushtool(hToolbar                            ...
    ,'CDATA',iconToolbarColormapeditor.cdata                            ...
    ,'TooltipString','Edit Colormap','ClickedCallback','colormapeditor;'...
    ,'Tag','toolbarColormapeditor')                                        ;


% =========================================================================
% --- reset toolbar controls of image figure
% =========================================================================
resetToolbar(hFigShowmaskpartition)                                        ;


% =========================================================================
% --- define some abbreviations
% =========================================================================
R1 = ccdimginfo.maskroi(1,2)                                               ; % row    start of ROI
R2 = ccdimginfo.maskroi(2,2)                                               ; % row    end   of ROI
C1 = ccdimginfo.maskroi(1,1)                                               ; % column start of ROI
C2 = ccdimginfo.maskroi(2,1)                                               ; % column end   of ROI


% =========================================================================
% --- create displayImage
% =========================================================================
displayImage = zeros(size(ccdimginfo.usermask))                            ;
for j = 1 : (ccdimginfo.lastslice - ccdimginfo.firstslice + 1)               % loop over good slices only
    if ( ccdimginfo.kinetics == 0 )
        sliceStart = R1                                                    ; % in full frame / roi mode use this as slice start
        sliceEnd   = R2                                                    ; % in full frame / roi mode use this as slice end
    else
        k          = (j-1) + ccdimginfo.firstslice                         ; % shift the counter in a way that ccdimginfo.firstslice == 1
        sliceStart = ccdimginfo.sliceinfo(k,1) + R1 - 1                    ; % get the sliceStart position for each slice
        sliceEnd   = ccdimginfo.sliceinfo(k,1) + R2 - 1                    ; % get the sliceEnd   position for each slice
    end
    % ---
    displayImage(R1:R2,C1:C2) = displayImage(R1:R2,C1:C2)               ...
                              + ccdimginfo.testimg(sliceStart:sliceEnd,C1:C2) ; % sum ROI of all good slices
end


% =========================================================================
% --- apply mask & upper level discrimination to displayImage
% =========================================================================
displayImage = displayImage .* ccdimginfo.usermask                         ;
if ( ccdimginfo.detector ~= 5 && ccdimginfo.detector ~= 6 )
    displayImage(displayImage > 1.1*(min(min(displayImage(:))+1500,max(displayImage(:))+1))) = ...
                                1.1*(min(min(displayImage(:))+1500,max(displayImage(:))+1))       ;
else
    displayImage(displayImage > 1.1*(min(min(displayImage(:))+ 100,max(displayImage(:))+1))) = ...
                                1.1*(min(min(displayImage(:))+ 100,max(displayImage(:))+1))       ;
end


% =========================================================================
% --- calculte binary mask of pixels on a partition border 
% =========================================================================
partitionmask = zeros(size(displayImage(R1:R2,C1:C2),1)                 ...
                     ,size(displayImage(R1:R2,C1:C2),2),2)                 ;
if ( isfield(ccdimginfo,'spartitionmaskindex') == 1 )
    dummy1 = zeros(size(displayImage(R1:R2,C1:C2)))                        ;
    dummy1(ccdimginfo.spartitionmaskindex) = 1                             ;
    partitionmask(:,:,1) = dummy1                                          ;
end
if ( isfield(ccdimginfo,'dpartitionmaskindex') == 1 )
    dummy2 = zeros(size(displayImage(R1:R2,C1:C2)))                        ;
    dummy2(ccdimginfo.dpartitionmaskindex) = 1                             ;
    partitionmask(:,:,2) = dummy2                                          ;
end
% ---
displayImageMask = imoverlaymask(displayImage(R1:R2,C1:C2)              ...
                                ,partitionmask,[1 0 0; 0 1 0])             ;
clear dummy1 dummy2 partitionmask                                          ;

% --- activate figure and plot average of images
CONTOURon = 0                                                              ;
if ( ccdimginfo.kinetics == 1 && (R2-R1+1) * (C2-C1+1) <= 200000 )
    CONTOURon = 1                                                          ;
end
figure(hFigShowmaskpartition)                                              ;
if ( CONTOURon == 1 )
    hImm = imagesc(displayImage(R1:R2,C1:C2))                              ; 
else
    hImm = image(displayImageMask)                                         ;    
end
% ---
if (   ccdimginfo.geometry == 0 &&                                      ...
       max(size(displayImage,1),size(displayImage,2))                   ...
     / min(size(displayImage,1),size(displayImage,2)) <= 4/3 )
    axis image                                                             ;
end

% ----
set(hImm,'Tag','showmaskpartition_fig_image')                              ;
uicontrol('Parent',hFigShowmaskpartition,'Style','Text'                 ...
         ,'BackgroundColor',[1 1 0.85],'units','normalized'             ...
         ,'String',' ','HorizontalAlignment','left'                     ...
         ,'position',[0.05 0.01 0.9 0.05],'Tag','mouseposition')           ;
set(get(hImm,'Parent'),'YDir','normal')                                    ;

% --- draw contour lines
if ( CONTOURon == 1 )
    hold on                                                                ;
    contour(ccdimginfo.qmap(R1:R2,C1:C2)  ,ccdimginfo.sqspan,  'r-')       ;
    contour(ccdimginfo.phimap(R1:R2,C1:C2),ccdimginfo.sphispan,'r-')       ;
    if ccdimginfo.analysistype == 1
        contour(ccdimginfo.qmap(R1:R2,C1:C2)  ,ccdimginfo.dqspan,  'g-')   ;
        contour(ccdimginfo.phimap(R1:R2,C1:C2),ccdimginfo.dphispan,'g-')   ;
    end
    hold off                                                               ;
end


%==========================================================================
% --- toolbar zoom callback
%==========================================================================
function toolbarZoomFcn(hObject,eventdata)
hFig         = gcbf                                                        ;
hToolbarZoom = findall(hFig,'Tag','toolbarZoom')                           ;
hToolbarPan  = findall(hFig,'Tag','toolbarPan')                            ;
pan off                                                                    ;
set(hToolbarPan,'state','off')                                             ;
zoom                                                                       ;


%==========================================================================
% --- toolbar pan callback
%==========================================================================
function toolbarPanFcn(hObject,eventdata)
hFig         = gcbf                                                        ;
hToolbarZoom = findall(hFig,'Tag','toolbarZoom')                           ;
hToolbarPan  = findall(hFig,'Tag','toolbarPan')                            ;
zoom off                                                                   ;
set(hToolbarZoom,'state','off')                                            ;
pan                                                                        ;


%==========================================================================
% --- reset toolbars
%==========================================================================
function resetToolbar(varargin)
hFigShowImage = varargin{1}                                                ;
hToolbarZoom  = findall(hFigShowImage,'Tag','toolbarZoom')                 ;
hToolbarPan   = findall(hFigShowImage,'Tag','toolbarPan')                  ;
set(hToolbarZoom,'state','off')                                            ;
set(hToolbarPan,'state','off')                                             ;
zoom(hFigShowImage,'off')                                                  ;
pan(hFigShowImage,'off')                                                   ;


%==========================================================================
% --- tracking mouse position
%==========================================================================
function showmaskpartition_WindowButtonMotionFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                         ;
set(gcf,'selected','on')                                                   ;
pointPosition = get(gca,'CurrentPoint')                                    ;
XLim=get(gca,'XLim')                                                       ;
YLim=get(gca,'YLim')                                                       ;
XLimFlag=(pointPosition(1,1)>=XLim(1) & pointPosition(1,1)<=XLim(2))       ;
YLimFlag=(pointPosition(1,2)>=YLim(1) & pointPosition(1,2)<=YLim(2))       ;
cdata = get(findall(gcf,'type','image'),'cdata')                           ;
xpos = round(pointPosition(1,1))                                           ;
ypos = round(pointPosition(1,2))                                           ;
if (  xpos >= 1 && xpos <= size(cdata,2) ...
   && ypos >= 1 && ypos <= size(cdata,1) && XLimFlag == 1 && YLimFlag ==1 )
    set(gcf,'Pointer','crosshair')                                         ;
    if ccdimginfo.geometry == 0
        display_str = ['x=',num2str(xpos),                              ...
                     ', y=',num2str(ypos),                              ...
                     ', value=',num2str(cdata(ypos,xpos)),              ...
                     ', q=',num2str(ccdimginfo.qmap(ypos,xpos)),        ...
                     ', phi=',num2str(ccdimginfo.phimap(ypos,xpos))]       ;
    else
        display_str = ['x=',num2str(xpos),                              ...
                     ', y=',num2str(ypos),                              ...
                     ', value=',num2str(cdata(ypos,xpos)),              ...
                     ', q||=',num2str(ccdimginfo.qmap(ypos,xpos)),      ...
                     ', qx=',num2str(ccdimginfo.qxmap(ypos,xpos)),      ...
                     ', qy=',num2str(ccdimginfo.qymap(ypos,xpos)),      ...
                     ', qz=',num2str(ccdimginfo.phimap(ypos,xpos))]        ;
    end
    set(findall(gcf,'Tag','mouseposition'),'string',display_str)           ;
        
else
    set(gcf,'Pointer','arrow')                                             ;
end
clear XLim YLim XLimFlag YLimFlag                                          ;
clear pointPosition xpos ypos cdata                                        ;


% ---
% EOF
