function hout=supertitle(str)
% =========================================================================
% --- SUPERTITLE Puts a title above all subplots.
% --- SUPERTITLE('text') adds text to the top of the figure
% --- above all subplots (a "super title"). Use this function
% --- after all subplot commands.
% ---
% --- Drea Thomas 6/15/95 drea@mathworks.com
% --- John Cristion 12/13/00 modified
% --- Mark Histed 03/13/04 histed@mit.edu: fix disappearing legend
% --- 
% --- $Id: SUPERTITLE.m,v 1.2 2004/03/13 22:17:47 histed Exp $
% ---
% --- Warning: If the figure or axis units are non-default, this
% --- will break.
% =========================================================================


% =========================================================================
% Parameters used to position the SUPERTITLE.
% =========================================================================
plotregion = 0.970                                                         ; % Amount of the figure window devoted to subplots
titleypos  = 0.965                                                         ; % Y position of title in normalized coordinates
fudge      = 0.01                                                          ; % Fudge factor to adjust y spacing between subplots
ha_old     = gca                                                           ; % old axis handle
fs         = get(gcf,'defaultaxesfontsize') + 2                            ; % Fontsize for SUPERTITLE
figunits   = get(gcf,'units')                                              ; % get figure units

% =========================================================================
% --- Get the (approximate) difference between full height
% --- (plot + title + xlabel) and bounding rectangle.
% =========================================================================
if (~strcmp(figunits,'pixels')) 
    set(gcf,'units','pixels')                                              ;
	pos = get(gcf,'position')                                              ;
	set(gcf,'units',figunits)                                              ;
else
	pos = get(gcf,'position')                                              ;
end
ff = (fs-4)*1.27*5/pos(4)*fudge                                            ;
% The 5 here reflects about 3 characters of height below
% an axis and 2 above. 1.27 is pixels per point.

% =========================================================================
% Determine the bounding rectange for all the plots
% =========================================================================
h = findobj(gcf,'Type','axes')                                             ;  % Change suggested by Stacy J. Hills
	
% =========================================================================
% --- create SUPERTITLE
% =========================================================================
max_y    = 0                                                               ;
min_y    = 1                                                               ;
oldtitle = 0                                                               ;
for i=1:length(h)
	if (~strcmp(get(h(i),'Tag'),'supertitle'))
		pos=get(h(i),'pos')                                                ;
		if (pos(2) < min_y)
            min_y = pos(2) - 3/5*ff                                        ;
        end
		if (pos(4)+pos(2) > max_y)
            max_y = pos(4) + pos(2) + 2/5*ff                               ;
        end
    else
		oldtitle = h(i)                                                    ;
	end
end

% =========================================================================
% --- if needed rescale the plotregion
% =========================================================================
if max_y > plotregion,
	scale = (plotregion-min_y)/(max_y-min_y)                               ;
	for i=1:length(h)
		pos    = get(h(i),'position')                                      ;
		pos(2) = (pos(2)-min_y)*scale+min_y                                ;
		pos(4) = pos(4)*scale-(1-scale)*ff/5*3                             ;
		set(h(i),'position',pos)                                           ;
	end
end

np = get(gcf,'nextplot')                                                   ;
set(gcf,'nextplot','add')                                                  ;
if (oldtitle)
	delete(oldtitle)                                                       ;
end

ha = axes('pos',[0 1 1 1],'visible','off','Tag','supertitle')              ;
ht = text(.5,titleypos-1,str)                                              ;
% set(ht,'horizontalalignment','center','Interpreter','tex','fontsize',fs)   ; % default setting changed by MS 200602 20
set(ht,'HorizontalAlignment','center','Interpreter','none'              ...
      ,'FontWeight','demi','FontSize',fs)                                  ; % setting for XPCSGUI titles
set(gcf,'nextplot',np)                                                     ;
axes(ha_old)                                                               ;

% =========================================================================
% --- fix legend if one exists
% =========================================================================
legH = legend                                                              ;
if ~isempty(legH)
    axes(legH)                                                             ;
end

% =========================================================================
% --- assign supertitle handle to hout if needed
% =========================================================================
if nargout
	hout = ht                                                              ;
end

% ---
% EOF
