function test_loadascii
% --- TEST_LOADASCII : demonstration function for the use of loadascii 
% --- 
% --- This is an example function how to load ASCII data files created
% --- by XPCSGUI and how to manipulate them
% ---
% --- by MS 20060217
% ---


% =========================================================================
% --- import the data from the ASCII files 
% =========================================================================
file = 'lat_m23C_c2_g2_0004_average.ASCII'                                 ; % define a filename (here a g2 function)
a = loadascii(file)                                                         % load the ASCII file and store in a

file = 'lat_m23C_c2_fitresults.ASCII'                                      ; % define a filename (here the fitresults summary)
b = loadascii(file)                                                         % load the ASCII file and store in b


% =========================================================================
% --- data preparation 
% =========================================================================
% --- to plot now e.g. f = sqrt((g2-baseline)/contrast) vs time
% --- first, one needs to assign the data
x = a.data(:,1)                                                            ; % the delays are store in column 1
y = a.data(:,2)                                                            ; % the measured g2 is stored in column 2
x = x(:); y = y(:)                                                         ; % transform x & y to column vectors
% --- get the fit parameter
contrast = b.data(4,5)                                                     ; % it's the fourth g2 & contrast values are in column 5
baseline = b.data(4,4)                                                     ; % it's the fourth g2 & baseline values are in column 4
% ---
y      = sqrt((y-baseline)/contrast)                                       ; % calculate f
y_imag = imag(y)                                                           ; % calculate imaginary part of f
% ---
Imags     = find(y_imag ~= 0)                                              ; % find positions of (g2-baseline) < 0
if ( numel(Imags) ~= 0 )
    firstImag = Imags(1)                                                   ; % find position of the first value of (g2-baseline) < 0
else
    firstImag = numel(y) +1                                                ; % find position of the first value of (g2-baseline) < 0    
end


% =========================================================================
% --- create a figure ( here g1 is shown in figure with 4 subplots)
% =========================================================================
delete(findall(0,'Tag','testFig'))                                         ; % close a possible previously created figure
% ---
testFig = figure('Tag','testFig')                                          ;
% ---
subplot(2,2,1)                                                             ;
plot(x(1:firstImag-1),y(1:firstImag-1),'ko-')                              ; % plot only up values with (g2-baseline) > 0
xlabel(a.columnName{1})                                                    ; % original labels are store in a.columnName
ylabel('f')                                                                ;
% ---
subplot(2,2,2)                                                             ;
semilogx(x(1:firstImag-1),y(1:firstImag-1),'ko-')                          ; % plot only up values with (g2-baseline) > 0
xlabel(a.columnName{1})                                                    ;
ylabel('f')                                                                ;
% --- btw : This is probably the plot Simon wants
subplot(2,2,3)                                                             ;
semilogy(x(1:firstImag-1),y(1:firstImag-1),'ko-')                          ; % plot only up values with (g2-baseline) > 0
xlabel(a.columnName{1})                                                    ;
ylabel('f')                                                                ;
% ---
subplot(2,2,4)                                                             ;
loglog(x(1:firstImag-1),y(1:firstImag-1),'ko-')                            ; % plot only up values with (g2-baseline) > 0
xlabel(a.columnName{1})                                                    ;
ylabel('f')                                                                ;


% ---
% EOF
