function uncompress_batch(nBatch)
%
% converts a batch of compressed data into an uncompressed '.imm' multifile 
%
% Before starting this routine please start 'xpcsgui' and load the
% batchinfofile
%
% Michael Sprung
% $Revision: 1.0b $  $Date: 2005/12/05 $
%


% =========================================================================
% --- get application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
pause(0)                                                                   ;


% =========================================================================
% --- start progress messages
% =========================================================================
ta = clock                                                                 ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
msgnew = sprintf('\r   Starting to uncompress the batch!')                 ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),                 ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
pause(0.005)                                                               ; % wait for 5ms to display message


% =========================================================================
% --- try to create directory to store uncompressed data
% =========================================================================
CDataDir  = ccdimginfo.imgPath                                             ;
UCDataDir = [CDataDir,'_ucp']                                              ; 
[dummy1 dummy2 dummy3] = mkdir(UCDataDir)                                  ; % try to create uncompressed data directory
if ( dummy1 == 0 )                                                           % no write permission to create directory
    msgstr = get(findall(hFigXPCSMain,'Tag',                            ...
                                      'xpcsmain_EditMessage'),'string')    ;
    set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'String',    ...
        [msgstr;{' ';'   Failed creating saving directory !'}],         ...
        'HorizontalAlignment','left')                                      ;
    pause(0.001)                                                           ;
    return
end
clear dummy1 dummy2 dummy3                                                 ;
pause(0)                                                                   ;


% =========================================================================
% --- assign name of compressed data of Batch# nBatch to 'CFileName'
% =========================================================================
CFileName = ccdimginfo.imagefile{nBatch,1}                                 ;


% =========================================================================
% --- create the file name for the uncompressed data multifile
% =========================================================================
frameBeg = sprintf('%04i',ccdimginfo.ndata0(nBatch))                       ;
frameEnd = sprintf('%04i',ccdimginfo.ndataend(nBatch))                     ;
UCFileName = fullfile(UCDataDir,[ccdimginfo.name,'ucp_',                ...
                                 frameBeg,'-',frameEnd,'.imm'])            ;


% =========================================================================
% --- copy the batchinfo file to the uncompressed data directory
% --- modify the 'name' & 'child' parameter of the UC batchinfo file
% =========================================================================
[pathstr,name,ext,versn] = fileparts(ccdimginfo.batchinfoFile)             ; % get batchinfo file parts
UCBatchinfoFile          = fullfile(UCDataDir,[name,ext])                  ; % create full name for the uncompressed batchinfo file

[Cfid,message] = fopen(ccdimginfo.batchinfoFile)                           ; % open file for modifications
if ( Cfid == -1 )                                                            % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(Cfid)                                                            ;
    return                                                                 ;
end
[UCfid,message] = fopen(UCBatchinfoFile,'w+')                              ; % open file for modifications
if ( UCfid == -1 )                                                           % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(UCfid)                                                          ;
    return                                                                 ;
end

k = 0                                                                      ;
while ( feof(Cfid) == 0 )
    k = k + 1                                                              ; % line number counter
    scanline      = fgetl(Cfid)                                            ;
    equal_pos     = find(scanline == '=')                                  ;
    left_str      = scanline(1:equal_pos-1)                                ;
    dummy         = left_str                                               ;
    left_str(findstr(left_str,' '))=''                                     ;
    if ( strcmp(left_str,'name') == 1 || strcmp(left_str,'child') == 1 )
       LineStart  = dummy                                                  ;
       right_str  = scanline(equal_pos+1:end)                              ;
       LineEnd    = right_str                                              ; 
       LineEndNew = [LineEnd(1:end-1),'ucp_',LineEnd(end)]                 ;
       LineNew    = [LineStart,'=',LineEndNew]                             ;
       fprintf(UCfid,[LineNew,'\n'])                                       ;
    else
       fprintf(UCfid,[scanline,'\n'])                                      ;
    end
end
clear LineStart LineEnd LineEndNew LineNew

fclose(Cfid)                                                               ; % close file
fclose(UCfid)                                                              ; % close file
                                 
                                 
% =========================================================================
% --- open the compressed data file and create uncompressed data file
% =========================================================================
[Cfid,message] = fopen(CFileName)                                          ;        
if ( Cfid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(Cfid)                                                           ;
    return                                                                 ;
end
[UCfid,message] = fopen(UCFileName,'w+')                                   ;        
if ( UCfid == -1 )                                                             % return if open fails
    uiwait(msgbox(message,'File Open Error','error','modal'))              ;
    fclose(UCfid)                                                          ;
    return                                                                 ;
end


% =========================================================================
% --- get the uncompressed image size
% =========================================================================
fseek(Cfid,108,'bof')                                                      ;
UCrows  = fread(Cfid,1 ,'int')                                             ;
fseek(Cfid,112,'bof')                                                      ;
UCcols  = fread(Cfid,1 ,'int')                                             ;
fseek(Cfid,116,'bof')                                                      ;
UCbytes = fread(Cfid,1 ,'int')                                             ;
fseek(Cfid,616,'bof')                                                      ;
UCimmversion = fread(Cfid,1 ,'int')                                        ;
if ( UCimmversion >= 11 )
    immSize = UCbytes * UCrows * UCcols + 1024                             ; % 'bytes' per pixel + 1024 Byte header
else
    immSize = 2 * UCrows * UCcols + 1024                                   ; % allways 2 bytes per pixel + 1024 Byte header for older versions
end


% =========================================================================
% --- start of loop over all frames
% =========================================================================
CPos = 0                                                                   ;
FilesInBatch = ccdimginfo.ndataend(nBatch) - ccdimginfo.ndata0(nBatch) + 1 ;
for i = 1 : ccdimginfo.ndataend(nBatch) - ccdimginfo.ndata0(nBatch) + 1
    

    % =====================================================================
    % --- show progress message
    % =====================================================================
    if ( i == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('\r   Convert image no. %04i of %04i images'   ...
                         ,i,FilesInBatch)                                  ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
        pause(0.001)                                                       ; % wait for 1ms to display message
    end
    if ( FilesInBatch < 1000 && (floor(i/10)-i/10+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Convert image no. %04i of %04i images',    ...
                         i,FilesInBatch)                                   ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end
    if ( FilesInBatch >= 1000 && (floor(i/100)-i/100+1) == 1 )
        msgstr = get(findall(hFigXPCSMain,'Tag',                        ...
                     'xpcsmain_EditMessage'),'string')                     ;
        msgnew = sprintf('   Convert image no. %04i of %04i images',    ...
                         i,FilesInBatch)                                   ;
        set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),         ...
            'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
            pause(0.001)                                                   ; % wait for 1ms to display message
    end

    % --- start to read and write headers
    fseek(Cfid ,CPos+(i-1)*1024,'bof')                                     ; % go to the next header start of the compressed file
    fseek(UCfid,(i-1)*immSize,'bof')                                       ; % go to the next header start of the uncompressed file
    header = fread(Cfid,1024,'uint8')                                      ; % read  complete header
    fwrite(UCfid,header,'uint8')                                           ; % write complete header
    clear header                                                           ;
    
    % --- go back to umcompressed header start and modify compression flag
    fseek(Cfid ,CPos+(i-1)*1024+4,'bof')                                   ; % go to the compression flag position of the   compressed file
    fseek(UCfid,(i-1)*immSize+4,'bof')                                     ; % go to the compression flag position of the uncompressed file
    compression = fread(Cfid,1,'*int')                                     ; % read  old compression flag
    fwrite(UCfid,compression-1,'int')                                      ; % write new compression flag
    clear compression                                                      ;
    % --- finish to read and write headers
    
    % --- read the number of compressed data points
    fseek(Cfid ,CPos+(i-1)*1024+152,'bof')                                 ; % go to the 'dlen' field of the compressed file
    dlen = fread(Cfid,1 ,'*int')                                           ; % read the 'dlen' field
    PixelNumber = floor(double(dlen)/8)                                    ;
    clear dlen                                                             ;
    
    % --- read the compressed data (fast code)
    fseek(Cfid ,CPos+i*1024,'bof')                                         ; % go to the next data start of the compressed file
    data       = fread(Cfid,2*PixelNumber,'uint32')                        ; % read the full data block 
    datavalues = data(2:2:end)                                             ; % create vector with intensity values
    PixelIndex = data(1:2:end-1) + 1                                       ; % create vector with pixel indices
    PixelValue = single(uread(uwrite(datavalues,'uint32')               ...
                             ,PixelNumber,'float32'))'                     ; % convert data values to binary to float32 to single
    clear data datavalues                                                  ;
                         
    % --- create uncompressed data
    UCData = zeros(UCrows,UCcols)                                          ;
    UCData(PixelIndex) = PixelValue                                        ;
    
    % --- write uncompressed data
    fseek(UCfid,(i-1)*immSize+1024,'bof')                                  ;
    fwrite(UCfid,UCData,'ushort')                                          ;

    % --- calculate CPos for the next image
    CPos  = CPos + 8 * PixelNumber                                         ; % calculate the next position of a hearder start
    clear PixelNumber PixelIndex PixelValue                                ;


end
% =========================================================================
% --- end of loop over all frames
% =========================================================================


% =========================================================================
% --- close the compressed data file and uncompressed data file
% =========================================================================
fclose(Cfid)                                                               ;
fclose(UCfid)                                                              ;


% =========================================================================
% --- finish message
% =========================================================================
tb = etime(clock,ta)                                                       ;
msgstr = get(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage'),'string')  ;
msgnew = sprintf(['\r   Finish to uncompress the batch in '             ...
                 ,num2str(tb),' seconds.'])                                ;
set(findall(hFigXPCSMain,'Tag','xpcsmain_EditMessage')                  ...
           ,'String',[msgstr;{msgnew}],'HorizontalAlignment','left')       ;
pause(0.005)                                                               ; % wait for 5ms to display message


% =========================================================================
% --- load the uncompressed batchinfo file
% =========================================================================
loadbatchinfo(UCBatchinfoFile)                                             ;


% ---
% EOF
