function viewanalysis(varargin)
% VIEWANALYSIS GUI to display and change analysis settings.
%
% Zhang Jiang & Michael Sprung
% $Revision: 1.0 $  $Date: 2005/01/07 $
% $Revision: 2.0 $  $Date: 2006/10/04 $


% =========================================================================
% --- get ccdimginfo
% =========================================================================
hFigXPCSMain   = findall(0,'Tag','xpcsmain_Fig')                           ;
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                     ;
else
    return                                                                 ;
end


% =========================================================================
% --- if viewanalysis fig exists, display it in front; else create it
% =========================================================================
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
if ~isempty(hFigViewanalysis)  
    figure(hFigViewanalysis)                                               ;
    return                                                                 ;
end


% =========================================================================
% --- figure layout
% =========================================================================
backgroundcolor = [1 1 0.85]                                               ;
facecolor = [1 1 0.9]                                                      ;
textcolor = [0.4 0.3 0]                                                    ; % text color
subtextcolor = 'b'                                                         ; % subtext color
MainPos = get(hFigXPCSMain,'position')                                     ;
figViewanalysisSize = [720 540]                                            ;
hFigViewanalysis = figure('BackingStore','on','Units','pixels'          ...
    ,'DockControls','off','Resize','off','PaperOrient','portrait'       ...
    ,'PaperPositionMode','auto','IntegerHandle','off'                   ...
    ,'NumberTitle','off','MenuBar','none','Toolbar','none'              ...
    ,'CloseRequestFcn',@viewanalysis_CloseRequestFcn                    ...
    ,'Name','XPCS - Analysis Settings','WindowStyle','modal'            ...
    ,'Position',[MainPos(1)+ 150,MainPos(2)+ 60,figViewanalysisSize]    ...
    ,'HandleVisibility','callback','Tag','viewanalysis_Fig','UserData',[]) ;
clear MainPos                                                              ;

hAxes = axes('Parent',hFigViewanalysis,'Units','pixels'                ...
    ,'Position',[0 0 figViewanalysisSize],'Tag','figviewanalysis_Axes')    ;

patch('Parent',hAxes,'XData',[0 1 1 0],'YData',[0 0 1 1]                ...
     ,'FaceColor',backgroundcolor,'EdgeColor',backgroundcolor)             ;
hGroupPatch = hggroup('Parent',hAxes)                                      ;

patch('Parent',hGroupPatch,'XData',[0.02 0.49 0.49 0.02]                ...
     ,'YData',[0.1 0.1 0.925 0.925])                                       ;
patch('Parent',hGroupPatch,'XData',[0.51 0.98 0.98 0.51]                ...
     ,'YData',[0.1 0.1 0.925 0.925])                                       ;
set(get(hGroupPatch,'Children'),'FaceColor',facecolor                   ...
                               ,'EdgeColor',[0.7 0.7 0.7])                 ;

hGroupText      = hggroup('Parent',hAxes)                                  ;
hGroupSubText   = hggroup('Parent',hAxes)                                  ;
hGroupSubText1  = hggroup('Parent',hAxes)                                  ;


% =========================================================================
% --- gui title
% =========================================================================
text('Parent',hAxes,'Position',[.415 .965],'String','Analysis Settings' ...
    ,'FontSize',12,'FontWeight','demi','color',[0.4 0.3 0])                ;


% =========================================================================
% --- layout of selection for analysis type
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.9],                         ...
     'String','Select Analysis Type:')                                     ;
 
uicontrol('Parent',hFigViewanalysis,'style','Radiobutton'               ...
         ,'Units','normalized','String','static analysis'               ...
         ,'FontSize',10,'BackgroundColor',facecolor                     ...
         ,'ForegroundColor',subtextcolor,'HorizontalAlignment','left'   ...
         ,'Position',[0.06 0.835 0.3 0.04]                              ...
         ,'Tag','figviewanalysis_RadiobuttonAnalysisTypeS'              ...
         ,'Callback',@figviewanalysis_RadiobuttonAnalysisTypeCallbackFcn)  ;

uicontrol('Parent',hFigViewanalysis,'style','Radiobutton'               ...
         ,'Units','normalized','String','dynamic analysis','FontSize',10 ...
         ,'BackgroundColor',facecolor,'ForegroundColor',subtextcolor    ...
         ,'HorizontalAlignment','left','Position',[0.06 0.79 0.3 0.04]  ...
         ,'Tag','figviewanalysis_RadiobuttonAnalysisTypeD'              ...
         ,'Callback',@figviewanalysis_RadiobuttonAnalysisTypeCallbackFcn)  ;


% =========================================================================
% ---define strings for the partition methods
% =========================================================================
qpartitionMethodString = {                                              ...
    'from existing result file (nw; uses evenly spaced)';               ...
    'evenly spaced';                                                    ...
    'equal dq/q';                                                       ...
    'manually set';                                                     ...
    'no partitions (use whole image)'}                                     ;

phipartitionMethodString = {                                            ...
    'from existing result file (nw; uses evenly spaced)';               ...
    'evenly spaced';                                                    ...
    'evenly spaced (trans.) or equal dq/q (reflec.)';                   ...
    'manually set';                                                     ...
    'no partitions (use whole image)'}                                     ;


% =========================================================================
% ---layout static q partition
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.77],...
     'String','Static Q Partition:')                                       ;
text('Parent',hGroupSubText,'Position',[0.06 0.73],'String','method')      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.715 0.28 0.04]',                                  ...
    'String',qpartitionMethodString,                                    ...
    'Value',ccdimginfo.sqMethod,                                        ...
    'Tag','figviewanalysis_PopupmenuSqMethod',                          ...
    'callback',@figviewanalysis_PopupmenuSqMethodCallbackFcn)              ;

text('Parent',hGroupSubText,'Position',[0.06 0.69],...
     'String','file/number/span')                                          ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.67 0.19 0.035]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.snoq),                                  ...
    'Tag','figviewanalysis_EditSnoq')                                      ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Browse ...',                                              ...
    'Position',[0.4 0.67 0.08 0.04],                                    ...
    'Enable','off',                                                     ...
    'Tag','figviewanalysis_PushbuttonSqMethod',                         ...
    'callback',@figviewanalysis_BrowseSQCallbackFcn)                       ;


% =========================================================================
% --- layout static phi partition
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.65],...
     'String','Static Phi Partition:')                                     ;
text('Parent',hGroupSubText,'Position',[0.06 0.61],'String','method')      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.595 0.28 0.04]',                                  ...
    'String',phipartitionMethodString,                                  ...
    'Value',ccdimginfo.sphiMethod,                                      ...
    'Tag','figviewanalysis_PopupmenuSphiMethod',                        ...
    'callback',@figviewanalysis_PopupmenuSphiMethodCallbackFcn)            ;

text('Parent',hGroupSubText,'Position',[0.06 0.57],...
     'String','file/number/span')                                          ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.55 0.19 0.035]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.snophi),                                ...
    'Tag','figviewanalysis_EditSnophi')                                    ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Browse ...',                                              ...
    'Position',[0.4 0.55 0.08 0.04],                                    ...
    'Enable','off',                                                     ...
    'Tag','figviewanalysis_PushbuttonSphiMethod',                       ...
    'callback',@figviewanalysis_BrowseSPHICallbackFcn)                     ;


% =========================================================================
% --- layout dynamic q partition
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.53],...
     'String','Dynamic Q Partition:')                                      ;
text('Parent',hGroupSubText,'Position',[0.06 0.49],'String','method')      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.475 0.28 0.04]',                                  ...
    'String',qpartitionMethodString,                                    ...
    'Value',ccdimginfo.dqMethod,                                        ...
    'Tag','figviewanalysis_PopupmenuDqMethod',                          ...
    'callback',@figviewanalysis_PopupmenuDqMethodCallbackFcn)              ;

text('Parent',hGroupSubText,'Position',[0.06 0.45],                     ...
     'String','file/number/span')                                          ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.43 0.19 0.035]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.dnoq),                                  ...
    'Tag','figviewanalysis_EditDnoq')                                      ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Browse ...',                                              ...
    'Position',[0.4 0.43 0.08 0.04],                                    ...
    'Enable','off',                                                     ...
    'Tag','figviewanalysis_PushbuttonDqMethod',                         ...
    'callback',@figviewanalysis_BrowseDQCallbackFcn)                       ;


% =========================================================================
% --- layout dynamic phi partition
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.41],                        ...
     'String','Dynamic Phi Partition:')                                    ;
text('Parent',hGroupSubText,'Position',[0.06 0.37],'String','method')      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.355 0.28 0.04]',                                  ...
    'String',phipartitionMethodString,                                  ...
    'Value',ccdimginfo.dphiMethod,                                      ...
    'Tag','figviewanalysis_PopupmenuDphiMethod',                        ...
    'callback',@figviewanalysis_PopupmenuDphiMethodCallbackFcn)            ;

text('Parent',hGroupSubText,'Position',[0.06 0.33],...
     'String','file/number/span')                                          ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.2 0.31 0.19 0.035]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.dnophi),                                ...
    'Tag','figviewanalysis_EditDnophi')                                    ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Browse ...',                                              ...
    'Position',[0.4 0.31 0.08 0.04],                                    ...
    'Enable','off',                                                     ...
    'Tag','figviewanalysis_PushbuttonDphiMethod',                       ...
    'callback',@figviewanalysis_BrowseDPHICallbackFcn)                     ;


% =========================================================================
% --- layout for mask selection
% ccdimginfo.maskMethod
% 1. no mask
% 2. new mask
% 3. from existing result file (not working yet)
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.29],'String','Mask Settings:') ;
hRadiobuttonMask1 = uicontrol('Parent',hFigViewanalysis,                ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String','no mask (use all pixles)',                                ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'HorizontalAlignment','left',                                       ...
    'Position',[0.06 0.23 0.3 0.04],                                    ...
    'Tag','figviewanalysis_RadiobuttonMask1',                           ...
    'Callback',@figviewanalysis_RadiobuttonMaskCallbackFcn)                ;

hRadiobuttonMask2 = uicontrol('Parent',hFigViewanalysis,                ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String','new mask',                                                ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'HorizontalAlignment','left',                                       ...
    'Position',[0.06 0.19 0.15 0.04],                                   ...
    'Tag','figviewanalysis_RadiobuttonMask2',                           ...
    'Callback',@figviewanalysis_RadiobuttonMaskCallbackFcn)                ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Mask Polygon',                                            ...
    'Position',[0.33 0.19 0.15 0.04],                                   ...
    'Enable','on',                                                      ...
    'Tag','figviewanalysis_PushbuttonNewMask',                          ...
    'TooltipString','Show/define mask',                                 ...
    'callback',@figviewanalysis_PushbuttonNewMaskCallbackFcn)             ;

hRadiobuttonMask3 = uicontrol('Parent',hFigViewanalysis,                ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String','from existing file',                                      ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'HorizontalAlignment','left',                                       ...
    'Position',[0.06 0.15 0.25 0.04],                                   ...
    'Tag','figviewanalysis_RadiobuttonMask3',                           ...
    'Callback',@figviewanalysis_RadiobuttonMaskCallbackFcn)                ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Browser ...',                                             ...
    'Position',[0.33 0.15 0.15 0.04],                                   ...
    'Tag','figviewanalysis_PushbuttonMaskFile',                         ...
    'callback',@figviewanalysis_BrowseMaskFileCallbackFcn)                 ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'String',ccdimginfo.maskfile,                                       ...
    'HorizontalAlignment','left',                                       ...
    'TooltipString','Browse existing result file containing mask',      ...
    'Position',[0.1 0.11 0.38 0.035],                                   ...
    'Tag','figviewanalysis_maskfile')                                      ;

% --- initialize gui objects of mask settings
switch ccdimginfo.maskMethod
    case 1
        figviewanalysis_RadiobuttonMaskCallbackFcn(hRadiobuttonMask1,[])   ;
    case 2
        figviewanalysis_RadiobuttonMaskCallbackFcn(hRadiobuttonMask2,[])   ;
    case 3
        figviewanalysis_RadiobuttonMaskCallbackFcn(hRadiobuttonMask3,[])   ;
end


% =========================================================================
% --- layout for dynamics analysis options
% =========================================================================
text('Parent',hGroupText,'Position',[0.53 0.9],                         ...
     'String','Dynamic Analysis Options:')                                 ;
text('Parent',hGroupSubText,'Position',[0.56 0.86],                     ...
     'String','# of delays per multiple tau level')                        ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.87 0.84 0.1 0.035]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.dpl),                                   ...
    'Tag','figviewanalysis_dpl')                                           ;

text('Parent',hGroupSubText,'Position',[0.56 0.82],                     ...
     'String','# of slice delays per multiple tau level')                  ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.87 0.8 0.1 0.035]',                                   ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.slicedpl),                              ...
    'Tag','figviewanalysis_slicedpl')                                      ;

text('Parent',hGroupSubText,'Position',[0.56 0.77],                     ...
     'String','use smoothed symmetric normalization')                      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.9 0.75 0.07 0.04]',                                   ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuSsn',                               ...
    'callback',@figviewanalysis_PopupmenuSsnCallbackFcn)                   ;

text('Parent',hGroupSubText,'Position',[0.56 0.72],                      ...
     'String','minimum # of pixels used to smooth')                        ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.87 0.70 0.1 0.04]',                                   ...
    'HorizontalAlignment','right',                                      ...
    'String',num2str(ccdimginfo.ssnmin),                                ...
    'Tag','figviewanalysis_ssnmin')                                        ;

if ( ccdimginfo.ssn == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuSsn'),'value',1,'Enable','off') ;
    set(findall(0,'Tag','figviewanalysis_ssnmin')      ,'Enable','off')           ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuSsn'),'value',2,'Enable','off') ;
    set(findall(0,'Tag','figviewanalysis_ssnmin')      ,'Enable','off')           ;
end


% =========================================================================
% --- layout for lld
% =========================================================================
text('Parent',hGroupText,'Position',[0.53 0.66],                        ...
     'String','Lower Level Discrimination (LLD):')                         ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String','no LLD',                                                  ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'Position',[0.56 0.59 0.1 0.04],                                    ...
    'Tag','figviewanalysis_RadiobuttonLLD1',                            ...
    'Callback',@figviewanalysis_RadiobuttonLLDCallbackFcn)                 ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String',' ',                                                       ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'HorizontalAlignment','right',                                      ...
    'Position',[0.56 0.545 0.04 0.04],                                  ...
    'Tag','figviewanalysis_RadiobuttonLLD2',                            ...
    'Callback',@figviewanalysis_RadiobuttonLLDCallbackFcn)                 ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'String','',                                                        ...
    'BackgroundColor','w',                                              ...
    'HorizontalAlignment','right',                                      ...
    'Position',[0.59 0.545 0.05 0.035],                                 ...
    'Tag','figviewanalysis_EditLLD2')                                      ;

text('Parent',hGroupSubText,'Position',[0.65 0.565],'String','X ADU')      ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Radiobutton',                                              ...
    'Units','normalized',                                               ...
    'String',' ',                                                       ...
    'FontSize',10,                                                      ...
    'BackgroundColor',facecolor,                                        ...
    'ForegroundColor',subtextcolor,                                     ...
    'HorizontalAlignment','right',                                      ...
    'Position',[0.56 0.50 0.04 0.04],                                   ...
    'Tag','figviewanalysis_RadiobuttonLLD3',                            ...
    'Callback',@figviewanalysis_RadiobuttonLLDCallbackFcn)                 ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'String','',                                                        ...
    'BackgroundColor','w',                                              ...
    'HorizontalAlignment','right',                                      ...
    'Position',[0.59 0.50 0.05 0.035],                                  ...
    'Tag','figviewanalysis_EditLLD3')                                      ;

text('Parent',hGroupSubText,'Position',[0.65 0.52],'String','X dark RMS')  ;
if ccdimginfo.lld < 0                                                       % use absolute lld (ADU)
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD1'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD2'),'value',1)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD3'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_EditLLD2'),'Enable','on',...
                  'string',num2str(abs(ccdimginfo.lld)))                   ;
    set(findall(0,'Tag','figviewanalysis_EditLLD3'),'Enable','off')        ; 
elseif ccdimginfo.lld > 0                                                   % relative lld with respect to dark RMS
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD1'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD2'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD3'),'value',1)      ;
    set(findall(0,'Tag','figviewanalysis_EditLLD2'),'Enable','off')        ;
    set(findall(0,'Tag','figviewanalysis_EditLLD3'),'Enable','on',...
                  'string',num2str(abs(ccdimginfo.lld)))                   ;
else
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD1'),'value',1)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD2'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonLLD3'),'value',0)      ;
    set(findall(0,'Tag','figviewanalysis_EditLLD2'),'Enable','off')        ;
    set(findall(0,'Tag','figviewanalysis_EditLLD3'),'Enable','off')        ;
end


% =========================================================================
% --- layout of saving options
% =========================================================================
text('Parent',hGroupText,'Position',[0.53 0.48],                        ...
     'String','Saving Settings:')                                          ;
text('Parent',hGroupSubText,'Position',[0.56 0.44],                     ...
     'String','save results to folder:')                                   ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','pushbutton',                                               ...
    'Units','normalized',                                               ...
    'String','Folder ...',                                              ...
    'Position',[0.86 0.42 0.1 0.04],                                    ...
    'Tag','figviewanalysis_PushbuttonSaveFolder',                       ...
    'TooltipString','Select/make folder',                               ...
    'callback',@figviewanalysis_BrowseSavingFolderCallbackFcn)             ;

uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'String',ccdimginfo.savepath,                                       ...
    'HorizontalAlignment','left',                                       ...
    'Enable','inactive',                                                ...
    'Position',[0.6 0.38 0.36 0.035],                                   ...
    'Tag','figviewanalysis_savepath')                                      ;

text('Parent',hGroupSubText,'Position',[0.56 0.36],                     ...
     'String','save results with file name prefix')                        ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'style','Edit',                                                     ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'String',ccdimginfo.name,                                           ...
    'HorizontalAlignment','left',                                       ...
    'Position',[0.6 0.30 0.36 0.035],                                   ...
    'Tag','figviewanalysis_prefix')                                        ;


% =========================================================================
% --- layout for display settings
% =========================================================================
text('Parent',hGroupText,'Position',[0.53 0.28],                        ...
     'String','Show result settings:')                                     ;

text('Parent',hGroupSubText,'Position',[0.56 0.25],                     ...
     'String','ASCII output')                                              ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.75 0.22 0.07 0.04]',                                   ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuASCII',                             ...
    'callback',@figviewanalysis_PopupmenuASCIICallbackFcn)                 ;
if ( ccdimginfo.ASCIIon == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuASCII'),'value',1)       ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuASCII'),'value',2)       ;
end

text('Parent',hGroupSubText,'Position',[0.56 0.22],                     ...
     'String','Perform Fit1')                                              ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.75 0.19 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuFIT1',                              ...
    'callback',@figviewanalysis_PopupmenuFIT1CallbackFcn)                  ;
if ( ccdimginfo.FIT1on == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuFIT1'),'value',1)        ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuFIT1'),'value',2)        ;
end

text('Parent',hGroupSubText,'Position',[0.56 0.19],                     ...
     'String','Perform Fit2')                                              ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.75 0.16 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuFIT2',                              ...
    'callback',@figviewanalysis_PopupmenuFIT2CallbackFcn)                  ;
if ( ccdimginfo.FIT2on == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuFIT2'),'value',1)        ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuFIT2'),'value',2)        ;
end

text('Parent',hGroupSubText,'Position',[0.56 0.16],                     ...
     'String','Perform custom fit')                                        ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.75 0.13 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuCUSTOMFIT',                         ...
    'callback',@figviewanalysis_PopupmenuCUSTOMFITCallbackFcn)             ;
if ( ccdimginfo.CUSTOMFITon == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuCUSTOMFIT'),'value',1)   ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuCUSTOMFIT'),'value',2)   ;
end

text('Parent',hGroupSubText,'Position',[0.56 0.13],                     ...
     'String','Create PNG Files')                                          ;
uicontrol('Parent',hFigViewanalysis,                                    ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.75 0.10 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figviewanalysis_PopupmenuPNG',                               ...
    'callback',@figviewanalysis_PopupmenuPNGCallbackFcn)                   ;
if ( ccdimginfo.PNGon == 1 )
    set(findall(0,'Tag','figviewanalysis_PopupmenuPNG'),'value',1)         ;
else
    set(findall(0,'Tag','figviewanalysis_PopupmenuPNG'),'value',2)         ;
end


% =========================================================================
% --- layout of status bar
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.05],                        ...
     'String','Status: Ready!','Tag','figviewanalysis_TextStatus')         ;

% --- set properties of group text and group sub text
set(get(hGroupText,'Children'),                                         ...
    'Units','normalized',                                               ...
    'HorizontalAlignment','left',                                       ...
    'Color',textcolor)                                                     ;

set(get(hGroupSubText,'Children'),                                      ...
    'Units','normalized',                                               ...
    'HorizontalAlignment','left',                                       ...
    'Color',subtextcolor)                                                  ;

set(get(hGroupSubText1,'Children'),                                     ...
    'Units','normalized',                                               ...
    'HorizontalAlignment','left',                                       ...
    'Color',subtextcolor)                                                  ;


% =========================================================================
% --- layout of close, apply and show pushbuttons
% =========================================================================
hPushbottonClose = uicontrol(hFigViewanalysis,'Style','pushbutton'      ...
    ,'Units','normalized','String','Close','position',[.5 .02 .125 .05] ...
    ,'Tag','viewanalysis_PushbuttonClose'                               ...
    ,'TooltipString','Close window'                                     ...
    ,'callback',@viewanalysis_CloseRequestFcn)                             ;

hPushbottonApply = uicontrol(hFigViewanalysis,'Style','pushbutton'      ...
    ,'Units','normalized','String','Apply','position',[.65 .02 .125 .05]...
    ,'Tag','viewanalysis_PushbuttonApply'                               ...
    ,'TooltipString','Apply changes'                                    ...
    ,'callback',@viewanalysis_ApplyFcn)                                    ;

hPushbottonShowMask = uicontrol(hFigViewanalysis,'Style','pushbutton'   ...
    ,'Units','normalized','String','Show Mask & Partitions'             ...
    ,'position',[.8 .02 .18 .05],'Tag','viewanalysis_PushbuttonShow'    ...
    ,'TooltipString','Apply changes & show image with mask & partitions'...
    ,'callback',@viewanalysis_ShowFcn)                                      ;


% =========================================================================
% --- for dynamics or static analysis initalize the enable properties of
% --- each object in the figure
% =========================================================================
if ccdimginfo.analysistype == 0                                             % static analysis
    set(findall(0,'Tag','figviewanalysis_RadiobuttonAnalysisTypeS'),    ...
                  'value',0)                                               ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonAnalysisTypeD'),    ...
                  'value',1)                                               ;
    figviewanalysis_RadiobuttonAnalysisTypeCallbackFcn(                 ...
        findall(hFigViewanalysis, 'Tag',                                ...
               'figviewanalysis_RadiobuttonAnalysisTypeS'),[])             ;
elseif ccdimginfo.analysistype == 1                                         % dynamic analysis
    set(findall(0,'Tag','figviewanalysis_RadiobuttonAnalysisTypeS'),    ...
                  'value',0)                                               ;
    set(findall(0,'Tag','figviewanalysis_RadiobuttonAnalysisTypeD'),    ...
                  'value',1)                                               ;
    figviewanalysis_RadiobuttonAnalysisTypeCallbackFcn(                 ...
        findall(hFigViewanalysis, 'Tag',                                ...
               'figviewanalysis_RadiobuttonAnalysisTypeD'),[])             ;
end

setappdata(hFigViewanalysis,'maskpoints',ccdimginfo.maskpoints)            ;


%==========================================================================
%==========================================================================
% --- Start to define Callback Functions & Subfunctions
%==========================================================================
%==========================================================================


%==========================================================================
% --- Browsing Callback Functions
%==========================================================================
function figviewanalysis_BrowseSQCallbackFcn(hObject,eventdata)
[filename filepath]=uigetfile('*.*','Load existing result file for static q partition');
if filename == 0
    clear filename filepath                                                ;
    return
end
file=fullfile(filepath,filename)                                           ;
set(findall(0,'Tag','figviewanalysis_EditSnoq'),'String',file           ...
             ,'HorizontalAlignment','right')                               ;
clear file filename filepath                                               ;

function figviewanalysis_BrowseSPHICallbackFcn(hObject,eventdata)
[filename filepath]=uigetfile('*.*','Load existing result file for static phi partition');
if filename == 0
    clear filename filepath                                                ;
    return
end
file=fullfile(filepath,filename)                                           ;
set(findall(0,'Tag','figviewanalysis_EditSnophi'),'String',file         ...
             ,'HorizontalAlignment','right')                               ;
clear file filename filepath                                               ;

function figviewanalysis_BrowseDQCallbackFcn(hObject,eventdata)
[filename filepath]=uigetfile('*.*','Load existing result file for dynamic q partition');
if filename == 0
    clear filename filepath                                                ;
    return
end
file=fullfile(filepath,filename)                                           ;
set(findall(0,'Tag','figviewanalysis_EditDnoq'),'String',file           ...
              ,'HorizontalAlignment','right')                              ;
clear file filename filepath                                               ;

function figviewanalysis_BrowseDPHICallbackFcn(hObject,eventdata)
[filename filepath]=uigetfile('*.*','Load existing result file for dynamic phi partition');
if filename == 0
    clear filename filepath                                                ;
    return
end
file=fullfile(filepath,filename)                                           ;
set(findall(0,'Tag','figviewanalysis_EditDnophi'),'String',file         ...
                   ,'HorizontalAlignment','right')                         ;
clear file filename filepath                                               ;

function figviewanalysis_BrowseMaskFileCallbackFcn(hObject,eventdata)
[filename filepath]=uigetfile('*.*','Load existing result file for mask')  ;
if filename == 0
    clear filename filepath                                                ;
    return
end
file=fullfile(filepath,filename)                                           ;
set(findall(0,'Tag','figviewanalysis_maskfile'),'String',file           ...
             ,'HorizontalAlignment','left')                                ;
clear file filename filepath                                               ;

function figviewanalysis_BrowseSavingFolderCallbackFcn(hObject,eventdata)
savepath = uigetdir(get(findall(0,'Tag','figviewanalysis_savepath'),'String')    ...
                                 ,'Select a folder to save XPCS analysis result')   ;
if savepath == 0
    clear savepath                                                         ;
    return
end
set(findall(0,'Tag','figviewanalysis_savepath'),'String',savepath       ...
             ,'HorizontalAlignment','left')                                ;
clear savepath                                                             ;


%==========================================================================
% --- Analysis Type radiobuttons callback fcn
%==========================================================================
function figviewanalysis_RadiobuttonAnalysisTypeCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig');
hRadiobuttonAnalysisTypeS = findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonAnalysisTypeS');
hRadiobuttonAnalysisTypeD = findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonAnalysisTypeD');
switch hObject
    case hRadiobuttonAnalysisTypeS                                           % static
        set(hRadiobuttonAnalysisTypeS,'value',1)                           ;
        set(hRadiobuttonAnalysisTypeD,'value',0)                           ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDqMethod')   ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDphiMethod') ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq')            ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi')          ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDqMethod')  ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDphiMethod'),'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_dpl')                 ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_slicedpl')            ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSsn')        ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_ssnmin')              ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuASCII')      ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT1')       ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT2')       ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuPNG')        ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuCUSTOMFIT')  ,'Enable','off');
        figviewanalysis_PopupmenuSqMethodCallbackFcn                       ;
        figviewanalysis_PopupmenuSphiMethodCallbackFcn                     ;
        figviewanalysis_PopupmenuASCIICallbackFcn                          ;
        figviewanalysis_PopupmenuFIT1CallbackFcn                           ;
        figviewanalysis_PopupmenuFIT2CallbackFcn                           ;
        figviewanalysis_PopupmenuPNGCallbackFcn                            ;
        figviewanalysis_PopupmenuCUSTOMFITCallbackFcn                      ;
    case hRadiobuttonAnalysisTypeD                                           % dynamic
        set(hRadiobuttonAnalysisTypeS,'value',0)                           ;
        set(hRadiobuttonAnalysisTypeD,'value',1)                           ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDqMethod')  ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDphiMethod'),'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_dpl')                ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_slicedpl')           ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSsn')       ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_ssnmin')             ,'Enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuASCII')     ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT1')      ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT2')      ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuPNG')       ,'Enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuCUSTOMFIT') ,'Enable','on') ;
        figviewanalysis_PopupmenuSqMethodCallbackFcn                       ;
        figviewanalysis_PopupmenuSphiMethodCallbackFcn                     ;
        figviewanalysis_PopupmenuDqMethodCallbackFcn                       ;
        figviewanalysis_PopupmenuDphiMethodCallbackFcn                     ;
        figviewanalysis_PopupmenuSsnCallbackFcn                            ;
        figviewanalysis_PopupmenuASCIICallbackFcn                          ;
        figviewanalysis_PopupmenuFIT1CallbackFcn                           ;
        figviewanalysis_PopupmenuFIT2CallbackFcn                           ;
        figviewanalysis_PopupmenuPNGCallbackFcn                            ;
        figviewanalysis_PopupmenuCUSTOMFITCallbackFcn                      ;
end


%==========================================================================
% --- PopupmenuSqMethod callback fcn
%==========================================================================
function figviewanalysis_PushbuttonNewMaskCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
maskccdimg                                                                 ;

%==========================================================================
% --- PopupmenuSqMethod callback fcn
%==========================================================================
function figviewanalysis_PopupmenuSqMethodCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
ccdimginfo = getappdata(findall(0,'Tag','xpcsmain_Fig'),'ccdimginfo')      ;
switch get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSqMethod'),'Value')
    case 1
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'), ...
            'Enable','on','String',ccdimginfo.sqfile,                   ...
            'TooltipString','Browse for existing result file')             ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSqMethod'),'Enable','on');
    case {2,3}
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'), ...
            'Enable','on','String',num2str(ccdimginfo.snoq),            ...
            'TooltipString','Enter number of partitions')                  ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSqMethod'),'Enable','off');
    case 4
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'), ...
            'Enable','on','String',ccdimginfo.sqspanstr,                ...
            'TooltipString','Enter static q span (start:step:end)')        ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSqMethod'),'Enable','off');
    case 5
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'),'Enable','off','String','');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSqMethod'),'Enable','off');
end


%==========================================================================
% --- PopupmenuSphiMethod callback fcn
%==========================================================================
function figviewanalysis_PopupmenuSphiMethodCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig');
ccdimginfo = getappdata(findall(0,'Tag','xpcsmain_Fig'),'ccdimginfo');
switch get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSphiMethod'),'Value')
    case 1
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),...
            'Enable','on','String',ccdimginfo.sphifile,...
            'TooltipString','Browse for existing result file');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSphiMethod'),'Enable','on');
    case {2,3}
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),...
            'Enable','on','String',num2str(ccdimginfo.snophi),...
            'TooltipString','Enter number of partitions');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSphiMethod'),'Enable','off');
    case 4
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),...
            'Enable','on','String',ccdimginfo.sphispanstr,...
            'TooltipString','Enter static phi span (start:step:end)');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSphiMethod'),'Enable','off');
    case 5
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),'Enable','off','String','');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonSphiMethod'),'Enable','off');
end


%==========================================================================
% --- PopupmenuDqMethod callback fcn
%==========================================================================
function figviewanalysis_PopupmenuDqMethodCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig');
ccdimginfo = getappdata(findall(0,'Tag','xpcsmain_Fig'),'ccdimginfo');
switch get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDqMethod'),'Value')
    case 1
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),...
            'Enable','on','String',ccdimginfo.dqfile,...
            'TooltipString','Browse for existing result file');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDqMethod'),'Enable','on');
    case {2,3}
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),...
            'Enable','on','String',num2str(ccdimginfo.dnoq),...
            'TooltipString','Enter number of partitions');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDqMethod'),'Enable','off');
    case 4
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),...
            'Enable','on','String',ccdimginfo.dqspanstr,...
            'TooltipString','Enter dynamic q span (start:step:end)');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDqMethod'),'Enable','off');
    case 5
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),'Enable','off','String','');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDqMethod'),'Enable','off');
end


%==========================================================================
% --- PopupmenuDphiMethod callback fcn
%==========================================================================
function figviewanalysis_PopupmenuDphiMethodCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig');
ccdimginfo = getappdata(findall(0,'Tag','xpcsmain_Fig'),'ccdimginfo');
switch get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDphiMethod'),'Value')
    case 1
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),...
            'Enable','on','String',ccdimginfo.dphifile,...
            'TooltipString','Browse for existing result file');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDphiMethod'),'Enable','on');
    case {2,3}
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),...
            'Enable','on','String',num2str(ccdimginfo.dnophi),...
            'TooltipString','Enter number of partitions');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDphiMethod'),'Enable','off');
    case 4
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),...
            'Enable','on','String',ccdimginfo.dphispanstr,...
            'TooltipString','Enter dynamic phi span (start:step:end)');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDphiMethod'),'Enable','off');
    case 5
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),'Enable','off','String','');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonDphiMethod'),'Enable','off');
end


%==========================================================================
% --- Radiobutton mask callback fcn
%==========================================================================
function figviewanalysis_RadiobuttonMaskCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
switch hObject
    case findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask1')
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask1')  ,'value',1)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask2')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask3')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonNewMask') ,'enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonMaskFile'),'enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_maskfile')          ,'enable','off');
    case findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask2')
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask1')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask2')  ,'value',1)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask3')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonNewMask') ,'enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonMaskFile'),'enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_maskfile')          ,'enable','off');        
    case findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask3')
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask1')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask2')  ,'value',0)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask3')  ,'value',1)     ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonNewMask') ,'enable','off');
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_PushbuttonMaskFile'),'enable','on') ;
        set(findall(hFigViewanalysis,'Tag','figviewanalysis_maskfile')          ,'enable','on') ;        
end 


%==========================================================================
%--- LLD radiobuttons callback fcn
%==========================================================================
function figviewanalysis_RadiobuttonLLDCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                              ;
hRadiobuttonLLD1 = findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonLLD1');
hRadiobuttonLLD2 = findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonLLD2');
hRadiobuttonLLD3 = findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonLLD3');
hEditLLD2        = findall(hFigViewanalysis,'Tag','figviewanalysis_EditLLD2')       ;
hEditLLD3        = findall(hFigViewanalysis,'Tag','figviewanalysis_EditLLD3')       ;
switch hObject
    case hRadiobuttonLLD1
        set(hRadiobuttonLLD1,'value',1)                                    ;
        set(hRadiobuttonLLD2,'value',0)                                    ;
        set(hRadiobuttonLLD3,'value',0)                                    ;
        set(hEditLLD2,'Enable','off')                                      ;
        set(hEditLLD3,'Enable','off')                                      ;
    case hRadiobuttonLLD2
        set(hRadiobuttonLLD1,'value',0)                                    ;
        set(hRadiobuttonLLD2,'value',1)                                    ;
        set(hRadiobuttonLLD3,'value',0)                                    ;
        set(hEditLLD2,'Enable','on')                                       ;
        set(hEditLLD3,'Enable','off')                                      ;
    case hRadiobuttonLLD3
        set(hRadiobuttonLLD1,'value',0)                                    ;
        set(hRadiobuttonLLD2,'value',0)                                    ;
        set(hRadiobuttonLLD3,'value',1)                                    ;
        set(hEditLLD2,'Enable','off')                                      ;
        set(hEditLLD3,'Enable','on')                                       ;
end


%==========================================================================
%--- Popupmenu Ssn callback fcn
%==========================================================================
function figviewanalysis_PopupmenuSsnCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                           ;
hPopupmenuSsn = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSsn')   ;
if ( get(hPopupmenuSsn,'value')==1 )
    set(findall(hFigViewanalysis,'Tag','figviewanalysis_ssnmin'),'Enable','off') ;
else
    set(findall(hFigViewanalysis,'Tag','figviewanalysis_ssnmin'),'Enable','off') ;
end


%==========================================================================
%--- Popupmenu display settings callback functions
%==========================================================================
function figviewanalysis_PopupmenuASCIICallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                                     ;
hPopupmenuASCII  = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuASCII')        ;

function figviewanalysis_PopupmenuFIT1CallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                                     ;
hPopupmenuFIT1   = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT1')         ;

function figviewanalysis_PopupmenuFIT2CallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                                     ;
hPopupmenuFIT2   = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT2')         ;

function figviewanalysis_PopupmenuPNGCallbackFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                                     ;
hPopupmenuPNG    = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuPNG')          ;

function figviewanalysis_PopupmenuCUSTOMFITCallbackFcn(hObject,eventdata)
hFigViewanalysis    = findall(0,'Tag','viewanalysis_Fig')                                  ;
hPopupmenuCUSTOMFIT = findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuCUSTOMFIT') ;


%==========================================================================
%--- Close figure close callback fcn
%==========================================================================
function viewanalysis_CloseRequestFcn(hObject,eventdata)
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
% --- call viewanalysis_ApplyFcn
viewanalysis_ApplyFcn                                                      ;
% --- close figure windows
delete(findall(0,'Tag','mask_Fig'))                                        ;
delete(findall(0,'Tag','showmaskpartition_Fig'))                           ;
delete(hFigViewanalysis)                                                   ;


%==========================================================================
% --- Apply pushbutton callback fcn (save information, do mapping)
%==========================================================================
function viewanalysis_ApplyFcn(hObject,eventdata)
hFigXPCSMain     = findall(0,'Tag','xpcsmain_Fig')                         ;
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
ccdimginfo       = getappdata(hFigXPCSMain,'ccdimginfo')                   ;


% =========================================================================
% --- change status bar and disable pushbuttons
% =========================================================================
hEditStatus  = findall(hFigViewanalysis,'Tag','figviewanalysis_TextStatus');
originalColor = get(hEditStatus,'Color')                                   ;
originalText  = get(hEditStatus,'String')                                  ;
set(hEditStatus,'String','Status: Applying changes and masking image ...','color','r')    ;
set(findall(hFigViewanalysis,'Tag','viewanalysis_PushbuttonClose')       ,'Enable','off') ;
set(findall(hFigViewanalysis,'Tag','viewanalysis_PushbuttonApply')       ,'Enable','off') ;
set(findall(hFigViewanalysis,'Tag','viewanalysis_PushbuttonShow')        ,'Enable','off') ;
pause(0.001)                                                               ;


% =========================================================================
% --- get analysis type
% =========================================================================
ccdimginfo.analysistype = 1                                                ; % assume dynamic analysis
if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonAnalysisTypeS'),'value') == 1 )
    ccdimginfo.analysistype = 0                                            ; % if needed change to static analysis
end


% =========================================================================
% --- get static q partition information
% =========================================================================
ccdimginfo.sqMethod = get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSqMethod'),'Value')    ;
switch ccdimginfo.sqMethod
    case 1          % get existing file name
        ccdimginfo.sqfile    = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'),'String')      ;
    case {2,3}      % get # of partitions
        ccdimginfo.snoq      = str2double(get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'),'String'))  ;
    case 4          % get q span string
        ccdimginfo.sqspanstr = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnoq'),'String')   ;
    case 5          % do nothing
end


% =========================================================================
% --- get static phi partition information
% =========================================================================
ccdimginfo.sphiMethod = get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuSphiMethod'),'Value')          ;
switch ccdimginfo.sphiMethod
    case 1          % get existing file name
        ccdimginfo.sphifile = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),'String')            ;
    case {2,3}      % get # of partitions
        ccdimginfo.snophi   = str2double(get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),'String'));
    case 4          % get q span string
        ccdimginfo.sphispanstr = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditSnophi'),'String')         ;
    case 5          % do nothing
end


% =========================================================================
% --- get dynamic partition information if necessary
% =========================================================================
if ( ccdimginfo.analysistype == 1 )
    % --- get dynamic q partition information
    ccdimginfo.dqMethod = get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDqMethod'),'Value')               ;
    switch ccdimginfo.dqMethod
        case 1          % get existing file name
            ccdimginfo.dqfile    = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),'String')              ;
        case {2,3}      % get # of partitions
            ccdimginfo.dnoq      = str2double(get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),'String'))  ;
        case 4          % get q span string
            ccdimginfo.dqspanstr = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnoq'),'String')              ;
        case 5          % do nothing
    end
    % --- get dynamic phi partition information
    ccdimginfo.dphiMethod = get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuDphiMethod'),'Value')              ;
    switch ccdimginfo.dphiMethod
        case 1          % get existing file name
            ccdimginfo.dphifile    = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),'String')             ;
        case {2,3}      % get # of partitions
            ccdimginfo.dnophi      = str2double(get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),'String')) ;
        case 4          % get q span string
            ccdimginfo.dphispanstr = get(findall(hFigViewanalysis,'Tag','figviewanalysis_EditDnophi'),'String')             ;
        case 5          % do nothing
    end
end


% =========================================================================
% --- get mask information
% =========================================================================
xpixels = ccdimginfo.col_end-ccdimginfo.col_beg+1                          ;
if ( ccdimginfo.kinetics == 1 )
    ypixels = ccdimginfo.kinwinsize                                        ;
else
    ypixels = ccdimginfo.row_end-ccdimginfo.row_beg+1                      ;
end
switch 1
    case get(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask1'),'Value')
        ccdimginfo.maskMethod = 1;
        ccdimginfo.maskpoints = [1       1;...
                                 xpixels  1;...
                                 xpixels  ypixels;...
                                 1        ypixels]                         ;
        % --- define mask roi as corner points of the whole slice/image roi
        if ccdimginfo.kinetics == 1
            ccdimginfo.maskroi = [1              1                    ; ...
                                  xpixels        ccdimginfo.kinwinsize ]   ;
        else
            ccdimginfo.maskroi = [1              1                 ;    ...
                                  xpixels        ypixels            ]      ;
        end
        ccdimginfo.maskfile   = ''                                         ;
        setappdata(hFigViewanalysis,'maskpoints',ccdimginfo.maskpoints)    ;
        
    case get(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask2'),'Value')
        ccdimginfo.maskMethod = 2                                          ;
        maskpoints = getappdata(hFigViewanalysis,'maskpoints')             ;
        if ( ~isempty(maskpoints) )
            ccdimginfo.maskpoints = maskpoints                             ;
            ccdimginfo.maskroi    =                                     ...
                [min(ccdimginfo.maskpoints(:,1)) min(ccdimginfo.maskpoints(:,2)); ...
                 max(ccdimginfo.maskpoints(:,1)) max(ccdimginfo.maskpoints(:,2))]    ;
        else
            hMSGBox = msgbox('Invalid polygon --> use whole image!'     ...
                            ,'Mask Warning Message','warn','modal')        ;
            set(hMSGBox,'color',[1 1 0.85])                                ;
            uiwait(hMSGBox)                                                ;
            pause(0.010)                                                   ;
            ccdimginfo.maskpoints = [1       1;...
                                     xpixels  1;...
                                     xpixels  ypixels;...
                                     1        ypixels];
            % --- define mask roi as corner points of the slice/image roi
            if ccdimginfo.kinetics == 1
                ccdimginfo.maskroi = [1          1                    ; ...
                                      xpixels    ccdimginfo.kinwinsize ]   ;
            else
                ccdimginfo.maskroi = [1          1                 ;    ...
                                      xpixels    ypixels            ]      ;
            end
        end
        ccdimginfo.maskfile   = ''                                         ;
        setappdata(hFigViewanalysis,'maskpoints',ccdimginfo.maskpoints)    ;

 
    case get(findall(hFigViewanalysis,'Tag','figviewanalysis_RadiobuttonMask3'),'Value')
        
        maskfile = get(findall(hFigViewanalysis,'Tag','figviewanalysis_maskfile'),'String')        ;
        custommaskload(maskfile)                                           ; % this updates the main application data
        ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                 ; % get the updated application data
        maskpoints = ccdimginfo.maskpoints                                 ;
        setappdata(hFigViewanalysis,'maskpoints',maskpoints)               ;
        clear maskfile maskpoints                                          ;

end


% =========================================================================
% --- calculate the logical (1/0) usermask
% =========================================================================
if isappdata(hFigViewanalysis,'maskflag')
    maskflag = getappdata(hFigViewanalysis,'maskflag')                     ;
    if ~isequal(maskflag,ccdimginfo.maskpoints)
        ccdimginfo.usermask =                                           ...
            outsidepolygonmask(ones(ypixels,xpixels),                   ...
                ccdimginfo.maskpoints(:,1),ccdimginfo.maskpoints(:,2) )    ;
    end
else
    ccdimginfo.usermask =                                               ...
        outsidepolygonmask(ones(ypixels,xpixels),                       ...
            ccdimginfo.maskpoints(:,1),ccdimginfo.maskpoints(:,2) )        ;
end
maskflag = ccdimginfo.maskpoints                                           ;
setappdata(hFigViewanalysis,'maskflag',maskflag)                           ;


% =========================================================================
% --- get dynamic analysis settings
% =========================================================================
if ccdimginfo.analysistype == 1
    if ~isnan(str2double(get(findall(hFigViewanalysis,'Tag',            ...
                            'figviewanalysis_dpl'),'string')))
        ccdimginfo.dpl = str2double(get(findall(hFigViewanalysis,'Tag', ...
                                       'figviewanalysis_dpl'),'string'))   ;
    end
    if ~isnan(str2double(get(findall(hFigViewanalysis,'Tag',            ...
                                    'figviewanalysis_slicedpl'),'string')));
        ccdimginfo.slicedpl = str2double(get(findall(hFigViewanalysis,  ...
                              'Tag','figviewanalysis_slicedpl'),'string')) ;
    end
    if ( get(findall(hFigViewanalysis,'Tag',                            ...
            'figviewanalysis_PopupmenuSsn'),'Value') == 1 )
        ccdimginfo.ssn = 1                                                 ;
        if ~isnan(str2double(get(findall(hFigViewanalysis,'Tag',...
                                'figviewanalysis_ssnmin'),'string')))
            ccdimginfo.ssnmin = str2double(get(findall(hFigViewanalysis,...
                                'Tag','figviewanalysis_ssnmin'),'string')) ;
        end
    elseif ( get(findall(hFigViewanalysis,'Tag',                        ...
                'figviewanalysis_PopupmenuSsn'),'Value') == 2 )
        ccdimginfo.ssn = 0                                                 ;
    end
end


% =========================================================================
% --- get lld
% =========================================================================
switch 1
    case get(findall(hFigViewanalysis,'Tag',                            ...
            'figviewanalysis_RadiobuttonLLD1'),'value')
        ccdimginfo.lld = 0                                                 ;
        
    case get(findall(hFigViewanalysis,'Tag',                            ...
            'figviewanalysis_RadiobuttonLLD2'),'value')
        if ~isnan(str2double(get(findall(hFigViewanalysis,'Tag',        ...
                            'figviewanalysis_EditLLD2'),'string')))
            ccdimginfo.lld = -abs(str2double(get(findall(               ...
                                  hFigViewanalysis,'Tag',               ...
                                  'figviewanalysis_EditLLD2'),'string')))  ;
        end
        
    case get(findall(hFigViewanalysis,'Tag',                            ...
            'figviewanalysis_RadiobuttonLLD3'),'value')
        if ~isnan(str2double(get(findall(hFigViewanalysis,'Tag',        ...
                                'figviewanalysis_EditLLD3'),'string')))
            ccdimginfo.lld = abs(str2double(get(findall(                ...
                                 hFigViewanalysis,'Tag',                ...
                                 'figviewanalysis_EditLLD3'),'string')))   ;
        end
end


% =========================================================================
% --- get saving folder information
% =========================================================================
savepath = get(findall(hFigViewanalysis,'Tag',                          ...
              'figviewanalysis_savepath'),'String')                        ;
if isdir(savepath)
    ccdimginfo.savepath = savepath                                         ; % directory exists already
else
    hEditStatus = findall(hFigViewanalysis,'Tag',                       ...
                         'figviewanalysis_TextStatus')                     ;
    str1 = 'Status: Try to create saving directory ...'                    ;
    set(hEditStatus,'String',str1,'color','r')                             ;
    pause(0.01)                                                            ;
    % ---
    [success,message,messageID] = mkdir(savepath)                          ;
    % ---
    if ( success == 1 )
        ccdimginfo.savepath = savepath                                     ; % the saving directory was successfully created
    else
        hEditStatus = findall(hFigViewanalysis,'Tag',                   ...
                             'figviewanalysis_TextStatus')                 ;
        str1 = {['Status: Failed to create saving directory ...'];      ...
                ['        Try to save in Matlab XPCSGUI folder!']}         ;
        set(hEditStatus,'String',str1,'color','r')                         ;
        pause(0.01)                                                        ;
        % ---
        savepath = fullfile(pwd,'result')                                  ; % try to create a subdirectory in the Matlab XPCSGUI folder
        % ---
        [success,message,messageID] = mkdir(savepath)                      ;
        % ---
        if ( success == 1 )
            ccdimginfo.savepath = savepath                                 ; % the 'emergency' saving directory was successfully created
        else
            hEditStatus = findall(hFigViewanalysis,'Tag',               ...
                                 'figviewanalysis_TextStatus')             ;
            str1 = {['Status: Failed to create saving directory !!!']}     ;
            set(hEditStatus,'String',str1,'color','r')                     ;
            pause(0.01)                                                    ;
            % ---
            ccdimginfo.savepath = pwd                                      ; % set the saving directory to the XPCSGUI folder
            set(findall(hFigViewanalysis,'Tag',                         ...
                'figviewanalysis_savepath'),'String',ccdimginfo.savepath)  ;
            % ---
            uiwait(msgbox('Failed in creating new directory ...',       ...
                          'Error Message','error','modal'))                ;
        end        
    end
end


% =========================================================================
% --- get saving prefix information
% =========================================================================
ccdimginfo.name = get(findall(hFigViewanalysis,'Tag',                   ...
                             'figviewanalysis_prefix'),'string')           ;


% =========================================================================
% --- get display settings
% =========================================================================
if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuASCII'),'Value') == 1 )
    ccdimginfo.ASCIIon = 1                                                 ;
elseif ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuASCII'),'Value') == 2 )
    ccdimginfo.ASCIIon = 0                                                 ;
end

if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT1'),'Value') == 1 )
    ccdimginfo.FIT1on = 1                                                 ;
elseif ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT1'),'Value') == 2 )
    ccdimginfo.FIT1on = 0                                                 ;
end

if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT2'),'Value') == 1 )
    ccdimginfo.FIT2on = 1                                                 ;
elseif ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuFIT2'),'Value') == 2 )
    ccdimginfo.FIT2on = 0                                                 ;
end

if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuPNG'),'Value') == 1 )
    ccdimginfo.PNGon = 1                                                 ;
elseif ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuPNG'),'Value') == 2 )
    ccdimginfo.PNGon = 0                                                 ;
end

if ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuCUSTOMFIT'),'Value') == 1 )
    ccdimginfo.CUSTOMFITon = 1                                                 ;
elseif ( get(findall(hFigViewanalysis,'Tag','figviewanalysis_PopupmenuCUSTOMFIT'),'Value') == 2 )
    ccdimginfo.CUSTOMFITon = 0                                                 ;
end

                         
% =========================================================================
% --- save ccdimginfo to main figure
% =========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;

% --- set status bar back to Ready
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonClose'),'Enable','on')                  ;
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonApply'),'Enable','on')                  ;
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonShow'),'Enable','on')                   ;
set(hEditStatus,'String',originalText,'color',originalColor)               ;


% =========================================================================
% --- Show Mask & Partitions pushbutton callback fcn
% =========================================================================
function viewanalysis_ShowFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
hFigViewanalysis = findall(0,'Tag','viewanalysis_Fig')                     ;
% --- call viewanalysis_ApplyFcn to save information
viewanalysis_ApplyFcn                                                      ;
% --- change status bar and disable pushbuttons
hEditStatus   = findall(hFigViewanalysis,'Tag',                         ...
                       'figviewanalysis_TextStatus')                       ;
originalColor = get(hEditStatus,'Color')                                   ;
originalText  = get(hEditStatus,'String')                                  ;
set(hEditStatus,'String',                                               ...
   'Status: Loading, mapping image and drawing contours ...','color','r')  ;
set(findall(hFigViewanalysis,'Tag',                                     ...
   'viewanalysis_PushbuttonClose'),'Enable','off')                         ;
set(findall(hFigViewanalysis,'Tag',                                     ...
   'viewanalysis_PushbuttonApply'),'Enable','off')                         ;
set(findall(hFigViewanalysis,'Tag',                                     ...
   'viewanalysis_PushbuttonShow'),'Enable','off')                          ;
pause(0)                                                                   ;


% =========================================================================
% --- try mapping and calculate span of sq, sphi,dq, dphi
% =========================================================================
mapccdpixel                                                                ;
getsdqphispan                                                              ;
qphipartition                                                              ;


% =========================================================================
% --- call showmaskpartition function to show
% =========================================================================
% profile on; profile clear;
showmaskpartition                                                          ;
% profreport('showmaskpartition');


% =========================================================================
% --- set status bar back to Ready
% =========================================================================
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonClose'),'Enable','on')                  ;
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonApply'),'Enable','on')                  ;
set(findall(hFigViewanalysis,'Tag',                                     ...
           'viewanalysis_PushbuttonShow'),'Enable','on')                   ;
set(hEditStatus,'String',originalText,'color',originalColor)               ;


% ---
% EOF
