function viewinfo(varargin)
% VIEWINFO GUI to display and edit batchinfo.
%
% Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/12/13 $

% --- get ccdimginfo
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig');
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo');
else
    return;
end

% --- if viewbatchinfo fig exists, display it in front; else create it
hFigViewinfo = findall(0,'Tag','viewinfo_Fig');
if ~isempty(hFigViewinfo)  
    figure(hFigViewinfo);
    return;
end

% --- figure layout
backgroundcolor = [1 1 0.85];
hFigXPCSMainPos = get(hFigXPCSMain,'position');
figViewinfoSize = [720 540];
hFigViewinfo = figure(...
    'BackingStore','on',...
    'Units','pixels',...
    'DockControls','off',...
    'Resize','off',...
    'PaperOrient','portrait',...
    'PaperPositionMode','auto',...
    'IntegerHandle','off',...
    'NumberTitle','off',...
    'MenuBar','none',...
    'Toolbar','none',...
    'Windowstyle','modal',...
    'CloseRequestFcn',@viewinfo_CloseRequestFcn,...
    'Name',['XPCS - ',ccdimginfo.info_name],...
    'Position',[hFigXPCSMainPos(1)+ 150,hFigXPCSMainPos(2)+ 60,figViewinfoSize],...
    'HandleVisibility','callback',...
    'Tag','viewinfo_Fig',...
    'UserData',[]);
hAxes = axes('Parent',hFigViewinfo,...
    'Units','pixels',...
    'Position',[0 0 figViewinfoSize],...
    'Tag','figviewinfo_Axes');
hPatch1 = patch('Parent',hAxes,'XData',[0 1 1 0],'YData',[0 0 1 1],'FaceColor',backgroundcolor,'EdgeColor',backgroundcolor);
hGroupPatch = hggroup('Parent',hAxes);
hPatch2 = patch('Parent',hGroupPatch,'XData',[0.02 0.49 0.49 0.02],'YData',[0.1 0.1 0.925 0.925]);
hPatch3 = patch('Parent',hGroupPatch,'XData',[0.51 0.98 0.98 0.51],'YData',[0.1 0.1 0.925 0.925]);
set(get(hGroupPatch,'Children'),'FaceColor',backgroundcolor+[0 0 0.05],'EdgeColor',[0.7 0.7 0.7]);
hGroupText      = hggroup('Parent',hAxes);
hGroupSubText   = hggroup('Parent',hAxes);
hGroupSubText1   = hggroup('Parent',hAxes);
text('Parent',hAxes,'Position',[0.4 0.965],'String','Batch Information','FontSize',12,'FontWeight','demi','color',[0.4 0.3 0]);
text('Parent',hGroupText,'Position',[0.03 0.9],'String','Sample-Detector Distance (mm):');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'units','normalized',...
    'backgroundcolor','w',...
    'Position',[0.38 0.88 0.1 0.035],...
    'HorizontalAlignment','right',...
    'String',num2str(ccdimginfo.rr),...
    'Tag','figviewinfo_rr');
text('Parent',hGroupText,'Position',[0.03 0.86],'String','Beam Zero Parameters:');
text('Parent',hGroupSubText,'Position',[0.06 0.82],'String','x-coordinate in full frame mode');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'backgroundcolor','w',...
    'Position',[0.38 0.8 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'String',num2str(ccdimginfo.x0),...
    'Tag','figviewinfo_x0');
text('Parent',hGroupSubText,'Position',[0.06 0.78],'String','y-coordinate in full frame mode');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.76 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.y0),...
    'Tag','figviewinfo_y0');
text('Parent',hGroupSubText,'Position',[0.06 0.74],'String','ccdx0 position in beam0 measurement');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.72 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ccdx0),...
    'Tag','figviewinfo_ccdx0');
text('Parent',hGroupSubText,'Position',[0.06 0.70],'String','ccdz0 position in beam0 measurement');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.68 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ccdz0),...
    'Tag','figviewinfo_ccdz0');
text('Parent',hGroupText,'Position',[0.03 0.66],'String','Specularly Reflected Beam Parameters:');
text('Parent',hGroupSubText,'Position',[0.06 0.62],'String','x-coordinate in full frame mode');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.60 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.xspec),...
    'Tag','figviewinfo_xspec');
text('Parent',hGroupSubText,'Position',[0.06 0.58],'String','y-coordinate in full frame mode');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.56 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.yspec),...
    'Tag','figviewinfo_yspec');
text('Parent',hGroupSubText,'Position',[0.06 0.54],'String','ccdxspec in specular measurment');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.52 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ccdxspec),...
    'Tag','figviewinfo_ccdxspec');
text('Parent',hGroupSubText,'Position',[0.06 0.50],'String','ccdzspec in specular measurment');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.48 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ccdzspec),...
    'Tag','figviewinfo_ccdzspec');
% --- for transmission geometry, disable parameters related to specularly
% reflected beam
if ccdimginfo.geometry == 0
    set(findall(hFigViewinfo,'Tag','figviewinfo_xspec'),'Enable','off');
    set(findall(hFigViewinfo,'Tag','figviewinfo_yspec'),'Enable','off');
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdxspec'),'Enable','off');
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdzspec'),'Enable','off');
end
text('Parent',hGroupText,'Position',[0.03 0.46],'String','Saved CCD Image Size:');
text('Parent',hGroupSubText,'Position',[0.1 0.38],'String','begin');
text('Parent',hGroupSubText,'Position',[0.1 0.34],'String','  end');
text('Parent',hGroupSubText,'Position',[0.16 0.42],'String','column (x)');
text('Parent',hGroupSubText,'Position',[0.33 0.42],'String','   row (y)');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.15 0.36 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.col_beg),...
    'Tag','figviewinfo_col_beg');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.15 0.32 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.col_end),...
    'Tag','figviewinfo_col_end');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.32 0.36 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.row_beg),...
    'Tag','figviewinfo_row_beg');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.32 0.32 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.row_end),...
    'Tag','figviewinfo_row_end');
text('Parent',hGroupText,'Position',[0.03 0.30],'String','CCD Position During Data Collection:');
text('Parent',hGroupSubText,'Position',[0.1 0.26],'String','ccdx');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.15 0.24 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
   'String',num2str(ccdimginfo.ccdx),...
    'Tag','figviewinfo_ccdx');
text('Parent',hGroupSubText1,'Position',[0.29 0.26],'String','ccdz');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.32 0.24 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ccdz),...
    'Tag','figviewinfo_ccdz');
text('Parent',hGroupText,'Position',[0.03 0.22],'String','Experiment Setup Geometry:');
if ccdimginfo.geometry == 1
    uicontrol('Parent',hFigViewinfo,...
        'Style','Edit',...
        'Enable','inactive',...
        'Units','normalized',...
        'Position',[0.32 0.20 0.16 0.035]',...
        'HorizontalAlignment','right',...
        'backgroundcolor','w',...
        'String','Reflection');
else
    uicontrol('Parent',hFigViewinfo,...
        'Style','Edit',...
        'Enable','inactive',...
        'Units','normalized',...
        'Position',[0.32 0.20 0.16 0.035]',...
        'HorizontalAlignment','right',...
        'backgroundcolor','w',...
        'String','Transmission');
end
text('Parent',hGroupText,'Position',[0.03 0.18],'String','X-Ray Photon Energy (KeV):');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.16 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.energy),...
    'Tag','figviewinfo_energy');
text('Parent',hGroupText,'Position',[0.03 0.14],'String','Nominal Angle:');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.38 0.12 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.nominal_angle),...
    'Tag','figviewinfo_nominal_angle');
text('Parent',hGroupText,'Position',[0.53 0.9],'String','CCD Working Mode:');
if ccdimginfo.kinetics == 1
    uicontrol('Parent',hFigViewinfo,...
        'Style','Edit',...
        'Units','normalized',...
        'Enable','inactive',...
        'Position',[0.85 0.88 0.12 0.035],...
        'HorizontalAlignment','right',...
        'backgroundcolor','w',...
        'String','Kinetics');
else
    uicontrol('Parent',hFigViewinfo,...
        'Style','Edit',...
        'Units','normalized',...
        'Enable','inactive',...
        'Position',[0.85 0.88 0.12 0.035],...
        'HorizontalAlignment','right',...
        'backgroundcolor','w',...
        'String','Non-Kinetics');
end
text('Parent',hGroupSubText,'Position',[0.56 0.86],'String','kinetics window size');
hEditKinetics1 = uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.87 0.84 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.kinwinsize),...
    'Tag','figviewinfo_kinwinsize');
text('Parent',hGroupSubText,'Position',[0.56 0.82],'String','top row number of visible slice');
hEditKinetics2 = uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.87 0.80 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'Enable','inactive',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.slicetop),...
    'Tag','figviewinfo_slicetop');
text('Parent',hGroupSubText,'Position',[0.56 0.78],'String','first usable kinetics slice');
hEditKinetics3 = uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.87 0.76 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.firstslice),...
    'Tag','figviewinfo_firstslice');
text('Parent',hGroupSubText,'Position',[0.56 0.74],'String','last usable kinetics slice');
hEditKinetics4 = uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.87 0.72 0.1 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.lastslice),...
    'Tag','figviewinfo_lastslice');
if ccdimginfo.kinetics ~= 1
    set([hEditKinetics1,hEditKinetics2,hEditKinetics3,hEditKinetics4],'Enable','off');
end
text('Parent',hGroupText,'Position',[0.53 0.6975],'String','View and Edit Batch #');
uicontrol('Parent',hFigViewinfo,...
    'Style','Popupmenu',...
    'Units','normalized',...
    'Position',[0.725 0.68 0.06 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',cellstr(num2str([1:length(ccdimginfo.ndata0)]')),...
    'Tag','figviewinfo_PopupmenuBatchnumber',...
    'callback',@viewinfo_PopupmenuBatchnumberCallbackFcn);
text('Parent',hGroupText,'Position',[0.8 0.6975],'String',['of ',num2str(length(ccdimginfo.ndata0)),' Batches:']);
text('Parent',hGroupSubText,'Position',[0.68 0.66],'String','start');
text('Parent',hGroupSubText,'Position',[0.78 0.66],'String','end');
text('Parent',hGroupSubText,'Position',[0.855 0.66],'String','time (sec)');
text('Parent',hGroupSubText,'Position',[0.6 0.62],'String','data');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.65 0.60 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ndata0todo(1)),...
    'Tag','figviewinfo_ndata0todo');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.75 0.60 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ndataendtodo(1)),...
    'Tag','figviewinfo_ndataendtodo');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.85 0.60 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'Enable','inactive',...
    'String',num2str(ccdimginfo.preset(1)),...
    'Tag','figviewinfo_preset');
text('Parent',hGroupSubText,'Position',[0.6 0.58],'String','dark');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.65 0.56 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ndark0todo(1)),...
    'Tag','figviewinfo_ndark0todo');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.75 0.56 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',num2str(ccdimginfo.ndarkendtodo(1)),...
    'Tag','figviewinfo_ndarkendtodo');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.85 0.56 0.085 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'Enable','inactive',...
    'String',num2str(ccdimginfo.dark_preset(1)),...
    'Tag','figviewinfo_dark_preset');
text('Parent',hGroupSubText,'Position',[0.6 0.54],'String',' flux');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.65 0.52 0.15 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'Enable','inactive',...
    'String',num2str(ccdimginfo.beam_i_vacuum(1),'%g'),...
    'Tag','figviewinfo_beam_i_vacuum');
text('Parent',hGroupSubText,'Position',[0.82 0.54],'String','(photons/sec)');
text('Parent',hGroupSubText,'Position',[0.58 0.5],'String','current');
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.65 0.48 0.15 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'Enable','inactive',...
    'String',num2str(ccdimginfo.ring_i_vacuum(1),'%g'),...
    'Tag','figviewinfo_ring_i_vacuum');
text('Parent',hGroupSubText,'Position',[0.82 0.5],'String','(mA)');
text('Parent',hGroupText,'Position',[0.53 0.46],'String','Batches To Be Analyzed:');
str_batchestodo = '';
for iBatches = 1:length(ccdimginfo.batchestodo)
    str_batchestodo = [str_batchestodo,',',int2str(ccdimginfo.batchestodo(iBatches));];
end
str_batchestodo(1) = '';
uicontrol('Parent',hFigViewinfo,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.8 0.44 0.15 0.035]',...
    'HorizontalAlignment','right',...
    'backgroundcolor','w',...
    'String',str_batchestodo,...
    'Tag','figviewinfo_batchestodo');
clear iBatches str_batchestodo;

% --- set status bar
text('Parent',hGroupText,'Position',[0.03 0.05],'String','Status: Ready!','Tag','figviewinfo_TextStatus');

% --- set gourp properties
set(get(hGroupText,'Children'),...
    'Units','normalized',...
    'HorizontalAlignment','left',...
    'Color',[0.4 0.3 0]);
set(get(hGroupSubText,'Children'),...
    'Units','normalized',...
    'HorizontalAlignment','left',...
    'Color','b');
set(get(hGroupSubText1,'Children'),...
    'Units','normalized',...
    'HorizontalAlignment','center',...
    'Color','b');
hPushbuttonClose = uicontrol(hFigViewinfo,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Close',...
    'position',[0.55 0.02 0.125 0.05],...
    'Tag','viewinfo_PushbuttonClose',...
    'TooltipString','Close window',...
    'callback',@viewinfo_CloseRequestFcn);
hPushbuttonApply = uicontrol(hFigViewinfo,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Apply',...
    'position',[0.7 0.02 0.125 0.05],...
    'Tag','viewinfo_PushbuttonApply',...
    'TooltipString','Apply changes',...
    'callback',@viewinfo_PushbuttonApplyFcn);
hPushbuttonApplyShowImage = uicontrol(hFigViewinfo,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Show Image',...
    'position',[0.85 0.02 0.125 0.05],...
    'Tag','viewinfo_PushbuttonApplyShowImage',...
    'TooltipString','Apply changes and show image',...
    'callback',@viewinfo_ApplyShowImageFcn);


%================================================================
% --- Viewinfo PopupmenuBatchnumber Callback Function
%================================================================
function viewinfo_PopupmenuBatchnumberCallbackFcn(hObject,eventdata)
iBatch = get(gco,'value')                                                                                      ;
ccdimginfo = getappdata(findall(0,'Tag','xpcsmain_Fig'),'ccdimginfo')                                          ;
set(findobj(gcf,'Tag','figviewinfo_ndata0todo')  ,'string',num2str(ccdimginfo.ndata0todo(iBatch)))             ;
set(findobj(gcf,'Tag','figviewinfo_ndataendtodo'),'string',num2str(ccdimginfo.ndataendtodo(iBatch)))           ;
set(findobj(gcf,'Tag','figviewinfo_preset')      ,'string',num2str(ccdimginfo.preset(iBatch)))                 ;
set(findobj(gcf,'Tag','figviewinfo_ndark0todo')  ,'string',num2str(ccdimginfo.ndark0todo(iBatch)))             ;
set(findobj(gcf,'Tag','figviewinfo_preset')      ,'string',num2str(ccdimginfo.preset(iBatch)))                 ;
set(findobj(gcf,'Tag','figviewinfo_dark_preset') ,'string',num2str(ccdimginfo.dark_preset(iBatch)))            ;
set(findobj(gcf,'Tag','figviewinfo_ndarkendtodo'),'string',num2str(ccdimginfo.ndarkendtodo(iBatch)))           ;
try 
    set(findobj(gcf,'Tag','figviewinfo_beam_i_vacuum'),'string',num2str(ccdimginfo.beam_i_vacuum(iBatch),'%g'));
catch
    set(findobj(gcf,'Tag','figviewinfo_beam_i_vacuum'),'string',num2str(ccdimginfo.beam_i_vacuum(1)     ,'%g'));
end
try
    set(findobj(gcf,'Tag','figviewinfo_ring_i_vacuum'),'string',num2str(ccdimginfo.ring_i_vacuum(iBatch),'%g'));
catch
    set(findobj(gcf,'Tag','figviewinfo_ring_i_vacuum'),'string',num2str(ccdimginfo.ring_i_vacuum(1)     ,'%g'));
end
clear ccdimginfo iBatch                                          ;


%================================================================
% --- Close figure close callback fcn
%================================================================
function viewinfo_CloseRequestFcn(hObject,eventdata)
delete(findall(0,'Tag','showimage_fig1'));
delete(findall(0,'Tag','showimage_fig2'));
delete(gcf);


%================================================================
% --- Apply pushbutton callback fcn (save changes only without update images)
%================================================================
function viewinfo_PushbuttonApplyFcn(hObject,eventdata)
viewinfo_Apply;


%================================================================
% --- Apply user changes to ccdimginfo
%================================================================
function viewinfo_Apply(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig');
hFigViewinfo = findall(0,'Tag','viewinfo_Fig');
ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo');
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_rr'),'string')))
    ccdimginfo.rr               = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_rr'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_rr'),'string',num2str(ccdimginfo.rr));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_x0'),'string')))
    ccdimginfo.x0               = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_x0'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_x0'),'string',num2str(ccdimginfo.x0));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_y0'),'string')))
    ccdimginfo.y0               = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_y0'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_y0'),'string',num2str(ccdimginfo.y0));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdx0'),'string')))
    ccdimginfo.ccdx0            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdx0'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdx0'),'string',num2str(ccdimginfo.ccdx0));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdz0'),'string')))
    ccdimginfo.ccdz0            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdz0'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdz0'),'string',num2str(ccdimginfo.ccdz0));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_xspec'),'string')))
    ccdimginfo.xspec            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_xspec'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_xspec'),'string',num2str(ccdimginfo.xspec));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_yspec'),'string')))
    ccdimginfo.yspec            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_yspec'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_yspec'),'string',num2str(ccdimginfo.yspec));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdxspec'),'string')))
    ccdimginfo.ccdxspec         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdxspec'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdxspec'),'string',num2str(ccdimginfo.ccdxspec));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdzspec'),'string')))
    ccdimginfo.ccdzspec         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdzspec'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdzspec'),'string',num2str(ccdimginfo.ccdzspec));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_col_beg'),'string')))
    ccdimginfo.col_beg         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_col_beg'),'string'));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_col_end'),'string')))
    ccdimginfo.col_end         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_col_end'),'string'));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_row_beg'),'string')))
    ccdimginfo.row_beg         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_row_beg'),'string'));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_row_end'),'string')))
    ccdimginfo.row_end         = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_row_end'),'string'));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdx'),'string')))
    ccdimginfo.ccdx            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdx'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdx'),'string',num2str(ccdimginfo.ccdx));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdz'),'string')))
    ccdimginfo.ccdz            = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ccdz'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ccdz'),'string',num2str(ccdimginfo.ccdz));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_nominal_angle'),'string')))
    ccdimginfo.nominal_angle    = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_nominal_angle'),'string'));
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_nominal_angle'),'string',num2str(ccdimginfo.nominal_angle));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_energy'),'string')))
    ccdimginfo.energy           = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_energy'),'string'));
end
if ccdimginfo.kinetics == 1
    if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_kinwinsize'),'string')))
        ccdimginfo.kinwinsize   = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_kinwinsize'),'string'));
    end
    if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_slicetop'),'string')))
        ccdimginfo.slicetop    = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_slicetop'),'string'));
    end    
    if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_firstslice'),'string')))
        ccdimginfo.firstslice  = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_firstslice'),'string'));
    else
        set(findall(hFigViewinfo,'Tag','figviewinfo_firstslice'),'string',num2str(ccdimginfo.firstslice));
    end
    if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_lastslice'),'string')))
        ccdimginfo.lastslice   = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_lastslice'),'string'));
    else
        set(findall(hFigViewinfo,'Tag','figviewinfo_lastslice'),'string',num2str(ccdimginfo.lastslice));
    end
end
iBatch = get(findall(hFigViewinfo,'Tag','figviewinfo_PopupmenuBatchnumber'),'value');
temp = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ndata0todo'),'string'));
if ~isnan(temp) & temp >= ccdimginfo.ndata0(iBatch) & temp <= ccdimginfo.ndataend(iBatch)
    ccdimginfo.ndata0todo(iBatch)           = temp;
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ndata0todo'),'string',num2str(ccdimginfo.ndata0todo(iBatch)));
end
temp = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ndataendtodo'),'string'));
if ~isnan(temp) & temp >= ccdimginfo.ndata0todo(iBatch) & temp <= ccdimginfo.ndataend(iBatch)
    ccdimginfo.ndataendtodo(iBatch)         = temp;
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ndataendtodo'),'string',num2str(ccdimginfo.ndataendtodo(iBatch)));
end
temp = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ndark0todo'),'string'));
if ~isnan(temp) & temp >= ccdimginfo.ndark0(iBatch) & temp <= ccdimginfo.ndarkend(iBatch)
    ccdimginfo.ndark0todo(iBatch)           = temp;
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ndark0todo'),'string',num2str(ccdimginfo.ndark0todo(iBatch)));
end
temp = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_ndarkendtodo'),'string'));
if ~isnan(temp) & temp >= ccdimginfo.ndark0todo(iBatch) & temp <= ccdimginfo.ndarkend(iBatch)
    ccdimginfo.ndarkendtodo(iBatch)         = temp;
else
    set(findall(hFigViewinfo,'Tag','figviewinfo_ndarkendtodo'),'string',num2str(ccdimginfo.ndarkendtodo(iBatch)));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_preset'),'string')))
    ccdimginfo.preset(iBatch)           = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_preset'),'string'));
end
if ~isnan(str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_dark_preset'),'string')))
    ccdimginfo.dark_preset(iBatch)      = str2double(get(findall(hFigViewinfo,'Tag','figviewinfo_dark_preset'),'string'));
end
batchestodo = round(str2num(get(findall(hFigViewinfo,'Tag','figviewinfo_batchestodo'),'string')));
if ~isempty(batchestodo) & min(batchestodo) >=1 & max(batchestodo) <= length(ccdimginfo.ndata0)
    ccdimginfo.batchestodo              = str2num(get(findall(hFigViewinfo,'Tag','figviewinfo_batchestodo'),'string'));
    ccdimginfo.batchestodo              = sort(ccdimginfo.batchestodo);
else
    str_batchestodo = '';
    for iBatches = 1:length(ccdimginfo.batchestodo)
        str_batchestodo = [str_batchestodo,',',int2str(ccdimginfo.batchestodo(iBatches));];
    end
    str_batchestodo(1) = '';
    set(findall(hFigViewinfo,'Tag','figviewinfo_batchestodo'),'string',str_batchestodo);
end
% --- save ccdimginfo to main figure
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo);
% --- clear memory
clear hFigXPCSMain hFigViewinfo iBatch batchestodo temp str_batchestodo;


%================================================================
% --- Show image pushbutton callback fcn
%================================================================
function viewinfo_ApplyShowImageFcn(hObject,eventdata)
hFigViewinfo = findall(0,'Tag','viewinfo_Fig');
% --- change status bar
hEditStatus  = findall(hFigViewinfo,'Tag','figviewinfo_TextStatus');
originalColor = get(hEditStatus,'Color');
originalText  = get(hEditStatus,'String');
set(hEditStatus,'String','Status: Loading Images ...','color','r');
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonClose'),'Enable','off');
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonApply'),'Enable','off');
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonApplyShowImage'),'Enable','off');
pause(0);
viewinfo_Apply;         % save changes by user
showimage;              % show image after changing parameters of batchinfo
% --- set status bar back
set(hEditStatus,'String',originalText,'color',originalColor);
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonClose'),'Enable','on');
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonApply'),'Enable','on');
set(findall(hFigViewinfo,'Tag','viewinfo_PushbuttonApplyShowImage'),'Enable','on');

% ---
% EOF