function viewresult(varargin)
% VIEWRESULT Choose options to display previously analyzed reults
%
% Michael Sprung
% $Revision: 1.0 $  $Date: 2006/01/12 $

% =========================================================================
% --- get ccdimginfo
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                     ;
else
    return                                                                 ;
end


% =========================================================================
% --- if viewresult fig exists, display it in front; else create it
% =========================================================================
hFigVR = findall(0,'Tag','VR_Fig')                                         ;
if ~isempty(hFigVR)
    figure(hFigVR)                                                         ;
    return                                                                 ;
end


% =========================================================================
% --- figure layout
% =========================================================================
backgroundcolor = [1 1 0.85]                                               ;
facecolor       = [1 1 0.9]                                                ;
textcolor       = [0.4 0.3 0]                                              ; % text color
subtextcolor    = 'b'                                                      ; % subtext color

hFigXPCSMainPos = get(hFigXPCSMain,'position')                             ;
figVRCorner     = [hFigXPCSMainPos(1)+ 150,hFigXPCSMainPos(2)+ 60]         ;
figVRSize       = [720 540]                                                ;
hFigVR    = figure('BackingStore','on','Units','pixels',                ...
                   'DockControls','off','Resize','off',                 ...
                   'PaperOrient','portrait',                            ...
                   'PaperPositionMode','auto',                          ...
                   'IntegerHandle','off','NumberTitle','off',           ...
                   'MenuBar','none','Toolbar','none',                   ...
                   'WindowStyle','normal',                              ...
                   'Name','XPCS - Result Display Settings',             ...
                   'Position',[figVRCorner,figVRSize],                  ...
                   'HandleVisibility','callback',                       ...
                   'Tag','VR_Fig','UserData',[])                           ;
               
hAxes = axes('Parent',hFigVR,'Units','pixels',...
             'Position',[0 0 figVRSize],'Tag','figVR_Axes')                ;
         
hPatch1 = patch('Parent',hAxes,'XData',[0 1 1 0],'YData',[0 0 1 1],     ...
                'FaceColor',backgroundcolor,'EdgeColor',backgroundcolor)   ;
            
% --- create the two panels
hGroupPatch = hggroup('Parent',hAxes)                                      ;
hPatch2 = patch('Parent',hGroupPatch,'XData',[0.02 0.49 0.49 0.02],     ...
                'YData',[0.1 0.1 0.925 0.925])                             ;
hPatch3 = patch('Parent',hGroupPatch,'XData',[0.51 0.98 0.98 0.51],     ...
                'YData',[0.1 0.1 0.925 0.925])                             ;
set(get(hGroupPatch,'Children'),'FaceColor',facecolor,                  ...
                                'EdgeColor',[0.7 0.7 0.7])                 ;
                            
hGroupText      = hggroup('Parent',hAxes)                                  ;
hGroupSubText   = hggroup('Parent',hAxes)                                  ;
hGroupSubText1  = hggroup('Parent',hAxes)                                  ;


% =========================================================================
% --- gui title
% =========================================================================
text('Parent',hAxes,'Position',[0.415 0.965],                           ...
     'String','Result Display Settings',                                ...
     'FontSize',16,'FontWeight','demi','color',[0.4 0.3 0])                ;
 

% =========================================================================
% --- layout for display settings
% =========================================================================

% --- subtitle
text('Parent',hGroupText,'Position',[0.03 0.9],                         ...
     'String','Show result settings:')                                     ;

% --- ASCII output field
text('Parent',hGroupSubText,'Position',[0.06 0.85],                     ...
     'String','ASCII output')                                              ;
uicontrol('Parent',hFigVR,                                              ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.25 0.82 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figVR_PopupmenuASCII',                                       ...
    'callback',@figVR_PopupmenuASCIICallbackFcn)                           ;
if ( isfield(ccdimginfo,'ASCIIon') == 1 && ccdimginfo.ASCIIon == 1 )
    set(findall(0,'Tag','figVR_PopupmenuASCII'),'value',1)                 ;
else
    set(findall(0,'Tag','figVR_PopupmenuASCII'),'value',2)                 ;
end

% --- Fit1 field
text('Parent',hGroupSubText,'Position',[0.06 0.82],                     ...
     'String','Perform Fit1')                                              ;
uicontrol('Parent',hFigVR,                                              ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.25 0.79 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figVR_PopupmenuFIT1',                                        ...
    'callback',@figVR_PopupmenuFIT1CallbackFcn)                            ;
if ( isfield(ccdimginfo,'FIT1on') == 1 && ccdimginfo.FIT1on == 1 )
    set(findall(0,'Tag','figVR_PopupmenuFIT1'),'value',1)                  ;
else
    set(findall(0,'Tag','figVR_PopupmenuFIT1'),'value',2)                  ;
end

% --- Fit2 field
text('Parent',hGroupSubText,'Position',[0.06 0.79],                     ...
     'String','Perform Fit2')                                              ;
uicontrol('Parent',hFigVR,                                              ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.25 0.76 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figVR_PopupmenuFIT2',                                        ...
    'callback',@figVR_PopupmenuFIT2CallbackFcn)                            ;
if ( isfield(ccdimginfo,'FIT2on') == 1 && ccdimginfo.FIT2on == 1 )
    set(findall(0,'Tag','figVR_PopupmenuFIT2'),'value',1)                  ;
else
    set(findall(0,'Tag','figVR_PopupmenuFIT2'),'value',2)                  ;
end

% --- Custom fit field
text('Parent',hGroupSubText,'Position',[0.06 0.76],                     ...
     'String','Perform custom fit')                                        ;
uicontrol('Parent',hFigVR,                                              ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.25 0.73 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figVR_PopupmenuCUSTOMFIT',                                   ...
    'callback',@figVR_PopupmenuCUSTOMFITCallbackFcn)                       ;
if ( isfield(ccdimginfo,'CUSTOMFITon') == 1 && ccdimginfo.CUSTOMFITon == 1)
    set(findall(0,'Tag','figVR_PopupmenuCUSTOMFIT'),'value',1)             ;
else
    set(findall(0,'Tag','figVR_PopupmenuCUSTOMFIT'),'value',2)             ;
end

% --- PNG output field
text('Parent',hGroupSubText,'Position',[0.06 0.73],                     ...
     'String','Create PNG Files')                                          ;
uicontrol('Parent',hFigVR,                                              ...
    'Style','popupmenu',                                                ...
    'Units','normalized',                                               ...
    'backgroundcolor','w',                                              ...
    'Position',[0.25 0.70 0.07 0.04]',                                  ...
    'HorizontalAlignment','right',                                      ...
    'String','yes|no',                                                  ...
    'Tag','figVR_PopupmenuPNG',                                         ...
    'callback',@figVR_PopupmenuPNGCallbackFcn)                             ;
if ( isfield(ccdimginfo,'PNGon') == 1 && ccdimginfo.PNGon == 1 )
    set(findall(0,'Tag','figVR_PopupmenuPNG'),'value',1)                   ;
else
    set(findall(0,'Tag','figVR_PopupmenuPNG'),'value',2)                   ;
end


% =========================================================================
% --- layout for choose batch
% =========================================================================

% --- subtitle
text('Parent',hGroupText,'Position',[0.03 0.6],                         ...
     'String','Choose to display results of batch # : ')                   ;

uicontrol('Parent',hFigVR,'Style','Popupmenu','Units','normalized',     ...
          'Position',[0.375 0.58 0.06 0.035]',                          ...
          'HorizontalAlignment','right','backgroundcolor','w',          ...
          'String',cellstr(num2str(ccdimginfo.batchestodo')),           ...
          'Tag','figVR_PopupmenuBatchnumber',                           ...
          'callback',@figVR_PopupmenuBatchnumberCallbackFcn)               ;
      
      
% =========================================================================
% --- layout for show function calls
% =========================================================================
% --- subtitle
text('Parent',hGroupText,'Position',[0.62 0.90],                        ...
     'String','Choose a ''showresult'' function : ')                       ;

hPushbottonResult  = uicontrol(hFigVR,'Style','pushbutton',             ...
                                      'Units','normalized',             ...
                                      'String','Show batch results',    ...
                                      'position',[0.65 0.82 0.18 0.05], ...
                                      'Tag','VR_PushbuttonShow',        ...
                                      'TooltipString','Show results',   ...
                                      'callback',@VR_ShowFcn)              ;

hPushbottonAverage = uicontrol(hFigVR,'Style','pushbutton',             ...
                                      'Units','normalized',             ...
                                      'String','Show average results',  ...
                                      'position',[0.65 0.76 0.18 0.05], ...
                                      'Tag','VR_PushbuttonShow',        ...
                                      'TooltipString','Average results',...
                                      'callback',@VR_ShowAverageFcn)       ;

hPushbottonMerge   = uicontrol(hFigVR,'Style','pushbutton',             ...
                                      'Units','normalized',             ...
                                      'String','Show merged g2 results',...
                                      'position',[0.65 0.70 0.18 0.05], ...
                                      'Tag','VR_PushbuttonShow',        ...
                                      'TooltipString','Merge results',  ...
                                      'callback',@VR_ShowMergeFcn)         ;

      
      

% =========================================================================
% --- layout of status bar
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.05],                        ...
     'String','Status: Ready!','Tag','figVR_TextStatus')                   ;

% --- set properties of group text and group sub text
set(get(hGroupText,'Children'),'Units','normalized',                    ...
    'HorizontalAlignment','left','Color',textcolor)                        ;

set(get(hGroupSubText,'Children'),'Units','normalized',                 ...
    'HorizontalAlignment','left','Color',subtextcolor)                     ;

set(get(hGroupSubText1,'Children'),'Units','normalized',                ...
    'HorizontalAlignment','left','Color',subtextcolor)                     ;


% =========================================================================
% --- layout of close, apply, show & show average pushbuttons
% =========================================================================
hPushbottonClose   = uicontrol(hFigVR,'Style','pushbutton',             ...
                                      'Units','normalized',             ...
                                      'String','Close',                 ...
                                      'position',[0.70 0.02 0.125 0.05],...
                                      'Tag','VR_PushbuttonClose',       ...
                                      'TooltipString','Close window',   ...
                                      'callback',@VR_CloseRequestFcn)      ;

hPushbottonApply   = uicontrol(hFigVR,'Style','pushbutton',             ...
                                      'Units','normalized',             ...
                                      'String','Apply',                 ...
                                      'position',[0.85 0.02 0.125 0.05],...
                                      'Tag','VR_PushbuttonApply',       ...
                                      'TooltipString','Apply changes',  ...
                                      'callback',@VR_ApplyFcn)             ;


%==========================================================================
%--- Popupmenu display settings callback functions
%==========================================================================
function figVR_PopupmenuASCIICallbackFcn(hObject,eventdata)
hFigVR              = findall(0,'Tag','VR_Fig')                            ;
hPopupmenuASCII     = findall(hFigVR,'Tag','figVR_PopupmenuASCII')         ;

function figVR_PopupmenuFIT1CallbackFcn(hObject,eventdata)
hFigVR              = findall(0,'Tag','VR_Fig')                            ;
hPopupmenuFIT1      = findall(hFigVR,'Tag','figVR_PopupmenuFIT1')          ;

function figVR_PopupmenuFIT2CallbackFcn(hObject,eventdata)
hFigVR              = findall(0,'Tag','VR_Fig')                            ;
hPopupmenuFIT2      = findall(hFigVR,'Tag','figVR_PopupmenuFIT2')          ;

function figVR_PopupmenuPNGCallbackFcn(hObject,eventdata)
hFigVR              = findall(0,'Tag','VR_Fig')                            ;
hPopupmenuPNG       = findall(hFigVR,'Tag','figVR_PopupmenuPNG')           ;

function figVR_PopupmenuCUSTOMFITCallbackFcn(hObject,eventdata)
hFigVR              = findall(0,'Tag','VR_Fig')                            ;
hPopupmenuCUSTOMFIT = findall(hFigVR,'Tag','figVR_PopupmenuCUSTOMFIT')     ;

function figVR_PopupmenuBatchnumberCallbackFcn(hObject,eventdata)
iBatch = get(gco,'value')                                                  ;


%==========================================================================
%--- Close figure close callback fcn
%==========================================================================
function VR_CloseRequestFcn(hObject,eventdata)

% --- 'Load&Show Result' window
delete(findall(0,'Tag','VR_Fig'))                                          ;

% --- Windows from 'showresult.m'
delete(findall(0,'Tag','staticaverages'))                                  ;
delete(findall(0,'Tag','intensityVSq'))                                    ;
delete(findall(0,'Tag','intensityVSphi'))                                  ;
for i = 1 : 10
    figuretag = sprintf('correlationfigure_%02i',i)                        ;
    delete(findall(0,'Tag',figuretag))                                     ;    
end
delete(findall(0,'Tag','fit1results'))                                     ;
delete(findall(0,'Tag','fit2results'))                                     ;
delete(findall(0,'Tag','customfitresults'))                                ;
delete(findall(0,'Tag','waterfall'))                                       ;
for i = 1 : 10
    figuretag = sprintf('intermediatescatteringfigure_%02i',i)             ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag

% --- Windows from 'showaverageresult.m'
delete(findall(0,'Tag','SAR_staticaverages'))                              ;
delete(findall(0,'Tag','SAR_intensityVSq'))                                ;
delete(findall(0,'Tag','SAR_intensityVSphi'))                              ;
for i = 1 : 10
    figuretag = sprintf('SAR_correlationfigure_%02i',i)                    ;
    delete(findall(0,'Tag',figuretag))                                     ;    
end
delete(findall(0,'Tag','SAR_fit1results'))                                 ;
delete(findall(0,'Tag','SAR_fit2results'))                                 ;
delete(findall(0,'Tag','SAR_customfitresults'))                            ;
delete(findall(0,'Tag','SAR_waterfall'))                                   ;
for i = 1 : 10
    figuretag = sprintf('SAR_intermediatescatteringfigure_%02i',i)         ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag

% --- Windows from 'showmergedg2results.m'
delete(findall(0,'Tag','SMR_staticaverages'))                              ;
delete(findall(0,'Tag','SMR_intensityVSq'))                                ;
delete(findall(0,'Tag','SMR_intensityVSphi'))                              ;
for i = 1 : 10
    figuretag = sprintf('SMR_correlationfigure_%02i',i)                    ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
delete(findall(0,'Tag','SMR_fit1results'))                                 ;
delete(findall(0,'Tag','SMR_fit2results'))                                 ;
delete(findall(0,'Tag','SMR_customfitresults'))                            ;
for i = 1 : 10
    figuretag = sprintf('SMR_intermediatescatteringfigure_%02i',i)         ;
    delete(findall(0,'Tag',figuretag))                                     ;
end
clear i figuretag


%==========================================================================
% --- Apply pushbutton callback fcn
%==========================================================================
function VR_ApplyFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
hFigVR       = findall(0,'Tag','VR_Fig')                                   ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;

% --- get display settings for show result
if ( get(findall(hFigVR,'Tag','figVR_PopupmenuASCII'),'Value') == 1 )
    ccdimginfo.ASCIIon = 1                                                 ;
elseif ( get(findall(hFigVR,'Tag','figVR_PopupmenuASCII'),'Value') == 2 )
    ccdimginfo.ASCIIon = 0                                                 ;
end

if ( get(findall(hFigVR,'Tag','figVR_PopupmenuFIT1'),'Value') == 1 )
    ccdimginfo.FIT1on = 1                                                  ;
elseif ( get(findall(hFigVR,'Tag','figVR_PopupmenuFIT1'),'Value') == 2 )
    ccdimginfo.FIT1on = 0                                                  ;
end

if ( get(findall(hFigVR,'Tag','figVR_PopupmenuFIT2'),'Value') == 1 )
    ccdimginfo.FIT2on = 1                                                  ;
elseif ( get(findall(hFigVR,'Tag','figVR_PopupmenuFIT2'),'Value') == 2 )
    ccdimginfo.FIT2on = 0                                                  ;
end

if ( get(findall(hFigVR,'Tag','figVR_PopupmenuPNG'),'Value') == 1 )
    ccdimginfo.PNGon = 1                                                   ;
elseif ( get(findall(hFigVR,'Tag','figVR_PopupmenuPNG'),'Value') == 2 )
    ccdimginfo.PNGon = 0                                                   ;
end

if ( get(findall(hFigVR,'Tag','figVR_PopupmenuCUSTOMFIT'),'Value') == 1 )
    ccdimginfo.CUSTOMFITon = 1                                             ;
elseif ( get(findall(hFigVR,'Tag','figVR_PopupmenuCUSTOMFIT'),'Value')== 2)
    ccdimginfo.CUSTOMFITon = 0                                             ;
end

iString = get(findall(hFigVR,'Tag','figVR_PopupmenuBatchnumber'),'String') ;
iValue  = get(findall(hFigVR,'Tag','figVR_PopupmenuBatchnumber'),'value')  ;
iBatch  = round(str2num(iString{iValue}))                                  ;

% --- save ccdimginfo to main figure
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;


% =========================================================================
% --- Show Batch results pushbutton callback fcn
% =========================================================================
function VR_ShowFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
hFigVR       = findall(0,'Tag','VR_Fig')                                   ;
textcolor    = [0.4 0.3 0]                                                 ; % text color

% --- call VR_PushbuttonApplyFcn to save information
VR_ApplyFcn                                                                ;

% --- change status bar
hEditStatus   = findall(hFigVR,'Tag','figVR_TextStatus')                   ;
originalColor = get(hEditStatus,'Color')                                   ;
originalText  = get(hEditStatus,'String')                                  ;

iString = get(findall(hFigVR,'Tag','figVR_PopupmenuBatchnumber'),'String') ;
iValue  = get(findall(hFigVR,'Tag','figVR_PopupmenuBatchnumber'),'value')  ;
iBatch  = round(str2num(iString{iValue}))                                  ;
messagestring = sprintf('Status: Display results of batch # %02i',iBatch)  ;  
set(hEditStatus,'String',messagestring,'color','r')                        ;

% --- disable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','off')             ;

% --- here goes the code to be executed
showresult(iBatch)                                                         ;

% --- enable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','on')              ;

% --- reset status bar
set(hEditStatus,'String','Status: Ready!','Color',textcolor)               ;


% =========================================================================
% --- Show Average result pushbutton callback fcn
% =========================================================================
function VR_ShowAverageFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
hFigVR       = findall(0,'Tag','VR_Fig')                                   ;
textcolor    = [0.4 0.3 0]                                                 ; % text color

% --- call VR_PushbuttonApplyFcn to save information
VR_ApplyFcn                                                                ;

% --- change status bar
hEditStatus   = findall(hFigVR,'Tag','figVR_TextStatus')                   ;
originalColor = get(hEditStatus,'Color')                                   ;
originalText  = get(hEditStatus,'String')                                  ;
messagestring = 'Status: Display average result'                           ;  
set(hEditStatus,'String',messagestring,'color','r')                        ;

% --- disable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','off')             ;

% --- here goes the code to be executed
showaverageresult                                                          ;

% --- enable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','on')              ;

% --- reset status bar
set(hEditStatus,'String','Status: Ready!','Color',textcolor)               ;


% =========================================================================
% --- Show Average result pushbutton callback fcn
% =========================================================================
function VR_ShowMergeFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
hFigVR       = findall(0,'Tag','VR_Fig')                                   ;
textcolor    = [0.4 0.3 0]                                                 ; % text color

% --- call VR_PushbuttonApplyFcn to save information
VR_ApplyFcn                                                                ;

% --- change status bar
hEditStatus   = findall(hFigVR,'Tag','figVR_TextStatus')                   ;
originalColor = get(hEditStatus,'Color')                                   ;
originalText  = get(hEditStatus,'String')                                  ;
messagestring = 'Status: Display merged g2 results'                        ;  
set(hEditStatus,'String',messagestring,'color','r')                        ;

% --- disable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','off')             ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','off')             ;

% --- here goes the code to be executed
showmergedg2result                                                          ;

% --- enable pushbuttons
set(findall(hFigVR,'Tag','VR_PushbuttonClose'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonApply'),'Enable','on')              ;
set(findall(hFigVR,'Tag','VR_PushbuttonShow') ,'Enable','on')              ;

% --- reset status bar
set(hEditStatus,'String','Status: Ready!','Color',textcolor)               ;


% ---
% EOF
