function viewsingleimage(varargin)
% VIEWSINGLEIMAGE View single image
%
% Copyright 2004 Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/12/12 $

hFigViewSingleImage = findall(0,'Tag','viewsingleimage_Fig');
if ~isempty(hFigViewSingleImage);
    figure(hFigViewSingleImage);
    return;
end

screensize = get(0,'screensize');
figureSize = [300 175];
figurePos = [100,screensize(4)-figureSize(2)-100,figureSize];
hFigViewSingleImage = figure(...
    'BackingStore','on',...
    'Units','pixels',...
    'DockControls','off',...
    'Resize','off',...
    'PaperOrient','portrait',...
    'PaperPositionMode','auto',...
    'IntegerHandle','off',...
    'NumberTitle','off',...
    'MenuBar','none',...
    'Toolbar','none',...
    'Name','XPCS - Single Image Viewer',...
    'Position',figurePos,...
    'HandleVisibility','callback',...
    'Tag','viewsingleimage_Fig',...
    'CloseRequestFcn',@viewsingleimage_CloseRequestFcn,...
    'UserData',[]);
hAxes = axes(...
    'Parent',hFigViewSingleImage,...
    'Units','pixels',...
    'Position',[0 0 figureSize],...
    'Tag','viewsingleimage_Axes');
hPatch1 = patch(...
    'Parent',hAxes,...
    'XData',[0 1 1 0],...
    'YData',[0 0 1 1],...
    'FaceColor',[1 1 0.85],...
    'EdgeColor',[1 1 0.85]);
hPatch2 = patch(...
    'Parent',hAxes,...
    'XData',[0.02 0.98 0.98 0.02],...
    'YData',[0.3 0.3 0.96 0.96],...
    'FaceColor',[1 1 0.9],...
    'EdgeColor',[0.7 0.7 0.7]);
text('Parent',hAxes,'Position',[0.04 0.85],'String','Enter Image File Name:','color',[0.4 0.3 0]);
uicontrol('Parent',hFigViewSingleImage,...
    'Style','Edit',...
    'units','normalized',...
    'backgroundcolor','w',...
    'Position',[0.1 0.6 0.6 0.125],...
    'HorizontalAlignment','right',...
    'String','',...
    'Tag','figviewsingleimage_EditFile');
uicontrol('Parent',hFigViewSingleImage,...
    'Style','Pushbutton',...
    'units','normalized',...
    'backgroundcolor','w',...
    'Position',[0.75 0.6 0.2 0.125],...
    'HorizontalAlignment','right',...
    'String','Browse ...',...
    'Tag','figviewsingleimage_BrowseFile',...
    'Callback',@viewsingleimage_BrowseRequestFcn);
text('Parent',hAxes,'Position',[0.04 0.45],'String','Enter Image Index Number(s):','color',[0.4 0.3 0]);
uicontrol('Parent',hFigViewSingleImage,...
    'Style','Edit',...
    'units','normalized',...
    'backgroundcolor','w',...
    'Position',[0.7 0.375 0.25 0.125],...
    'HorizontalAlignment','right',...
    'String','',...
    'Tag','figviewsingleimage_EditImageIndex');
hPushbottonClose = uicontrol(hFigViewSingleImage,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Close',...
    'position',[0.52 0.075 0.2 0.125],...
    'Tag','viewsystem_PushbuttonClose',...
    'callback',@viewsingleimage_CloseRequestFcn);
hPushbottonView = uicontrol(hFigViewSingleImage,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','View',...
    'position',[0.75 0.075 0.2 0.125],...
    'Tag','viewsystem_PushbuttonView',...
    'callback',@viewsingleimage_PushbttonViewFcn);
text('Parent',hAxes,'Position',[0.04 0.15],'String','Status: Ready','color',[0.4 0.3 0],'Tag','status');


%================================================================
% --- browse request function of viewsingleimage
%================================================================
function viewsingleimage_BrowseRequestFcn(hObject,eventdata)
[filename,filepath] = uigetfile('*.*','Load Image File');        
if ( filename == 0 )
    clear filename filepath                                      ;
    return                                                       ;
end
file=fullfile(filepath,filename);
set(findall(0,'Tag','figviewsingleimage_EditFile')            ...
             ,'String',file,'HorizontalAlignment','right')       ;
clear file filename filepath                                     ;


%================================================================
% --- close request function of viewsingleimage
%================================================================
function viewsingleimage_CloseRequestFcn(hObject,eventdata)
delete(findall(0,'Tag','viewsingleimage_openedimages'));
delete(findall(0,'Tag','viewsingleimage_Fig'));


%================================================================
% --- view request function of viewsingleimage
%================================================================
function viewsingleimage_PushbttonViewFcn(hObject,eventdata)
hFigViewSingleImage = findall(0,'Tag','viewsingleimage_Fig')     ;
if ~isempty(str2num(get(findall(hFigViewSingleImage,'Tag','figviewsingleimage_EditImageIndex'),'string')))
    imageIndex = str2num(get(findall(hFigViewSingleImage,'Tag','figviewsingleimage_EditImageIndex'),'string'));
    if length(imageIndex) > 20      % stop if too many images to display
        return;
    end
else
    return;
end
file = get(findall(hFigViewSingleImage,'Tag','figviewsingleimage_EditFile'),'String');
initialTextColor = get(findall(hFigViewSingleImage,'Tag','status'),'Color');
set(findall(hFigViewSingleImage,'Tag','status'),'String','Status: Loading ...','Color','r');
pause(0);
for iIndex = 1:length(imageIndex)
    try
        imm = openfile(file,imageIndex(iIndex));
    catch
        set(findall(hFigViewSingleImage,'Tag','status'),'String','Status: Ready','Color',initialTextColor);
        return;
    end
    [filepath,filename,ext,versn] = fileparts(file);
    hFigure = figure(...
        'BackingStore','off',...
        'IntegerHandle','off',...
        'DockControls','off',...
        'Menubar','figure',...
        'Toolbar','figure',...
        'color',[1 1 0.85],...
        'NumberTitle','off',...
        'WindowButtonMotionFcn',@imagefigure_WindowButtonMotionFcn,...
        'Name',['Image # ',num2str(imageIndex(iIndex)),' of file ',filename],...
        'Tag','viewsingleimage_openedimages');
    hImm = imagesc(imm.imm,[0 max(imm.imm(:))]);
    uicontrol('Parent',hFigure,...
        'Style','Text',...
        'BackgroundColor',[1 1 0.85],...
        'String',' ',...
        'units','normalized',...
        'HorizontalAlignment','left',...
        'position',[0.05 0.01 0.5 0.05],...
        'Tag','mouseposition');
    set(get(hImm,'Parent'),'YDir','normal');
    pause(0);
end
set(findall(hFigViewSingleImage,'Tag','status'),'String','Status: Ready','Color',initialTextColor);


%================================================================
% --- tracking mouse position
%================================================================
function imagefigure_WindowButtonMotionFcn(hObject,eventdata)
set(gcf,'selected','on');
pointPosition = get(gca,'CurrentPoint');
XLim=get(gca,'XLim');
YLim=get(gca,'YLim');
XLimFlag=(pointPosition(1,1)>=XLim(1) & pointPosition(1,1)<=XLim(2));
YLimFlag=(pointPosition(1,2)>=YLim(1) & pointPosition(1,2)<=YLim(2));
cdata = get(findall(gcf,'type','image'),'cdata');
if iscell(cdata)
    intensity = cdata{2};
else
    intensity = cdata;
end
xpos = round(pointPosition(1,1));
ypos = round(pointPosition(1,2));
if xpos >= 1 & xpos <= size(intensity,2) & ypos >= 1 & ypos <= size(intensity,1) & XLimFlag == 1 & YLimFlag ==1
    set(gcf,'Pointer','crosshair');
    set(findall(gcf,'Tag','mouseposition'),'string',['x=',num2str(xpos),' y=',num2str(ypos),' value=',num2str(intensity(ypos,xpos))]);
else
    set(gcf,'Pointer','arrow');
end
clear XLim YLim XLimFlag YLimFlag;
clear pointPosition xpos ypos cdata intensity;
