function viewsystem(varargin)
% VIEWSYSTEM Display and edit system information.
%
% Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/12/11 $

% =========================================================================
% --- get ccdimginfo
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
if isappdata(hFigXPCSMain,'ccdimginfo')
    ccdimginfo = getappdata(hFigXPCSMain,'ccdimginfo')                     ;
else
    return                                                                 ;
end


% =========================================================================
% --- if viewsystem fig exists, display it in front; else create it
% =========================================================================
hFigViewsystem = findall(0,'Tag','viewsystem_Fig')                         ;
if ~isempty(hFigViewsystem)
    figure(hFigViewsystem)                                                 ;
    return                                                                 ;
end


% =========================================================================
% --- figure layout
% =========================================================================
backgroundcolor = [1 1 0.85]                                               ;
facecolor       = [1 1 0.9]                                                ;
textcolor       = [0.4 0.3 0]                                              ; % text color
subtextcolor    = 'b'                                                      ; % subtext color

hFigXPCSMainPos   = get(hFigXPCSMain,'position')                           ;
figViewsystemSize = [720 540]                                              ;
hFigViewsystem    = figure('BackingStore','on','Units','pixels',        ...
                           'DockControls','off','Resize','off',         ...
                           'PaperOrient','portrait',                    ...
                           'PaperPositionMode','auto',                  ...
                           'IntegerHandle','off','NumberTitle','off',   ...
                           'MenuBar','none','Toolbar','none',           ...
                           'WindowStyle','modal',                       ...
                           'Name','XPCS - System Settings',             ...
                           'Position',[hFigXPCSMainPos(1)+ 150          ...
                                      ,hFigXPCSMainPos(2)+ 60           ...
                                      ,figViewsystemSize],              ...
                           'HandleVisibility','callback',...
                           'Tag','viewsystem_Fig',...
                           'UserData',[])                                  ;
                       
hAxes = axes('Parent',hFigViewsystem,'Units','pixels',...
             'Position',[0 0 figViewsystemSize],'Tag','figviewsystem_Axes');
                                 
hPatch1     = patch('Parent',hAxes,'XData',[0 1 1 0],'YData',[0 0 1 1], ...
                'FaceColor',backgroundcolor,'EdgeColor',backgroundcolor)   ;
hGroupPatch = hggroup('Parent',hAxes)                                      ;

hPatch2 = patch('Parent',hGroupPatch,'XData',[0.02 0.49 0.49 0.02],     ...
                'YData',[0.1 0.1 0.925 0.925])                             ;
hPatch3 = patch('Parent',hGroupPatch,'XData',[0.51 0.98 0.98 0.51],     ...
                'YData',[0.1 0.1 0.925 0.925])                             ;
set(get(hGroupPatch,'Children'),'FaceColor',facecolor,                  ...
                                'EdgeColor',[0.7 0.7 0.7])                 ;
                            
hGroupText    = hggroup('Parent',hAxes)                                    ;
hGroupSubText = hggroup('Parent',hAxes)                                    ;


% =========================================================================
% --- CCD and System information
% =========================================================================
text('Parent',hAxes,'Position',[0.415 0.965],'String','System Settings',...
     'FontSize',12,'FontWeight','demi','color',[0.4 0.3 0])                ;
text('Parent',hGroupText,'Position',[0.03 0.9],'String','CCD Settings:')   ;

text('Parent',hGroupSubText,'Position',[0.06 0.86],                     ...
     'String','Pixel Size (mm)')                                           ;
uicontrol('Parent',hFigViewsystem,'Style','Edit','Units','normalized',  ...
          'backgroundcolor','w','Position',[0.38 0.84 0.1 0.035]',      ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.dpix),'Tag','figviewsystem_dpix')    ;

text('Parent',hGroupSubText,'Position',[0.06 0.82],                     ...
     'String','ADC saturation level [ADU]')                                ;
uicontrol('Parent',hFigViewsystem,'Style','Edit','Units','normalized',  ...
          'backgroundcolor','w','Position',[0.38 0.8 0.1 0.035]',       ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.saturation),                      ...
          'Tag','figviewsystem_saturation');

text('Parent',hGroupSubText,'Position',[0.06 0.78],                     ...
     'String','ADU per photon')                                            ;
uicontrol('Parent',hFigViewsystem,'Style','Edit','Units','normalized',  ...
          'backgroundcolor','w','Position',[0.38 0.76 0.1 0.035]',      ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.adupphot),                        ...
          'Tag','figviewsystem_adupphot')                                  ;

text('Parent',hGroupSubText,'Position',[0.06 0.74],                     ...
     'String','CCD efficiency')                                            ;
uicontrol('Parent',hFigViewsystem,'Style','Edit',                       ...
          'Units','normalized','backgroundcolor','w',                   ...
          'Position',[0.38 0.72 0.1 0.035]',                            ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.efficiency),                      ...
          'Tag','figviewsystem_efficiency')                                ;

text('Parent',hGroupSubText,'Position',[0.06 0.70],'String','CCD gain')    ;
uicontrol('Parent',hFigViewsystem,'Style','Edit',                       ...
          'Units','normalized','backgroundcolor','w',                   ...
          'Position',[0.38 0.68 0.1 0.035]',                            ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.gain),                            ...
          'Tag','figviewsystem_gain')                                      ;

text('Parent',hGroupText,'Position',[0.03 0.26],                     ...
     'String','Available RAM (MB):')                                       ;
uicontrol('Parent',hFigViewsystem,'Style','Edit','Units','normalized',  ...
          'backgroundcolor','w','Position',[0.38 0.24 0.1 0.035]',      ...
          'HorizontalAlignment','right',                                ...
          'String',num2str(ccdimginfo.memory),                          ...
          'Tag','figviewsystem_memory')                                    ;


% =========================================================================
% --- layout for CCD correction settings
% =========================================================================
text('Parent',hGroupText,'Position',[0.03 0.63],                        ...
     'String','CCD correction settings:')                                  ;
% --- blemish correction
text('Parent',hGroupSubText,'Position',[0.06 0.60],                     ...
     'String','Use blemish file')                                          ;
uicontrol('Parent',hFigViewsystem,'Style','popupmenu'                   ...
    ,'Units','normalized','backgroundcolor','w'                         ...
    ,'Position',[0.25 0.57 0.07 0.04]'                                  ...
    ,'HorizontalAlignment','right','String','yes|no'                    ...
    ,'Tag','figviewsystem_Blemish'                                      ...
    ,'callback',@figViewsystem_BlemishCallbackFcn)                         ;
if ( ccdimginfo.blemish == 1 )
    set(findall(0,'Tag','figviewsystem_Blemish'),'value',1)                ;
else
    set(findall(0,'Tag','figviewsystem_Blemish'),'value',2)                ;
end
% --- flatfield correction
text('Parent',hGroupSubText,'Position',[0.06 0.57],                     ...
     'String','Use flatfield file')                                        ;
uicontrol('Parent',hFigViewsystem,'Style','popupmenu'                   ...
    ,'Units','normalized','backgroundcolor','w'                         ...
    ,'Position',[0.25 0.54 0.07 0.04]','HorizontalAlignment','right'    ...
    ,'String','yes|no','Tag','figviewsystem_Flatfield'                  ...
    ,'callback',@figViewsystem_FlatfieldCallbackFcn)                       ;
if ( ccdimginfo.flatfield == 1 )
    set(findall(0,'Tag','figviewsystem_Flatfield'),'value',1)              ;
else
    set(findall(0,'Tag','figviewsystem_Flatfield'),'value',2)              ;
end
% --- distortion correction
text('Parent',hGroupSubText,'Position',[0.06 0.54],                     ...
     'String','Use distortion file')                                       ;
uicontrol('Parent',hFigViewsystem,'Style','popupmenu'                   ...
    ,'Units','normalized','backgroundcolor','w','Enable','off'          ...
    ,'Position',[0.25 0.51 0.07 0.04]','HorizontalAlignment','right'    ...
    ,'String','yes|no','Tag','figviewsystem_Distortion'                 ...
    ,'callback',@figViewsystem_DistortionCallbackFcn)                      ;
if ( ccdimginfo.distortion == 1 )
    set(findall(0,'Tag','figviewsystem_Distortion'),'value',1)             ;
else
    set(findall(0,'Tag','figviewsystem_Distortion'),'value',2)             ;
end
% --- parasitic correction
text('Parent',hGroupSubText,'Position',[0.06 0.51],                     ...
     'String','Use parasitic file')                                        ;
uicontrol('Parent',hFigViewsystem,'Style','popupmenu'                   ...
    ,'Units','normalized','backgroundcolor','w','Enable','off'          ...
    ,'Position',[0.25 0.48 0.07 0.04]','HorizontalAlignment','right'    ...
    ,'String','yes|no','Tag','figviewsystem_Parasitic'                  ...
    ,'callback',@figViewsystem_ParasiticCallbackFcn)                       ;
if ( ccdimginfo.parasitic == 1 )
    set(findall(0,'Tag','figviewsystem_Parasitic'),'value',1)              ;
else
    set(findall(0,'Tag','figviewsystem_Parasitic'),'value',2)              ;
end


% =========================================================================
% --- layout for the thickness normalization options
% =========================================================================
% --- group title
text('Parent',hGroupText,'Position',[0.53 0.90],                        ...
     'String','Thickness normalization options (transmission mode):')      ;

% --- no thickness normalization
uicontrol('Parent',hFigViewsystem,'style','Radiobutton',                ...
          'Units','normalized',                                         ...
          'String','no thickness normalization','FontSize',10,          ...
          'BackgroundColor',facecolor,'ForegroundColor',subtextcolor,   ...
          'Position',[0.56 0.83 0.4 0.04],                              ...
          'Tag','figViewsystem_RadiobuttonLLD1',                        ...
          'Callback',@figViewsystem_RadiobuttonLLDCallbackFcn)             ;

% --- sample thickness is known
uicontrol('Parent',hFigViewsystem,'style','Radiobutton',                ...
          'Units','normalized','String','','FontSize',10,               ...
          'BackgroundColor',facecolor,'ForegroundColor',subtextcolor,   ...
          'HorizontalAlignment','right',                                ...
          'Position',[0.56 0.785 0.04 0.04],                            ...
          'Tag','figViewsystem_RadiobuttonLLD2',                        ...
          'Callback',@figViewsystem_RadiobuttonLLDCallbackFcn)             ;
uicontrol('Parent',hFigViewsystem,'style','Edit','Units','normalized',  ...
          'String','','BackgroundColor','w',                            ...
          'HorizontalAlignment','right',                                ...
          'Position',[0.59 0.785 0.05 0.035],                           ...
          'Tag','figViewsystem_EditLLD2')                                  ;
text('Parent',hGroupSubText,'Position',[0.65 0.805],                    ...
     'String','sample thickness in [mm]')                                  ;

% --- sample attenuation length is known
uicontrol('Parent',hFigViewsystem,'style','Radiobutton',                ...
          'Units','normalized','String','','FontSize',10,               ...
          'BackgroundColor',facecolor,'ForegroundColor',subtextcolor,   ...
          'HorizontalAlignment','right',                                ...
          'Position',[0.56 0.74 0.04 0.04],                             ...
          'Tag','figViewsystem_RadiobuttonLLD3',                        ...
          'Callback',@figViewsystem_RadiobuttonLLDCallbackFcn)             ;
uicontrol('Parent',hFigViewsystem,'style','Edit','Units','normalized',  ...
          'String','','BackgroundColor','w',                            ...
          'HorizontalAlignment','right',                                ...
          'Position',[0.59 0.74 0.05 0.035],                            ...
          'Tag','figViewsystem_EditLLD3')                                  ;
text('Parent',hGroupSubText,'Position',[0.65 0.76],                     ...
     'String','sample attenuation length [mm]')                            ;


if ( ccdimginfo.geometry == 0 )                                              % enable the radiobuttons in transmission mode
    if ( ccdimginfo.thicknessflag == 0 )                                     % default startup mode
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD1'),'value',1,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD2'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD3'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_EditLLD2'),'Enable','off','string',num2str(ccdimginfo.thickness))    ;
        set(findall(0,'Tag','figViewsystem_EditLLD3'),'Enable','off','string',num2str(ccdimginfo.atten_length)) ;
    end
    if ( ccdimginfo.thicknessflag == 1 )                                     % sample thickness has been entered 
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD1'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD2'),'value',1,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD3'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_EditLLD2'),'Enable','on' ,'string',num2str(ccdimginfo.thickness))    ;
        set(findall(0,'Tag','figViewsystem_EditLLD3'),'Enable','off','string',num2str(ccdimginfo.atten_length)) ;
    end
    if ( ccdimginfo.thicknessflag == 2 )                                     % attenuation length has been entered
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD1'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD2'),'value',0,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_RadiobuttonLLD3'),'value',1,'Enable','on')                           ;
        set(findall(0,'Tag','figViewsystem_EditLLD2'),'Enable','off','string',num2str(ccdimginfo.thickness))    ;
        set(findall(0,'Tag','figViewsystem_EditLLD3'),'Enable','on' ,'string',num2str(ccdimginfo.atten_length)) ;
    end
else                                                                         % disable everything in reflection mode
    set(findall(0,'Tag','figViewsystem_RadiobuttonLLD1'),'value',1,'Enable','off')                              ;
    set(findall(0,'Tag','figViewsystem_RadiobuttonLLD2'),'value',0,'Enable','off')                              ;
    set(findall(0,'Tag','figViewsystem_RadiobuttonLLD3'),'value',0,'Enable','off')                              ;
    set(findall(0,'Tag','figViewsystem_EditLLD2'),'Enable','off','string',num2str(ccdimginfo.thickness))        ;
    set(findall(0,'Tag','figViewsystem_EditLLD3'),'Enable','off','string',num2str(ccdimginfo.atten_length))     ;
end


% =========================================================================
% --- define layout for 'Apply' & 'Close' button
% =========================================================================
set(get(hGroupText,'Children'),'Units','normalized',                    ...
    'HorizontalAlignment','left','Color',textcolor)                        ;
set(get(hGroupSubText,'Children'),'Units','normalized',                 ...
    'HorizontalAlignment','left','Color',subtextcolor)                     ;
hPushbottonClose = uicontrol(hFigViewsystem,'Style','pushbutton',       ...
                             'Units','normalized','String','Close',     ...
                             'position',[0.7 0.02 0.125 0.05],          ...
                             'Tag','viewsystem_PushbuttonClose',        ...
                             'TooltipString','Close window',            ...
                             'callback',@viewsystem_CloseRequestFcn)       ;
hPushbottonApply = uicontrol(hFigViewsystem,'Style','pushbutton',       ...
                             'Units','normalized','String','Apply',     ...
                             'position',[0.85 0.02 0.125 0.05],         ...
                             'Tag','viewsystem_PushbuttonApply',        ...
                             'TooltipString','Apply changes',           ...
                             'callback',@viewsystem_ApplyFcn)              ;


%==========================================================================
%==========================================================================
% --- define the callback functions below
%==========================================================================
%==========================================================================


%==========================================================================
%--- Popupmenu CCD correction settings callback functions
%==========================================================================
function figViewsystem_BlemishCallbackFcn(hObject,eventdata)
hFigViewsystem       = findall(0,'Tag','viewsystem_Fig')                         ;
hPopupmenuBlemish    = findall(hFigViewsystem,'Tag','figviewsystem_Blemish')     ;

function figViewsystem_FlatfieldCallbackFcn(hObject,eventdata)
hFigViewsystem       = findall(0,'Tag','viewsystem_Fig')                         ;
hPopupmenuFlatfield  = findall(hFigViewsystem,'Tag','figviewsystem_Flatfield')   ;

function figViewsystem_DistortionCallbackFcn(hObject,eventdata)
hFigViewsystem       = findall(0,'Tag','viewsystem_Fig')                         ;
hPopupmenuDistortion = findall(hFigViewsystem,'Tag','figviewsystem_Distortion')  ;

function figViewsystem_ParasiticCallbackFcn(hObject,eventdata)
hFigViewsystem       = findall(0,'Tag','viewsystem_Fig')                         ;
hPopupmenuParasitic  = findall(hFigViewsystem,'Tag','figviewsystem_Parasitic')   ;


%==========================================================================
%--- LLD radiobuttons callback fcn (thickness normalization)
%==========================================================================
function figViewsystem_RadiobuttonLLDCallbackFcn(hObject,eventdata)
hFigViewsystem   = findall(0,'Tag','viewsystem_Fig')                             ;
hRadiobuttonLLD1 = findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD1') ;
hRadiobuttonLLD2 = findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD2') ;
hRadiobuttonLLD3 = findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD3') ;
hEditLLD2        = findall(hFigViewsystem,'Tag','figViewsystem_EditLLD2')        ;
hEditLLD3        = findall(hFigViewsystem,'Tag','figViewsystem_EditLLD3')        ;
switch hObject
    case hRadiobuttonLLD1
        set(hRadiobuttonLLD1,'value',1)                                    ;
        set(hRadiobuttonLLD2,'value',0)                                    ;
        set(hRadiobuttonLLD3,'value',0)                                    ;
        set(hEditLLD2,'Enable','off')                                      ;
        set(hEditLLD3,'Enable','off')                                      ;
    case hRadiobuttonLLD2
        set(hRadiobuttonLLD1,'value',0)                                    ;
        set(hRadiobuttonLLD2,'value',1)                                    ;
        set(hRadiobuttonLLD3,'value',0)                                    ;
        set(hEditLLD2,'Enable','on')                                       ;
        set(hEditLLD3,'Enable','off')                                      ;
    case hRadiobuttonLLD3
        set(hRadiobuttonLLD1,'value',0)                                    ;
        set(hRadiobuttonLLD2,'value',0)                                    ;
        set(hRadiobuttonLLD3,'value',1)                                    ;
        set(hEditLLD2,'Enable','off')                                      ;
        set(hEditLLD3,'Enable','on')                                       ;
end


%==========================================================================
%--- Close figure close callback fcn
%==========================================================================
function viewsystem_CloseRequestFcn(hObject,eventdata)
hFigViewsystem = findall(0,'Tag','viewsystem_Fig')                         ;
viewsystem_ApplyFcn                                                        ; % apply changes
delete(hFigViewsystem)                                                     ; % delete figure


%==========================================================================
% --- Apply pushbutton callback fcn
%==========================================================================
function viewsystem_ApplyFcn(hObject,eventdata)
hFigXPCSMain   = findall(0,'Tag','xpcsmain_Fig')                           ;
hFigViewsystem = findall(0,'Tag','viewsystem_Fig')                         ;
ccdimginfo     = getappdata(hFigXPCSMain,'ccdimginfo')                     ;

% =========================================================================
% --- get detector and analysis system info
% =========================================================================
if ~isnan(str2num(get(findall(hFigViewsystem,'Tag','figviewsystem_dpix'),'string')))
    ccdimginfo.dpix       = str2num(get(findall(hFigViewsystem,'Tag','figviewsystem_dpix'),'string'))          ;
end
if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_saturation'),'string')))
    ccdimginfo.saturation = str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_saturation'),'string')) ;
end
if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_adupphot'),'string')))
    ccdimginfo.adupphot   = str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_adupphot'),'string'))   ;
end
if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_efficiency'),'string')))
    ccdimginfo.efficiency = str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_efficiency'),'string')) ;
end
if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_gain'),'string')))
    ccdimginfo.gain       = str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_gain'),'string'))       ;
end
if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_memory'),'string')))
    ccdimginfo.memory     = str2double(get(findall(hFigViewsystem,'Tag','figviewsystem_memory'),'string'))     ;
end

% =========================================================================
% --- get detector correction options
% =========================================================================
if ( get(findall(hFigViewsystem,'Tag','figviewsystem_Blemish'),'Value') == 1 )
    ccdimginfo.blemish = 1                                                           ;
elseif ( get(findall(hFigViewsystem,'Tag','figviewsystem_Blemish'),'Value') == 2 )
    ccdimginfo.blemish = 0                                                           ;
end

if ( get(findall(hFigViewsystem,'Tag','figviewsystem_Flatfield'),'Value') == 1 )
    ccdimginfo.flatfield = 1                                                         ;
elseif ( get(findall(hFigViewsystem,'Tag','figviewsystem_Flatfield'),'Value') == 2 )
    ccdimginfo.flatfield = 0                                                         ;
end

if ( get(findall(hFigViewsystem,'Tag','figviewsystem_Distortion'),'Value') == 1 )
    ccdimginfo.distortion = 1                                                        ;
elseif ( get(findall(hFigViewsystem,'Tag','figviewsystem_Distortion'),'Value') == 2 )
    ccdimginfo.distortion = 0                                                        ;
end

if ( get(findall(hFigViewsystem,'Tag','figviewsystem_Parasitic'),'Value') == 1 )
    ccdimginfo.parasitic = 1                                                        ;
elseif ( get(findall(hFigViewsystem,'Tag','figviewsystem_Parasitic'),'Value') == 2 )
    ccdimginfo.parasitic = 0                                                        ;
end


% =========================================================================
% --- get thickness normalization options
% =========================================================================
switch 1
    % ---
    case get(findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD1'),'value')
        if ~isfield(ccdimginfo,'thickness')
            ccdimginfo.thickness     = -1                                  ; % just create a dummy value for older versions
        end
        if ~isfield(ccdimginfo,'atten_length')
            ccdimginfo.atten_length  = -1                                  ; % just create a dummy value for older versions
        end
        ccdimginfo.thicknessflag = 0                                       ; % neither thickness value nor attenuation length value is known 
    % ---
    case get(findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD2'),'value')
        if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figViewsystem_EditLLD2'),'string')))
            ccdimginfo.thickness     = abs(str2double(get(findall(hFigViewsystem,'Tag','figViewsystem_EditLLD2'),'string')))  ;
            if ~isfield(ccdimginfo,'atten_length')
                ccdimginfo.atten_length  = -1                              ; % just create a dummy value for older versions
            end
            ccdimginfo.thicknessflag = 1                                   ; % set thicknessflag to one (sample thickness is known)
        end
    % ---
    case get(findall(hFigViewsystem,'Tag','figViewsystem_RadiobuttonLLD3'),'value')
        if ~isnan(str2double(get(findall(hFigViewsystem,'Tag','figViewsystem_EditLLD3'),'string')))
            ccdimginfo.atten_length  = abs(str2double(get(findall(hFigViewsystem,'Tag','figViewsystem_EditLLD3'),'string')))   ;
            if ~isfield(ccdimginfo,'thickness')
                ccdimginfo.thickness     = -1                              ; % just create a dummy value for older versions
            end
            ccdimginfo.thicknessflag = 2                                   ; % set thicknessflag to two (attenuation length is known)
        end
    % ---    
end


%==========================================================================
% --- save ccdimginfo to main figure
%==========================================================================
setappdata(hFigXPCSMain,'ccdimginfo',ccdimginfo)                           ;

% ---
% EOF