function writeclustercfg(nBatch)        
% --- this function writes the configuration file for cluster based 
% --- multi-tau calculations
% ---
% --- by MS 20061027


% =========================================================================
% --- get some variables from the application data
% =========================================================================
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig')                             ;
ccdimginfo   = getappdata(hFigXPCSMain,'ccdimginfo')                       ;
% ---
imagepath  = ccdimginfo.imgPath                                            ; % path, where the data resides
imagefile  = ccdimginfo.imagefile{nBatch,1}                                ; % full file name of the data file
startindex = ccdimginfo.ndata0todo(nBatch)                                 ; % get the index of the first selected image
endindex   = ccdimginfo.ndataendtodo(nBatch)                               ; % get the index of the last selected image
name       = ccdimginfo.name                                               ; % data file name without numbering and path
flatfield  = ccdimginfo.flatfield                                          ; % check if flatfield is needed
lld        = ccdimginfo.lld                                                ; % lower level discriminination value
saturation = ccdimginfo.saturation                                         ; % saturation value of a pixel on the CCD
% ---
clear hFigXPCSMain ccdimginfo nBatch                                       ;


% =========================================================================
% --- open the configuration file
% =========================================================================
filedir   = imagepath                                                      ;
filetitle = [name,'cluster.cfg']                                           ; % define name for configuration file
filename  = fullfile(filedir,filetitle)                                    ; % build full file name
out       = fopen(filename,'w')                                            ; % open the configuration file
clear filedir filetitle filename                                           ;


% =========================================================================
% --- determine network location
% =========================================================================
if ( strcmp(imagepath(1:3),'W:\') == 1 )
    networkstart = '//users-nfs/8-idi-a/'                                  ;
elseif ( strcmp(imagepath(1:3),'X:\') == 1 )
    networkstart = '//users-nfs/8-idi-b/'                                  ;
elseif ( strcmp(imagepath(1:3),'Y:\') == 1 )
    networkstart = '//users-nfs/8-idi-c/'                                  ;
elseif ( strcmp(imagepath(1:3),'Z:\') == 1 )
    networkstart = '//users-nfs/8-idi-d/'                                  ;
elseif ( strcmp(imagepath(1:3),'D:\') == 1 )
    networkstart = '//users-nfs/8-idi-a/'                                  ;
end
% ---
dummy       = imagepath(4:end)                                             ;
dummyindex  = strfind(dummy,'\')                                           ;
if ( ~isempty(dummyindex) )
    dummy(dummyindex) = '/'                                                ;
end
if ( isempty(strfind(dummy(end),'/')) )
    dummy(end+1) = '/'                                                     ;
end
networkdatapath    = ['''''',networkstart,dummy,'''''']                    ;
networkresultspath = ['''''',networkstart,dummy,'clusterresults/','''''']  ;
clear networkstart dummy dummyindex imagepath                              ;


% =========================================================================
% --- write configuration
% =========================================================================
% --- directory in which the data resides
fprintf(out,'<Data Top Directory>')                                        ;
fprintf(out,networkdatapath)                                               ;
fprintf(out,'</Data Top Directory>\n')                                     ;
clear networkdatapath                                                      ;
% --- directory to store the cluster results
fprintf(out,'<Output Directory>')                                          ;
fprintf(out,networkresultspath)                                            ;
fprintf(out,'</Output Directory>\n')                                       ;
clear networkresultspath                                                   ;
% --- data file name
[imagedir,imagename,imageextension] = fileparts(imagefile)                 ;
dataname = ['''''',imagename,imageextension,'''''']                        ;
fprintf(out,'<Data File Name>')                                            ;
fprintf(out,dataname)                                                      ;
fprintf(out,'</Data File Name>\n')                                         ;
clear imagefile imagedir imagename imageextension dataname                 ;
% --- data start index
fprintf(out,'<Data Start Index>')                                          ;
fprintf(out,num2str(startindex))                                           ;
fprintf(out,'</Data Start Index>\n')                                       ;
clear startindex                                                           ;
% --- data end index
fprintf(out,'<Data End Index>')                                            ;
fprintf(out,num2str(endindex))                                             ;
fprintf(out,'</Data End Index>\n')                                         ;
clear endindex                                                             ;
% --- averaged dark image
darkname = ['''''',name,'dark.mat','''''']                                 ;
fprintf(out,'<Dark Field File>')                                           ;
fprintf(out,darkname)                                                      ;
fprintf(out,'</Dark Field File>\n')                                        ;
clear darkname                                                             ;
% --- static q phi partition map
staticname = ['''''',name,'static.mat','''''']                             ;
fprintf(out,'<Static Map File>')                                           ;
fprintf(out,staticname)                                                    ;
fprintf(out,'</Static Map File>\n')                                        ;
clear staticname                                                           ;
% --- dynamic q phi partition map
dynamicname = ['''''',name,'dynamic.mat','''''']                           ;
fprintf(out,'<Dynamic Map File>')                                          ;
fprintf(out,dynamicname)                                                   ;
fprintf(out,'</Dynamic Map File>\n')                                       ;
clear dynamicname                                                          ;
% --- flatfield file
if ( flatfield == 0 )
    fprintf(out,'<Flat Field File>')                                       ;
    fprintf(out,'''''NONE''''')                                            ;
    fprintf(out,'</Flat Field File>\n')                                    ;
elseif ( flatfield == 1 )
    flatfieldname = ['''''',name,'flatfield.mat','''''']                   ;
    fprintf(out,'<Flat Field File>')                                       ;
    fprintf(out,flatfieldname)                                             ;
    fprintf(out,'</Flat Field File>\n')                                    ;    
    clear flatfieldname                                                    ;
end
clear flatfield                                                            ;
% --- saturation value
fprintf(out,'<Saturation Value>')                                          ;
fprintf(out,num2str(saturation))                                           ;
fprintf(out,'</Saturation Value>\n')                                       ;
clear saturation                                                           ;
% --- lower level discrimination value
fprintf(out,'<LLD Value>')                                                 ;
fprintf(out,num2str(lld))                                                  ;
fprintf(out,'</LLD Value>\n')                                              ;
% --- rms of averaged dark image
if ( lld > 0 )
    darkrmsname = ['''''',name,'darkrms.mat','''''']                       ;
    fprintf(out,'<Dark RMS Field File>')                                   ;
    fprintf(out,darkrmsname)                                               ;
    fprintf(out,'</Dark RMS Field File>\n')                                ;
    clear darkrmsname                                                      ;
elseif ( lld <= 0 )
    fprintf(out,'<Dark RMS Field File>')                                   ;
    fprintf(out,'''''NONE''''')                                            ;
    fprintf(out,'</Dark RMS Field File>\n')                                ;    
end
clear lld name                                                             ;
% --- end flag
fprintf(out,'<End>\n')                                                     ;

% =========================================================================
% --- close configuration file & clear leftover variables
% =========================================================================
fclose(out)                                                                ;
clear out                                                                  ;

% EOF
% ---
