function xpcsgui(varargin)
% XPCSGUI Open GUI to analyze XPCS data
%
% Copyright 2004 Zhang Jiang
% $Revision: 1.0 $  $Date: 2004/11/27 $

hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig');
if isempty(hFigXPCSMain)
    initFigure;
else
    figure(hFigXPCSMain);
end

%================================================================
% --- initialize main figure layout
%================================================================
function initFigure
% --------------------------------
% --- main figure handle
% --------------------------------
figureSize = [720 540];
figurePos  = [100 100 figureSize];
hFigXPCSMain = figure(...
    'BackingStore','on',...
    'Units','pixels',...
    'DockControls','off',...
    'Resize','off',...
    'PaperOrient','portrait',...
    'PaperPositionMode','auto',...
    'IntegerHandle','off',...
    'NumberTitle','off',...
    'MenuBar','none',...
    'Toolbar','none',...
    'Name','XPCS Analysis Program (APS 8-ID-I)',...
    'Position',figurePos,...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_Fig',...
    'CloseRequestFcn',@xpcsmain_CloseRequestFcn,...
    'UserData',[]);
hAxes = axes(...
    'Parent',hFigXPCSMain,...
    'Units','pixels',...
    'Position',[0 0 figureSize],...
    'Tag','xpcsmain_Axes');
hPatchMain = patch(...
    'Parent',hAxes,...
    'XData',[0 1 1 0],...
    'YData',[0 0 1 1],...
    'FaceColor',[1 1 0.85],...
    'EdgeColor',[1 1 0.85]);
hGroupPatch = hggroup('Parent',hAxes);
hPatchButton = patch(...
    'Parent',hGroupPatch,...
    'XData',[0 0.2 0.2 0],...
    'YData',[0.1 0.1 1 1]);
hPatchMessage = patch(...
    'Parent',hGroupPatch,...
    'XData',[0.2 1 1 0.2],...
    'YData',[0.1 0.1 1 1]);
hPatchTimebar = patch(...
    'Parent',hGroupPatch,...
    'XData',[0 1 1 0],...
    'YData',[0 0 0.1 0.1]);
set(get(hGroupPatch,'Children'),...
    'FaceColor',[1 1 0.85],...
    'EdgeColor',[0.7 0.7 0.7]);


% --------------------------------
% --- file menu handles
% --------------------------------
hMenuFile = uimenu(hFigXPCSMain,...
    'Label','&File',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuFile');
hMenuFileLoadBatchinfo = uimenu(hMenuFile,...
    'Label','&Load Batchinfo...',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuFileLoadBatchInfo',...
    'Accelerator','L',...
    'callback',@loadbatchinfo);
hMenuFileLoadResult = uimenu(hMenuFile,...
    'Label','&Load and Show Results ...',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuFileLoadResult',...
    'Accelerator','R',...
    'callback',@loadresult);
hMenuFileClose = uimenu(hMenuFile,...
    'Label','&Close',...
    'HandleVisibility','callback',...
    'Separator','on',...
    'Tag','xpcsmain_MenuFileClose',...
    'Accelerator','W',...
    'callback',@xpcsmain_CloseRequestFcn);

% hMenuFileLoadImage = uimenu(hMenuFile,...
%     'Label','&Load Image Header...',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuFileLoadImage',...
%     'Accelerator','L',...
%     'callback',@loadimage);
% hMenuFileSave = uimenu(hMenuFile,...
%     'Label','&Save',...
%     'Position',3,...
%     'Separator','on',...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFileSave',...
%     'Accelerator','S',...
%     'callback','curvesave');
% hMenuFileSave2Workspace = uimenu(hMenuFile,...
%     'Label','Save to &Workspace',...
%     'Position',4,...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFileSave2Workspace',...
%     'callback','save2workspace');
% hMenuFilePreferences = uimenu(hMenuFile,...
%     'Label','Pre&ferences...',...
%     'Position',5,...
%     'Separator','on',...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFilePreferences',...
%     'callback','preferences');
% hMenuFilePageSetup = uimenu(hMenuFile,...
%     'Label','Pa&ge Setup...',...
%     'Position',6,...
%     'Separator','on',...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFilePageSetup',...
%     'callback','pagesetupdlg');
% hMenuFilePrintPreview = uimenu(hMenuFile,...
%     'Label','Print Pre&view...',...
%     'Position',7,...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFilePrintPreview',...
%     'callback',@specr_PrintPreviewFcn);
% hMenuFilePrint = uimenu(hMenuFile,...
%     'Label','&Print...',...
%     'Position',8,...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFilePrint',...
%     'Accelerator','P',...
%     'callback',@specr_PrintFcn);
% hMenuFilePrintToFigure = uimenu(hMenuFile,...
%     'Label','Print to Figure',...
%     'Position',9,...
%     'HandleVisibility','callback',...
%     'Tag','specr_MenuFilePrintToFigure',...
%     'callback','print2figure');

% 
% % --- Setup menu handles ---
% hMenuSetup = uimenu(hFigXPCSMain,...
%     'Label','&Setup',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuSetup');
% hMenuSetupBatchinfo = uimenu(hMenuSetup,...
%     'Label','&Batch Information',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuSetupBatchinfo',...
%     'callback',@viewinfo);    
% hMenuSetupAnalysis = uimenu(hMenuSetup,...
%     'Label','&Analysis Settings',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuSetupAnalysis',...
%     'callback',@viewanalysis);
% hMenuSetupSystem = uimenu(hMenuSetup,...
%     'Label','&System Settings',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuSetupSystem',...
%     'callback',@viewsystem);


% % --- Run menu handles ---
% hMenuRun = uimenu(hFigXPCSMain,...
%     'Label','&Run',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuRun');
% hMenuRunDynamics = uimenu(hMenuRun,...
%     'Label','&Dynamics',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuRunDynamics');
% %'callback',@viewinfo);
% hMenuRunStatic = uimenu(hMenuRun,...
%     'Label','&Static',...
%     'HandleVisibility','callback',...
%     'Tag','xpcsmain_MenuRunStatic');
% %'callback',@viewsystem);


% --- Tools menu handles ---
hMenuTools = uimenu(hFigXPCSMain,...
    'Label','&Tools',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuTools');
hMenuToolsViewSingleImage = uimenu(hMenuTools,...
    'Label','&Single Image Viewer',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuToolsViewSingleImage',...
    'callback',@viewsingleimage);
hMenuToolsMovie = uimenu(hMenuTools,...
    'Label','&Movie Maker',...
    'Separator','on',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuToolsMovie',...
    'callback',@xpcsmovie);


% --- Help menu handles ---
hMenuHelp = uimenu(hFigXPCSMain,...
    'Label','&Help',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuHelp');
hMenuHelpAnalysis = uimenu(hMenuHelp,...
    'Label','&XPCS Analysis Help',...
    'HandleVisibility','callback',...
    'Tag','xpcsmain_MenuHelpAnalysis');
%'callback',@viewinfo);
hMenuHelpAbout = uimenu(hMenuHelp,...
    'Label','&About XPCS Analysis Program',...
    'HandleVisibility','callback',...
    'Separator','on',...
    'Tag','xpcsmain_MenuHelpAbout',...
    'callback',@xpcsmain_MenuHelpAboutCallbackFcn);

% --- layout of control panel putshbuttons
text('Parent',hAxes,...
    'Position',[0.1 0.96],...
    'String','Control Panel',...
    'Units','normalized',...
    'HorizontalAlignment','Center',...
    'Color',[0.4 0.3 0]);
hPushbuttonLoad = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Load information file',...
    'position',[0.025 0.88 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonLoad',...
    'TooltipString','Load information file',...
    'callback',@loadbatchinfo);
hPushbuttonBatchinfo = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Batch Information',...
    'position',[0.025 0.82 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonBatchinfo',...
    'Enable','off',...
    'TooltipString','View and change batch information',...
    'callback',@viewinfo);
hPushbuttonAnalysis = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Analysis & Display',...
    'position',[0.025 0.76 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonAnalysis',...
    'Enable','off',...
    'TooltipString','View and change analysis & display settings',...
    'callback',@viewanalysis);
hPushbuttonSystem = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','System',...
    'position',[0.025 0.70 0.15 0.05],...
    'Enable','off',...
    'Tag','xpcsmain_PushbuttonSystem',...
    'TooltipString','View and change system settings: CCD, RAM ...',...
    'callback',@viewsystem);
hPushbuttonRun = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Run',...
    'position',[0.025 0.64 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonRun',...
    'TooltipString','Start analysis',...
    'Enable','off',...
    'callback',@runxpcs);
hPushbuttonStop = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Stop',...
    'Enable','off',...
    'position',[0.025 0.58 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonStop',...
    'TooltipString','Stop analysis');
hPushbuttonResult = uicontrol(hFigXPCSMain,...
    'Style','pushbutton',...
    'Units','normalized',...
    'String','Load&Show Results',...
    'position',[0.025 0.48 0.15 0.05],...
    'Tag','xpcsmain_PushbuttonResult',...
    'TooltipString','Show previously generated results',...
    'Enable','on',...
    'callback',@loadresult);


% --- layout of message panel
start_str = {'';...
    '';...
    '';...
    '';...
    '';...
    '';...
    '';...
    '';...
    '';...
    '';...
    '';...
    'X-Ray Photon Correlation Spectroscopy (XPCS) Analysis Program';...
    '';...
    'University of California at San Diego';...
    'Argonne National Laboratory';...
    };
uicontrol('Parent',hFigXPCSMain,...
    'Style','Edit',...
    'Units','normalized',...
    'Position',[0.2 0.1 0.8 0.9]',...
    'HorizontalAlignment','center',...
    'backgroundcolor',[1 1 0.85],...
    'ForegroundColor',[0.4 0.3 0],...
    'Enable','Inactive',...
    'Max',2,...
    'Min',0,...
    'String',start_str,...
    'Tag','xpcsmain_EditMessage');

    
%================================================================
% --- close request function of xpcsmain fig 
%================================================================
function xpcsmain_CloseRequestFcn(hObject,eventdata)
hFigXPCSMain = findall(0,'Tag','xpcsmain_Fig');
delete(findall(0,'Tag','viewinfo_Fig'));
delete(findall(0,'Tag','viewsystem_Fig'));
delete(findall(0,'Tag','mask_Fig'));
delete(findall(0,'Tag','viewanalysis_Fig'));
delete(findall(0,'Tag','showimage_fig1'));
delete(findall(0,'Tag','showimage_fig2'));
delete(findall(0,'Tag','showmaskpartition_Fig'));
delete(hFigXPCSMain);
return;
quitButton = questdlg('Quit without saving ?',...
    'Confirm Close','Quit','Cancel','Cancel');
switch quitButton
    case 'Cancel'
        return;
    case 'Quit'
        delete(hFigXPCSMain);
end


%================================================================
% --- Help About callback function
%================================================================
function xpcsmain_MenuHelpAboutCallbackFcn(hObject,eventdata)
text_string = {...
    'XPCS Analysis Program 2.0';...
    ' ';...
    'X-Ray and Soft Condensed Matter Group';...
    'Department of Physics';...
    'University of California, San Diego';...
    ' ';...
    'X-Ray Photon Correlation Spectroscopy Beamline 8-ID-I';...
    'Advanced Photon Source';...
    'Argonne National Laboratory';...
    ' ';...
    'Copyright 2006 Zhang Jiang, Michael Sprung'};
[logo.cdata,logo.colormap]=imread('Hlogo.gif');
hMSGBox = msgbox(text_string,'About XPCS Analysis Program','custom',logo.cdata,logo.colormap,'modal');
set(hMSGBox,'color',[1 1 0.85]);


